// unit.cpp

#include "CvGameCoreDLL.h"
#include "CvUnit.h"
#include "CvArea.h"
#include "CvPlot.h"
#include "CvCity.h"
#include "CvGlobals.h"
#include "CvGameCoreUtils.h"
#include "CvGameAI.h"
#include "CvMap.h"
#include "CvPlayerAI.h"
#include "CvRandom.h"
#include "CvTeamAI.h"
#include "CvGameCoreUtils.h"
#include "CyUnit.h"
#include "CyArgsList.h"
#include "CyPlot.h"
#include "CvDLLEntityIFaceBase.h"
#include "CvDLLInterfaceIFaceBase.h"
#include "CvDLLEngineIFaceBase.h"
#include "CvEventReporter.h"
#include "CvDLLPythonIFaceBase.h"
#include "CvDLLFAStarIFaceBase.h"
#include "CvInfos.h"
#include "FProfiler.h"
#include "CvPopupInfo.h"
#include "CvArtFileMgr.h"

// Public Functions...


CvUnit::CvUnit()
{
	m_aiExtraDomainModifier = new int[NUM_DOMAIN_TYPES];

	m_pabHasPromotion = NULL;

	m_paiTerrainDoubleMoveCount = NULL;
	m_paiFeatureDoubleMoveCount = NULL;
	m_paiExtraTerrainAttackPercent = NULL;
	m_paiExtraTerrainDefensePercent = NULL;
	m_paiExtraFeatureAttackPercent = NULL;
	m_paiExtraFeatureDefensePercent = NULL;
	m_paiExtraUnitCombatModifier = NULL;

	CvDLLEntity::createUnitEntity(this);		// create and attach entity to unit

	reset(0, NO_UNIT, NO_PLAYER, true);
}


CvUnit::~CvUnit()
{
	if (!gDLL->GetDone() && GC.IsGraphicsInitialized())						// don't need to remove entity when the app is shutting down, or crash can occur
	{
		gDLL->getEntityIFace()->RemoveUnitFromBattle(this);
		CvDLLEntity::removeEntity();		// remove entity from engine
	}

	CvDLLEntity::destroyEntity();			// delete CvUnitEntity and detach from us

	uninit();

	SAFE_DELETE_ARRAY(m_aiExtraDomainModifier);
}

void CvUnit::reloadEntity()
{
	//destroy old entity
	if (!gDLL->GetDone() && GC.IsGraphicsInitialized())						// don't need to remove entity when the app is shutting down, or crash can occur
	{
		gDLL->getEntityIFace()->RemoveUnitFromBattle(this);
		CvDLLEntity::removeEntity();		// remove entity from engine
	}

	CvDLLEntity::destroyEntity();			// delete CvUnitEntity and detach from us

	//creat new one
	CvDLLEntity::createUnitEntity(this);		// create and attach entity to unit
	setupGraphical();
}


void CvUnit::init(int iID, UnitTypes eUnit, UnitAITypes eUnitAI, PlayerTypes eOwner, int iX, int iY, DirectionTypes eFacingDirection)
{
	CvWString szBuffer;
	int iUnitName;
	int iI, iJ;

	FAssert(NO_UNIT != eUnit);

	//--------------------------------
	// Init saved data
	reset(iID, eUnit, eOwner);

	if(eFacingDirection == NO_DIRECTION)
		m_eFacingDirection = DIRECTION_SOUTH;
	else
		m_eFacingDirection = eFacingDirection;

	//--------------------------------
	// Init containers

	//--------------------------------
	// Init pre-setup() data
	setXY(iX, iY, false, false);

	//--------------------------------
	// Init non-saved data
	setupGraphical();

	//--------------------------------
	// Init other game data
	plot()->updateCenterUnit();

	plot()->setFlagDirty(true);

	iUnitName = GC.getGameINLINE().getUnitCreatedCount(getUnitType());
	int iNumNames = m_pUnitInfo->getNumUnitNames();
	if (iUnitName < iNumNames)
	{
		int iOffset = GC.getGameINLINE().getSorenRandNum(iNumNames, "Unit name selection");

		for (iI = 0; iI < iNumNames; iI++)
		{
			int iIndex = (iI + iOffset) % iNumNames;
			CvWString szName = gDLL->getText(m_pUnitInfo->getUnitNames(iIndex));
			if (!GC.getGameINLINE().isGreatPersonBorn(szName))
			{
				setName(szName);
				GC.getGameINLINE().addGreatPersonBornName(szName);
				break;
			}
		}
	}

	setGameTurnCreated(GC.getGameINLINE().getGameTurn());

	GC.getGameINLINE().incrementUnitCreatedCount(getUnitType());

	GC.getGameINLINE().incrementUnitClassCreatedCount((UnitClassTypes)(m_pUnitInfo->getUnitClassType()));
	GET_TEAM(getTeam()).changeUnitClassCount(((UnitClassTypes)(m_pUnitInfo->getUnitClassType())), 1);
	GET_PLAYER(getOwnerINLINE()).changeUnitClassCount(((UnitClassTypes)(m_pUnitInfo->getUnitClassType())), 1);

	GET_PLAYER(getOwnerINLINE()).changeExtraUnitCost(m_pUnitInfo->getExtraCost());

	if (m_pUnitInfo->getNukeRange() != -1)
	{
		GET_PLAYER(getOwnerINLINE()).changeNumNukeUnits(1);
	}

	if (m_pUnitInfo->isMilitarySupport())
	{
		GET_PLAYER(getOwnerINLINE()).changeNumMilitaryUnits(1);
	}

	GET_PLAYER(getOwnerINLINE()).changeAssets(m_pUnitInfo->getAssetValue());

	GET_PLAYER(getOwnerINLINE()).changePower(m_pUnitInfo->getPowerValue());

	for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	{
		if (m_pUnitInfo->getFreePromotions(iI))
		{
			setHasPromotion(((PromotionTypes)iI), true);
		}
	}

	FAssertMsg((GC.getNumTraitInfos() > 0), "GC.getNumTraitInfos() is less than or equal to zero but is expected to be larger than zero in CvUnit::init");
	for (iI = 0; iI < GC.getNumTraitInfos(); iI++)
	{
		if (GET_PLAYER(getOwnerINLINE()).hasTrait((TraitTypes)iI))
		{
			for (iJ = 0; iJ < GC.getNumPromotionInfos(); iJ++)
			{
				if (GC.getTraitInfo((TraitTypes) iI).isFreePromotion(iJ))
				{
					if ((getUnitCombatType() != NO_UNITCOMBAT) && GC.getTraitInfo((TraitTypes) iI).isFreePromotionUnitCombat(getUnitCombatType()))
					{
						setHasPromotion(((PromotionTypes)iJ), true);
					}
				}
			}

			//Sanguo Mod Civilization Trait, start, Liu Chong, TRAIT_ENGINEERING, add by nk, 11.23.08
			changeExtraFirstStrikes(GC.getTraitInfo((TraitTypes) iI).getNumFreeFirstStrikesUnitClass(getUnitClassType()));
			//Sanguo Mod Civilization Trait, end
		}
	}

	if (NO_UNITCOMBAT != getUnitCombatType())
	{
		for (iJ = 0; iJ < GC.getNumPromotionInfos(); iJ++)
		{
			if (GET_PLAYER(getOwnerINLINE()).isFreePromotion(getUnitCombatType(), (PromotionTypes)iJ))
			{
				setHasPromotion(((PromotionTypes)iJ), true);
			}
		}
	}

	if (NO_UNITCLASS != getUnitClassType())
	{
		for (iJ = 0; iJ < GC.getNumPromotionInfos(); iJ++)
		{
			if (GET_PLAYER(getOwnerINLINE()).isFreePromotion(getUnitClassType(), (PromotionTypes)iJ))
			{
				setHasPromotion(((PromotionTypes)iJ), true);
			}
		}
	}

	if (getDomainType() == DOMAIN_LAND)
	{
		if (baseCombatStr() > 0)
		{
			if ((GC.getGameINLINE().getBestLandUnit() == NO_UNIT) || (baseCombatStr() > GC.getGameINLINE().getBestLandUnitCombat()))
			{
				GC.getGameINLINE().setBestLandUnit(getUnitType());
			}
		}
	}

	if (getOwnerINLINE() == GC.getGameINLINE().getActivePlayer())
	{
		gDLL->getInterfaceIFace()->setDirty(GameData_DIRTY_BIT, true);
	}

	if (isWorldUnitClass((UnitClassTypes)(m_pUnitInfo->getUnitClassType())))
	{
		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_TEAM(getTeam()).isHasMet(GET_PLAYER((PlayerTypes)iI).getTeam()))
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_SOMEONE_CREATED_UNIT", GET_PLAYER(getOwnerINLINE()).getNameKey(), getNameKey());
					gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_WONDER_UNIT_BUILD", MESSAGE_TYPE_MAJOR_EVENT, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_UNIT_TEXT"), getX_INLINE(), getY_INLINE(), true, true);
				}
				else
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_UNKNOWN_CREATED_UNIT", getNameKey());
					gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_WONDER_UNIT_BUILD", MESSAGE_TYPE_MAJOR_EVENT, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_UNIT_TEXT"));
				}
			}
		}

		szBuffer = gDLL->getText("TXT_KEY_MISC_SOMEONE_CREATED_UNIT", GET_PLAYER(getOwnerINLINE()).getNameKey(), getNameKey());
		GC.getGameINLINE().addReplayMessage(REPLAY_MESSAGE_MAJOR_EVENT, getOwnerINLINE(), szBuffer, getX_INLINE(), getY_INLINE(), (ColorTypes)GC.getInfoTypeForString("COLOR_UNIT_TEXT"));
	}

    // Sanguo Mod Hero, start, added by poyuzhe 01.08.09
    if (m_pUnitInfo->isHiddenNationality())
    {
        changeHiddenNationalityCount(1);
    }

    if (m_pUnitInfo->isAlwaysHostile())
    {
		changeAlwaysHostileCount(1);
	}
    // Sanguo Mod Hero, end

	AI_init(eUnitAI);

	// Sanguo Mod tutorial, start, by poyuzhe 09.27.09
	// Temporary, why unit created event do not work?
	if (AI_getUnitAIType() == UNITAI_EMPEROR)
	{
		if (!GET_PLAYER(getOwnerINLINE()).isFeatAccomplished(FEAT_CAPTURE_EMPEROR))
		{
			GET_PLAYER(getOwnerINLINE()).setFeatAccomplished(FEAT_CAPTURE_EMPEROR, true);
			if (GET_PLAYER(getOwnerINLINE()).isFeatPopup())
			{
				CyArgsList argsList;

				argsList.add(getOwnerINLINE());
				argsList.add(getID());

				gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doUnitCreatedFeat", argsList.makeFunctionArgs());
			}
		}
	}
	// Sanguo Mod Tutorial, end

	CvEventReporter::getInstance().unitCreated(this);
}


void CvUnit::uninit()
{
	SAFE_DELETE_ARRAY(m_pabHasPromotion);

	SAFE_DELETE_ARRAY(m_paiTerrainDoubleMoveCount);
	SAFE_DELETE_ARRAY(m_paiFeatureDoubleMoveCount);
	SAFE_DELETE_ARRAY(m_paiExtraTerrainAttackPercent);
	SAFE_DELETE_ARRAY(m_paiExtraTerrainDefensePercent);
	SAFE_DELETE_ARRAY(m_paiExtraFeatureAttackPercent);
	SAFE_DELETE_ARRAY(m_paiExtraFeatureDefensePercent);
	SAFE_DELETE_ARRAY(m_paiExtraUnitCombatModifier);
}


// FUNCTION: reset()
// Initializes data members that are serialized.
void CvUnit::reset(int iID, UnitTypes eUnit, PlayerTypes eOwner, bool bConstructorCall)
{
	int iI;

	//--------------------------------
	// Uninit class
	uninit();

	m_iID = iID;
	m_iGroupID = FFreeList::INVALID_INDEX;
	m_iHotKeyNumber = -1;
	m_iX = INVALID_PLOT_COORD;
	m_iY = INVALID_PLOT_COORD;
	m_iLastMoveTurn = 0;
	m_iReconX = INVALID_PLOT_COORD;
	m_iReconY = INVALID_PLOT_COORD;
	m_iGameTurnCreated = 0;
	m_iDamage = 0;
	m_iMoves = 0;
	m_iExperience = 0;
	m_iLevel = 1;
	m_iCargo = 0;
	m_iAttackPlotX = INVALID_PLOT_COORD;
	m_iAttackPlotY = INVALID_PLOT_COORD;
	m_iCombatTimer = 0;
	m_iCombatFirstStrikes = 0;
	m_iFortifyTurns = 0;
	m_iBlitzCount = 0;
	m_iAmphibCount = 0;
	m_iRiverCount = 0;
	m_iEnemyRouteCount = 0;
	m_iAlwaysHealCount = 0;
	m_iHillsDoubleMoveCount = 0;
	m_iImmuneToFirstStrikesCount = 0;
	m_iExtraVisibilityRange = 0;
	m_iExtraMoves = 0;
	m_iExtraMoveDiscount = 0;
	m_iExtraAirRange = 0;
	m_iExtraIntercept = 0;
	m_iExtraEvasion = 0;
	m_iExtraFirstStrikes = 0;
	m_iExtraChanceFirstStrikes = 0;
	m_iExtraWithdrawal = 0;
	m_iExtraCollateralDamage = 0;
	m_iExtraBombardRate = 0;
	m_iExtraEnemyHeal = 0;
	m_iExtraNeutralHeal = 0;
	m_iExtraFriendlyHeal = 0;
	m_iSameTileHeal = 0;
	m_iAdjacentTileHeal = 0;
	m_iExtraCombatPercent = 0;
	m_iExtraCityAttackPercent = 0;
	m_iExtraCityDefensePercent = 0;
	m_iExtraHillsAttackPercent = 0;
	m_iExtraHillsDefensePercent = 0;
	m_iRevoltProtection = 0;
	m_iCollateralDamageProtection = 0;
	m_iPillageChange = 0;
	m_iUpgradeDiscount = 0;
	m_iExperiencePercent = 0;
	m_iKamikazePercent = 0;
	m_eFacingDirection = DIRECTION_SOUTH;
	m_iImmobileTimer = 0;

    // Sanguo Mod Hero, start, added by poyuzhe 01.08.09
    m_eHeroType = NO_HERO;
    m_eLegionHero = NO_HERO;
    m_eLeaderHero = NO_HERO;
    m_bTempCheck = false;
    m_iTempStrengthModifier = 0;
    m_iSurroundingEnemyHealthChange = 0;
    m_iDCMBombRange = 0;
    m_iDefendFromChargeCount = 0;
    m_iInvisibleType = NO_INVISIBLE;
    m_iHiddenNationalityCount = 0;
    m_iAlwaysHostileCount = 0;
    m_iAdditionalAttackChance = 0;
	m_iWarGodCount = 0;
	m_iIgnoreDefenderTerrainCount = 0;
	m_iWaterSideStrModifier = 0;
	m_iPlotDefenseModifier = 0;
	m_iGuardCount = 0;
	m_iLegionTurn = 0;
    // Sanguo Mod Hero, end

	m_bMadeAttack = false;
	m_bMadeInterception = false;
	m_bPromotionReady = false;
	m_bDeathDelay = false;
	m_bCombatFocus = false;
	m_bInfoBarDirty = false;
	m_bBlockading = false;
	m_bAirCombat = false;

	m_eOwner = eOwner;
	m_eCapturingPlayer = NO_PLAYER;
	m_eUnitType = eUnit;
	m_pUnitInfo = (NO_UNIT != m_eUnitType) ? &GC.getUnitInfo(m_eUnitType) : NULL;
	m_iBaseCombat = (NO_UNIT != m_eUnitType) ? m_pUnitInfo->getCombat() : 0;
	m_eLeaderUnitType = NO_UNIT;
	m_iCargoCapacity = (NO_UNIT != m_eUnitType) ? m_pUnitInfo->getCargoSpace() : 0;

	m_combatUnit.reset();
	m_transportUnit.reset();

	for (iI = 0; iI < NUM_DOMAIN_TYPES; iI++)
	{
		m_aiExtraDomainModifier[iI] = 0;
	}

	m_szName.clear();
	m_szScriptData ="";

	if (!bConstructorCall)
	{
		FAssertMsg((0 < GC.getNumPromotionInfos()), "GC.getNumPromotionInfos() is not greater than zero but an array is being allocated in CvUnit::reset");
		m_pabHasPromotion = new bool[GC.getNumPromotionInfos()];
		for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		{
			m_pabHasPromotion[iI] = false;
		}

		FAssertMsg((0 < GC.getNumTerrainInfos()), "GC.getNumTerrainInfos() is not greater than zero but a float array is being allocated in CvUnit::reset");
		m_paiTerrainDoubleMoveCount = new int[GC.getNumTerrainInfos()];
		m_paiExtraTerrainAttackPercent = new int[GC.getNumTerrainInfos()];
		m_paiExtraTerrainDefensePercent = new int[GC.getNumTerrainInfos()];
		for (iI = 0; iI < GC.getNumTerrainInfos(); iI++)
		{
			m_paiTerrainDoubleMoveCount[iI] = 0;
			m_paiExtraTerrainAttackPercent[iI] = 0;
			m_paiExtraTerrainDefensePercent[iI] = 0;
		}

		FAssertMsg((0 < GC.getNumFeatureInfos()), "GC.getNumFeatureInfos() is not greater than zero but a float array is being allocated in CvUnit::reset");
		m_paiFeatureDoubleMoveCount = new int[GC.getNumFeatureInfos()];
		m_paiExtraFeatureDefensePercent = new int[GC.getNumFeatureInfos()];
		m_paiExtraFeatureAttackPercent = new int[GC.getNumFeatureInfos()];
		for (iI = 0; iI < GC.getNumFeatureInfos(); iI++)
		{
			m_paiFeatureDoubleMoveCount[iI] = 0;
			m_paiExtraFeatureAttackPercent[iI] = 0;
			m_paiExtraFeatureDefensePercent[iI] = 0;
		}

		FAssertMsg((0 < GC.getNumUnitCombatInfos()), "GC.getNumUnitCombatInfos() is not greater than zero but an array is being allocated in CvUnit::reset");
		m_paiExtraUnitCombatModifier = new int[GC.getNumUnitCombatInfos()];
		for (iI = 0; iI < GC.getNumUnitCombatInfos(); iI++)
		{
			m_paiExtraUnitCombatModifier[iI] = 0;
		}

		AI_reset();
	}
}


//////////////////////////////////////
// graphical only setup
//////////////////////////////////////
void CvUnit::setupGraphical()
{
	if (!GC.IsGraphicsInitialized())
	{
		return;
	}

	CvDLLEntity::setup();

	if (getGroup()->getActivityType() == ACTIVITY_INTERCEPT)
	{
		airCircle(true);
	}
}


void CvUnit::convert(CvUnit* pUnit)
{
	CvPlot* pPlot = plot();

	for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	{
		setHasPromotion(((PromotionTypes)iI), (pUnit->isHasPromotion((PromotionTypes)iI) || m_pUnitInfo->getFreePromotions(iI)));
	}

	setGameTurnCreated(pUnit->getGameTurnCreated());
	setDamage(pUnit->getDamage());
	setMoves(pUnit->getMoves());

	setLevel(pUnit->getLevel());
	int iOldModifier = std::max(1, 100 + GET_PLAYER(pUnit->getOwnerINLINE()).getLevelExperienceModifier());
	int iOurModifier = std::max(1, 100 + GET_PLAYER(getOwnerINLINE()).getLevelExperienceModifier());
	setExperience(std::max(0, (pUnit->getExperience() * iOurModifier) / iOldModifier));

	setName(pUnit->getNameNoDesc());
	setLeaderUnitType(pUnit->getLeaderUnitType());

	CvUnit* pTransportUnit = pUnit->getTransportUnit();
	if (pTransportUnit != NULL)
	{
		pUnit->setTransportUnit(NULL);
		setTransportUnit(pTransportUnit);
	}

	std::vector<CvUnit*> aCargoUnits;
	pUnit->getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
/************************************************************************************************/
/* UNOFFICIAL_PATCH                       10/30/09                     Mongoose & jdog5000      */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
/* original BTS code
		aCargoUnits[i]->setTransportUnit(this);
*/
		// From Mongoose SDK
		// Check cargo types and capacity when upgrading transports
		if (cargoSpaceAvailable(aCargoUnits[i]->getSpecialUnitType(), aCargoUnits[i]->getDomainType()) > 0)
		{
			aCargoUnits[i]->setTransportUnit(this);
		}
		else
		{
			aCargoUnits[i]->setTransportUnit(NULL);
			aCargoUnits[i]->jumpToNearestValidPlot();
		}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/
	}

	// Sanguo Mod Hero, start, added by poyuzhe 01.18.09
	setTempStrengthModifier(pUnit->getTempStrengthModifier());
	setHeroType(pUnit->getHeroType());
	setLegionHero(pUnit->getLegionHero());
	setLeaderHero(pUnit->getLeaderHero());
	m_iLegionTurn = pUnit->getLegionTurn();
	if (pUnit->getLegionHero() != NO_HERO)
	{
	    // If the unit is a legion member
	    CvUnit* pHero = GC.getGameINLINE().getHeroUnit(getLegionHero());
	    FAssert (pHero != NULL);
	    if (GC.getGameINLINE().getHeroCoreTroop(pUnit->getLegionHero()) == pUnit->getID())
	    {
	        // If the unit is a core troop
	        FAssert(pUnit->getHeroType() == pUnit->getLegionHero());
	        FAssert(pHero->getID() == pUnit->getID());
	        GC.getGameINLINE().setHeroUnitID(getLegionHero(), getID());
			GC.getGameINLINE().tempRemoveHeroCoreTroop(getLegionHero());
	        GC.getGameINLINE().addHeroCoreTroop(getLegionHero(), getID());

			// if this unit is in the Army
			if (pUnit->getLeaderHero() != NO_HERO)
			{
				if (pUnit->isLeaderTroop())
				{
					GC.getGameINLINE().setHeroUnitID(getLeaderHero(), getID());
					GC.getGameINLINE().tempRemoveHeroCoreTroop(getLeaderHero());
					GC.getGameINLINE().addHeroCoreTroop(getLeaderHero(), getID());
				}
				else
				{
					GC.getGameINLINE().deleteHeroLegionMember(getLeaderHero(), pUnit->getID());
					GC.getGameINLINE().addHeroLegionMember(getLeaderHero(), getID());
				}
				pUnit->setLeaderHero(NO_HERO);
			}

	        // Set the old unit's hero type and leader unit to none
	        pUnit->setHeroType(NO_HERO);
	        pUnit->setLeaderUnitType(NO_UNIT);
	    }
	    else
	    {
	        // If the unit is just a plain member
	        GC.getGameINLINE().deleteHeroLegionMember(getLegionHero(), pUnit->getID());
	        GC.getGameINLINE().addHeroLegionMember(getLegionHero(), getID());
			pUnit->setLeaderHero(NO_HERO);
	    }
	    // Set the old unit's legion hero to none
        pUnit->setLegionHero(NO_HERO);
        // Now the old unit is nothing but a plain normal unit
	}
    // Sanguo Mod Hero, end

	pUnit->kill(true);
}


void CvUnit::kill(bool bDelay, PlayerTypes ePlayer)
{
	PROFILE_FUNC();

	CLLNode<IDInfo>* pUnitNode;
	CvUnit* pTransportUnit;
	CvUnit* pLoopUnit;
	CvPlot* pPlot;
	CvWString szBuffer;
	PlayerTypes eOwner;
	PlayerTypes eCapturingPlayer;
	UnitTypes eCaptureUnitType;
	// Sanguo Mod Hero, start, added by poyuzhe 07.22.09
	HeroTypes eHero = getHeroType();
	// Sanguo Mod Hero, end

	pPlot = plot();
	FAssertMsg(pPlot != NULL, "Plot is not assigned a valid value");

	static std::vector<IDInfo> oldUnits;
	oldUnits.clear();
	pUnitNode = pPlot->headUnitNode();

	while (pUnitNode != NULL)
	{
		oldUnits.push_back(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);
	}

	for (uint i = 0; i < oldUnits.size(); i++)
	{
		pLoopUnit = ::getUnit(oldUnits[i]);

		if (pLoopUnit != NULL)
		{
			if (pLoopUnit->getTransportUnit() == this)
			{
				//save old units because kill will clear the static list
				std::vector<IDInfo> tempUnits = oldUnits;

				if (pPlot->isValidDomainForLocation(*pLoopUnit))
				{
					pLoopUnit->setCapturingPlayer(NO_PLAYER);
				}

				// Sanguo Mod Hero, start, added by poyuzhe 02.16.09
				if (pLoopUnit->getLegionHero() != NO_HERO)
				{
					pLoopUnit->doLegionMemberDefeated(NULL, false, false, ePlayer);
				}
				// Sanguo Mod Hero, end

				pLoopUnit->kill(false, ePlayer);

				oldUnits = tempUnits;
			}
		}
	}

	if (ePlayer != NO_PLAYER && bDelay)
	{
        // Sanguo Mod Hero, start, added by poyuzhe 02.08.09
        if (getHeroType() == NO_HERO)
        {
            // the following line indented once
     		CvEventReporter::getInstance().unitKilled(this, ePlayer);
        }
        // Sanguo Mod Hero, end

		if (NO_UNIT != getLeaderUnitType())
		{
			// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
			if (getHeroType() == NO_HERO)
			{
			    // the following lines are indented once
                for (int iI = 0; iI < MAX_PLAYERS; iI++)
                {
                    if (GET_PLAYER((PlayerTypes)iI).isAlive())
                    {
                        szBuffer = gDLL->getText("TXT_KEY_MISC_GENERAL_KILLED", getNameNoDesc().GetCString());
                        gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
                    }
                }
                // end indented
			}
			// This is when a Core Troop is killed
			else if (getLegionHero() != NO_HERO)
			{
			    for (int iI = 0; iI < MAX_PLAYERS; iI++)
                {
                    if (GET_PLAYER((PlayerTypes)iI).isAlive())
                    {
                        szBuffer = gDLL->getText("TXT_KEY_MISC_GENERAL_KILLED", GC.getHeroInfo(getLegionHero()).getDescription());
                        gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
                    }
                }
			}
			// Sanguo Mod Hero, end
		}
	}

	if (bDelay)
	{
		startDelayedDeath();
		return;
	}

	if (isMadeAttack() && nukeRange() != -1)
	{
		CvPlot* pTarget = getAttackPlot();
		if (pTarget)
		{
			pTarget->nukeExplosion(nukeRange(), this);
			setAttackPlot(NULL, false);
		}
	}

	finishMoves();

	if (IsSelected())
	{
		if (gDLL->getInterfaceIFace()->getLengthSelectionList() == 1)
		{
			if (!(gDLL->getInterfaceIFace()->isFocused()) && !(gDLL->getInterfaceIFace()->isCitySelection()) && !(gDLL->getInterfaceIFace()->isDiploOrPopupWaiting()))
			{
				GC.getGameINLINE().updateSelectionList();
			}

			if (IsSelected())
			{
				gDLL->getInterfaceIFace()->setCycleSelectionCounter(1);
			}
			else
			{
				gDLL->getInterfaceIFace()->setDirty(SelectionCamera_DIRTY_BIT, true);
			}
		}
	}

	gDLL->getInterfaceIFace()->removeFromSelectionList(this);

	// XXX this is NOT a hack, without it, the game crashes.
	gDLL->getEntityIFace()->RemoveUnitFromBattle(this);

	FAssertMsg(!isCombat(), "isCombat did not return false as expected");

	// Sanguo Mod Hero, start, added by poyuzhe 01.18.09
	// if we killed our own core troop for some weird reason
	// actually not weird enough, it happens when we are eliminated
	// It's not an enemy core troop because doLegionMemberDefeated() is called before kill()
	if (isCoreTroop())
	{
		if (GC.getGameINLINE().getHeroLegionSize(getHeroType()) == 1)
		{
			GC.getGameINLINE().clearHeroLegionMember(getHeroType());
			CvUnit* pHero = NULL;
			if (GET_PLAYER(getOwnerINLINE()).getNumCities() > 0 || !GET_PLAYER(getOwnerINLINE()).isFoundedFirstCity())
			{
			    pHero = GET_PLAYER(getOwnerINLINE()).initHero(getHeroType(), plot()->getX_INLINE(), plot()->getY_INLINE(), GC.getGameINLINE().getHeroDebutStatus(getHeroType()) != getOwnerINLINE());
			}
			if (pHero != NULL)
			{
				if (!plot()->isValidDomainForLocation(*pHero))
				{
					CvCity* pCapital = GET_PLAYER(getOwnerINLINE()).getCapitalCity();
					if (pCapital != NULL)
					{
						pHero->setXY(pCapital->getX_INLINE(), pCapital->getY_INLINE());
					}
					else
					{
						pHero->jumpToNearestValidPlot();
					}
				}
			}
			else
			{
				GC.getGameINLINE().setHeroDebutStatus(getHeroType(), -2);
				GC.getGameINLINE().setHeroUnitID(getHeroType(), -1);
				GET_PLAYER(getOwnerINLINE()).changeHeroMaintenance(getHeroType(), false, true, 0);
			}
		}
		else
		{
			FAssert (getHeroType() >= 0 && getHeroType() < GC.getNumHeroInfos());
			GC.getGameINLINE().backUpHeroLegionMember(getHeroType());
			GC.getGameINLINE().deleteHeroCoreTroop(getHeroType());
			processLegion(true, false, false, false);
		}
	}
	// if we killed our own legion member
	else if (getLegionHero() != NO_HERO)
	{
		GC.getGameINLINE().deleteHeroLegionMember(getLegionHero(), getID());
		setLegionHero(NO_HERO);
	}
	// if we killed our own naked hero (most likely to happen when our civ is destroyed and have to kill every unit alive)
	// another case would be when our hero is executed, died in combat or of natural age
	else if (ePlayer == NO_PLAYER && isNakedHero() && GC.getGameINLINE().getHeroUnitID(getHeroType()) == getID())
	{
		if (GC.getGameINLINE().getHeroVitality(getHeroType()) > 0)
		{
			GC.getGameINLINE().setHeroDebutStatus(getHeroType(), -2);
		}
		else
		{
			GC.getGameINLINE().setHeroDebutStatus(getHeroType(), -3);
		}
		GC.getGameINLINE().setHeroUnitID(getHeroType(), -1);
		GET_PLAYER(getOwnerINLINE()).changeHeroMaintenance(getHeroType(), false, true, 0);
	}

	if (ePlayer != NO_PLAYER && isNakedHero())
	{
		if (GC.getGameINLINE().getHeroVitality(getHeroType()) > 0)
		{
			FAssert (GC.getGameINLINE().getHeroUnitID(getHeroType()) == getID());

			// If it's a naked hero unit
			// Do the hero escape and surrender stuff and else
			int iDieRand = GC.getGameINLINE().getSorenRandNum(100, "Hero Die Rand");
			bool bHeroProcessed = false;
			bool bCurrentLeader = (GC.getHeroInfo(getHeroType()).getLeaderType() == GET_PLAYER(getOwnerINLINE()).getLeaderType());
			bool bCurrentCiv = (GC.getGameINLINE().getCurrentDefaultCivilization(getHeroType()) == getCivilizationType());

			if (!bHeroProcessed && !isBarbarian() && ((GET_PLAYER(ePlayer).isBarbarian())
				? true : (iDieRand < (isBarbarian() ? 0 : GC.getDefineINT("HERO_ESCAPE_CHANCE")))))
			{
				CvCity* pCity = GET_PLAYER(getOwnerINLINE()).getCapitalCity();
				if (pCity)
				{
					// note that if the hero is at capital city, which means the capital has fallen, the hero will be captured if we only have 1 city left		
					if (!at(pCity->getX_INLINE(), pCity->getY_INLINE()) || GET_PLAYER(getOwnerINLINE()).getNumCities() > 1)
					{
						CvCity* pNearestCity = GC.getMapINLINE().findCity(getX_INLINE(), getY_INLINE(), getOwnerINLINE(), NO_TEAM, !(plot()->isWater()), false, NO_TEAM, NO_DIRECTION, (plot()->isCity() ? plot()->getPlotCity() : NULL));
						if (pNearestCity == NULL)
						{
							pNearestCity = GC.getMapINLINE().findCity(plot()->getX_INLINE(), plot()->getY_INLINE(), getOwnerINLINE(), NO_TEAM, false, false, NO_TEAM, NO_DIRECTION, (plot()->isCity() ? plot()->getPlotCity() : NULL));
						}
						FAssert (pNearestCity != NULL);
						setXY(pNearestCity->getX_INLINE(), pNearestCity->getY_INLINE());
						bHeroProcessed = true;
					}

					if (bHeroProcessed)
					{
						setDamage(0);
						finishMoves();
						m_eCapturingPlayer = NO_PLAYER;
						szBuffer = gDLL->getText("TXT_KEY_MISC_HERO_ESCAPED", getNameKey());
						gDLL->getInterfaceIFace()->addMessage(ePlayer, false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_MAJOR_EVENT);
						return;
					}
				}
				else
				{
					GC.getGameINLINE().setHeroUnitID(getHeroType(), -1);
					GC.getGameINLINE().setHeroDebutStatus(getHeroType(), -2);

					m_eCapturingPlayer = NO_PLAYER;

					bHeroProcessed = true;
				}
			}

			// Hero Capture
			if (!bHeroProcessed)
			{
				FAssert (!GET_PLAYER(ePlayer).isBarbarian());
				GC.getGameINLINE().setHeroUnitID(getHeroType(), -1);
				GET_PLAYER(getOwnerINLINE()).changeHeroMaintenance(getHeroType(), false, true, 0);

				GC.getGameINLINE().setHeroCapturePlayer(eHero, ePlayer);

				if (GET_PLAYER(ePlayer).isHuman())
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_CAPTURED_HERO", GC.getHeroInfo(eHero).getDescription());
					gDLL->getInterfaceIFace()->addMessage(ePlayer, false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO);
				}
				if (GET_PLAYER(getOwnerINLINE()).isHuman())
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_OUR_HERO_CAPTURED", GC.getHeroInfo(eHero).getDescription(), GET_PLAYER(ePlayer).getCivilizationDescription());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO);
				}

				// Sanguo Mod tutorial, start, by poyuzhe 09.27.09
				if (!GET_PLAYER(ePlayer).isFeatAccomplished(FEAT_CAPTURE_HERO))
				{
					GET_PLAYER(ePlayer).setFeatAccomplished(FEAT_CAPTURE_HERO, true);
					if (GET_PLAYER(ePlayer).isFeatPopup())
					{
						CyArgsList argsList;

						argsList.add(getOwnerINLINE());
						argsList.add(eHero);

						gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doCaptureHeroFeat", argsList.makeFunctionArgs());
					}
				}
				// Sanguo Mod Tutorial, end
			}
		}
    }
    // Sanguo Mod Hero, end

	pTransportUnit = getTransportUnit();

	if (pTransportUnit != NULL)
	{
		setTransportUnit(NULL);
	}

	setReconPlot(NULL);
	setBlockading(false);

	FAssertMsg(getAttackPlot() == NULL, "The current unit instance's attack plot is expected to be NULL");
	FAssertMsg(getCombatUnit() == NULL, "The current unit instance's combat unit is expected to be NULL");

	GET_TEAM(getTeam()).changeUnitClassCount((UnitClassTypes)m_pUnitInfo->getUnitClassType(), -1);
	GET_PLAYER(getOwnerINLINE()).changeUnitClassCount((UnitClassTypes)m_pUnitInfo->getUnitClassType(), -1);

	GET_PLAYER(getOwnerINLINE()).changeExtraUnitCost(-(m_pUnitInfo->getExtraCost()));

	if (m_pUnitInfo->getNukeRange() != -1)
	{
		GET_PLAYER(getOwnerINLINE()).changeNumNukeUnits(-1);
	}

	if (m_pUnitInfo->isMilitarySupport())
	{
		GET_PLAYER(getOwnerINLINE()).changeNumMilitaryUnits(-1);
	}

	GET_PLAYER(getOwnerINLINE()).changeAssets(-(m_pUnitInfo->getAssetValue()));

	GET_PLAYER(getOwnerINLINE()).changePower(-(m_pUnitInfo->getPowerValue()));
	GET_PLAYER(getOwnerINLINE()).AI_changeNumAIUnits(AI_getUnitAIType(), -1);

	eOwner = getOwnerINLINE();
	eCapturingPlayer = getCapturingPlayer();
	eCaptureUnitType = ((eCapturingPlayer != NO_PLAYER) ? getCaptureUnitType(GET_PLAYER(eCapturingPlayer).getCivilizationType()) : NO_UNIT);

	setXY(INVALID_PLOT_COORD, INVALID_PLOT_COORD, true);

	joinGroup(NULL, false, false);

	CvEventReporter::getInstance().unitLost(this);

	// Sanguo Mod Hero, start, added by poyuzhe 02.16.09
	if (m_pUnitInfo->isHero())
	{
		GC.getGameINLINE().decrementUnitCreatedCount(getUnitType());
		GC.getGameINLINE().decrementUnitClassCreatedCount(getUnitClassType());
	}
	// Sanguo Mod Hero, end

	GET_PLAYER(getOwnerINLINE()).deleteUnit(getID());

	if ((eCapturingPlayer != NO_PLAYER) && (eCaptureUnitType != NO_UNIT) && !(GET_PLAYER(eCapturingPlayer).isBarbarian()))
	{
		if (GET_PLAYER(eCapturingPlayer).isHuman() || GET_PLAYER(eCapturingPlayer).AI_captureUnit(eCaptureUnitType, pPlot) || 0 == GC.getDefineINT("AI_CAN_DISBAND_UNITS"))
		{
			CvUnit* pkCapturedUnit = GET_PLAYER(eCapturingPlayer).initUnit(eCaptureUnitType, pPlot->getX_INLINE(), pPlot->getY_INLINE());

			if (pkCapturedUnit != NULL)
			{
				// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
				// These lines should never execute now since heroes do not have a capture unit class now. They are imprisoned by default
				if (pkCapturedUnit->getUnitInfo().isHero())
				{
				    if (eHero != NO_HERO)
				    {
                        pkCapturedUnit->setHeroType(eHero);
                        if (GC.getGameINLINE().getHeroDebutStatus(eHero) != eCapturingPlayer)
                        {
                            GC.getGameINLINE().setHeroDebutStatus(eHero, eCapturingPlayer);
                            GET_PLAYER(eCapturingPlayer).changeHeroMaintenance(eHero, true, false, 0);
                        }
                        GC.getGameINLINE().setHeroUnitID(eHero, pkCapturedUnit->getID());
				    }

				    szBuffer = gDLL->getText("TXT_KEY_HERO_SURRENDERED", GC.getHeroInfo(eHero).getDescription());
                    gDLL->getInterfaceIFace()->addMessage(eCapturingPlayer, true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO, pkCapturedUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

                    szBuffer = gDLL->getText("TXT_KEY_HERO_SURRENDER_TO_ENEMY", GC.getHeroInfo(eHero).getDescription());
                    gDLL->getInterfaceIFace()->addMessage(eOwner, true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO, pkCapturedUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

                    CyArgsList argsList;

                    argsList.add(eHero);
                    argsList.add(eCapturingPlayer);
                    argsList.add(eOwner);

                    gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doHeroSurrender", argsList.makeFunctionArgs());
                }
				else
				{
				    // the following 2 lines are indented once
                    szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_CAPTURED_UNIT", GC.getUnitInfo(eCaptureUnitType).getTextKeyWide());
                    gDLL->getInterfaceIFace()->addMessage(eCapturingPlayer, true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO, pkCapturedUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
                }

                if (!pPlot->isValidDomainForLocation(*pkCapturedUnit))
				{
					CvCity* pEnemyCapital = GET_PLAYER(eCapturingPlayer).getCapitalCity();
					if (pEnemyCapital != NULL)
					{
						pkCapturedUnit->setXY(pEnemyCapital->getX_INLINE(), pEnemyCapital->getY_INLINE());
					}
					else
					{
						pkCapturedUnit->jumpToNearestValidPlot();
					}
				}
                // Sanguo Mod Hero, end

				// Add a captured mission
				CvMissionDefinition kMission;
				kMission.setMissionTime(GC.getMissionInfo(MISSION_CAPTURED).getTime() * gDLL->getSecsPerTurn());
				kMission.setUnit(BATTLE_UNIT_ATTACKER, pkCapturedUnit);
				kMission.setUnit(BATTLE_UNIT_DEFENDER, NULL);
				kMission.setPlot(pPlot);
				kMission.setMissionType(MISSION_CAPTURED);
				gDLL->getEntityIFace()->AddMission(&kMission);

				pkCapturedUnit->finishMoves();

				if (!GET_PLAYER(eCapturingPlayer).isHuman())
				{
					CvPlot* pPlot = pkCapturedUnit->plot();
					if (pPlot && !pPlot->isCity(false))
					{
						// Sanguo Mod Emperor, start, added by poyuzhe 12.25.08
						//if (GET_PLAYER(eCapturingPlayer).AI_getPlotDanger(pPlot) && GC.getDefineINT("AI_CAN_DISBAND_UNITS"))
						if (GET_PLAYER(eCapturingPlayer).AI_getPlotDanger(pPlot) && GC.getDefineINT("AI_CAN_DISBAND_UNITS") && GC.getUnitInfo(eCaptureUnitType).getDefaultUnitAIType() != UNITAI_EMPEROR && !GC.getUnitInfo(eCaptureUnitType).isHero())
						// Sanguo Mod Emperor, end
						{
							pkCapturedUnit->kill(false);
						}
					}
				}
			}
		}
	}
}


void CvUnit::NotifyEntity(MissionTypes eMission)
{
	gDLL->getEntityIFace()->NotifyEntity(getUnitEntity(), eMission);
}


void CvUnit::doTurn()
{
	PROFILE("CvUnit::doTurn()")

	FAssertMsg(!isDead(), "isDead did not return false as expected");
	FAssertMsg(getGroup() != NULL, "getGroup() is not expected to be equal with NULL");

	testPromotionReady();

	if (isBlockading())
	{
		collectBlockadeGold();
	}

	if (isSpy() && isIntruding() && !isCargo())
	{
		TeamTypes eTeam = plot()->getTeam();
		if (NO_TEAM != eTeam)
		{
			if (GET_TEAM(getTeam()).isOpenBorders(eTeam))
			{
				testSpyIntercepted(plot()->getOwnerINLINE(), GC.getDefineINT("ESPIONAGE_SPY_NO_INTRUDE_INTERCEPT_MOD"));
			}
			else
			{
				testSpyIntercepted(plot()->getOwnerINLINE(), GC.getDefineINT("ESPIONAGE_SPY_INTERCEPT_MOD"));
			}
		}
	}

	if (baseCombatStr() > 0)
	{
		FeatureTypes eFeature = plot()->getFeatureType();
		if (NO_FEATURE != eFeature)
		{
			if (0 != GC.getFeatureInfo(eFeature).getTurnDamage())
			{
				// Sanguo Mod Swamp, start, added by poyuzhe 07.07.09
				bool bNoHealthChange = false;
				PROFILE_BEGIN("UnitDoTurn::featureDamage");
				for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		        {
		            if (isHasPromotion((PromotionTypes)iI))
                    {
                        if ((FeatureTypes)GC.getPromotionInfo((PromotionTypes)iI).getNoHealthPercentFeature() == eFeature)
                        {
		                     bNoHealthChange = true;
		                     break;
                        }
		            }
		        }
		        if (!bNoHealthChange)
		        {
                    changeDamage(GC.getFeatureInfo(eFeature).getTurnDamage(), NO_PLAYER);
		        }
				PROFILE_END();
		        //changeDamage(GC.getFeatureInfo(eFeature).getTurnDamage(), NO_PLAYER);
		        // Sanguo Mod Swamp, end
			}
		}

		// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
		if (m_iSurroundingEnemyHealthChange < 0)
		{
			PROFILE_BEGIN("UnitDoTurn::surroudingDamage");
			int iI;
		    CvPlot* pLoopPlot;
		    CvUnit* pLoopUnit;
		    for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
            {
                pLoopPlot = plotDirection(plot()->getX_INLINE(), plot()->getY_INLINE(), ((DirectionTypes)iI));

                if (pLoopPlot != NULL && !pLoopPlot->isCity())
                {
                    if (pLoopPlot->area() == plot()->area())
                    {
                        CLLNode<IDInfo>* pUnitNode = pLoopPlot->headUnitNode();

                        while (pUnitNode != NULL)
                        {
                            pLoopUnit = ::getUnit(pUnitNode->m_data);
                            pUnitNode = pLoopPlot->nextUnitNode(pUnitNode);

                            if (pLoopUnit->isEnemy(getTeam()))
                            {
                                if (pLoopUnit->currHitPoints() * 100 > pLoopUnit->maxHitPoints() * (GC.getDefineINT("HERO_SURROUNDING_ENEMY_HEALTH_CHANGE_LIMIT")))
                                {
                                    int iHealthChange = GC.getGameINLINE().getSorenRandNum(-m_iSurroundingEnemyHealthChange, "Decrease surrounding enemy's health");
                                    pLoopUnit->changeDamage(iHealthChange, getOwnerINLINE());
                                }
                            }
                        }
                    }
                }
            }
			PROFILE_END();
		}
        // Sanguo Mod Hero, end
	}

	if (hasMoved())
	{
		if (isAlwaysHeal())
		{
			doHeal();
		}
	}
	else
	{
		if (isHurt())
		{
			doHeal();
		}

		if (!isCargo())
		{
			changeFortifyTurns(1);
		}
	}

	changeImmobileTimer(-1);

	setMadeAttack(false);
	setMadeInterception(false);

	setReconPlot(NULL);

	setMoves(0);

	// Sanguo Mod Hero, start, added by poyuzhe 01.24.09
	if (getTempStrengthModifier() > 0)
	{
	    changeTempStrengthModifier(-std::max(getTempStrengthModifier() / 2, GC.getGameINLINE().getSorenRandNum(getTempStrengthModifier(), "Temp Strength Wear off")));
	}
	else
	{
	    changeTempStrengthModifier(std::max(-getTempStrengthModifier() / 2, GC.getGameINLINE().getSorenRandNum(-getTempStrengthModifier(), "Temp Strength Wear off")));
	}

	if (getTempStrengthModifier() >= -5 && getTempStrengthModifier() <= 5)
	{
		setTempStrengthModifier(0);
	}

#ifdef _DEBUG
	if (isCoreTroop())
	{
		FAssert (getID() == GC.getGameINLINE().getHeroUnitID(getHeroType()));
		FAssert (getOwnerINLINE() == GC.getGameINLINE().getHeroDebutStatus(getHeroType()));
		for (int iIndex = 1; iIndex < GC.getGameINLINE().getHeroLegionSize(getHeroType()); iIndex++)
		{
			CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroLegionMemberAt(getHeroType(), iIndex));
			FAssert (pMember != NULL);
			FAssert (pMember->getUnitCombatType() == getUnitCombatType());
		}
	}
	else if (getLegionHero() != NO_HERO)
	{
		CvUnit* pHero = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroUnitID(getLegionHero()));
		FAssert (pHero != NULL);
		FAssert (GC.getGameINLINE().isHeroHasLegionMember(getLegionHero(), getID()));
	}
	else if (isNakedHero())
	{
		FAssert (GC.getGameINLINE().getHeroLegionSize(getHeroType()) == 0);
		FAssert (getID() == GC.getGameINLINE().getHeroUnitID(getHeroType()));
		FAssert (getOwnerINLINE() == GC.getGameINLINE().getHeroDebutStatus(getHeroType()));
	}
#endif
	if (getHeroType() != NO_HERO)
	{
		if (GC.getGameINLINE().isHeroWounded(getHeroType()))
		{
			finishMoves();
			GC.getGameINLINE().setHeroWounded(getHeroType(), false);
		}
		if (GC.getGameINLINE().getHeroVitality(getHeroType()) == 0)
		{
            GC.getGameINLINE().doHeroDeath(getHeroType(), true);
			return;
		}
		if (!GC.getGameINLINE().isOption(GAMEOPTION_NO_HERO_DEATH_FROM_AGE)	&& 
			GC.getGameINLINE().getGameTurnYear() > GC.getHeroInfo(getHeroType()).getHistoricalDieYear())
		{
			if (GC.getHeroInfo(getHeroType()).getMurdererCiv() == NO_CIVILIZATION || GC.getGameINLINE().getGameTurnYear() > GC.getHeroInfo(getHeroType()).getStartingYear() + (GC.getGameINLINE().isOption(GAMEOPTION_HIGH_HERO_DEATH_CHANCE) ? 35 : 45))
			{
				if (GC.getGameINLINE().getSorenRandNum(100, "Random Vitality Decrease") < GC.getDefineINT("HERO_OLD_AGE_DECERASE_VITALITY_CHANCE_PER_TURN") * ((GC.getGameINLINE().isOption(GAMEOPTION_HIGH_HERO_DEATH_CHANCE)) ? 2 : 1))
				{
					GC.getGameINLINE().changeHeroVitality(getHeroType(), -1);
				}
			}
		}
	}
	if (getLegionHero() != NO_HERO)
	{
		m_iLegionTurn++;
	}
    // Sanguo Mod Hero, end
}


void CvUnit::updateAirStrike(CvPlot* pPlot, bool bQuick, bool bFinish)
{
	bool bVisible = false;

	if (!bFinish)
	{
		if (isFighting())
		{
			return;
		}

		if (!bQuick)
		{
			bVisible = isCombatVisible(NULL);
		}

		if (!airStrike(pPlot))
		{
			return;
		}

		if (bVisible)
		{
			CvAirMissionDefinition kAirMission;
			kAirMission.setMissionType(MISSION_AIRSTRIKE);
			kAirMission.setUnit(BATTLE_UNIT_ATTACKER, this);
			kAirMission.setUnit(BATTLE_UNIT_DEFENDER, NULL);
			kAirMission.setDamage(BATTLE_UNIT_DEFENDER, 0);
			kAirMission.setDamage(BATTLE_UNIT_ATTACKER, 0);
			kAirMission.setPlot(pPlot);
			setCombatTimer(GC.getMissionInfo(MISSION_AIRSTRIKE).getTime());
			GC.getGameINLINE().incrementTurnTimer(getCombatTimer());
			kAirMission.setMissionTime(getCombatTimer() * gDLL->getSecsPerTurn());

			if (pPlot->isActiveVisible(false))
			{
				gDLL->getEntityIFace()->AddMission(&kAirMission);
			}

			return;
		}
	}

	CvUnit *pDefender = getCombatUnit();
	if (pDefender != NULL)
	{
		pDefender->setCombatUnit(NULL);
	}
	setCombatUnit(NULL);
	setAttackPlot(NULL, false);

	getGroup()->clearMissionQueue();

	if (isSuicide() && !isDead())
	{
		kill(true);
	}
}

void CvUnit::resolveAirCombat(CvUnit* pInterceptor, CvPlot* pPlot, CvAirMissionDefinition& kBattle)
{
	CvWString szBuffer;

	int iTheirStrength = (DOMAIN_AIR == pInterceptor->getDomainType() ? pInterceptor->airCurrCombatStr(this) : pInterceptor->currCombatStr(NULL, NULL));
	int iOurStrength = (DOMAIN_AIR == getDomainType() ? airCurrCombatStr(pInterceptor) : currCombatStr(NULL, NULL));
	int iTotalStrength = iOurStrength + iTheirStrength;
	if (0 == iTotalStrength)
	{
		FAssert(false);
		return;
	}

	/********************************************************************************/
	/**		BETTER_BTS_AI_MOD						10/19/08	Roland J & jdog5000	*/
	/**																				*/
	/**		Combat mechanics														*/
	/********************************************************************************/
	/*
	int iOurOdds = (100 * iOurStrength) / std::max(1, iTotalStrength);

	int iOurRoundDamage = (pInterceptor->currInterceptionProbability() * GC.getDefineINT("MAX_INTERCEPTION_DAMAGE")) / 100;
	int iTheirRoundDamage = (currInterceptionProbability() * GC.getDefineINT("MAX_INTERCEPTION_DAMAGE")) / 100;
	if (getDomainType() == DOMAIN_AIR)
	{
		iTheirRoundDamage = std::max(GC.getDefineINT("MIN_INTERCEPTION_DAMAGE"), iTheirRoundDamage);
	}

	//original BTS code
	int iTheirDamage = 0;
	int iOurDamage = 0;

	for (int iRound = 0; iRound < GC.getDefineINT("INTERCEPTION_MAX_ROUNDS"); ++iRound)
	*/
	// For air v air, more rounds and factor in strength for per round damage
	int iOurOdds = (100 * iOurStrength) / std::max(1, iTotalStrength);
	int iMaxRounds = 0;
	int iOurRoundDamage = 0;
	int iTheirRoundDamage = 0;

	// Air v air is more like standard comabt
	// Round damage in this case will now depend on strength and interception probability
	if( GC.getDefineINT("BBAI_AIR_COMBAT") > 0 && (DOMAIN_AIR == pInterceptor->getDomainType() && DOMAIN_AIR == getDomainType()) )
	{
		int iBaseDamage = GC.getDefineINT("AIR_COMBAT_DAMAGE");
		int iOurFirepower = ((airMaxCombatStr(pInterceptor) + iOurStrength + 1) / 2);
		int iTheirFirepower = ((pInterceptor->airMaxCombatStr(this) + iTheirStrength + 1) / 2);

		int iStrengthFactor = ((iOurFirepower + iTheirFirepower + 1) / 2);

		int iTheirInterception = std::max(pInterceptor->maxInterceptionProbability(),2*GC.getDefineINT("MIN_INTERCEPTION_DAMAGE"));
		int iOurInterception = std::max(maxInterceptionProbability(),2*GC.getDefineINT("MIN_INTERCEPTION_DAMAGE"));

		iOurRoundDamage = std::max(1, ((iBaseDamage * (iTheirFirepower + iStrengthFactor) * iTheirInterception) / ((iOurFirepower + iStrengthFactor) * 100)));
		iTheirRoundDamage = std::max(1, ((iBaseDamage * (iOurFirepower + iStrengthFactor) * iOurInterception) / ((iTheirFirepower + iStrengthFactor) * 100)));

		iMaxRounds = 2*GC.getDefineINT("INTERCEPTION_MAX_ROUNDS") - 1;
	}
	else
	{
		iOurRoundDamage = (pInterceptor->currInterceptionProbability() * GC.getDefineINT("MAX_INTERCEPTION_DAMAGE")) / 100;
		iTheirRoundDamage = (currInterceptionProbability() * GC.getDefineINT("MAX_INTERCEPTION_DAMAGE")) / 100;
		if (getDomainType() == DOMAIN_AIR)
		{
			iTheirRoundDamage = std::max(GC.getDefineINT("MIN_INTERCEPTION_DAMAGE"), iTheirRoundDamage);
		}

		iMaxRounds = GC.getDefineINT("INTERCEPTION_MAX_ROUNDS");
	}

	int iTheirDamage = 0;
	int iOurDamage = 0;

	for (int iRound = 0; iRound < iMaxRounds; ++iRound)
	/********************************************************************************/
	/**		BETTER_BTS_AI_MOD						END								*/
	/********************************************************************************/
	{
		if (GC.getGameINLINE().getSorenRandNum(100, "Air combat") < iOurOdds)
		{
			if (DOMAIN_AIR == pInterceptor->getDomainType())
			{
				iTheirDamage += iTheirRoundDamage;
				pInterceptor->changeDamage(iTheirRoundDamage, getOwnerINLINE());
				if (pInterceptor->isDead())
				{
					break;
				}
			}
		}
		else
		{
			iOurDamage += iOurRoundDamage;
			changeDamage(iOurRoundDamage, pInterceptor->getOwnerINLINE());
			if (isDead())
			{
				break;
			}
		}
	}

	if (isDead())
	{
		if (iTheirRoundDamage > 0)
		{
			int iExperience = attackXPValue();
			iExperience = (iExperience * iOurStrength) / std::max(1, iTheirStrength);
			iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
			pInterceptor->changeExperience(iExperience, maxXPValue(), true, pPlot->getOwnerINLINE() == pInterceptor->getOwnerINLINE(), !isBarbarian());
		}
	}
	else if (pInterceptor->isDead())
	{
		int iExperience = pInterceptor->defenseXPValue();
		iExperience = (iExperience * iTheirStrength) / std::max(1, iOurStrength);
		iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
		changeExperience(iExperience, pInterceptor->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pInterceptor->isBarbarian());
	}
	else if (iOurDamage > 0)
	{
		if (iTheirRoundDamage > 0)
		{
			pInterceptor->changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), maxXPValue(), true, pPlot->getOwnerINLINE() == pInterceptor->getOwnerINLINE(), !isBarbarian());
		}
	}
	else if (iTheirDamage > 0)
	{
		changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), pInterceptor->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pInterceptor->isBarbarian());
	}

	kBattle.setDamage(BATTLE_UNIT_ATTACKER, iOurDamage);
	kBattle.setDamage(BATTLE_UNIT_DEFENDER, iTheirDamage);
}


void CvUnit::updateAirCombat(bool bQuick)
{
	CvUnit* pInterceptor = NULL;
	bool bFinish = false;

	FAssert(getDomainType() == DOMAIN_AIR || getDropRange() > 0);

	if (getCombatTimer() > 0)
	{
		changeCombatTimer(-1);

		if (getCombatTimer() > 0)
		{
			return;
		}
		else
		{
			bFinish = true;
		}
	}

	CvPlot* pPlot = getAttackPlot();
	if (pPlot == NULL)
	{
		return;
	}

	if (bFinish)
	{
		pInterceptor = getCombatUnit();
	}
	else
	{
		pInterceptor = bestInterceptor(pPlot);
	}


	if (pInterceptor == NULL)
	{
		setAttackPlot(NULL, false);
		setCombatUnit(NULL);

		getGroup()->clearMissionQueue();

		return;
	}

	//check if quick combat
	bool bVisible = false;
	if (!bQuick)
	{
		bVisible = isCombatVisible(pInterceptor);
	}

	//if not finished and not fighting yet, set up combat damage and mission
	if (!bFinish)
	{
		if (!isFighting())
		{
			if (plot()->isFighting() || pPlot->isFighting())
			{
				return;
			}

			setMadeAttack(true);

			setCombatUnit(pInterceptor, true);
			pInterceptor->setCombatUnit(this, false);
		}

		FAssertMsg(pInterceptor != NULL, "Defender is not assigned a valid value");

		FAssertMsg(plot()->isFighting(), "Current unit instance plot is not fighting as expected");
		FAssertMsg(pInterceptor->plot()->isFighting(), "pPlot is not fighting as expected");

		CvAirMissionDefinition kAirMission;
		if (DOMAIN_AIR != getDomainType())
		{
			kAirMission.setMissionType(MISSION_PARADROP);
		}
		else
		{
			kAirMission.setMissionType(MISSION_AIRSTRIKE);
		}
		kAirMission.setUnit(BATTLE_UNIT_ATTACKER, this);
		kAirMission.setUnit(BATTLE_UNIT_DEFENDER, pInterceptor);

		resolveAirCombat(pInterceptor, pPlot, kAirMission);

		if (!bVisible)
		{
			bFinish = true;
		}
		else
		{
			kAirMission.setPlot(pPlot);
			kAirMission.setMissionTime(GC.getMissionInfo(MISSION_AIRSTRIKE).getTime() * gDLL->getSecsPerTurn());
			setCombatTimer(GC.getMissionInfo(MISSION_AIRSTRIKE).getTime());
			GC.getGameINLINE().incrementTurnTimer(getCombatTimer());

			if (pPlot->isActiveVisible(false))
			{
				gDLL->getEntityIFace()->AddMission(&kAirMission);
			}
		}

		changeMoves(GC.getMOVE_DENOMINATOR());
		if (DOMAIN_AIR != pInterceptor->getDomainType())
		{
			pInterceptor->setMadeInterception(true);
		}

		if (isDead())
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_SHOT_DOWN_ENEMY", pInterceptor->getNameKey(), getNameKey(), getVisualCivAdjective(pInterceptor->getTeam()));
			gDLL->getInterfaceIFace()->addMessage(pInterceptor->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPT", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_SHOT_DOWN", getNameKey(), pInterceptor->getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPTED", MESSAGE_TYPE_INFO, pInterceptor->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}
		else if (kAirMission.getDamage(BATTLE_UNIT_ATTACKER) > 0)
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_HURT_ENEMY_AIR", pInterceptor->getNameKey(), getNameKey(), -(kAirMission.getDamage(BATTLE_UNIT_ATTACKER)), getVisualCivAdjective(pInterceptor->getTeam()));
			gDLL->getInterfaceIFace()->addMessage(pInterceptor->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPT", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_AIR_UNIT_HURT", getNameKey(), pInterceptor->getNameKey(), -(kAirMission.getDamage(BATTLE_UNIT_ATTACKER)));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPTED", MESSAGE_TYPE_INFO, pInterceptor->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}

		if (pInterceptor->isDead())
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_SHOT_DOWN_ENEMY", getNameKey(), pInterceptor->getNameKey(), pInterceptor->getVisualCivAdjective(getTeam()));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPT", MESSAGE_TYPE_INFO, pInterceptor->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_SHOT_DOWN", pInterceptor->getNameKey(), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(pInterceptor->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPTED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}
		else if (kAirMission.getDamage(BATTLE_UNIT_DEFENDER) > 0)
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_DAMAGED_ENEMY_AIR", getNameKey(), pInterceptor->getNameKey(), -(kAirMission.getDamage(BATTLE_UNIT_DEFENDER)), pInterceptor->getVisualCivAdjective(getTeam()));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPT", MESSAGE_TYPE_INFO, pInterceptor->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

			szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_AIR_UNIT_DAMAGED", pInterceptor->getNameKey(), getNameKey(), -(kAirMission.getDamage(BATTLE_UNIT_DEFENDER)));
			gDLL->getInterfaceIFace()->addMessage(pInterceptor->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPTED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}

		if (0 == kAirMission.getDamage(BATTLE_UNIT_ATTACKER) + kAirMission.getDamage(BATTLE_UNIT_DEFENDER))
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ABORTED_ENEMY_AIR", pInterceptor->getNameKey(), getNameKey(), getVisualCivAdjective(getTeam()));
			gDLL->getInterfaceIFace()->addMessage(pInterceptor->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPT", MESSAGE_TYPE_INFO, pInterceptor->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

			szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_AIR_UNIT_ABORTED", getNameKey(), pInterceptor->getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_INTERCEPTED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}
	}

	if (bFinish)
	{
		setAttackPlot(NULL, false);
		setCombatUnit(NULL);
		pInterceptor->setCombatUnit(NULL);

		if (!isDead() && isSuicide())
		{
			kill(true);
		}
	}
}

void CvUnit::resolveCombat(CvUnit* pDefender, CvPlot* pPlot, CvBattleDefinition& kBattle)
{
	PROFILE_FUNC();

	CombatDetails cdAttackerDetails;
	CombatDetails cdDefenderDetails;

	int iAttackerStrength = currCombatStr(NULL, NULL, &cdAttackerDetails);
	int iAttackerFirepower = currFirepower(NULL, NULL);
	int iDefenderStrength;
	int iAttackerDamage;
	int iDefenderDamage;
	int iDefenderOdds;

	getDefenderCombatValues(*pDefender, pPlot, iAttackerStrength, iAttackerFirepower, iDefenderOdds, iDefenderStrength, iAttackerDamage, iDefenderDamage, &cdDefenderDetails);

	// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
	bool bNoAttackerWithdrawal = false;
	int iAttackNullifyChance = 0;
	int iDefendNullifyChance = 0;
	int iAttackerTotalWinStrThreshold = 0;
	int iDefenderTotalWinStrThreshold = 0;
	int iAttackerNoDeathThreshold = 100;
	int iDefenderNoDeathThreshold = 100;
	bool bDefenderForceWithdrawal = false;
	CvWString szBuffer;

	if (getLegionHero() != NO_HERO || pDefender->getLegionHero() != NO_HERO)
	{
	    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	    {
	        if (pDefender->getLegionHero() != NO_HERO)
	        {
                if (pDefender->isHasPromotion((PromotionTypes)iI))
                {
                    if (GC.getPromotionInfo((PromotionTypes)iI).isNoEnemyWithdrawal())
                    {
                        bNoAttackerWithdrawal = true;
                    }

					if (!pDefender->plot()->isCity() && GC.getPromotionInfo((PromotionTypes)iI).isForceDefensiveWithdrawal())
					{
						bDefenderForceWithdrawal = true;
					}

                    iAttackNullifyChance += GC.getPromotionInfo((PromotionTypes)iI).getEnemyAttackNullifyChance();
					if (GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold() > 0)
					{
						iDefenderTotalWinStrThreshold = (iDefenderTotalWinStrThreshold == 0) ? GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold() : (std::min(iDefenderTotalWinStrThreshold, GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold()));
					}
					iDefenderNoDeathThreshold = std::min(iDefenderNoDeathThreshold, 100 - (GC.getPromotionInfo((PromotionTypes)iI).getNoDeathCombatOddsThreshold()));
                }
	        }

	        if (getLegionHero() != NO_HERO)
	        {
                if (isHasPromotion((PromotionTypes)iI))
                {
                    if (GC.getPromotionInfo((PromotionTypes)iI).isNoEnemyWithdrawal())
                    {
                        bNoAttackerWithdrawal = true;
                    }
                    iDefendNullifyChance += GC.getPromotionInfo((PromotionTypes)iI).getEnemyAttackNullifyChance();
					if (GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold() > 0)
					{
						iAttackerTotalWinStrThreshold = (iAttackerTotalWinStrThreshold == 0) ? GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold() : (std::min(iAttackerTotalWinStrThreshold, GC.getPromotionInfo((PromotionTypes)iI).getTotalWinStrengthThreshold()));
					}
					iAttackerNoDeathThreshold = std::min(iAttackerNoDeathThreshold, 100 - (GC.getPromotionInfo((PromotionTypes)iI).getNoDeathCombatOddsThreshold()));
                }
	        }
	    }
	}

	if (bDefenderForceWithdrawal)
	{
		bool bNoWhereToRun = true;
		if (pDefender->plot()->plotCount(PUF_isPlayer, pDefender->getOwnerINLINE(), -1, NO_PLAYER, NO_TEAM, PUF_canDefend) == 0)
		{
			for (int iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
			{
				CvPlot* pAdjacentPlot = plotDirection(pDefender->getX_INLINE(), pDefender->getY_INLINE(), (DirectionTypes)iI);
				if (pAdjacentPlot != NULL && pAdjacentPlot != pDefender->plot())
				{
					if (pAdjacentPlot->plotCount(PUF_isEnemy, getTeam(), 1, NO_PLAYER, NO_TEAM, PUF_canDefend) == 0)
					{
						if (pAdjacentPlot->isValidDomainForLocation(*pDefender))
						{
							if (pDefender->canMoveInto(pAdjacentPlot))
							{
								bNoWhereToRun = false;
								break;
							}
						}
					}
				}
			}
		}
		if (bNoWhereToRun)
		{
			bDefenderForceWithdrawal = false;
		}
	}

	bool bAttackerSureWithdrawal = false;
	bool bDefenderSureWithdrawal = bDefenderForceWithdrawal;
	if (iAttackerNoDeathThreshold < 100 || iDefenderNoDeathThreshold < 100)
	{
	    int iCombatOdds = getCombatOdds(this, pDefender);

	    if (iAttackerNoDeathThreshold * 10 < iCombatOdds)
	    {
	        bAttackerSureWithdrawal = true;
	    }

	    if (iDefenderNoDeathThreshold * 10 < (1000 - iCombatOdds))
	    {
	        bDefenderSureWithdrawal = true;
	    }
	}

	int iAttackerKillOdds;
	if (bNoAttackerWithdrawal && !bAttackerSureWithdrawal)
    {
        iAttackerKillOdds = iDefenderOdds;
    }
    else
    {
        // the following line is indented once
	    iAttackerKillOdds = iDefenderOdds * (100 - withdrawalProbability()) / 100;
    }
    // Sanguo Mod Hero, end

	if (isHuman() || pDefender->isHuman())
	{
		//Added ST
		CyArgsList pyArgsCD;
		pyArgsCD.add(gDLL->getPythonIFace()->makePythonObject(&cdAttackerDetails));
		pyArgsCD.add(gDLL->getPythonIFace()->makePythonObject(&cdDefenderDetails));
		pyArgsCD.add(getCombatOdds(this, pDefender));
		CvEventReporter::getInstance().genericEvent("combatLogCalc", pyArgsCD.makeFunctionArgs());
	}

	collateralCombat(pPlot, pDefender);

	while (true)
	{
		// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
        if (iAttackerTotalWinStrThreshold != 0 && iAttackerStrength > iDefenderStrength * iAttackerTotalWinStrThreshold)
        {
            pDefender->setDamage(GC.getMAX_HIT_POINTS());

            flankingStrikeCombat(pPlot, iAttackerStrength, iAttackerFirepower, iAttackerKillOdds, iDefenderDamage, pDefender);

            int iExperience = pDefender->attackXPValue();
            iExperience = ((iExperience * iDefenderStrength) / iAttackerStrength);
            iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
            changeExperience(iExperience, pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian(), pDefender->plot()->getTerrainType());
            if (isBarbarian())
            {
                GC.getGameINLINE().changeCombatExperienceFromBarbarian(false, iExperience);
            }
            break;
        }

        if (iDefenderTotalWinStrThreshold != 0 && iDefenderStrength > iAttackerStrength * iDefenderTotalWinStrThreshold)
        {
            setDamage(GC.getMAX_HIT_POINTS());
            int iExperience = defenseXPValue();
            iExperience = ((iExperience * iAttackerStrength) / iDefenderStrength);
            iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
            pDefender->changeExperience(iExperience, maxXPValue(), true, pPlot->getOwnerINLINE() == pDefender->getOwnerINLINE(), !isBarbarian(), pDefender->plot()->getTerrainType());
            if (pDefender->isBarbarian())
            {
                GC.getGameINLINE().changeCombatExperienceFromBarbarian(false, iExperience);
            }
            break;
        }
        // Sanguo Mod Hero, end

		if (GC.getGameINLINE().getSorenRandNum(GC.getDefineINT("COMBAT_DIE_SIDES"), "Combat") < iDefenderOdds)
		{
			if (getCombatFirstStrikes() == 0)
			{
				// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
				if (iDefendNullifyChance > 0 && GC.getGameINLINE().getSorenRandNum(100, "Defender Attack Nullification") < iDefendNullifyChance)
				{
				    szBuffer = gDLL->getText("TXT_KEY_ENEMY_ATTACK_NULLIFIED");
                    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
				    szBuffer = gDLL->getText("TXT_KEY_OUR_ATTACK_NULLIFIED");
                    gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
				    continue;
				}

                // the following line been commented out
//				if (getDamage() + iAttackerDamage >= maxHitPoints() && GC.getGameINLINE().getSorenRandNum(100, "Withdrawal") < withdrawalProbability())
				if (getDamage() + iAttackerDamage >= maxHitPoints() && (bAttackerSureWithdrawal || GC.getGameINLINE().getSorenRandNum(100, "Withdrawal") < withdrawalProbability()))
				// Sanguo Mod Hero, end
				{
					// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
					if (!bNoAttackerWithdrawal || bAttackerSureWithdrawal)
					{
					    // the following lines are indented once
                        flankingStrikeCombat(pPlot, iAttackerStrength, iAttackerFirepower, iAttackerKillOdds, iDefenderDamage, pDefender);
                        //Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
                        //Sun Ce, TRAIT_SEAFARING
                        //changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian());
                        changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian(), pDefender->plot()->getTerrainType());
                        // Huang Jin, TRAIT_PEASANT, added by poyuzhe 12.22.08
                        if (isBarbarian())
                        {
                            GC.getGameINLINE().changeCombatExperienceFromBarbarian(true, 0);
                        }
                        //Sanguo Mod Civilization Trait, end
                        if (bAttackerSureWithdrawal)
                        {
                            szBuffer = gDLL->getText("TXT_KEY_OUR_COMMANDER_WITHDRAWAL");
                            gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
                            szBuffer = gDLL->getText("TXT_KEY_ENEMY_COMMANDER_WITHDRAWAL");
                            gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
                        }
                        break;
                        // end indented
					}
					else
					{
					    szBuffer = gDLL->getText("TXT_KEY_WE_CANNOT_WITHDRAW");
                        gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
                        szBuffer = gDLL->getText("TXT_KEY_THEY_CANNOT_WITHDRAW");
                        gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
					}
					// Sanguo Mod Hero, end
				}

				changeDamage(iAttackerDamage, pDefender->getOwnerINLINE());

				if (pDefender->getCombatFirstStrikes() > 0 && pDefender->isRanged())
				{
					kBattle.addFirstStrikes(BATTLE_UNIT_DEFENDER, 1);
					kBattle.addDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_RANGED, iAttackerDamage);
				}

				cdAttackerDetails.iCurrHitPoints = currHitPoints();

				if (isHuman() || pDefender->isHuman())
				{
					CyArgsList pyArgs;
					pyArgs.add(gDLL->getPythonIFace()->makePythonObject(&cdAttackerDetails));
					pyArgs.add(gDLL->getPythonIFace()->makePythonObject(&cdDefenderDetails));
					pyArgs.add(1);
					pyArgs.add(iAttackerDamage);
					CvEventReporter::getInstance().genericEvent("combatLogHit", pyArgs.makeFunctionArgs());
				}
			}
		}
		else
		{
			if (pDefender->getCombatFirstStrikes() == 0)
			{
				// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
				if (iAttackNullifyChance > 0 && GC.getGameINLINE().getSorenRandNum(100, "Attacker Attack Nullification") < iAttackNullifyChance)
				{
				    szBuffer = gDLL->getText("TXT_KEY_OUR_ATTACK_NULLIFIED");
                    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
				    szBuffer = gDLL->getText("TXT_KEY_ENEMY_ATTACK_NULLIFIED");
                    gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
				    continue;
				}

				if (bDefenderSureWithdrawal && pDefender->getDamage() + iDefenderDamage >= pDefender->maxHitPoints())
				{
                    pDefender->changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), maxXPValue(), true, pPlot->getOwnerINLINE() == pDefender->getOwnerINLINE(), !isBarbarian(), pDefender->plot()->getTerrainType());
                    if (pDefender->isBarbarian())
                    {
                        GC.getGameINLINE().changeCombatExperienceFromBarbarian(true, 0);
                    }
                    if (bDefenderForceWithdrawal)
					{
						szBuffer = gDLL->getText("TXT_KEY_HERO_RUNNER", GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_ENEMY_COMMANDER_WITHDRAWAL");
					}
                    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
				    if (bDefenderForceWithdrawal)
					{
						szBuffer = gDLL->getText("TXT_KEY_HERO_RUNNER", GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_OUR_COMMANDER_WITHDRAWAL");
					}
                    gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITCAPTURE", MESSAGE_TYPE_INFO);
                    break;
				}
				// Sanguo Mod Hero, end

				if (std::min(GC.getMAX_HIT_POINTS(), pDefender->getDamage() + iDefenderDamage) > combatLimit())
				{
					//Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
                    //Sun Ce, TRAIT_SEAFARING
					//changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian());
					changeExperience(GC.getDefineINT("EXPERIENCE_FROM_WITHDRAWL"), pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian(), pDefender->plot()->getTerrainType());
					// Huang Jin, TRAIT_PEASANT, added by poyuzhe 12.22.08
					if (isBarbarian())
					{
					    GC.getGameINLINE().changeCombatExperienceFromBarbarian(true, 0);
					}
					//Sanguo Mod Civilization Trait, end
					pDefender->setDamage(combatLimit(), getOwnerINLINE());
					break;
				}

				pDefender->changeDamage(iDefenderDamage, getOwnerINLINE());

				if (getCombatFirstStrikes() > 0 && isRanged())
				{
					kBattle.addFirstStrikes(BATTLE_UNIT_ATTACKER, 1);
					kBattle.addDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_RANGED, iDefenderDamage);
				}

				cdDefenderDetails.iCurrHitPoints=pDefender->currHitPoints();

				if (isHuman() || pDefender->isHuman())
				{
					CyArgsList pyArgs;
					pyArgs.add(gDLL->getPythonIFace()->makePythonObject(&cdAttackerDetails));
					pyArgs.add(gDLL->getPythonIFace()->makePythonObject(&cdDefenderDetails));
					pyArgs.add(0);
					pyArgs.add(iDefenderDamage);
					CvEventReporter::getInstance().genericEvent("combatLogHit", pyArgs.makeFunctionArgs());
				}
			}
		}

		if (getCombatFirstStrikes() > 0)
		{
			changeCombatFirstStrikes(-1);
		}

		if (pDefender->getCombatFirstStrikes() > 0)
		{
			pDefender->changeCombatFirstStrikes(-1);
		}

		if (isDead() || pDefender->isDead())
		{
			if (isDead())
			{
				int iExperience = defenseXPValue();
				iExperience = ((iExperience * iAttackerStrength) / iDefenderStrength);
				iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
				//Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
                //Sun Ce, TRAIT_SEAFARING
				//pDefender->changeExperience(iExperience, maxXPValue(), true, pPlot->getOwnerINLINE() == pDefender->getOwnerINLINE(), !isBarbarian());
				pDefender->changeExperience(iExperience, maxXPValue(), true, pPlot->getOwnerINLINE() == pDefender->getOwnerINLINE(), !isBarbarian(), pDefender->plot()->getTerrainType());
				// Huang Jin, TRAIT_PEASANT, added by poyuzhe 12.22.08
                if (pDefender->isBarbarian())
                {
                    GC.getGameINLINE().changeCombatExperienceFromBarbarian(false, iExperience);
                }
				//Sanguo Mod Civilization Trait, end
			}
			else
			{
				flankingStrikeCombat(pPlot, iAttackerStrength, iAttackerFirepower, iAttackerKillOdds, iDefenderDamage, pDefender);

				int iExperience = pDefender->attackXPValue();
				iExperience = ((iExperience * iDefenderStrength) / iAttackerStrength);
				iExperience = range(iExperience, GC.getDefineINT("MIN_EXPERIENCE_PER_COMBAT"), GC.getDefineINT("MAX_EXPERIENCE_PER_COMBAT"));
				//Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
                //Sun Ce, TRAIT_SEAFARING
				//changeExperience(iExperience, pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian());
				changeExperience(iExperience, pDefender->maxXPValue(), true, pPlot->getOwnerINLINE() == getOwnerINLINE(), !pDefender->isBarbarian(), pDefender->plot()->getTerrainType());
				if (isBarbarian())
                {
                    GC.getGameINLINE().changeCombatExperienceFromBarbarian(false, iExperience);
                }
				//Sanguo Mod Civilization Trait, end
			}

			break;
		}
	}
}

// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
// commented 1 line out
void CvUnit::updateCombat(bool bQuick, CvUnit* pDefender)
// Sanguo Mod Hero, end
{
	PROFILE_FUNC();

	CvWString szBuffer;

	bool bFinish = false;
	bool bVisible = false;
	// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
	bool bAttackerTerrified = false;
    // Sanguo Mod Hero, end

	if (getCombatTimer() > 0)
	{
		changeCombatTimer(-1);

		if (getCombatTimer() > 0)
		{
			return;
		}
		else
		{
			bFinish = true;
		}
	}

	CvPlot* pPlot = getAttackPlot();

	if (pPlot == NULL)
	{
		return;
	}

	if (getDomainType() == DOMAIN_AIR)
	{
		updateAirStrike(pPlot, bQuick, bFinish);
		return;
	}

    // Sanguo Mod Hero, start, added by poyuzhe 02.05.09
    // commented 1 line out
//	CvUnit* pDefender = NULL;
	// Sanguo Mod Hero, end
	if (bFinish)
	{
		pDefender = getCombatUnit();
	}
	else
	{
		// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
        // commented 1 line out
//		pDefender = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
        if (pDefender == NULL)
        {
            pDefender = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
        }
		// Sanguo Mod Hero, end
	}

	if (pDefender == NULL)
	{
		setAttackPlot(NULL, false);
		setCombatUnit(NULL);

		getGroup()->groupMove(pPlot, true, ((canAdvance(pPlot, 0)) ? this : NULL));

		getGroup()->clearMissionQueue();

		return;
	}
	// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
	else if (getCombatTimer() == 0 && pDefender->isCoreTroop())
	{
	    int iTerrifyAttackingEnemyChance = 0;
	    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	    {
	        if (pDefender->isHasPromotion((PromotionTypes)iI))
	        {
	            iTerrifyAttackingEnemyChance += GC.getPromotionInfo((PromotionTypes)iI).getTerrifyAttackingEnemyChance();
	        }
	    }

	    if (iTerrifyAttackingEnemyChance > 0)
	    {
	        if (GC.getGameINLINE().getSorenRandNum(100, "Terrify attacking enemy") < iTerrifyAttackingEnemyChance)
	        {
	            bFinish = true;
	            bAttackerTerrified = true;
	        }
	    }
	}
    // Sanguo Mod Hero, end

	//check if quick combat
	if (!bQuick)
	{
		bVisible = isCombatVisible(pDefender);
	}

	//FAssertMsg((pPlot == pDefender->plot()), "There is not expected to be a defender or the defender's plot is expected to be pPlot (the attack plot)");

	//if not finished and not fighting yet, set up combat damage and mission
	if (!bFinish)
	{
		if (!isFighting())
		{
			if (plot()->isFighting() || pPlot->isFighting())
			{
				return;
			}

			// Sanguo Mod Hero, start, added by poyuzhe 03.10.09
			if (!isTempCheck())
			{
			    setMadeAttack(true);
			}
			// Sanguo Mod Hero, end

			//rotate to face plot
			DirectionTypes newDirection = estimateDirection(this->plot(), pDefender->plot());
			if (newDirection != NO_DIRECTION)
			{
				setFacingDirection(newDirection);
			}

			//rotate enemy to face us
			newDirection = estimateDirection(pDefender->plot(), this->plot());
			if (newDirection != NO_DIRECTION)
			{
				pDefender->setFacingDirection(newDirection);
			}

			setCombatUnit(pDefender, true);
			pDefender->setCombatUnit(this, false);

			pDefender->getGroup()->clearMissionQueue();

			bool bFocused = (bVisible && isCombatFocus() && gDLL->getInterfaceIFace()->isCombatFocus());

			if (bFocused)
			{
				DirectionTypes directionType = directionXY(plot(), pPlot);
				//								N			NE				E				SE					S				SW					W				NW
				NiPoint2 directions[8] = {NiPoint2(0, 1), NiPoint2(1, 1), NiPoint2(1, 0), NiPoint2(1, -1), NiPoint2(0, -1), NiPoint2(-1, -1), NiPoint2(-1, 0), NiPoint2(-1, 1)};
				NiPoint3 attackDirection = NiPoint3(directions[directionType].x, directions[directionType].y, 0);
				float plotSize = GC.getPLOT_SIZE();
				NiPoint3 lookAtPoint(plot()->getPoint().x + plotSize / 2 * attackDirection.x, plot()->getPoint().y + plotSize / 2 * attackDirection.y, (plot()->getPoint().z + pPlot->getPoint().z) / 2);
				attackDirection.Unitize();
				gDLL->getInterfaceIFace()->lookAt(lookAtPoint, (((getOwnerINLINE() != GC.getGameINLINE().getActivePlayer()) || gDLL->getGraphicOption(GRAPHICOPTION_NO_COMBAT_ZOOM)) ? CAMERALOOKAT_BATTLE : CAMERALOOKAT_BATTLE_ZOOM_IN), attackDirection);
			}
			else
			{
				PlayerTypes eAttacker = getVisualOwner(pDefender->getTeam());
				CvWString szMessage;
				if (BARBARIAN_PLAYER != eAttacker)
				{
					szMessage = gDLL->getText("TXT_KEY_MISC_YOU_UNITS_UNDER_ATTACK", GET_PLAYER(getOwnerINLINE()).getNameKey());
				}
				else
				{
					szMessage = gDLL->getText("TXT_KEY_MISC_YOU_UNITS_UNDER_ATTACK_UNKNOWN");
				}

				gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szMessage, "AS2D_COMBAT", MESSAGE_TYPE_DISPLAY_ONLY, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true);
			}
		}

		FAssertMsg(pDefender != NULL, "Defender is not assigned a valid value");

		FAssertMsg(plot()->isFighting(), "Current unit instance plot is not fighting as expected");
		FAssertMsg(pPlot->isFighting(), "pPlot is not fighting as expected");

		if (!pDefender->canDefend())
		{
			if (!bVisible)
			{
				bFinish = true;
			}
			else
			{
				CvMissionDefinition kMission;
				kMission.setMissionTime(getCombatTimer() * gDLL->getSecsPerTurn());
				kMission.setMissionType(MISSION_SURRENDER);
				kMission.setUnit(BATTLE_UNIT_ATTACKER, this);
				kMission.setUnit(BATTLE_UNIT_DEFENDER, pDefender);
				kMission.setPlot(pPlot);
				gDLL->getEntityIFace()->AddMission(&kMission);

				// Surrender mission
				setCombatTimer(GC.getMissionInfo(MISSION_SURRENDER).getTime());

				GC.getGameINLINE().incrementTurnTimer(getCombatTimer());
			}

			// Kill them!
			pDefender->setDamage(GC.getMAX_HIT_POINTS());
		}
		else
		{
			CvBattleDefinition kBattle;
			kBattle.setUnit(BATTLE_UNIT_ATTACKER, this);
			kBattle.setUnit(BATTLE_UNIT_DEFENDER, pDefender);
			kBattle.setDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_BEGIN, getDamage());
			kBattle.setDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_BEGIN, pDefender->getDamage());

			resolveCombat(pDefender, pPlot, kBattle);

			if (!bVisible)
			{
				bFinish = true;
			}
			else
			{
				kBattle.setDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_END, getDamage());
				kBattle.setDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_END, pDefender->getDamage());
				kBattle.setAdvanceSquare(canAdvance(pPlot, 1));

				if (isRanged() && pDefender->isRanged())
				{
					kBattle.setDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_END));
					kBattle.setDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_END));
				}
				else
				{
					kBattle.addDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_BEGIN));
					kBattle.addDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_BEGIN));
				}

				int iTurns = planBattle( kBattle);
				kBattle.setMissionTime(iTurns * gDLL->getSecsPerTurn());
				setCombatTimer(iTurns);

				GC.getGameINLINE().incrementTurnTimer(getCombatTimer());

				if (pPlot->isActiveVisible(false))
				{
					ExecuteMove(0.5f, true);
					gDLL->getEntityIFace()->AddMission(&kBattle);
				}
			}
		}
	}

	if (bFinish)
	{
		if (bVisible)
		{
			if (isCombatFocus() && gDLL->getInterfaceIFace()->isCombatFocus())
			{
				if (getOwnerINLINE() == GC.getGameINLINE().getActivePlayer())
				{
					gDLL->getInterfaceIFace()->releaseLockedCamera();
				}
			}
		}

		//end the combat mission if this code executes first
		gDLL->getEntityIFace()->RemoveUnitFromBattle(this);
		gDLL->getEntityIFace()->RemoveUnitFromBattle(pDefender);
		setAttackPlot(NULL, false);
		setCombatUnit(NULL);
		pDefender->setCombatUnit(NULL);
		NotifyEntity(MISSION_DAMAGE);
		pDefender->NotifyEntity(MISSION_DAMAGE);

		if (isDead())
		{
			if (isBarbarian())
			{
				GET_PLAYER(pDefender->getOwnerINLINE()).changeWinsVsBarbs(1);
			}

			// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
			// commented out 1 line
//			if (!m_pUnitInfo->isHiddenNationality() && !pDefender->getUnitInfo().isHiddenNationality())
			if (!isHiddenNationality() && !pDefender->isHiddenNationality())
			// Sanguo Mod Hero, end
			{
				GET_TEAM(getTeam()).changeWarWeariness(pDefender->getTeam(), *pPlot, GC.getDefineINT("WW_UNIT_KILLED_ATTACKING"));
				GET_TEAM(pDefender->getTeam()).changeWarWeariness(getTeam(), *pPlot, GC.getDefineINT("WW_KILLED_UNIT_DEFENDING"));
				GET_TEAM(pDefender->getTeam()).AI_changeWarSuccess(getTeam(), GC.getDefineINT("WAR_SUCCESS_DEFENDING"));
			}

			// Sanguo Mod Hero, start, added by poyuzhe 01.24.09
			if (getLegionHero() != NO_HERO)
			{
				if (pDefender->getLegionHero() != NO_HERO)
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_DIED_ATTACKING_LEGION", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_KILLED_ENEMY_LEGION_UNIT", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey(),  GC.getHeroInfo(getLegionHero()).getDescription());
					gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
				}
				else
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_DIED_ATTACKING", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_KILLED_ENEMY_LEGION_UNIT", pDefender->getNameKey(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
					gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
				}
			}
			else
			{
				if (pDefender->getLegionHero() != NO_HERO)
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DIED_ATTACKING_LEGION", getNameKey(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_KILLED_ENEMY", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey());
					gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
				
				}
				else
				{
				// the following 4 lines indented twice
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DIED_ATTACKING", getNameKey(), pDefender->getNameKey());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_KILLED_ENEMY_UNIT", pDefender->getNameKey(), getNameKey(), getVisualCivAdjective(pDefender->getTeam()));
					gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
				}
			}

			pDefender->processCombatWin(this, false);
            if (getLegionHero() != NO_HERO)
            {
                doLegionMemberDefeated(pDefender, true, false);
            }
            // Sanguo Mod Hero, end

			// report event to Python, along with some other key state
			CvEventReporter::getInstance().combatResult(pDefender, this);
		}
		else if (pDefender->isDead())
		{
			if (pDefender->isBarbarian())
			{
				GET_PLAYER(getOwnerINLINE()).changeWinsVsBarbs(1);
			}

			// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
			// commented out 1 line
//			if (!m_pUnitInfo->isHiddenNationality() && !pDefender->getUnitInfo().isHiddenNationality())
			if (!isHiddenNationality() && !pDefender->isHiddenNationality())
			// Sanguo Mod Hero, end
			{
				GET_TEAM(pDefender->getTeam()).changeWarWeariness(getTeam(), *pPlot, GC.getDefineINT("WW_UNIT_KILLED_DEFENDING"));
				GET_TEAM(getTeam()).changeWarWeariness(pDefender->getTeam(), *pPlot, GC.getDefineINT("WW_KILLED_UNIT_ATTACKING"));
				GET_TEAM(getTeam()).AI_changeWarSuccess(pDefender->getTeam(), GC.getDefineINT("WAR_SUCCESS_ATTACKING"));
			}

			// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
			int iFlyingGeneralCount = 0;
			if (pDefender->isNakedHero())
			{
			    szBuffer = szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DEFEATED_NAKED_HERO", getNameKey(), pDefender->getNameKey());
			}
			else
			{
			    if (getLegionHero() != NO_HERO)
			    {
					if (pDefender->getLegionHero() != NO_HERO)
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_DESTROYED_ENEMY_LEGION", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_DESTROYED_ENEMY", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey());
					}
				}
				else
				{
					if (pDefender->getLegionHero() != NO_HERO)
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DESTROYED_ENEMY_LEGION", getNameKey(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
					}
					else
					{
						// the following line indented 3 times
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DESTROYED_ENEMY", getNameKey(), pDefender->getNameKey());
					}
				}
			}
			// Sanguo Mod Hero, end

			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			if (getVisualOwner(pDefender->getTeam()) != getOwnerINLINE())
			{
				// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
                if (pDefender->isNakedHero())
                {
                    szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_HERO_WAS_DEFEATED_UNKNOWN", pDefender->getNameKey(), getNameKey());
                }
                else
                {
					if (getLegionHero() != NO_HERO)
                    {
						if (pDefender->getLegionHero() != NO_HERO)
						{
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WAS_DESTROYED_LEGION", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						}
						else
						{
						   szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WAS_DESTROYED_LEGION", pDefender->getNameKey(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						}
					}
					else
					{
						if (pDefender->getLegionHero() != NO_HERO)
						{
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WAS_DESTROYED", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey());
						}
						else
						{
							// the following line indented 3 times
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WAS_DESTROYED_UNKNOWN", pDefender->getNameKey(), getNameKey());
						}
					}
				}
                // Sanguo Mod Hero, end
			}
			else
			{
			    // Sanguo Mod Hero, start, added by poyuzhe 02.08.09
                if (pDefender->isNakedHero())
                {
                    szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_HERO_WAS_DEFEATED", pDefender->getNameKey(), getNameKey(), getVisualCivAdjective(pDefender->getTeam()));
                }
                else
                {
                    if (getLegionHero() != NO_HERO)
                    {
						if (pDefender->getLegionHero() != NO_HERO)
						{
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WAS_DESTROYED_LEGION", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						}
						else
						{
						   szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WAS_DESTROYED_LEGION", pDefender->getNameKey(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						}
					}
					else
					{
						if (pDefender->getLegionHero() != NO_HERO)
						{
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WAS_DESTROYED", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey());
						}
						else
						{
							// the following line indented 3 times
							szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WAS_DESTROYED", pDefender->getNameKey(), getNameKey(), getVisualCivAdjective(pDefender->getTeam()));
						}
					}
				}
                // Sanguo Mod Hero, end
            }
			gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer,GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

			// report event to Python, along with some other key state
			CvEventReporter::getInstance().combatResult(this, pDefender);

			bool bAdvance = false;

			if (isSuicide())
			{
                // Sanguo Mod Hero, start, added by poyuzhe 02.11.09
                if (getLegionHero() != NO_HERO)
                {
                    doLegionMemberDefeated(pDefender, true, true);
                }
                // Sanguo Mod Hero, end
				kill(true);

				// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
				// commented out 1 line
//				pDefender->kill(false);
                if (pDefender->getLegionHero() != NO_HERO)
                {
					bool bTempHiddenNationality = (isHiddenNationality() && !m_pUnitInfo->isHiddenNationality());
                    pDefender->doLegionMemberDefeated(this, false, true);
					if (bTempHiddenNationality)
					{
						m_bTempCheck = true;
					}
                }
                pDefender->kill(false, getOwnerINLINE());
                FAssert (pDefender != NULL);
                if (pDefender->isNakedHero())
                {
                    pDefender->stopDelayedDeath();
                }
				// Sanguo Mod Hero, end
				pDefender = NULL;
			}
			else
			{
				bAdvance = canAdvance(pPlot, ((pDefender->canDefend()) ? 1 : 0));

				if (bAdvance)
				{
					if (!isNoCapture())
					{
						pDefender->setCapturingPlayer(getOwnerINLINE());
					}
				}

                // Sanguo Mod Hero, start, added by poyuzhe 01.24.09
                // Commented out the following 2 lines and insert them later
//				pDefender->kill(false);
//				pDefender = NULL;

                // commented out 1 line
//				if (!bAdvance)
				if (isCoreTroop() && pDefender->getLeaderUnitType() != NO_UNIT)
				{
					for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
					{
						if (GC.getPromotionInfo((PromotionTypes)iI).isDefeatGeneralRestoreMoves())
						{
							if (isHasPromotion((PromotionTypes)iI))
							{
								iFlyingGeneralCount += 1;
							}
						}
					}
				}

				if (!bAdvance && !m_bTempCheck)  // m_bTempCheck is used here to exclude Free Additional Attack targeting pursuing enemy
				// Sanguo Mod Hero, end
				{
					changeMoves(std::max(GC.getMOVE_DENOMINATOR(), pPlot->movementCost(this, plot())));
					checkRemoveSelectionAfterAttack();
				}

				// Sanguo Mod Hero, start, added by poyuzhe 01.24.09
			    processCombatWin(pDefender, true);

			    if (pDefender->getLegionHero() != NO_HERO)
			    {
					bool bTempHiddenNationality = (pDefender->isHiddenNationality() && !pDefender->getUnitInfo().isHiddenNationality());
			        pDefender->doLegionMemberDefeated(this, false, false);
					if (bTempHiddenNationality)
					{
						pDefender->setTempCheck(true);
					}
			    }
			    pDefender->kill(false, getOwnerINLINE());
			    FAssert (pDefender != NULL);
			    if (pDefender->isNakedHero())
                {
                    pDefender->stopDelayedDeath();
                }
				pDefender = NULL;
				bAdvance = canAdvance(pPlot, 0);
                // Sanguo Mod Hero, end
			}

			// Sanguo Mod Hero, start, added by poyuzhe 02.06.09
			// commented out 1 line
//			if (pPlot->getNumVisibleEnemyDefenders(this) == 0)
//			{
//				getGroup()->groupMove(pPlot, true, ((bAdvance) ? this : NULL));
//			}
			if (pPlot->getNumVisibleEnemyDefenders(this) == 0 && !m_bTempCheck)
			{
				getGroup()->groupMove(pPlot, true, ((bAdvance) ? this : NULL), false, iFlyingGeneralCount > 0);
			}
			// Sanguo Mod Hero, end

			// Sanguo Mod Hero, start, added by poyuzhe 02.06.09
			if (m_bTempCheck && getLegionHero() != NO_HERO)
			{
				for (int iI = 0; iI < GC.getGameINLINE().getHeroLegionSize(getLegionHero()); iI++)
				{
					GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroLegionMemberAt(getLegionHero(), iI))->setTempCheck(false);
				}
			}
			m_bTempCheck = false;
            // Sanguo Mod Hero, end

			// This is is put before the plot advancement, the unit will always try to walk back
			// to the square that they came from, before advancing.
			getGroup()->clearMissionQueue();
		}
		// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
		// commented out 1 line
//		else
        else if (!bAttackerTerrified)
		{
			bool bAttackerWithdrawal = (currHitPoints() <= pDefender->currHitPoints());
			if (getLegionHero() != NO_HERO)
			{
				if (pDefender->getLegionHero() != NO_HERO)
				{
					if (bAttackerWithdrawal)
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WITHDRAW_LEGION", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_LEGION_UNIT_WITHDRAW_LEGION", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WITHDRAW_LEGION", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_LEGION_UNIT_WITHDRAW_LEGION", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
				}
				else
				{
					if (bAttackerWithdrawal)
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WITHDRAW", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_LEGION_UNIT_WITHDRAW", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription(), pDefender->getNameKey());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WITHDRAW_LEGION", pDefender->getNameKey(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_UNIT_WITHDRAW_LEGION", pDefender->getNameKey(), getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
				}
			}
			else
			{
				if (pDefender->getLegionHero() != NO_HERO)
				{
					if (bAttackerWithdrawal)
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WITHDRAW_LEGION", getNameKey(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_UNIT_WITHDRAW_LEGION", getNameKey(), pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_WITHDRAW", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_LEGION_UNIT_WITHDRAW", pDefender->getNameKey(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), getNameKey());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}

				}
				else
				{
					if (bAttackerWithdrawal)
					{
						// The following 4 lines indented 3 times
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WITHDRAW", getNameKey(), pDefender->getNameKey());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_UNIT_WITHDRAW", getNameKey(), pDefender->getNameKey());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
					else
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_WITHDRAW", pDefender->getNameKey(), getNameKey());
						gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_OUR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
						szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_UNIT_WITHDRAW", pDefender->getNameKey(), getNameKey());
						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_THEIR_WITHDRAWL", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
					}
				}
			}
			if (!m_bTempCheck)
			{
				changeMoves(std::max(GC.getMOVE_DENOMINATOR(), pPlot->movementCost(this, plot())));
			}
		    // Sanguo Mod Hero, end

			checkRemoveSelectionAfterAttack();

			// Sanguo Mod Hero, start, added by poyuzhe 01.24.09
			processCombatWithdrawal(pDefender, bAttackerWithdrawal);

			if (getGroup() != NULL)
			{
				// This line indented once
				getGroup()->clearMissionQueue();
			}
			// Sanguo Mod Hero, end
		}
		// Sanguo Mod Hero, start, added by poyuzhe 02.08.09
		else
		{
		    szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_TERRIFIED", GC.getHeroInfo(pDefender->getHeroType()).getDescription(), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_CITYRAZED", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_UNIT_TERRIFIED", GC.getHeroInfo(pDefender->getHeroType()).getDescription(), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_CITYRAZED", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

			finishMoves();
			checkRemoveSelectionAfterAttack();

			getGroup()->clearMissionQueue();
		}
        // Sanguo Mod Hero, end
	}
}

void CvUnit::checkRemoveSelectionAfterAttack()
{
	if (!canMove() || !isBlitz())
	{
		if (IsSelected())
		{
			if (gDLL->getInterfaceIFace()->getLengthSelectionList() > 1)
			{
				gDLL->getInterfaceIFace()->removeFromSelectionList(this);
			}
		}
	}
}


bool CvUnit::isActionRecommended(int iAction)
{
	CvCity* pWorkingCity;
	CvPlot* pPlot;
	ImprovementTypes eImprovement;
	ImprovementTypes eFinalImprovement;
	BuildTypes eBuild;
	RouteTypes eRoute;
	BonusTypes eBonus;
	int iIndex;

	if (getOwnerINLINE() != GC.getGameINLINE().getActivePlayer())
	{
		return false;
	}

	if (GET_PLAYER(getOwnerINLINE()).isOption(PLAYEROPTION_NO_UNIT_RECOMMENDATIONS))
	{
		return false;
	}

	CyUnit* pyUnit = new CyUnit(this);
	CyArgsList argsList;
	argsList.add(gDLL->getPythonIFace()->makePythonObject(pyUnit));	// pass in unit class
	argsList.add(iAction);
	long lResult=0;
	gDLL->getPythonIFace()->callFunction(PYGameModule, "isActionRecommended", argsList.makeFunctionArgs(), &lResult);
	delete pyUnit;	// python fxn must not hold on to this pointer
	if (lResult == 1)
	{
		return true;
	}

	pPlot = gDLL->getInterfaceIFace()->getGotoPlot();

	if (pPlot == NULL)
	{
		if (gDLL->shiftKey())
		{
			pPlot = getGroup()->lastMissionPlot();
		}
	}

	if (pPlot == NULL)
	{
		pPlot = plot();
	}

	if (GC.getActionInfo(iAction).getMissionType() == MISSION_FORTIFY)
	{
		if (pPlot->isCity(true, getTeam()))
		{
			if (canDefend(pPlot))
			{
				if (pPlot->getNumDefenders(getOwnerINLINE()) < ((atPlot(pPlot)) ? 2 : 1))
				{
					return true;
				}
			}
		}
	}

	if (GC.getActionInfo(iAction).getMissionType() == MISSION_HEAL)
	{
		if (isHurt())
		{
			if (!hasMoved())
			{
				if ((pPlot->getTeam() == getTeam()) || (healTurns(pPlot) < 4))
				{
					return true;
				}
			}
		}
	}

	if (GC.getActionInfo(iAction).getMissionType() == MISSION_FOUND)
	{
		if (canFound(pPlot))
		{
			if (pPlot->isBestAdjacentFound(getOwnerINLINE()))
			{
				return true;
			}
		}
	}

	if (GC.getActionInfo(iAction).getMissionType() == MISSION_BUILD)
	{
		if (pPlot->getOwnerINLINE() == getOwnerINLINE())
		{
			eBuild = ((BuildTypes)(GC.getActionInfo(iAction).getMissionData()));
			FAssert(eBuild != NO_BUILD);
			FAssertMsg(eBuild < GC.getNumBuildInfos(), "Invalid Build");

			if (canBuild(pPlot, eBuild))
			{
				eImprovement = ((ImprovementTypes)(GC.getBuildInfo(eBuild).getImprovement()));
				eRoute = ((RouteTypes)(GC.getBuildInfo(eBuild).getRoute()));
				eBonus = pPlot->getBonusType(getTeam());
				pWorkingCity = pPlot->getWorkingCity();

				if (pPlot->getImprovementType() == NO_IMPROVEMENT)
				{
					if (pWorkingCity != NULL)
					{
						iIndex = pWorkingCity->getCityPlotIndex(pPlot);

						if (iIndex != -1)
						{
							if (pWorkingCity->AI_getBestBuild(iIndex) == eBuild)
							{
								return true;
							}
						}
					}

					if (eImprovement != NO_IMPROVEMENT)
					{
						if (eBonus != NO_BONUS)
						{
							if (GC.getImprovementInfo(eImprovement).isImprovementBonusTrade(eBonus))
							{
								return true;
							}
						}

						if (pPlot->getImprovementType() == NO_IMPROVEMENT)
						{
							if (!(pPlot->isIrrigated()) && pPlot->isIrrigationAvailable(true))
							{
								if (GC.getImprovementInfo(eImprovement).isCarriesIrrigation())
								{
									return true;
								}
							}

							if (pWorkingCity != NULL)
							{
								if (GC.getImprovementInfo(eImprovement).getYieldChange(YIELD_FOOD) > 0)
								{
									return true;
								}

								if (pPlot->isHills())
								{
									if (GC.getImprovementInfo(eImprovement).getYieldChange(YIELD_PRODUCTION) > 0)
									{
										return true;
									}
								}
								else
								{
									if (GC.getImprovementInfo(eImprovement).getYieldChange(YIELD_COMMERCE) > 0)
									{
										return true;
									}
								}
							}
						}
					}
				}

				if (eRoute != NO_ROUTE)
				{
					if (!(pPlot->isRoute()))
					{
						if (eBonus != NO_BONUS)
						{
							return true;
						}

						if (pWorkingCity != NULL)
						{
							if (pPlot->isRiver())
							{
								return true;
							}
						}
					}

					eFinalImprovement = eImprovement;

					if (eFinalImprovement == NO_IMPROVEMENT)
					{
						eFinalImprovement = pPlot->getImprovementType();
					}

					if (eFinalImprovement != NO_IMPROVEMENT)
					{
						if ((GC.getImprovementInfo(eFinalImprovement).getRouteYieldChanges(eRoute, YIELD_FOOD) > 0) ||
							(GC.getImprovementInfo(eFinalImprovement).getRouteYieldChanges(eRoute, YIELD_PRODUCTION) > 0) ||
							(GC.getImprovementInfo(eFinalImprovement).getRouteYieldChanges(eRoute, YIELD_COMMERCE) > 0))
						{
							return true;
						}
					}
				}
			}
		}
	}

	if (GC.getActionInfo(iAction).getCommandType() == COMMAND_PROMOTION)
	{
		return true;
	}

	// Sanguo Mod Hero, start, added by poyuzhe 01.17.09
	if (GC.getActionInfo(iAction).getCommandType() == COMMAND_PROMOTION_HERO)
	{
		return true;
	}
    // Sanguo Mod Hero, end

	return false;
}


bool CvUnit::isBetterDefenderThan(const CvUnit* pDefender, const CvUnit* pAttacker) const
{
	PROFILE_FUNC();
	
	int iOurDefense;
	int iTheirDefense;

	if (pDefender == NULL)
	{
		return true;
	}

	TeamTypes eAttackerTeam = NO_TEAM;
	if (NULL != pAttacker)
	{
		eAttackerTeam = pAttacker->getTeam();
	}

	if (canCoexistWithEnemyUnit(eAttackerTeam))
	{
		return false;
	}

	if (!canDefend())
	{
		return false;
	}

	if (canDefend() && !(pDefender->canDefend()))
	{
		return true;
	}

	if (pAttacker)
	{
		if (isTargetOf(*pAttacker) && !pDefender->isTargetOf(*pAttacker))
		{
			return true;
		}

		if (!isTargetOf(*pAttacker) && pDefender->isTargetOf(*pAttacker))
		{
			return false;
		}

		if (pAttacker->canAttack(*pDefender) && !pAttacker->canAttack(*this))
		{
			return false;
		}

		if (pAttacker->canAttack(*this) && !pAttacker->canAttack(*pDefender))
		{
			return true;
		}
	}

	// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
	bool bAttackerMarksMan = false;
	if (pAttacker != NULL && pAttacker->isCoreTroop())
	{
		PROFILE_BEGIN("BetterDefender::loop");
		bool bUsCoreTroop = isCoreTroop();
        bool bDefenderCoreTroop = pDefender->isCoreTroop();
        bool bUsGuardian = isGuard();
        bool bDefenderGuardian = pDefender->isGuard();
        bool bAttackerAttackCoreTroop = false;

        for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
        {
            if (GC.getPromotionInfo((PromotionTypes)iI).isTargetWeakestFirst())
            {
                if (pAttacker->isHasPromotion((PromotionTypes)iI))
                {
                    bAttackerMarksMan = true;
                }
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isTargetCoreTroopFirst())
            {
                if (pAttacker->isHasPromotion((PromotionTypes)iI))
                {
                    bAttackerAttackCoreTroop = true;
                }
            }
        }

		PROFILE_END();

        if (bAttackerMarksMan)
        {
            if (bUsGuardian && !bDefenderGuardian)
            {
                return true;
            }
            if (!bUsGuardian && bDefenderGuardian)
            {
                return false;
            }
        }

        if (bAttackerAttackCoreTroop)
        {
            if (bUsCoreTroop && !bDefenderCoreTroop )
            {
                if (!bDefenderGuardian || bUsGuardian)
                {
                    return true;
                }
                return false;
            }
            if (!bUsCoreTroop && bDefenderCoreTroop)
            {
                if (!bUsGuardian || bDefenderGuardian)
                {
                    return false;
                }
                return true;
            }
        }
	}
    // Sanguo Mod Hero, end

	iOurDefense = currCombatStr(plot(), pAttacker);
	if (::isWorldUnitClass(getUnitClassType()))
	{
		iOurDefense /= 2;
	}

	if (NULL == pAttacker)
	{
		if (pDefender->collateralDamage() > 0)
		{
			iOurDefense *= (100 + pDefender->collateralDamage());
			iOurDefense /= 100;
		}

		if (pDefender->currInterceptionProbability() > 0)
		{
			iOurDefense *= (100 + pDefender->currInterceptionProbability());
			iOurDefense /= 100;
		}
	}
	else
	{
		if (!(pAttacker->immuneToFirstStrikes()))
		{
			iOurDefense *= ((((firstStrikes() * 2) + chanceFirstStrikes()) * ((GC.getDefineINT("COMBAT_DAMAGE") * 2) / 5)) + 100);
			iOurDefense /= 100;
		}

		if (immuneToFirstStrikes())
		{
			iOurDefense *= ((((pAttacker->firstStrikes() * 2) + pAttacker->chanceFirstStrikes()) * ((GC.getDefineINT("COMBAT_DAMAGE") * 2) / 5)) + 100);
			iOurDefense /= 100;
		}
	}

	int iAssetValue = std::max(1, getUnitInfo().getAssetValue());
	int iCargoAssetValue = 0;
	std::vector<CvUnit*> aCargoUnits;
	getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
		iCargoAssetValue += aCargoUnits[i]->getUnitInfo().getAssetValue();
	}
	iOurDefense = iOurDefense * iAssetValue / std::max(1, iAssetValue + iCargoAssetValue);

	iTheirDefense = pDefender->currCombatStr(plot(), pAttacker);
	if (::isWorldUnitClass(pDefender->getUnitClassType()))
	{
		iTheirDefense /= 2;
	}

	if (NULL == pAttacker)
	{
		if (collateralDamage() > 0)
		{
			iTheirDefense *= (100 + collateralDamage());
			iTheirDefense /= 100;
		}

		if (currInterceptionProbability() > 0)
		{
			iTheirDefense *= (100 + currInterceptionProbability());
			iTheirDefense /= 100;
		}
	}
	else
	{
		if (!(pAttacker->immuneToFirstStrikes()))
		{
			iTheirDefense *= ((((pDefender->firstStrikes() * 2) + pDefender->chanceFirstStrikes()) * ((GC.getDefineINT("COMBAT_DAMAGE") * 2) / 5)) + 100);
			iTheirDefense /= 100;
		}

		if (pDefender->immuneToFirstStrikes())
		{
			iTheirDefense *= ((((pAttacker->firstStrikes() * 2) + pAttacker->chanceFirstStrikes()) * ((GC.getDefineINT("COMBAT_DAMAGE") * 2) / 5)) + 100);
			iTheirDefense /= 100;
		}
	}

	iAssetValue = std::max(1, pDefender->getUnitInfo().getAssetValue());
	iCargoAssetValue = 0;
	pDefender->getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
		iCargoAssetValue += aCargoUnits[i]->getUnitInfo().getAssetValue();
	}
	iTheirDefense = iTheirDefense * iAssetValue / std::max(1, iAssetValue + iCargoAssetValue);

	if (iOurDefense == iTheirDefense)
	{
		if (NO_UNIT == getLeaderUnitType() && NO_UNIT != pDefender->getLeaderUnitType())
		{
			++iOurDefense;
		}
		else if (NO_UNIT != getLeaderUnitType() && NO_UNIT == pDefender->getLeaderUnitType())
		{
			++iTheirDefense;
		}
		else if (isBeforeUnitCycle(this, pDefender))
		{
			++iOurDefense;
		}
	}

    // Sanguo Mod Hero, start, added by poyuzhe 02.01.09
    // The following line is commented out
//	return (iOurDefense > iTheirDefense);
    return (bAttackerMarksMan ? (iOurDefense < iTheirDefense) : (iOurDefense > iTheirDefense));
	// Sanguo Mod Hero, end
}


bool CvUnit::canDoCommand(CommandTypes eCommand, int iData1, int iData2, bool bTestVisible, bool bTestBusy)
{
	CvUnit* pUnit;

	if (bTestBusy && getGroup()->isBusy())
	{
		return false;
	}

	switch (eCommand)
	{
	case COMMAND_PROMOTION:
		if (canPromote((PromotionTypes)iData1, iData2))
		{
			return true;
		}
		break;

	case COMMAND_UPGRADE:
		if (canUpgrade(((UnitTypes)iData1), bTestVisible))
		{
			return true;
		}
		break;

	case COMMAND_AUTOMATE:
		if (canAutomate((AutomateTypes)iData1))
		{
			return true;
		}
		break;

	case COMMAND_WAKE:
		if (!isAutomated() && isWaiting())
		{
			return true;
		}
		break;

	case COMMAND_CANCEL:
	case COMMAND_CANCEL_ALL:
		if (!isAutomated() && (getGroup()->getLengthMissionQueue() > 0))
		{
			return true;
		}
		break;

	case COMMAND_STOP_AUTOMATION:
		if (isAutomated())
		{
			return true;
		}
		break;

	case COMMAND_DELETE:
		if (canScrap())
		{
			return true;
		}
		break;

	case COMMAND_GIFT:
		if (canGift(bTestVisible))
		{
			return true;
		}
		break;

	case COMMAND_LOAD:
		if (canLoad(plot()))
		{
			return true;
		}
		break;

	case COMMAND_LOAD_UNIT:
		pUnit = ::getUnit(IDInfo(((PlayerTypes)iData1), iData2));
		if (pUnit != NULL)
		{
			if (canLoadUnit(pUnit, plot()))
			{
				return true;
			}
		}
		break;

	case COMMAND_UNLOAD:
		if (canUnload())
		{
			return true;
		}
		break;

	case COMMAND_UNLOAD_ALL:
		if (canUnloadAll())
		{
			return true;
		}
		break;

	case COMMAND_HOTKEY:
		if (isGroupHead())
		{
			return true;
		}
		break;

    // Sanguo Mod Hero, start, added by poyuzhe 01.17.09
    case COMMAND_PROMOTION_HERO:
		if (GC.getGameINLINE().canPromoteHero(getHeroType(), (PromotionTypes)iData1))
		{
			return true;
		}
        if (getLeaderHero() != NO_HERO)
        {
            if (GC.getGameINLINE().getHeroCoreTroop(getLeaderHero()) == getID())
            {
                if (GC.getGameINLINE().canPromoteHero(getLeaderHero(), (PromotionTypes)iData1))
                {
                    return true;
                }
            }
        }
		break;
    // Sanguo Mod Hero, end

	default:
		FAssert(false);
		break;
	}

	return false;
}


void CvUnit::doCommand(CommandTypes eCommand, int iData1, int iData2)
{
	CvUnit* pUnit;
	bool bCycle;

	bCycle = false;

	FAssert(getOwnerINLINE() != NO_PLAYER);

	if (canDoCommand(eCommand, iData1, iData2))
	{
		switch (eCommand)
		{
		case COMMAND_PROMOTION:
			promote((PromotionTypes)iData1, iData2);
			break;

		case COMMAND_UPGRADE:
			upgrade((UnitTypes)iData1);
			bCycle = true;
			break;

		case COMMAND_AUTOMATE:
			automate((AutomateTypes)iData1);
			bCycle = true;
			break;

		case COMMAND_WAKE:
			getGroup()->setActivityType(ACTIVITY_AWAKE);
			break;

		case COMMAND_CANCEL:
			getGroup()->popMission();
			break;

		case COMMAND_CANCEL_ALL:
			getGroup()->clearMissionQueue();
			break;

		case COMMAND_STOP_AUTOMATION:
			getGroup()->setAutomateType(NO_AUTOMATE);
			break;

		case COMMAND_DELETE:
			scrap();
			bCycle = true;
			break;

		case COMMAND_GIFT:
			gift();
			bCycle = true;
			break;

		case COMMAND_LOAD:
			load();
			bCycle = true;
			break;

		case COMMAND_LOAD_UNIT:
			pUnit = ::getUnit(IDInfo(((PlayerTypes)iData1), iData2));
			if (pUnit != NULL)
			{
				loadUnit(pUnit);
				bCycle = true;
			}
			break;

		case COMMAND_UNLOAD:
			unload();
			bCycle = true;
			break;

		case COMMAND_UNLOAD_ALL:
			unloadAll();
			bCycle = true;
			break;

		case COMMAND_HOTKEY:
			setHotKeyNumber(iData1);
			break;

        // Sanguo Mod Hero, start, added by poyuzhe 01.17.09
        case COMMAND_PROMOTION_HERO:
			if (getLeaderHero() == NO_HERO || GC.getGameINLINE().canPromoteHero(getHeroType(), (PromotionTypes)iData1))
			{
				GC.getGameINLINE().promoteHero(getHeroType(), (PromotionTypes)iData1);
			}
			else
			{
				FAssert (getLeaderHero() != NO_HERO);
				FAssert (GC.getGameINLINE().canPromoteHero(getLeaderHero(), (PromotionTypes)iData1));
				GC.getGameINLINE().promoteHero(getLeaderHero(), (PromotionTypes)iData1);
			}
			break;
        // Sanguo Mod Hero, end

		default:
			FAssert(false);
			break;
		}
	}

	if (bCycle)
	{
		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setCycleSelectionCounter(1);
		}
	}

	getGroup()->doDelayedDeath();
}


FAStarNode* CvUnit::getPathLastNode() const
{
	return getGroup()->getPathLastNode();
}


CvPlot* CvUnit::getPathEndTurnPlot() const
{
	return getGroup()->getPathEndTurnPlot();
}


bool CvUnit::generatePath(const CvPlot* pToPlot, int iFlags, bool bReuse, int* piPathTurns) const
{
	return getGroup()->generatePath(plot(), pToPlot, iFlags, bReuse, piPathTurns);
}


bool CvUnit::canEnterTerritory(TeamTypes eTeam, bool bIgnoreRightOfPassage) const
{
	if (GET_TEAM(getTeam()).isFriendlyTerritory(eTeam))
	{
		return true;
	}

	if (eTeam == NO_TEAM)
	{
		return true;
	}

	if (isEnemy(eTeam))
	{
		return true;
	}

	if (isRivalTerritory())
	{
		return true;
	}

	if (alwaysInvisible())
	{
		return true;
	}

	if (m_pUnitInfo->isHiddenNationality())
	{
		return true;
	}

	if (!bIgnoreRightOfPassage)
	{
		if (GET_TEAM(getTeam()).isOpenBorders(eTeam))
		{
			return true;
		}
	}

	return false;
}


bool CvUnit::canEnterArea(TeamTypes eTeam, const CvArea* pArea, bool bIgnoreRightOfPassage) const
{
	if (!canEnterTerritory(eTeam, bIgnoreRightOfPassage))
	{
		return false;
	}

	if (isBarbarian() && DOMAIN_LAND == getDomainType())
	{
		if (eTeam != NO_TEAM && eTeam != getTeam())
		{
			if (pArea && pArea->isBorderObstacle(eTeam))
			{
				return false;
			}
		}
	}

	return true;
}


// Returns the ID of the team to declare war against
TeamTypes CvUnit::getDeclareWarMove(const CvPlot* pPlot) const
{
	CvUnit* pUnit;
	TeamTypes eRevealedTeam;

	FAssert(isHuman());

	if (getDomainType() != DOMAIN_AIR)
	{
		eRevealedTeam = pPlot->getRevealedTeam(getTeam(), false);

		if (eRevealedTeam != NO_TEAM)
		{
			if (!canEnterArea(eRevealedTeam, pPlot->area()) || (getDomainType() == DOMAIN_SEA && !canCargoEnterArea(eRevealedTeam, pPlot->area(), false) && getGroup()->isAmphibPlot(pPlot)))
			{
				if (GET_TEAM(getTeam()).canDeclareWar(pPlot->getTeam()))
				{
					return eRevealedTeam;
				}
			}
		}
		else
		{
			if (pPlot->isActiveVisible(false))
			{
				if (canMoveInto(pPlot, true, true, true))
				{
					pUnit = pPlot->plotCheck(PUF_canDeclareWar, getOwnerINLINE(), isAlwaysHostile(pPlot), NO_PLAYER, NO_TEAM, PUF_isVisible, getOwnerINLINE());

					if (pUnit != NULL)
					{
						return pUnit->getTeam();
					}
				}
			}
		}
	}

	return NO_TEAM;
}

bool CvUnit::willRevealByMove(const CvPlot* pPlot) const
{
	int iRange = visibilityRange() + 1;
	for (int i = -iRange; i <= iRange; ++i)
	{
		for (int j = -iRange; j <= iRange; ++j)
		{
			CvPlot* pLoopPlot = ::plotXY(pPlot->getX_INLINE(), pPlot->getY_INLINE(), i, j);
			if (NULL != pLoopPlot)
			{
				if (!pLoopPlot->isRevealed(getTeam(), false) && pPlot->canSeePlot(pLoopPlot, getTeam(), visibilityRange(), NO_DIRECTION))
				{
					return true;
				}
			}
		}
	}

	return false;
}

bool CvUnit::canMoveInto(const CvPlot* pPlot, bool bAttack, bool bDeclareWar, bool bIgnoreLoad) const
{
	PROFILE_FUNC();

	FAssertMsg(pPlot != NULL, "Plot is not assigned a valid value");

	if (atPlot(pPlot))
	{
		return false;
	}

	if (pPlot->isImpassable())
	{
		if (!canMoveImpassable())
		{
			return false;
		}
	}

	// Cannot move around in unrevealed land freely
	if (m_pUnitInfo->isNoRevealMap() && willRevealByMove(pPlot))
	{
		return false;
	}

	if (GC.getUSE_SPIES_NO_ENTER_BORDERS())
	{
		if (isSpy() && NO_PLAYER != pPlot->getOwnerINLINE())
		{
			if (!GET_PLAYER(getOwnerINLINE()).canSpiesEnterBorders(pPlot->getOwnerINLINE()))
			{
				return false;
			}
		}
	}

	CvArea *pPlotArea = pPlot->area();
	TeamTypes ePlotTeam = pPlot->getTeam();
	bool bCanEnterArea = canEnterArea(ePlotTeam, pPlotArea);
	if (bCanEnterArea)
	{
		if (pPlot->getFeatureType() != NO_FEATURE)
		{
			if (m_pUnitInfo->getFeatureImpassable(pPlot->getFeatureType()))
			{
				TechTypes eTech = (TechTypes)m_pUnitInfo->getFeaturePassableTech(pPlot->getFeatureType());
				if (NO_TECH == eTech || !GET_TEAM(getTeam()).isHasTech(eTech))
				{
					if (DOMAIN_SEA != getDomainType() || pPlot->getTeam() != getTeam())  // sea units can enter impassable in own cultural borders
					{
						return false;
					}
				}
			}
		}
		// Sanguo Mod Civilization Trait, start, by poyuzhe 11.26.08
		// Zhang Yang, TRAIT_FOREIGNER
		else if (m_pUnitInfo->isCamp())
		{
			if (pPlot->isCity())
			{
				return false;
			}
		}
		// Sanguo Mod Civilization Trait, end
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                       09/17/09                         TC01 & jdog5000      */
/**                                                                                              */
/** Bugfix				                                                                         */
/*************************************************************************************************/
/* original bts code
		else
*/
		// always check terrain also
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                        END                                                  */
/*************************************************************************************************/

		{
			if (m_pUnitInfo->getTerrainImpassable(pPlot->getTerrainType()))
			{
				TechTypes eTech = (TechTypes)m_pUnitInfo->getTerrainPassableTech(pPlot->getTerrainType());
				if (NO_TECH == eTech || !GET_TEAM(getTeam()).isHasTech(eTech))
				{
					if (DOMAIN_SEA != getDomainType() || pPlot->getTeam() != getTeam())  // sea units can enter impassable in own cultural borders
					{
						if (bIgnoreLoad || !canLoad(pPlot))
						{
							return false;
						}
					}
				}
			}
		}
	}

	switch (getDomainType())
	{
	case DOMAIN_SEA:
		if (!pPlot->isWater() && !canMoveAllTerrain())
		{
			if (!pPlot->isFriendlyCity(*this, true) || !pPlot->isCoastalLand())
			{
				return false;
			}
		}
		break;

	case DOMAIN_AIR:
		if (!bAttack)
		{
			bool bValid = false;

			if (pPlot->isFriendlyCity(*this, true))
			{
				bValid = true;

				if (m_pUnitInfo->getAirUnitCap() > 0)
				{
					if (pPlot->airUnitSpaceAvailable(getTeam()) <= 0)
					{
						bValid = false;
					}
				}
			}

			if (!bValid)
			{
				if (bIgnoreLoad || !canLoad(pPlot))
				{
					return false;
				}
			}
		}

		break;

	case DOMAIN_LAND:
		if (pPlot->isWater() && !canMoveAllTerrain())
		{
			if (!pPlot->isCity() || 0 == GC.getDefineINT("LAND_UNITS_CAN_ATTACK_WATER_CITIES"))
			{
				if (bIgnoreLoad || !isHuman() || plot()->isWater() || !canLoad(pPlot))
				{
					return false;
				}
			}
		}
		break;

	case DOMAIN_IMMOBILE:
		return false;
		break;

	default:
		FAssert(false);
		break;
	}

	if (isAnimal())
	{
		if (pPlot->isOwned())
		{
			return false;
		}

		if (!bAttack)
		{
			if (pPlot->getBonusType() != NO_BONUS)
			{
				return false;
			}

			if (pPlot->getImprovementType() != NO_IMPROVEMENT)
			{
				return false;
			}

			if (pPlot->getNumUnits() > 0)
			{
				return false;
			}
		}
	}

	if (isNoCapture())
	{
		if (!bAttack)
		{
			if (pPlot->isEnemyCity(*this))
			{
				return false;
			}
		}
	}

	if (bAttack)
	{
		// Sanguo Mod Hero, start, added by poyuzhe 03.10.09
		// UNOFFICIAL_PATCH start
		//if ((!isTempCheck() && isMadeAttack()) && !isBlitz())
		if ((!isTempCheck() && isMadeAttack()) && !isBlitz() && (pPlot->getNumVisibleEnemyDefenders(this) > 0))
		// UNOFFICIAL_PATCH start
		// Sanguo Mod Hero, end
		{
			return false;
		}
	}

	if (getDomainType() == DOMAIN_AIR)
	{
		if (bAttack)
		{
			if (!canAirStrike(pPlot))
			{
				return false;
			}
		}
	}
	else
	{
		if (canAttack())
		{
			if (bAttack || !canCoexistWithEnemyUnit(NO_TEAM))
			{
				if (!isHuman() || (pPlot->isVisible(getTeam(), false)))
				{
					if (pPlot->isVisibleEnemyUnit(this) != bAttack)
					{
						//FAssertMsg(isHuman() || (!bDeclareWar || (pPlot->isVisibleOtherUnit(getOwnerINLINE()) != bAttack)), "hopefully not an issue, but tracking how often this is the case when we dont want to really declare war");
						if (!bDeclareWar || (pPlot->isVisibleOtherUnit(getOwnerINLINE()) != bAttack && !(bAttack && pPlot->getPlotCity() && !isNoCapture())))
						{
							return false;
						}
					}
				}
			}

			if (bAttack)
			{
				CvUnit* pDefender = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
				if (NULL != pDefender)
				{
					if (!canAttack(*pDefender))
					{
						return false;
					}
				}
			}
		}
		else
		{
			if (bAttack)
			{
				return false;
			}

			if (!canCoexistWithEnemyUnit(NO_TEAM))
			{
				if (!isHuman() || pPlot->isVisible(getTeam(), false))
				{
					if (pPlot->isEnemyCity(*this))
					{
						return false;
					}

					if (pPlot->isVisibleEnemyUnit(this))
					{
						return false;
					}
				}
			}
		}

		if (isHuman())
		{
			ePlotTeam = pPlot->getRevealedTeam(getTeam(), false);
			bCanEnterArea = canEnterArea(ePlotTeam, pPlotArea);
		}

		if (!bCanEnterArea)
		{
			FAssert(ePlotTeam != NO_TEAM);

			if (!(GET_TEAM(getTeam()).canDeclareWar(ePlotTeam)))
			{
				return false;
			}

			if (isHuman())
			{
				if (!bDeclareWar)
				{
					return false;
				}
			}
			else
			{
				if (GET_TEAM(getTeam()).AI_isSneakAttackReady(ePlotTeam))
				{
					if (!(getGroup()->AI_isDeclareWar(pPlot)))
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}
		}
	}

	if (GC.getUSE_UNIT_CANNOT_MOVE_INTO_CALLBACK())
	{
		// Python Override
		CyArgsList argsList;
		argsList.add(getOwnerINLINE());	// Player ID
		argsList.add(getID());	// Unit ID
		argsList.add(pPlot->getX());	// Plot X
		argsList.add(pPlot->getY());	// Plot Y
		long lResult=0;
		gDLL->getPythonIFace()->callFunction(PYGameModule, "unitCannotMoveInto", argsList.makeFunctionArgs(), &lResult);

		if (lResult != 0)
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::canMoveOrAttackInto(const CvPlot* pPlot, bool bDeclareWar) const
{
	return (canMoveInto(pPlot, false, bDeclareWar) || canMoveInto(pPlot, true, bDeclareWar));
}


bool CvUnit::canMoveThrough(const CvPlot* pPlot) const
{
	return canMoveInto(pPlot, false, false, true);
}


// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
// comment 1 line out
//void CvUnit::attack(CvPlot* pPlot, bool bQuick)
void CvUnit::attack(CvPlot* pPlot, bool bQuick, CvUnit* pDefender)
// Sanguo Mod Hero, end
{
	FAssert(canMoveInto(pPlot, true));
	FAssert(getCombatTimer() == 0);

	setAttackPlot(pPlot, false);

	// Sanguo Mod Hero, start, by poyuzhe 09.27.09
	if (getLegionHero() != NO_HERO)
	{
		GC.getGameINLINE().setLegionUnitMoved(getLegionHero(), true);
		if (getLeaderHero() != NO_HERO)
		{
			GC.getGameINLINE().setLegionUnitMoved(getLeaderHero(), true);
		}
	}
	// Sanguo Mod Hero, end

	// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
    // comment 1 line out
//	updateCombat(bQuick);
	updateCombat(bQuick, pDefender);
	// Sanguo Mod Hero, end
}

void CvUnit::fightInterceptor(const CvPlot* pPlot, bool bQuick)
{
	FAssert(getCombatTimer() == 0);

	setAttackPlot(pPlot, true);

	updateAirCombat(bQuick);
}

void CvUnit::attackForDamage(CvUnit *pDefender, int attackerDamageChange, int defenderDamageChange)
{
	FAssert(getCombatTimer() == 0);
	FAssert(pDefender != NULL);
	FAssert(!isFighting());

	if(pDefender == NULL)
	{
		return;
	}

	setAttackPlot(pDefender->plot(), false);

	CvPlot* pPlot = getAttackPlot();
	if (pPlot == NULL)
	{
		return;
	}

	//rotate to face plot
	DirectionTypes newDirection = estimateDirection(this->plot(), pDefender->plot());
	if(newDirection != NO_DIRECTION)
	{
		setFacingDirection(newDirection);
	}

	//rotate enemy to face us
	newDirection = estimateDirection(pDefender->plot(), this->plot());
	if(newDirection != NO_DIRECTION)
	{
		pDefender->setFacingDirection(newDirection);
	}

	//check if quick combat
	bool bVisible = isCombatVisible(pDefender);

	//if not finished and not fighting yet, set up combat damage and mission
	if (!isFighting())
	{
		if (plot()->isFighting() || pPlot->isFighting())
		{
			return;
		}

		setCombatUnit(pDefender, true);
		pDefender->setCombatUnit(this, false);

		pDefender->getGroup()->clearMissionQueue();

		bool bFocused = (bVisible && isCombatFocus() && gDLL->getInterfaceIFace()->isCombatFocus());

		if (bFocused)
		{
			DirectionTypes directionType = directionXY(plot(), pPlot);
			//								N			NE				E				SE					S				SW					W				NW
			NiPoint2 directions[8] = {NiPoint2(0, 1), NiPoint2(1, 1), NiPoint2(1, 0), NiPoint2(1, -1), NiPoint2(0, -1), NiPoint2(-1, -1), NiPoint2(-1, 0), NiPoint2(-1, 1)};
			NiPoint3 attackDirection = NiPoint3(directions[directionType].x, directions[directionType].y, 0);
			float plotSize = GC.getPLOT_SIZE();
			NiPoint3 lookAtPoint(plot()->getPoint().x + plotSize / 2 * attackDirection.x, plot()->getPoint().y + plotSize / 2 * attackDirection.y, (plot()->getPoint().z + pPlot->getPoint().z) / 2);
			attackDirection.Unitize();
			gDLL->getInterfaceIFace()->lookAt(lookAtPoint, (((getOwnerINLINE() != GC.getGameINLINE().getActivePlayer()) || gDLL->getGraphicOption(GRAPHICOPTION_NO_COMBAT_ZOOM)) ? CAMERALOOKAT_BATTLE : CAMERALOOKAT_BATTLE_ZOOM_IN), attackDirection);
		}
		else
		{
			PlayerTypes eAttacker = getVisualOwner(pDefender->getTeam());
			CvWString szMessage;
			if (BARBARIAN_PLAYER != eAttacker)
			{
				szMessage = gDLL->getText("TXT_KEY_MISC_YOU_UNITS_UNDER_ATTACK", GET_PLAYER(getOwnerINLINE()).getNameKey());
			}
			else
			{
				szMessage = gDLL->getText("TXT_KEY_MISC_YOU_UNITS_UNDER_ATTACK_UNKNOWN");
			}

			gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szMessage, "AS2D_COMBAT", MESSAGE_TYPE_DISPLAY_ONLY, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true);
		}
	}

	FAssertMsg(plot()->isFighting(), "Current unit instance plot is not fighting as expected");
	FAssertMsg(pPlot->isFighting(), "pPlot is not fighting as expected");

	//setup battle object
	CvBattleDefinition kBattle;
	kBattle.setUnit(BATTLE_UNIT_ATTACKER, this);
	kBattle.setUnit(BATTLE_UNIT_DEFENDER, pDefender);
	kBattle.setDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_BEGIN, getDamage());
	kBattle.setDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_BEGIN, pDefender->getDamage());

	changeDamage(attackerDamageChange, pDefender->getOwnerINLINE());
	pDefender->changeDamage(defenderDamageChange, getOwnerINLINE());

	if (bVisible)
	{
		kBattle.setDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_END, getDamage());
		kBattle.setDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_END, pDefender->getDamage());
		kBattle.setAdvanceSquare(canAdvance(pPlot, 1));

		kBattle.addDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_ATTACKER, BATTLE_TIME_BEGIN));
		kBattle.addDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_RANGED, kBattle.getDamage(BATTLE_UNIT_DEFENDER, BATTLE_TIME_BEGIN));

		int iTurns = planBattle( kBattle);
		kBattle.setMissionTime(iTurns * gDLL->getSecsPerTurn());
		setCombatTimer(iTurns);

		GC.getGameINLINE().incrementTurnTimer(getCombatTimer());

		if (pPlot->isActiveVisible(false))
		{
			ExecuteMove(0.5f, true);
			gDLL->getEntityIFace()->AddMission(&kBattle);
		}
	}
	else
	{
		setCombatTimer(1);
	}
}

// Sanguo Mod Hero, start, modified by poyuzhe, 08.06.09
// void CvUnit::move(CvPlot* pPlot, bool bShow)
void CvUnit::move(CvPlot* pPlot, bool bShow, bool bFlyingGeneral)
// Sanguo Mod Hero, end
{
	FAssert(canMoveOrAttackInto(pPlot) || isMadeAttack());

	CvPlot* pOldPlot = plot();

	// Sanguo Mod Hero, start, modified by poyuzhe, 08.06.09
	if (!bFlyingGeneral)
	{
		changeMoves(pPlot->movementCost(this, plot()));
	}
	// Sanguo Mod Hero, end

	setXY(pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true, bShow && pPlot->isVisibleToWatchingHuman(), bShow);

	//change feature
	FeatureTypes featureType = pPlot->getFeatureType();
	if(featureType != NO_FEATURE)
	{
		CvString featureString(GC.getFeatureInfo(featureType).getOnUnitChangeTo());
		if(!featureString.IsEmpty())
		{
			FeatureTypes newFeatureType = (FeatureTypes) GC.getInfoTypeForString(featureString);
			pPlot->setFeatureType(newFeatureType);
		}
	}

	// Sanguo Mod Civilization Trait, start, by poyuzhe 11.25.08
	// Zhang Yang, TRAIT_FOREIGNER
	if (m_pUnitInfo->isCamp())
	{
		SanguoCampTypes eCamp = getCampType();
		FAssert(eCamp != NO_SANGUOCAMP);
		bool bUpdate = false;

		if (pOldPlot->getOwnerINLINE() == getOwnerINLINE()) // update the yield of pOldPlot if an own camp used to be on it
		{
			BonusTypes eBonus = pOldPlot->getBonusType();
			if (eBonus != NO_BONUS && GET_TEAM(getTeam()).isHasTech((TechTypes)(GC.getBonusInfo(eBonus).getTechReveal())))
			{
				for (int iI = 0; iI < NUM_YIELD_TYPES; iI++)
				{
					if (GC.getSanguoCampInfo(eCamp).getBonusYieldChange(eBonus, iI) != 0)
					{
						bUpdate = true;
						break;
					}
				}

				if (bUpdate)
				{
					pOldPlot->updateYield();

				}
			}
		}

		if (pPlot->getOwnerINLINE() == getOwnerINLINE()) // update the yield of pPlot if an own camp is moving onto it
		{
			BonusTypes eBonus = pPlot->getBonusType();
			if (eBonus != NO_BONUS && GET_TEAM(getTeam()).isHasTech((TechTypes)(GC.getBonusInfo(eBonus).getTechReveal())))
			{
				bUpdate = false;

				for (int iI = 0; iI < NUM_YIELD_TYPES; iI++)
				{
					if (GC.getSanguoCampInfo(eCamp).getBonusYieldChange(eBonus, iI) != 0)
					{
						bUpdate = true;
						break;
					}
				}

				if (bUpdate)
				{
					pPlot->updateYield();
				}
			}
		}
	}
	// Sanguo Mod Civilization Trait, end

	if (getOwnerINLINE() == GC.getGameINLINE().getActivePlayer())
	{
		if (!(pPlot->isOwned()))
		{
			//spawn birds if trees present - JW
			if (featureType != NO_FEATURE)
			{
				if (GC.getASyncRand().get(100) < GC.getFeatureInfo(featureType).getEffectProbability())
				{
					EffectTypes eEffect = (EffectTypes)GC.getInfoTypeForString(GC.getFeatureInfo(featureType).getEffectType());
					gDLL->getEngineIFace()->TriggerEffect(eEffect, pPlot->getPoint(), (float)(GC.getASyncRand().get(360)));
					gDLL->getInterfaceIFace()->playGeneralSound("AS3D_UN_BIRDS_SCATTER", pPlot->getPoint());
				}
			}
		}
	}

	CvEventReporter::getInstance().unitMove(pPlot, this, pOldPlot);
}

// false if unit is killed
bool CvUnit::jumpToNearestValidPlot()
{
	CvCity* pNearestCity;
	CvPlot* pLoopPlot;
	CvPlot* pBestPlot;
	int iValue;
	int iBestValue;
	int iI;

	FAssertMsg(!isAttacking(), "isAttacking did not return false as expected");
	FAssertMsg(!isFighting(), "isFighting did not return false as expected");

	pNearestCity = GC.getMapINLINE().findCity(getX_INLINE(), getY_INLINE(), getOwnerINLINE());

	iBestValue = MAX_INT;
	pBestPlot = NULL;

	for (iI = 0; iI < GC.getMapINLINE().numPlotsINLINE(); iI++)
	{
		pLoopPlot = GC.getMapINLINE().plotByIndexINLINE(iI);

		if (pLoopPlot->isValidDomainForLocation(*this))
		{
			if (canMoveInto(pLoopPlot))
			{
				if (canEnterArea(pLoopPlot->getTeam(), pLoopPlot->area()) && !isEnemy(pLoopPlot->getTeam(), pLoopPlot))
				{
					FAssertMsg(!atPlot(pLoopPlot), "atPlot(pLoopPlot) did not return false as expected");

					if ((getDomainType() != DOMAIN_AIR) || pLoopPlot->isFriendlyCity(*this, true))
					{
						if (pLoopPlot->isRevealed(getTeam(), false))
						{
							iValue = (plotDistance(getX_INLINE(), getY_INLINE(), pLoopPlot->getX_INLINE(), pLoopPlot->getY_INLINE()) * 2);

							if (pNearestCity != NULL)
							{
								iValue += plotDistance(pLoopPlot->getX_INLINE(), pLoopPlot->getY_INLINE(), pNearestCity->getX_INLINE(), pNearestCity->getY_INLINE());
							}

							if (getDomainType() == DOMAIN_SEA && !plot()->isWater())
							{
								if (!pLoopPlot->isWater() || !pLoopPlot->isAdjacentToArea(area()))
								{
									iValue *= 3;
								}
							}
							else
							{
								if (pLoopPlot->area() != area())
								{
									iValue *= 3;
								}
							}

							if (iValue < iBestValue)
							{
								iBestValue = iValue;
								pBestPlot = pLoopPlot;
							}
						}
					}
				}
			}
		}
	}

	bool bValid = true;
	if (pBestPlot != NULL)
	{
		setXY(pBestPlot->getX_INLINE(), pBestPlot->getY_INLINE());
	}
	else
	{
		kill(false);
		bValid = false;
	}

	return bValid;
}


bool CvUnit::canAutomate(AutomateTypes eAutomate) const
{
	if (eAutomate == NO_AUTOMATE)
	{
		return false;
	}

	if (!isGroupHead())
	{
		return false;
	}

	switch (eAutomate)
	{
	case AUTOMATE_BUILD:
		if ((AI_getUnitAIType() != UNITAI_WORKER) && (AI_getUnitAIType() != UNITAI_WORKER_SEA))
		{
			return false;
		}
		break;

	case AUTOMATE_NETWORK:
		if ((AI_getUnitAIType() != UNITAI_WORKER) || !canBuildRoute())
		{
			return false;
		}
		break;

	case AUTOMATE_CITY:
		if (AI_getUnitAIType() != UNITAI_WORKER)
		{
			return false;
		}
		break;

	case AUTOMATE_EXPLORE:
/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                      06/02/09                                jdog5000      */
/**                                                                                              */
/** Player Interface                                                                             */
/*************************************************************************************************/
		// Enable exploration for air units
		if ((!canFight() && (getDomainType() != DOMAIN_SEA) && (getDomainType() != DOMAIN_AIR)) || (getDomainType() == DOMAIN_IMMOBILE))
		{
			return false;
		}

		if( getDomainType() == DOMAIN_AIR && !canRecon(NULL) )
		{
			return false;
		}
/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                       END                                                  */
/*************************************************************************************************/		

		break;

	case AUTOMATE_RELIGION:
		if (AI_getUnitAIType() != UNITAI_MISSIONARY)
		{
			return false;
		}
		break;

	default:
		FAssert(false);
		break;
	}

	return true;
}


void CvUnit::automate(AutomateTypes eAutomate)
{
	if (!canAutomate(eAutomate))
	{
		return;
	}

	getGroup()->setAutomateType(eAutomate);
}


bool CvUnit::canScrap() const
{
	if (plot()->isFighting())
	{
		return false;
	}

	return true;
}


void CvUnit::scrap()
{
	if (!canScrap())
	{
		return;
	}

	kill(true);
}


bool CvUnit::canGift(bool bTestVisible, bool bTestTransport)
{
	CvPlot* pPlot = plot();
	CvUnit* pTransport = getTransportUnit();

	// Sanguo Mod Hero, start, added by poyuzhe 01.18.09
	if (getHeroType() != NO_HERO)
	{
	    return false;
	}

	if (getLegionHero() != NO_HERO)
	{
	    return false;
	}
    // Sanguo Mod Hero, end

	if (!(pPlot->isOwned()))
	{
		return false;
	}

	if (pPlot->getOwnerINLINE() == getOwnerINLINE())
	{
		return false;
	}

	if (pPlot->isVisibleEnemyUnit(this))
	{
		return false;
	}

	if (pPlot->isVisibleEnemyUnit(pPlot->getOwnerINLINE()))
	{
		return false;
	}

	if (!pPlot->isValidDomainForLocation(*this) && NULL == pTransport)
	{
		return false;
	}

	for (int iCorp = 0; iCorp < GC.getNumCorporationInfos(); ++iCorp)
	{
		if (m_pUnitInfo->getCorporationSpreads(iCorp) > 0)
		{
			return false;
		}
	}

	if (bTestTransport)
	{
		if (pTransport && pTransport->getTeam() != pPlot->getTeam())
		{
			return false;
		}
	}

	if (!bTestVisible)
	{
		if (GET_TEAM(pPlot->getTeam()).isUnitClassMaxedOut(getUnitClassType(), GET_TEAM(pPlot->getTeam()).getUnitClassMaking(getUnitClassType())))
		{
			return false;
		}

		if (GET_PLAYER(pPlot->getOwnerINLINE()).isUnitClassMaxedOut(getUnitClassType(), GET_PLAYER(pPlot->getOwnerINLINE()).getUnitClassMaking(getUnitClassType())))
		{
			return false;
		}

		if (!(GET_PLAYER(pPlot->getOwnerINLINE()).AI_acceptUnit(this)))
		{
			return false;
		}
	}

	return !atWar(pPlot->getTeam(), getTeam());
}


void CvUnit::gift(bool bTestTransport)
{
	CvUnit* pGiftUnit;
	CvWString szBuffer;
	PlayerTypes eOwner;

	if (!canGift(false, bTestTransport))
	{
		return;
	}

	std::vector<CvUnit*> aCargoUnits;
	getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
		aCargoUnits[i]->gift(false);
	}

	FAssertMsg(plot()->getOwnerINLINE() != NO_PLAYER, "plot()->getOwnerINLINE() is not expected to be equal with NO_PLAYER");
	pGiftUnit = GET_PLAYER(plot()->getOwnerINLINE()).initUnit(getUnitType(), getX_INLINE(), getY_INLINE(), AI_getUnitAIType());

	FAssertMsg(pGiftUnit != NULL, "GiftUnit is not assigned a valid value");

	eOwner = getOwnerINLINE();

	pGiftUnit->convert(this);

	GET_PLAYER(pGiftUnit->getOwnerINLINE()).AI_changePeacetimeGrantValue(eOwner, (pGiftUnit->getUnitInfo().getProductionCost() / 5));

	szBuffer = gDLL->getText("TXT_KEY_MISC_GIFTED_UNIT_TO_YOU", GET_PLAYER(eOwner).getNameKey(), pGiftUnit->getNameKey());
	gDLL->getInterfaceIFace()->addMessage(pGiftUnit->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNITGIFTED", MESSAGE_TYPE_INFO, pGiftUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_WHITE"), pGiftUnit->getX_INLINE(), pGiftUnit->getY_INLINE(), true, true);

	// Python Event
	CvEventReporter::getInstance().unitGifted(pGiftUnit, getOwnerINLINE(), plot());
}


bool CvUnit::canLoadUnit(const CvUnit* pUnit, const CvPlot* pPlot) const
{
	FAssert(pUnit != NULL);
	FAssert(pPlot != NULL);

	if (pUnit == this)
	{
		return false;
	}

	if (pUnit->getTeam() != getTeam())
	{
		return false;
	}

/************************************************************************************************/
/* UNOFFICIAL_PATCH                       06/23/10                     Mongoose & jdog5000      */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
	// From Mongoose SDK
	if (isCargo() && getTransportUnit() == pUnit)
	{
		return false;
	}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/

	if (getCargo() > 0)
	{
		return false;
	}

	if (pUnit->isCargo())
	{
		return false;
	}

	if (!(pUnit->cargoSpaceAvailable(getSpecialUnitType(), getDomainType())))
	{
		return false;
	}

	if (!(pUnit->atPlot(pPlot)))
	{
		return false;
	}

	// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
	// commented out 1 line
//	if (!m_pUnitInfo->isHiddenNationality() && pUnit->getUnitInfo().isHiddenNationality())
	if (!isHiddenNationality() && pUnit->isHiddenNationality())
	// Sanguo Mod Hero, end
	{
		return false;
	}

	if (NO_SPECIALUNIT != getSpecialUnitType())
	{
		if (GC.getSpecialUnitInfo(getSpecialUnitType()).isCityLoad())
		{
			if (!pPlot->isCity(true, getTeam()))
			{
				return false;
			}
		}
	}

	return true;
}


void CvUnit::loadUnit(CvUnit* pUnit)
{
	if (!canLoadUnit(pUnit, plot()))
	{
		return;
	}

	setTransportUnit(pUnit);
}

bool CvUnit::shouldLoadOnMove(const CvPlot* pPlot) const
{
	if (isCargo())
	{
		return false;
	}

	switch (getDomainType())
	{
	case DOMAIN_LAND:
/************************************************************************************************/
/* UNOFFICIAL_PATCH                       10/30/09                     Mongoose & jdog5000      */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
/* original bts code
		if (pPlot->isWater())
*/
		// From Mongoose SDK
		if (pPlot->isWater() && !canMoveAllTerrain())
		{
			return true;
		}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/
		break;
	case DOMAIN_AIR:
		if (!pPlot->isFriendlyCity(*this, true))
		{
			return true;
		}

		if (m_pUnitInfo->getAirUnitCap() > 0)
		{
			if (pPlot->airUnitSpaceAvailable(getTeam()) <= 0)
			{
				return true;
			}
		}
		break;
	default:
		break;
	}

	if (m_pUnitInfo->getTerrainImpassable(pPlot->getTerrainType()))
	{
		TechTypes eTech = (TechTypes)m_pUnitInfo->getTerrainPassableTech(pPlot->getTerrainType());
		if (NO_TECH == eTech || !GET_TEAM(getTeam()).isHasTech(eTech))
		{
			return true;
		}
	}

	return false;
}


bool CvUnit::canLoad(const CvPlot* pPlot) const
{
	PROFILE_FUNC();

	FAssert(pPlot != NULL);

	CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();
	while (pUnitNode != NULL)
	{
		CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (canLoadUnit(pLoopUnit, pPlot))
		{
			return true;
		}
	}

	return false;
}


void CvUnit::load()
{
	CLLNode<IDInfo>* pUnitNode;
	CvUnit* pLoopUnit;
	CvPlot* pPlot;
	int iPass;

	if (!canLoad(plot()))
	{
		return;
	}

	pPlot = plot();

	for (iPass = 0; iPass < 2; iPass++)
	{
		pUnitNode = pPlot->headUnitNode();

		while (pUnitNode != NULL)
		{
			pLoopUnit = ::getUnit(pUnitNode->m_data);
			pUnitNode = pPlot->nextUnitNode(pUnitNode);

			if (canLoadUnit(pLoopUnit, pPlot))
			{
				if ((iPass == 0) ? (pLoopUnit->getOwnerINLINE() == getOwnerINLINE()) : (pLoopUnit->getTeam() == getTeam()))
				{
					setTransportUnit(pLoopUnit);
					break;
				}
			}
		}

		if (isCargo())
		{
			break;
		}
	}
}


bool CvUnit::canUnload() const
{
	CvPlot& kPlot = *(plot());

	if (getTransportUnit() == NULL)
	{
		return false;
	}

	if (!kPlot.isValidDomainForLocation(*this))
	{
		return false;
	}

	if (getDomainType() == DOMAIN_AIR)
	{
		if (kPlot.isFriendlyCity(*this, true))
		{
			int iNumAirUnits = kPlot.countNumAirUnits(getTeam());
			CvCity* pCity = kPlot.getPlotCity();
			if (NULL != pCity)
			{
				if (iNumAirUnits >= pCity->getAirUnitCapacity(getTeam()))
				{
					return false;
				}
			}
			else
			{
				if (iNumAirUnits >= GC.getDefineINT("CITY_AIR_UNIT_CAPACITY"))
				{
					return false;
				}
			}
		}
	}

	return true;
}


void CvUnit::unload()
{
	if (!canUnload())
	{
		return;
	}

	setTransportUnit(NULL);
}


bool CvUnit::canUnloadAll() const
{
	if (getCargo() == 0)
	{
		return false;
	}

	return true;
}


void CvUnit::unloadAll()
{
	if (!canUnloadAll())
	{
		return;
	}

	std::vector<CvUnit*> aCargoUnits;
	getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
		CvUnit* pCargo = aCargoUnits[i];
		if (pCargo->canUnload())
		{
			pCargo->setTransportUnit(NULL);
		}
		else
		{
			FAssert(isHuman() || pCargo->getDomainType() == DOMAIN_AIR);
			pCargo->getGroup()->setActivityType(ACTIVITY_AWAKE);
		}
	}
}


bool CvUnit::canHold(const CvPlot* pPlot) const
{
	return true;
}


bool CvUnit::canSleep(const CvPlot* pPlot) const
{
	if (isFortifyable())
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

	return true;
}


bool CvUnit::canFortify(const CvPlot* pPlot) const
{
	if (!isFortifyable())
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

	return true;
}


bool CvUnit::canAirPatrol(const CvPlot* pPlot) const
{
	if (getDomainType() != DOMAIN_AIR)
	{
		return false;
	}

	if (!canAirDefend(pPlot))
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

	return true;
}


bool CvUnit::canSeaPatrol(const CvPlot* pPlot) const
{
	if (!pPlot->isWater())
	{
		return false;
	}

	if (getDomainType() != DOMAIN_SEA)
	{
		return false;
	}

	if (!canFight() || isOnlyDefensive())
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

	return true;
}


void CvUnit::airCircle(bool bStart)
{
	if (!GC.IsGraphicsInitialized())
	{
		return;
	}

	if ((getDomainType() != DOMAIN_AIR) || (maxInterceptionProbability() == 0))
	{
		return;
	}

	//cancel previos missions
	gDLL->getEntityIFace()->RemoveUnitFromBattle( this );

	if (bStart)
	{
		CvAirMissionDefinition kDefinition;
		kDefinition.setPlot(plot());
		kDefinition.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefinition.setUnit(BATTLE_UNIT_DEFENDER, NULL);
		kDefinition.setMissionType(MISSION_AIRPATROL);
		kDefinition.setMissionTime(1.0f); // patrol is indefinite - time is ignored

		gDLL->getEntityIFace()->AddMission( &kDefinition );
	}
}


bool CvUnit::canHeal(const CvPlot* pPlot) const
{
	if (!isHurt())
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

/*************************************************************************************************/
/* UNOFFICIAL_PATCH                       06/30/10                           LunarMongoose       */
/*                                                                                               */
/* Bugfix                                                                                        */
/*************************************************************************************************/
/* original bts code
	if (healRate(pPlot) <= 0)
	{
		return false;
	}
*/
	// Mongoose FeatureDamageFix
	if (healTurns(pPlot) == MAX_INT)
	{
		return false;
	}
/*************************************************************************************************/
/* UNOFFICIAL_PATCH                         END                                                  */
/*************************************************************************************************/

	return true;
}


bool CvUnit::canSentry(const CvPlot* pPlot) const
{
	if (!canDefend(pPlot))
	{
		return false;
	}

	if (isWaiting())
	{
		return false;
	}

	return true;
}


int CvUnit::healRate(const CvPlot* pPlot) const
{
	PROFILE_FUNC();

	CLLNode<IDInfo>* pUnitNode;
	CvCity* pCity;
	CvUnit* pLoopUnit;
	CvPlot* pLoopPlot;
	int iTotalHeal;
	int iHeal;
	int iBestHeal;
	int iI;

	pCity = pPlot->getPlotCity();

	iTotalHeal = 0;

	if (pPlot->isCity(true, getTeam()))
	{
		iTotalHeal += GC.getDefineINT("CITY_HEAL_RATE") + (GET_TEAM(getTeam()).isFriendlyTerritory(pPlot->getTeam()) ? getExtraFriendlyHeal() : getExtraNeutralHeal());
		if (pCity && !pCity->isOccupation())
		{
			iTotalHeal += pCity->getHealRate();
		}
	}
	else
	{
		if (!GET_TEAM(getTeam()).isFriendlyTerritory(pPlot->getTeam()))
		{
			if (isEnemy(pPlot->getTeam(), pPlot))
			{
				iTotalHeal += (GC.getDefineINT("ENEMY_HEAL_RATE") + getExtraEnemyHeal());
			}
			else
			{
				iTotalHeal += (GC.getDefineINT("NEUTRAL_HEAL_RATE") + getExtraNeutralHeal());
			}
		}
		else
		{
			iTotalHeal += (GC.getDefineINT("FRIENDLY_HEAL_RATE") + getExtraFriendlyHeal());
		}
	}

	// XXX optimize this (save it?)
	iBestHeal = 0;

	pUnitNode = pPlot->headUnitNode();

	while (pUnitNode != NULL)
	{
		pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit->getTeam() == getTeam()) // XXX what about alliances?
		{
			iHeal = pLoopUnit->getSameTileHeal();

			if (iHeal > iBestHeal)
			{
				iBestHeal = iHeal;
			}
		}
	}

	for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
	{
		pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

		if (pLoopPlot != NULL)
		{
			if (pLoopPlot->area() == pPlot->area())
			{
				pUnitNode = pLoopPlot->headUnitNode();

				while (pUnitNode != NULL)
				{
					pLoopUnit = ::getUnit(pUnitNode->m_data);
					pUnitNode = pLoopPlot->nextUnitNode(pUnitNode);

					if (pLoopUnit->getTeam() == getTeam()) // XXX what about alliances?
					{
						iHeal = pLoopUnit->getAdjacentTileHeal();

						if (iHeal > iBestHeal)
						{
							iBestHeal = iHeal;
						}
					}
				}
			}
		}
	}

	iTotalHeal += iBestHeal;
	// XXX

	return iTotalHeal;
}


int CvUnit::healTurns(const CvPlot* pPlot) const
{
	int iHeal;
	int iTurns;

	if (!isHurt())
	{
		return 0;
	}

	iHeal = healRate(pPlot);

/*************************************************************************************************/
/* UNOFFICIAL_PATCH                       06/02/10                           LunarMongoose       */
/*                                                                                               */
/* Bugfix                                                                                        */
/*************************************************************************************************/
	// Mongoose FeatureDamageFix
	FeatureTypes eFeature = pPlot->getFeatureType();
	if (eFeature != NO_FEATURE)
	{
		iHeal -= GC.getFeatureInfo(eFeature).getTurnDamage();
	}
/*************************************************************************************************/
/* UNOFFICIAL_PATCH                         END                                                  */
/*************************************************************************************************/

	if (iHeal > 0)
	{
		iTurns = (getDamage() / iHeal);

		if ((getDamage() % iHeal) != 0)
		{
			iTurns++;
		}

		return iTurns;
	}
	else
	{
		return MAX_INT;
	}
}


void CvUnit::doHeal()
{
	changeDamage(-(healRate(plot())));
}


bool CvUnit::canAirlift(const CvPlot* pPlot) const
{
	CvCity* pCity;

	if (getDomainType() != DOMAIN_LAND)
	{
		return false;
	}

	if (hasMoved())
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (pCity->getCurrAirlift() >= pCity->getMaxAirlift())
	{
		return false;
	}

	if (pCity->getTeam() != getTeam())
	{
		return false;
	}

	return true;
}


bool CvUnit::canAirliftAt(const CvPlot* pPlot, int iX, int iY) const
{
	CvPlot* pTargetPlot;
	CvCity* pTargetCity;

	if (!canAirlift(pPlot))
	{
		return false;
	}

	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (!canMoveInto(pTargetPlot))
	{
		return false;
	}

	pTargetCity = pTargetPlot->getPlotCity();

	if (pTargetCity == NULL)
	{
		return false;
	}

	if (pTargetCity->isAirliftTargeted())
	{
		return false;
	}

	if (pTargetCity->getTeam() != getTeam() && !GET_TEAM(pTargetCity->getTeam()).isVassal(getTeam()))
	{
		return false;
	}

	return true;
}


bool CvUnit::airlift(int iX, int iY)
{
	CvCity* pCity;
	CvCity* pTargetCity;
	CvPlot* pTargetPlot;

	if (!canAirliftAt(plot(), iX, iY))
	{
		return false;
	}

	pCity = plot()->getPlotCity();
	FAssert(pCity != NULL);
	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);
	FAssert(pTargetPlot != NULL);
	pTargetCity = pTargetPlot->getPlotCity();
	FAssert(pTargetCity != NULL);
	FAssert(pCity != pTargetCity);

	pCity->changeCurrAirlift(1);
	if (pTargetCity->getMaxAirlift() == 0)
	{
		pTargetCity->setAirliftTargeted(true);
	}

	finishMoves();

	setXY(pTargetPlot->getX_INLINE(), pTargetPlot->getY_INLINE());

	return true;
}


bool CvUnit::isNukeVictim(const CvPlot* pPlot, TeamTypes eTeam) const
{
	CvPlot* pLoopPlot;
	int iDX, iDY;

	if (!(GET_TEAM(eTeam).isAlive()))
	{
		return false;
	}

	if (eTeam == getTeam())
	{
		return false;
	}

	for (iDX = -(nukeRange()); iDX <= nukeRange(); iDX++)
	{
		for (iDY = -(nukeRange()); iDY <= nukeRange(); iDY++)
		{
			pLoopPlot	= plotXY(pPlot->getX_INLINE(), pPlot->getY_INLINE(), iDX, iDY);

			if (pLoopPlot != NULL)
			{
				if (pLoopPlot->getTeam() == eTeam)
				{
					return true;
				}

				if (pLoopPlot->plotCheck(PUF_isCombatTeam, eTeam, getTeam()) != NULL)
				{
					return true;
				}
			}
		}
	}

	return false;
}


bool CvUnit::canNuke(const CvPlot* pPlot) const
{
	if (nukeRange() == -1)
	{
		return false;
	}

	return true;
}


bool CvUnit::canNukeAt(const CvPlot* pPlot, int iX, int iY) const
{
	CvPlot* pTargetPlot;
	int iI;

	if (!canNuke(pPlot))
	{
		return false;
	}

	int iDistance = plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), iX, iY);
	if (iDistance <= nukeRange())
	{
		return false;
	}

	if (airRange() > 0 && iDistance > airRange())
	{
		return false;
	}

	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (isNukeVictim(pTargetPlot, ((TeamTypes)iI)))
		{
			if (!isEnemy((TeamTypes)iI, pPlot))
			{
				return false;
			}
		}
	}

	return true;
}


bool CvUnit::nuke(int iX, int iY)
{
	CvPlot* pPlot;
	CvWString szBuffer;
	bool abTeamsAffected[MAX_TEAMS];
	TeamTypes eBestTeam;
	int iBestInterception;
	int iI, iJ, iK;

	if (!canNukeAt(plot(), iX, iY))
	{
		return false;
	}

	pPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		abTeamsAffected[iI] = isNukeVictim(pPlot, ((TeamTypes)iI));
	}

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (abTeamsAffected[iI])
		{
			if (!isEnemy((TeamTypes)iI))
			{
				GET_TEAM(getTeam()).declareWar(((TeamTypes)iI), false, WARPLAN_LIMITED);
			}
		}
	}

	iBestInterception = 0;
	eBestTeam = NO_TEAM;

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (abTeamsAffected[iI])
		{
			if (GET_TEAM((TeamTypes)iI).getNukeInterception() > iBestInterception)
			{
				iBestInterception = GET_TEAM((TeamTypes)iI).getNukeInterception();
				eBestTeam = ((TeamTypes)iI);
			}
		}
	}

	iBestInterception *= (100 - m_pUnitInfo->getEvasionProbability());
	iBestInterception /= 100;

	setReconPlot(pPlot);

	if (GC.getGameINLINE().getSorenRandNum(100, "Nuke") < iBestInterception)
	{
		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_NUKE_INTERCEPTED", GET_PLAYER(getOwnerINLINE()).getNameKey(), getNameKey(), GET_TEAM(eBestTeam).getName().GetCString());
				gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), (((PlayerTypes)iI) == getOwnerINLINE()), GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_NUKE_INTERCEPTED", MESSAGE_TYPE_MAJOR_EVENT, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
			}
		}

		if (pPlot->isActiveVisible(false))
		{
			// Nuke entity mission
			CvMissionDefinition kDefiniton;
			kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_NUKE).getTime() * gDLL->getSecsPerTurn());
			kDefiniton.setMissionType(MISSION_NUKE);
			kDefiniton.setPlot(pPlot);
			kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
			kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, this);

			// Add the intercepted mission (defender is not NULL)
			gDLL->getEntityIFace()->AddMission(&kDefiniton);
		}

		kill(true);
		return true; // Intercepted!!! (XXX need special event for this...)
	}

	if (pPlot->isActiveVisible(false))
	{
		// Nuke entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_NUKE).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_NUKE);
		kDefiniton.setPlot(pPlot);
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, NULL);

		// Add the non-intercepted mission (defender is NULL)
		gDLL->getEntityIFace()->AddMission(&kDefiniton);
	}

	setMadeAttack(true);
	setAttackPlot(pPlot, false);

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (abTeamsAffected[iI])
		{
			GET_TEAM((TeamTypes)iI).changeWarWeariness(getTeam(), 100 * GC.getDefineINT("WW_HIT_BY_NUKE"));
			GET_TEAM(getTeam()).changeWarWeariness(((TeamTypes)iI), 100 * GC.getDefineINT("WW_ATTACKED_WITH_NUKE"));
			GET_TEAM(getTeam()).AI_changeWarSuccess(((TeamTypes)iI), GC.getDefineINT("WAR_SUCCESS_NUKE"));
		}
	}

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive())
		{
			if (iI != getTeam())
			{
				if (abTeamsAffected[iI])
				{
					for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
					{
						if (GET_PLAYER((PlayerTypes)iJ).isAlive())
						{
							if (GET_PLAYER((PlayerTypes)iJ).getTeam() == ((TeamTypes)iI))
							{
								GET_PLAYER((PlayerTypes)iJ).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_NUKED_US, 1);
							}
						}
					}
				}
				else
				{
					for (iJ = 0; iJ < MAX_TEAMS; iJ++)
					{
						if (GET_TEAM((TeamTypes)iJ).isAlive())
						{
							if (abTeamsAffected[iJ])
							{
								if (GET_TEAM((TeamTypes)iI).isHasMet((TeamTypes)iJ))
								{
									if (GET_TEAM((TeamTypes)iI).AI_getAttitude((TeamTypes)iJ) >= ATTITUDE_CAUTIOUS)
									{
										for (iK = 0; iK < MAX_PLAYERS; iK++)
										{
											if (GET_PLAYER((PlayerTypes)iK).isAlive())
											{
												if (GET_PLAYER((PlayerTypes)iK).getTeam() == ((TeamTypes)iI))
												{
													GET_PLAYER((PlayerTypes)iK).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_NUKED_FRIEND, 1);
												}
											}
										}
										break;
									}
								}
							}
						}
					}
				}
			}
		}
	}

	// XXX some AI should declare war here...

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_NUKE_LAUNCHED", GET_PLAYER(getOwnerINLINE()).getNameKey(), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), (((PlayerTypes)iI) == getOwnerINLINE()), GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_NUKE_EXPLODES", MESSAGE_TYPE_MAJOR_EVENT, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
		}
	}

	if (isSuicide())
	{
		kill(true);
	}

	return true;
}


bool CvUnit::canRecon(const CvPlot* pPlot) const
{
	if (getDomainType() != DOMAIN_AIR)
	{
		return false;
	}

	if (airRange() == 0)
	{
		return false;
	}

	if (m_pUnitInfo->isSuicide())
	{
		return false;
	}

	return true;
}



bool CvUnit::canReconAt(const CvPlot* pPlot, int iX, int iY) const
{
	if (!canRecon(pPlot))
	{
		return false;
	}

	int iDistance = plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), iX, iY);
	if (iDistance > airRange() || 0 == iDistance)
	{
		return false;
	}

	return true;
}


bool CvUnit::recon(int iX, int iY)
{
	CvPlot* pPlot;

	if (!canReconAt(plot(), iX, iY))
	{
		return false;
	}

	pPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	setReconPlot(pPlot);

	finishMoves();

	if (pPlot->isActiveVisible(false))
	{
		CvAirMissionDefinition kAirMission;
		kAirMission.setMissionType(MISSION_RECON);
		kAirMission.setUnit(BATTLE_UNIT_ATTACKER, this);
		kAirMission.setUnit(BATTLE_UNIT_DEFENDER, NULL);
		kAirMission.setDamage(BATTLE_UNIT_DEFENDER, 0);
		kAirMission.setDamage(BATTLE_UNIT_ATTACKER, 0);
		kAirMission.setPlot(pPlot);
		kAirMission.setMissionTime(GC.getMissionInfo((MissionTypes)MISSION_RECON).getTime() * gDLL->getSecsPerTurn());
		gDLL->getEntityIFace()->AddMission(&kAirMission);
	}

	return true;
}


bool CvUnit::canParadrop(const CvPlot* pPlot) const
{
	if (getDropRange() <= 0)
	{
		return false;
	}

	if (hasMoved())
	{
		return false;
	}

	if (!pPlot->isFriendlyCity(*this, true))
	{
		return false;
	}

	return true;
}



bool CvUnit::canParadropAt(const CvPlot* pPlot, int iX, int iY) const
{
	if (!canParadrop(pPlot))
	{
		return false;
	}

	CvPlot* pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);
	if (NULL == pTargetPlot || pTargetPlot == pPlot)
	{
		return false;
	}

	if (!pTargetPlot->isVisible(getTeam(), false))
	{
		return false;
	}

	if (!canMoveInto(pTargetPlot, false, false, true))
	{
		return false;
	}

	if (plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), iX, iY) > getDropRange())
	{
		return false;
	}

	if (!canCoexistWithEnemyUnit(NO_TEAM))
	{
		if (pTargetPlot->isEnemyCity(*this))
		{
			return false;
		}

		if (pTargetPlot->isVisibleEnemyUnit(this))
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::paradrop(int iX, int iY)
{
	if (!canParadropAt(plot(), iX, iY))
	{
		return false;
	}

	CvPlot* pPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	changeMoves(GC.getMOVE_DENOMINATOR() / 2);
	setMadeAttack(true);

	setXY(pPlot->getX_INLINE(), pPlot->getY_INLINE());

	//check if intercepted
	if(interceptTest(pPlot))
	{
		return true;
	}

	//play paradrop animation by itself
	if (pPlot->isActiveVisible(false))
	{
		CvAirMissionDefinition kAirMission;
		kAirMission.setMissionType(MISSION_PARADROP);
		kAirMission.setUnit(BATTLE_UNIT_ATTACKER, this);
		kAirMission.setUnit(BATTLE_UNIT_DEFENDER, NULL);
		kAirMission.setDamage(BATTLE_UNIT_DEFENDER, 0);
		kAirMission.setDamage(BATTLE_UNIT_ATTACKER, 0);
		kAirMission.setPlot(pPlot);
		kAirMission.setMissionTime(GC.getMissionInfo((MissionTypes)MISSION_PARADROP).getTime() * gDLL->getSecsPerTurn());
		gDLL->getEntityIFace()->AddMission(&kAirMission);
	}

	return true;
}


bool CvUnit::canAirBomb(const CvPlot* pPlot) const
{
	if (getDomainType() != DOMAIN_AIR)
	{
		return false;
	}

	if (airBombBaseRate() == 0)
	{
		return false;
	}

	if (isMadeAttack())
	{
		return false;
	}

	return true;
}


bool CvUnit::canAirBombAt(const CvPlot* pPlot, int iX, int iY) const
{
	CvCity* pCity;
	CvPlot* pTargetPlot;

	if (!canAirBomb(pPlot))
	{
		return false;
	}

	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), pTargetPlot->getX_INLINE(), pTargetPlot->getY_INLINE()) > airRange())
	{
		return false;
	}

	if (pTargetPlot->isOwned())
	{
		if (!potentialWarAction(pTargetPlot))
		{
			return false;
		}
	}

	pCity = pTargetPlot->getPlotCity();

	if (pCity != NULL)
	{
		if (!(pCity->isBombardable(this)))
		{
			return false;
		}
	}
	else
	{
		if (pTargetPlot->getImprovementType() == NO_IMPROVEMENT)
		{
			return false;
		}

		if (GC.getImprovementInfo(pTargetPlot->getImprovementType()).isPermanent())
		{
			return false;
		}

		if (GC.getImprovementInfo(pTargetPlot->getImprovementType()).getAirBombDefense() == -1)
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::airBomb(int iX, int iY)
{
	CvCity* pCity;
	CvPlot* pPlot;
	CvWString szBuffer;

	if (!canAirBombAt(plot(), iX, iY))
	{
		return false;
	}

	pPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (!isEnemy(pPlot->getTeam()))
	{
		getGroup()->groupDeclareWar(pPlot, true);
	}

	if (!isEnemy(pPlot->getTeam()))
	{
		return false;
	}

	if (interceptTest(pPlot))
	{
		return true;
	}

	pCity = pPlot->getPlotCity();

	if (pCity != NULL)
	{
		pCity->changeDefenseModifier(-airBombCurrRate());

		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_DEFENSES_REDUCED_TO", pCity->getNameKey(), pCity->getDefenseModifier(false), getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE(), true, true);

		szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_DEFENSES_REDUCED_TO", getNameKey(), pCity->getNameKey(), pCity->getDefenseModifier(false));
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pCity->getX_INLINE(), pCity->getY_INLINE());
	}
	else
	{
		if (pPlot->getImprovementType() != NO_IMPROVEMENT)
		{
			if (GC.getGameINLINE().getSorenRandNum(airBombCurrRate(), "Air Bomb - Offense") >=
					GC.getGameINLINE().getSorenRandNum(GC.getImprovementInfo(pPlot->getImprovementType()).getAirBombDefense(), "Air Bomb - Defense"))
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DESTROYED_IMP", getNameKey(), GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

				if (pPlot->isOwned())
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_IMP_WAS_DESTROYED", GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide(), getNameKey(), getVisualCivAdjective(pPlot->getTeam()));
					gDLL->getInterfaceIFace()->addMessage(pPlot->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
				}

				pPlot->setImprovementType((ImprovementTypes)(GC.getImprovementInfo(pPlot->getImprovementType()).getImprovementPillage()));
			}
			else
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_FAIL_DESTROY_IMP", getNameKey(), GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMB_FAILS", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			}
		}
	}

	setReconPlot(pPlot);

	setMadeAttack(true);
	changeMoves(GC.getMOVE_DENOMINATOR());

	if (pPlot->isActiveVisible(false))
	{
		CvAirMissionDefinition kAirMission;
		kAirMission.setMissionType(MISSION_AIRBOMB);
		kAirMission.setUnit(BATTLE_UNIT_ATTACKER, this);
		kAirMission.setUnit(BATTLE_UNIT_DEFENDER, NULL);
		kAirMission.setDamage(BATTLE_UNIT_DEFENDER, 0);
		kAirMission.setDamage(BATTLE_UNIT_ATTACKER, 0);
		kAirMission.setPlot(pPlot);
		kAirMission.setMissionTime(GC.getMissionInfo((MissionTypes)MISSION_AIRBOMB).getTime() * gDLL->getSecsPerTurn());

		gDLL->getEntityIFace()->AddMission(&kAirMission);
	}

	if (isSuicide())
	{
		kill(true);
	}

	return true;
}


CvCity* CvUnit::bombardTarget(const CvPlot* pPlot) const
{
	int iBestValue = MAX_INT;
	CvCity* pBestCity = NULL;

	for (int iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
	{
		CvPlot* pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

		if (pLoopPlot != NULL)
		{
			CvCity* pLoopCity = pLoopPlot->getPlotCity();

			if (pLoopCity != NULL)
			{
				if (pLoopCity->isBombardable(this))
				{
					int iValue = pLoopCity->getDefenseDamage();

					// always prefer cities we are at war with
					if (isEnemy(pLoopCity->getTeam(), pPlot))
					{
						iValue *= 128;
					}

					if (iValue < iBestValue)
					{
						iBestValue = iValue;
						pBestCity = pLoopCity;
					}
				}
			}
		}
	}

	return pBestCity;
}


bool CvUnit::canBombard(const CvPlot* pPlot) const
{
// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
	// Dale - RB: Field Bombard START
	if(getDCMBombRange() > 0)
	{
        return false;
	}
	else if (bombardTarget(pPlot) == NULL)
	{
		return false;
	}
	// Dale - RB: Field Bombard END
// Sanguo Mod Hero, end
	if (bombardRate() <= 0)
	{
		return false;
	}

	if (isMadeAttack())
	{
		return false;
	}

	if (isCargo())
	{
		return false;
	}

	if (bombardTarget(pPlot) == NULL)
	{
		return false;
	}

	return true;
}


bool CvUnit::bombard()
{
	CvPlot* pPlot = plot();
	if (!canBombard(pPlot))
	{
		return false;
	}

	CvCity* pBombardCity = bombardTarget(pPlot);
	FAssertMsg(pBombardCity != NULL, "BombardCity is not assigned a valid value");

	CvPlot* pTargetPlot = pBombardCity->plot();
	if (!isEnemy(pTargetPlot->getTeam()))
	{
		getGroup()->groupDeclareWar(pTargetPlot, true);
	}

	if (!isEnemy(pTargetPlot->getTeam()))
	{
		return false;
	}

	int iBombardModifier = 0;
	if (!ignoreBuildingDefense())
	{
		iBombardModifier -= pBombardCity->getBuildingBombardDefense();
	}

	pBombardCity->changeDefenseModifier(-(bombardRate() * std::max(0, 100 + iBombardModifier)) / 100);

	setMadeAttack(true);
	changeMoves(GC.getMOVE_DENOMINATOR());

	CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_DEFENSES_IN_CITY_REDUCED_TO", pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false), GET_PLAYER(getOwnerINLINE()).getNameKey());
	gDLL->getInterfaceIFace()->addMessage(pBombardCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE(), true, true);

	szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_REDUCE_CITY_DEFENSES", getNameKey(), pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false));
	gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE());

	if (pPlot->isActiveVisible(false))
	{
		CvUnit *pDefender = pBombardCity->plot()->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);

		// Bombard entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_BOMBARD).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_BOMBARD);
		kDefiniton.setPlot(pBombardCity->plot());
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, pDefender);
		gDLL->getEntityIFace()->AddMission(&kDefiniton);
	}

	return true;
}


bool CvUnit::canPillage(const CvPlot* pPlot) const
{
	if (!(m_pUnitInfo->isPillage()))
	{
		return false;
	}

	if (pPlot->isCity())
	{
		return false;
	}

/************************************************************************************************/
/* UNOFFICIAL_PATCH                       06/23/10                     Mongoose & jdog5000      */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
	// From Mongoose SDK
	if (isCargo())
	{
		return false;
	}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/

	if (pPlot->getImprovementType() == NO_IMPROVEMENT)
	{
		if (!(pPlot->isRoute()))
		{
			return false;
		}
	}
	else
	{
		if (GC.getImprovementInfo(pPlot->getImprovementType()).isPermanent())
		{
			return false;
		}
	}

	if (pPlot->isOwned())
	{
		if (!potentialWarAction(pPlot))
		{
			if ((pPlot->getImprovementType() == NO_IMPROVEMENT) || (pPlot->getOwnerINLINE() != getOwnerINLINE()))
			{
				return false;
			}
		}
	}

	if (!(pPlot->isValidDomainForAction(*this)))
	{
		return false;
	}

	return true;
}


bool CvUnit::pillage()
{
	CvWString szBuffer;
	int iPillageGold;
	long lPillageGold;
	ImprovementTypes eTempImprovement = NO_IMPROVEMENT;
	RouteTypes eTempRoute = NO_ROUTE;

	CvPlot* pPlot = plot();

	if (!canPillage(pPlot))
	{
		return false;
	}

	if (pPlot->isOwned())
	{
		// we should not be calling this without declaring war first, so do not declare war here
		if (!isEnemy(pPlot->getTeam(), pPlot))
		{
			if ((pPlot->getImprovementType() == NO_IMPROVEMENT) || (pPlot->getOwnerINLINE() != getOwnerINLINE()))
			{
				return false;
			}
		}
	}

	if (pPlot->isWater())
	{
		CvUnit* pInterceptor = bestSeaPillageInterceptor(this, GC.getDefineINT("COMBAT_DIE_SIDES") / 2);
		if (NULL != pInterceptor)
		{
			setMadeAttack(false);

			int iWithdrawal = withdrawalProbability();
			changeExtraWithdrawal(-iWithdrawal); // no withdrawal since we are really the defender
			attack(pInterceptor->plot(), false);
			changeExtraWithdrawal(iWithdrawal);

			return false;
		}
	}

	if (pPlot->getImprovementType() != NO_IMPROVEMENT)
	{
		eTempImprovement = pPlot->getImprovementType();

		if (pPlot->getTeam() != getTeam())
		{
			// Use python to determine pillage amounts...
			lPillageGold = 0;

			CyPlot* pyPlot = new CyPlot(pPlot);
			CyUnit* pyUnit = new CyUnit(this);

			CyArgsList argsList;
			argsList.add(gDLL->getPythonIFace()->makePythonObject(pyPlot));	// pass in plot class
			argsList.add(gDLL->getPythonIFace()->makePythonObject(pyUnit));	// pass in unit class

			gDLL->getPythonIFace()->callFunction(PYGameModule, "doPillageGold", argsList.makeFunctionArgs(),&lPillageGold);

			delete pyPlot;	// python fxn must not hold on to this pointer
			delete pyUnit;	// python fxn must not hold on to this pointer

			iPillageGold = (int)lPillageGold;

			if (iPillageGold > 0)
			{
				GET_PLAYER(getOwnerINLINE()).changeGold(iPillageGold);

				szBuffer = gDLL->getText("TXT_KEY_MISC_PLUNDERED_GOLD_FROM_IMP", iPillageGold, GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

				if (pPlot->isOwned())
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_IMP_DESTROYED", GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide(), getNameKey(), getVisualCivAdjective(pPlot->getTeam()));
					gDLL->getInterfaceIFace()->addMessage(pPlot->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
				}
			}
		}
		// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
		// Processing Destroy Own Town
		else
		{
		    bool bDestroyOwnTown = false;
		    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		    {
		        if (isHasPromotion((PromotionTypes)iI))
		        {
		            if (GC.getPromotionInfo((PromotionTypes)iI).canDestroyOwnTown())
		            {
		                bDestroyOwnTown = true;
		                break;
		            }
		        }
		    }

		    if (bDestroyOwnTown)
		    {
		        // Use python to determine the resulting UnitClassType of pillaging own improvement...
                long lUnitClass = -1;
                CyPlot* pyPlot = new CyPlot(pPlot);

                CyArgsList argsList;
                argsList.add(getOwnerINLINE());  // pass in Player ID
                argsList.add(gDLL->getPythonIFace()->makePythonObject(pyPlot));	// pass in plot class

                gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doDestroyOwnTown", argsList.makeFunctionArgs(), &lUnitClass);

                delete pyPlot;	// python fxn must not hold on to this pointer

                UnitClassTypes eUnitClass = (UnitClassTypes)lUnitClass;

                if (eUnitClass != NO_UNITCLASS)
                {
                    UnitTypes eUnit = (UnitTypes)GC.getCivilizationInfo(GET_PLAYER(getOwnerINLINE()).getCivilizationType()).getCivilizationUnits(eUnitClass);
                    GET_PLAYER(getOwnerINLINE()).initUnit(eUnit, pPlot->getX_INLINE(), pPlot->getY_INLINE());
                    szBuffer = gDLL->getText("TXT_KEY_HERO_DESTROY_OWN_TOWN_CONSCRIPT", GC.getUnitInfo(eUnit).getDescription(), GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGE", MESSAGE_TYPE_INFO, GC.getUnitInfo(eUnit).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
                }
		    }
		}
        // Sanguo Mod Hero, end

		pPlot->setImprovementType((ImprovementTypes)(GC.getImprovementInfo(pPlot->getImprovementType()).getImprovementPillage()));
	}
	else if (pPlot->isRoute())
	{
		eTempRoute = pPlot->getRouteType();
		pPlot->setRouteType(NO_ROUTE, true); // XXX downgrade rail???
	}

	changeMoves(GC.getMOVE_DENOMINATOR());

	if (pPlot->isActiveVisible(false))
	{
		// Pillage entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_PILLAGE).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_PILLAGE);
		kDefiniton.setPlot(pPlot);
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, NULL);
		gDLL->getEntityIFace()->AddMission(&kDefiniton);
	}

	if (eTempImprovement != NO_IMPROVEMENT || eTempRoute != NO_ROUTE)
	{
		CvEventReporter::getInstance().unitPillage(this, eTempImprovement, eTempRoute, getOwnerINLINE());
	}

	// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
	// Process to destroy surrounding improvement
	if (pPlot->getTeam() != getTeam())
	{
	    int iDestroySurroundingImprovementChance = 0;
	    int iI;
	    for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	    {
	        if (isHasPromotion((PromotionTypes)iI))
	        {
	            if (GC.getPromotionInfo((PromotionTypes)iI).getDestroySurroundingImprovementChance() > 0)
	            {
	                iDestroySurroundingImprovementChance += GC.getPromotionInfo((PromotionTypes)iI).getDestroySurroundingImprovementChance();
	            }
	        }
	    }

	    if (iDestroySurroundingImprovementChance > 0)
	    {
	        CvPlot* pLoopPlot;
	        for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
	        {
                pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

                if (pLoopPlot != NULL)
                {
                    if (pLoopPlot->isOwned())
                    {
                        if (GET_PLAYER(pLoopPlot->getOwnerINLINE()).getTeam() == GET_PLAYER(getOwnerINLINE()).getTeam())
                        {
                            continue;
                        }
                    }

                    if (!canPillage(pLoopPlot))
                    {
                        continue;
                    }

                    if (pLoopPlot->isOwned())
                    {
                        if (!isEnemy(pLoopPlot->getTeam(), pLoopPlot))
                        {
                            if ((pLoopPlot->getImprovementType() == NO_IMPROVEMENT) || (pLoopPlot->getOwnerINLINE() != getOwnerINLINE()))
                            {
                                continue;
                            }
                        }
                    }

                    if (GC.getGameINLINE().getSorenRandNum(100, "Destroy Surrounding Improvement") >= iDestroySurroundingImprovementChance)
                    {
                        continue;
                    }

                    if (pLoopPlot->getImprovementType() != NO_IMPROVEMENT)
                    {
                        eTempImprovement = pLoopPlot->getImprovementType();

                        if (pLoopPlot->getTeam() != getTeam())
                        {
                            // Use python to determine pillage amounts...
                            lPillageGold = 0;

                            CyPlot* pyPlot = new CyPlot(pLoopPlot);
                            CyUnit* pyUnit = new CyUnit(this);

                            CyArgsList argsList;
                            argsList.add(gDLL->getPythonIFace()->makePythonObject(pyPlot));	// pass in plot class
                            argsList.add(gDLL->getPythonIFace()->makePythonObject(pyUnit));	// pass in unit class

                            gDLL->getPythonIFace()->callFunction(PYGameModule, "doPillageGold", argsList.makeFunctionArgs(),&lPillageGold);

                            delete pyPlot;	// python fxn must not hold on to this pointer
                            delete pyUnit;	// python fxn must not hold on to this pointer

                            iPillageGold = (int)lPillageGold;

                            if (iPillageGold > 0)
                            {
                                GET_PLAYER(getOwnerINLINE()).changeGold(iPillageGold);

                                szBuffer = gDLL->getText("TXT_KEY_MISC_PLUNDERED_GOLD_FROM_IMP", iPillageGold, GC.getImprovementInfo(pLoopPlot->getImprovementType()).getTextKeyWide());
                                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pLoopPlot->getX_INLINE(), pLoopPlot->getY_INLINE());

                                if (pLoopPlot->isOwned())
                                {
                                    szBuffer = gDLL->getText("TXT_KEY_MISC_IMP_DESTROYED", GC.getImprovementInfo(pLoopPlot->getImprovementType()).getTextKeyWide(), getNameKey(), getVisualCivAdjective(pLoopPlot->getTeam()));
                                    gDLL->getInterfaceIFace()->addMessage(pLoopPlot->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_PILLAGED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoopPlot->getX_INLINE(), pLoopPlot->getY_INLINE(), true, true);
                                }
                            }
                        }

                        pLoopPlot->setImprovementType((ImprovementTypes)(GC.getImprovementInfo(pLoopPlot->getImprovementType()).getImprovementPillage()));
                    }
                    else if (pLoopPlot->isRoute())
                    {
                        eTempRoute = pLoopPlot->getRouteType();
                        pLoopPlot->setRouteType(NO_ROUTE, true);
                    }

                    if (eTempImprovement != NO_IMPROVEMENT || eTempRoute != NO_ROUTE)
                    {
                        CvEventReporter::getInstance().unitPillage(this, eTempImprovement, eTempRoute, getOwnerINLINE());
                    }
                }
            }
        }
    }
    // Sanguo Mod Hero, end

	return true;
}


bool CvUnit::canPlunder(const CvPlot* pPlot, bool bTestVisible) const
{
	if (getDomainType() != DOMAIN_SEA)
	{
		return false;
	}

	if (!(m_pUnitInfo->isPillage()))
	{
		return false;
	}

	if (!pPlot->isWater())
	{
		return false;
	}

	if (pPlot->isFreshWater())
	{
		return false;
	}

	if (!pPlot->isValidDomainForAction(*this))
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (pPlot->getTeam() == getTeam())
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::plunder()
{
	CvPlot* pPlot = plot();

	if (!canPlunder(pPlot))
	{
		return false;
	}

	setBlockading(true);

	finishMoves();

	return true;
}


void CvUnit::updatePlunder(int iChange, bool bUpdatePlotGroups)
{
	int iBlockadeRange = GC.getDefineINT("SHIP_BLOCKADE_RANGE");

	bool bOldTradeNet;
	bool bChanged = false;

/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                      06/01/09                                jdog5000      */
/**                                                                                              */
/** Bugfix                                                                                       */
/*************************************************************************************************/
	//gDLL->getFAStarIFace()->ForceReset(&GC.getStepFinder());
	bool bValid = false;
	for (int i = -iBlockadeRange; i <= iBlockadeRange; ++i)
	{
		for (int j = -iBlockadeRange; j <= iBlockadeRange; ++j)
		{
			CvPlot* pLoopPlot = ::plotXY(getX_INLINE(), getY_INLINE(), i, j);

			if (NULL != pLoopPlot && pLoopPlot->isWater() && pLoopPlot->area() == area())
			{

				int iPathDist = GC.getMapINLINE().calculatePathDistance(plot(),pLoopPlot);
				
				// BBAI NOTES:  There are rare issues where the path finder will return incorrect results
				// for unknown reasons.  Seems to find a suboptimal path sometimes in partially repeatable 
				// circumstances.  The fix below is a hack to address the permanent one or two tile blockades which 
				// would appear randomly, it should cause extra blockade clearing only very rarely.
				/*
				if( iPathDist > iBlockadeRange )
				{
					// No blockading on other side of an isthmus
					continue;
				}
				*/
				
				if( (iPathDist >= 0) && (iPathDist <= iBlockadeRange + 2) )
				{
					for (int iTeam = 0; iTeam < MAX_TEAMS; ++iTeam)
					{
						// Sanguo Mod sea lord, start, by poyuzhe 10.15.09
						// if (isEnemy((TeamTypes)iTeam))
						if (isEnemy((TeamTypes)iTeam) || isTempCheck())
						// Sanguo Mod sea lord, end
						{
							bValid = (iPathDist <= iBlockadeRange);
							if( !bValid && (iChange == -1 && pLoopPlot->getBlockadedCount((TeamTypes)iTeam) > 0) )
							{
								bValid = true;
							}

							if( bValid )
							{
								if (!bChanged)
								{
									bOldTradeNet = pLoopPlot->isTradeNetwork((TeamTypes)iTeam);
								}

								pLoopPlot->changeBlockadedCount((TeamTypes)iTeam, iChange);

								if (!bChanged)
								{
									bChanged = (bOldTradeNet != pLoopPlot->isTradeNetwork((TeamTypes)iTeam));
								}
							}
						}
					}
				}
			}
		}
	}

	if (bChanged)
	{
		gDLL->getInterfaceIFace()->setDirty(BlockadedPlots_DIRTY_BIT, true);

		if (bUpdatePlotGroups)
		{
			GC.getGameINLINE().updatePlotGroups();
		}
	}
/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                       END                                                  */
/*************************************************************************************************/

	// Sanguo Mod sea lord, start, by poyuzhe 10.15.09
	setTempCheck(false);
	// Sanguo Mod sea lord, end
}


int CvUnit::sabotageCost(const CvPlot* pPlot) const
{
	return GC.getDefineINT("BASE_SPY_SABOTAGE_COST");
}


// XXX compare with destroy prob...
int CvUnit::sabotageProb(const CvPlot* pPlot, ProbabilityTypes eProbStyle) const
{
	CvPlot* pLoopPlot;
	int iDefenseCount;
	int iCounterSpyCount;
	int iProb;
	int iI;

	iProb = 0; // XXX

	if (pPlot->isOwned())
	{
		iDefenseCount = pPlot->plotCount(PUF_canDefend, -1, -1, NO_PLAYER, pPlot->getTeam());
		iCounterSpyCount = pPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());

		for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
		{
			pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

			if (pLoopPlot != NULL)
			{
				iCounterSpyCount += pLoopPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());
			}
		}
	}
	else
	{
		iDefenseCount = 0;
		iCounterSpyCount = 0;
	}

	if (eProbStyle == PROBABILITY_HIGH)
	{
		iCounterSpyCount = 0;
	}

	iProb += (40 / (iDefenseCount + 1)); // XXX

	if (eProbStyle != PROBABILITY_LOW)
	{
		iProb += (50 / (iCounterSpyCount + 1)); // XXX
	}

	return iProb;
}


bool CvUnit::canSabotage(const CvPlot* pPlot, bool bTestVisible) const
{
	if (!(m_pUnitInfo->isSabotage()))
	{
		return false;
	}

	if (pPlot->getTeam() == getTeam())
	{
		return false;
	}

	if (pPlot->isCity())
	{
		return false;
	}

	if (pPlot->getImprovementType() == NO_IMPROVEMENT)
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (GET_PLAYER(getOwnerINLINE()).getGold() < sabotageCost(pPlot))
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::sabotage()
{
	CvCity* pNearestCity;
	CvPlot* pPlot;
	CvWString szBuffer;
	bool bCaught;

	if (!canSabotage(plot()))
	{
		return false;
	}

	pPlot = plot();

	bCaught = (GC.getGameINLINE().getSorenRandNum(100, "Spy: Sabotage") > sabotageProb(pPlot));

	GET_PLAYER(getOwnerINLINE()).changeGold(-(sabotageCost(pPlot)));

	if (!bCaught)
	{
		pPlot->setImprovementType((ImprovementTypes)(GC.getImprovementInfo(pPlot->getImprovementType()).getImprovementPillage()));

		finishMoves();

		pNearestCity = GC.getMapINLINE().findCity(pPlot->getX_INLINE(), pPlot->getY_INLINE(), pPlot->getOwnerINLINE(), NO_TEAM, false);

		if (pNearestCity != NULL)
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_SABOTAGED", getNameKey(), pNearestCity->getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_SABOTAGE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

			if (pPlot->isOwned())
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_SABOTAGE_NEAR", pNearestCity->getNameKey());
				gDLL->getInterfaceIFace()->addMessage(pPlot->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_SABOTAGE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
			}
		}

		if (pPlot->isActiveVisible(false))
		{
			NotifyEntity(MISSION_SABOTAGE);
		}
	}
	else
	{
		if (pPlot->isOwned())
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_CAUGHT_AND_KILLED", GET_PLAYER(getOwnerINLINE()).getCivilizationAdjective(), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(pPlot->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSE", MESSAGE_TYPE_INFO);
		}

		szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_SPY_CAUGHT", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSED", MESSAGE_TYPE_INFO);

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_SURRENDER);
		}

		if (pPlot->isOwned())
		{
			if (!isEnemy(pPlot->getTeam(), pPlot))
			{
				GET_PLAYER(pPlot->getOwnerINLINE()).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_SPY_CAUGHT, 1);
			}
		}

		kill(true, pPlot->getOwnerINLINE());
	}

	return true;
}


int CvUnit::destroyCost(const CvPlot* pPlot) const
{
	CvCity* pCity;
	bool bLimited;

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return 0;
	}

	bLimited = false;

	if (pCity->isProductionUnit())
	{
		bLimited = isLimitedUnitClass((UnitClassTypes)(GC.getUnitInfo(pCity->getProductionUnit()).getUnitClassType()));
	}
	else if (pCity->isProductionBuilding())
	{
		bLimited = isLimitedWonderClass((BuildingClassTypes)(GC.getBuildingInfo(pCity->getProductionBuilding()).getBuildingClassType()));
	}
	else if (pCity->isProductionProject())
	{
		bLimited = isLimitedProject(pCity->getProductionProject());
	}

	return (GC.getDefineINT("BASE_SPY_DESTROY_COST") + (pCity->getProduction() * ((bLimited) ? GC.getDefineINT("SPY_DESTROY_COST_MULTIPLIER_LIMITED") : GC.getDefineINT("SPY_DESTROY_COST_MULTIPLIER"))));
}


int CvUnit::destroyProb(const CvPlot* pPlot, ProbabilityTypes eProbStyle) const
{
	CvCity* pCity;
	CvPlot* pLoopPlot;
	int iDefenseCount;
	int iCounterSpyCount;
	int iProb;
	int iI;

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return 0;
	}

	iProb = 0; // XXX

	iDefenseCount = pPlot->plotCount(PUF_canDefend, -1, -1, NO_PLAYER, pPlot->getTeam());

	iCounterSpyCount = pPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());

	for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
	{
		pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

		if (pLoopPlot != NULL)
		{
			iCounterSpyCount += pLoopPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());
		}
	}

	if (eProbStyle == PROBABILITY_HIGH)
	{
		iCounterSpyCount = 0;
	}

	iProb += (25 / (iDefenseCount + 1)); // XXX

	if (eProbStyle != PROBABILITY_LOW)
	{
		iProb += (50 / (iCounterSpyCount + 1)); // XXX
	}

	iProb += std::min(25, pCity->getProductionTurnsLeft()); // XXX

	return iProb;
}


bool CvUnit::canDestroy(const CvPlot* pPlot, bool bTestVisible) const
{
	CvCity* pCity;

	if (!(m_pUnitInfo->isDestroy()))
	{
		return false;
	}

	if (pPlot->getTeam() == getTeam())
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (pCity->getProduction() == 0)
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (GET_PLAYER(getOwnerINLINE()).getGold() < destroyCost(pPlot))
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::destroy()
{
	CvCity* pCity;
	CvWString szBuffer;
	bool bCaught;

	if (!canDestroy(plot()))
	{
		return false;
	}

	bCaught = (GC.getGameINLINE().getSorenRandNum(100, "Spy: Destroy") > destroyProb(plot()));

	pCity = plot()->getPlotCity();
	FAssertMsg(pCity != NULL, "City is not assigned a valid value");

	GET_PLAYER(getOwnerINLINE()).changeGold(-(destroyCost(plot())));

	if (!bCaught)
	{
		pCity->setProduction(pCity->getProduction() / 2);

		finishMoves();

		szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_DESTROYED_PRODUCTION", getNameKey(), pCity->getProductionNameKey(), pCity->getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_DESTROY", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pCity->getX_INLINE(), pCity->getY_INLINE());

		szBuffer = gDLL->getText("TXT_KEY_MISC_CITY_PRODUCTION_DESTROYED", pCity->getProductionNameKey(), pCity->getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_DESTROY", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE(), true, true);

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_DESTROY);
		}
	}
	else
	{
		szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_CAUGHT_AND_KILLED", GET_PLAYER(getOwnerINLINE()).getCivilizationAdjective(), getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSE", MESSAGE_TYPE_INFO);

		szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_SPY_CAUGHT", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSED", MESSAGE_TYPE_INFO);

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_SURRENDER);
		}

		if (!isEnemy(pCity->getTeam()))
		{
			GET_PLAYER(pCity->getOwnerINLINE()).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_SPY_CAUGHT, 1);
		}

		kill(true, pCity->getOwnerINLINE());
	}

	return true;
}


int CvUnit::stealPlansCost(const CvPlot* pPlot) const
{
	CvCity* pCity;

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return 0;
	}

	return (GC.getDefineINT("BASE_SPY_STEAL_PLANS_COST") + ((GET_TEAM(pCity->getTeam()).getTotalLand() + GET_TEAM(pCity->getTeam()).getTotalPopulation()) * GC.getDefineINT("SPY_STEAL_PLANS_COST_MULTIPLIER")));
}


// XXX compare with destroy prob...
int CvUnit::stealPlansProb(const CvPlot* pPlot, ProbabilityTypes eProbStyle) const
{
	CvCity* pCity;
	CvPlot* pLoopPlot;
	int iDefenseCount;
	int iCounterSpyCount;
	int iProb;
	int iI;

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return 0;
	}

	iProb = ((pCity->isGovernmentCenter()) ? 20 : 0); // XXX

	iDefenseCount = pPlot->plotCount(PUF_canDefend, -1, -1, NO_PLAYER, pPlot->getTeam());

	iCounterSpyCount = pPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());

	for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
	{
		pLoopPlot = plotDirection(pPlot->getX_INLINE(), pPlot->getY_INLINE(), ((DirectionTypes)iI));

		if (pLoopPlot != NULL)
		{
			iCounterSpyCount += pLoopPlot->plotCount(PUF_isCounterSpy, -1, -1, NO_PLAYER, pPlot->getTeam());
		}
	}

	if (eProbStyle == PROBABILITY_HIGH)
	{
		iCounterSpyCount = 0;
	}

	iProb += (20 / (iDefenseCount + 1)); // XXX

	if (eProbStyle != PROBABILITY_LOW)
	{
		iProb += (50 / (iCounterSpyCount + 1)); // XXX
	}

	return iProb;
}


bool CvUnit::canStealPlans(const CvPlot* pPlot, bool bTestVisible) const
{
	CvCity* pCity;

	if (!(m_pUnitInfo->isStealPlans()))
	{
		return false;
	}

	if (pPlot->getTeam() == getTeam())
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (GET_PLAYER(getOwnerINLINE()).getGold() < stealPlansCost(pPlot))
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::stealPlans()
{
	CvCity* pCity;
	CvWString szBuffer;
	bool bCaught;

	if (!canStealPlans(plot()))
	{
		return false;
	}

	bCaught = (GC.getGameINLINE().getSorenRandNum(100, "Spy: Steal Plans") > stealPlansProb(plot()));

	pCity = plot()->getPlotCity();
	FAssertMsg(pCity != NULL, "City is not assigned a valid value");

	GET_PLAYER(getOwnerINLINE()).changeGold(-(stealPlansCost(plot())));

	if (!bCaught)
	{
		GET_TEAM(getTeam()).changeStolenVisibilityTimer(pCity->getTeam(), 2);

		finishMoves();

		szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_STOLE_PLANS", getNameKey(), pCity->getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_STEALPLANS", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pCity->getX_INLINE(), pCity->getY_INLINE());

		szBuffer = gDLL->getText("TXT_KEY_MISC_PLANS_STOLEN", pCity->getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_STEALPLANS", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE(), true, true);

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_STEAL_PLANS);
		}
	}
	else
	{
		szBuffer = gDLL->getText("TXT_KEY_MISC_SPY_CAUGHT_AND_KILLED", GET_PLAYER(getOwnerINLINE()).getCivilizationAdjective(), getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSE", MESSAGE_TYPE_INFO);

		szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_SPY_CAUGHT", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSED", MESSAGE_TYPE_INFO);

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_SURRENDER);
		}

		if (!isEnemy(pCity->getTeam()))
		{
			GET_PLAYER(pCity->getOwnerINLINE()).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_SPY_CAUGHT, 1);
		}

		kill(true, pCity->getOwnerINLINE());
	}

	return true;
}


bool CvUnit::canFound(const CvPlot* pPlot, bool bTestVisible) const
{
	if (!isFound())
	{
		return false;
	}

	if (!(GET_PLAYER(getOwnerINLINE()).canFound(pPlot->getX_INLINE(), pPlot->getY_INLINE(), bTestVisible)))
	{
		return false;
	}

	return true;
}


bool CvUnit::found()
{
	if (!canFound(plot()))
	{
		return false;
	}

	if (GC.getGameINLINE().getActivePlayer() == getOwnerINLINE())
	{
		gDLL->getInterfaceIFace()->lookAt(plot()->getPoint(), CAMERALOOKAT_NORMAL);
	}

	GET_PLAYER(getOwnerINLINE()).found(getX_INLINE(), getY_INLINE());

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_FOUND);
	}

	kill(true);

	return true;
}


bool CvUnit::canSpread(const CvPlot* pPlot, ReligionTypes eReligion, bool bTestVisible) const
{
	CvCity* pCity;

/*************************************************************************************************/
/** UNOFFICIAL_PATCH                       08/19/09                                jdog5000      */
/**                                                                                              */
/** Efficiency                                                                                   */
/*************************************************************************************************/
/* orginal bts code
	if (GC.getUSE_USE_CANNOT_SPREAD_RELIGION_CALLBACK())
	{
		CyArgsList argsList;
		argsList.add(getOwnerINLINE());
		argsList.add(getID());
		argsList.add((int) eReligion);
		argsList.add(pPlot->getX());
		argsList.add(pPlot->getY());
		long lResult=0;
		gDLL->getPythonIFace()->callFunction(PYGameModule, "cannotSpreadReligion", argsList.makeFunctionArgs(), &lResult);
		if (lResult > 0)
		{
			return false;
		}
	}
*/
				// UP efficiency: Moved below faster calls
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                        END                                                  */
/*************************************************************************************************/

	if (eReligion == NO_RELIGION)
	{
		return false;
	}

	if (m_pUnitInfo->getReligionSpreads(eReligion) <= 0)
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (pCity->isHasReligion(eReligion))
	{
		return false;
	}

	if (!canEnterArea(pPlot->getTeam(), pPlot->area()))
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (pCity->getTeam() != getTeam())
		{
			if (GET_PLAYER(pCity->getOwnerINLINE()).isNoNonStateReligionSpread())
			{
				if (eReligion != GET_PLAYER(pCity->getOwnerINLINE()).getStateReligion())
				{
					return false;
				}
			}
		}
	}

/*************************************************************************************************/
/** UNOFFICIAL_PATCH                       08/19/09                                jdog5000      */
/**                                                                                              */
/** Efficiency                                                                                   */
/*************************************************************************************************/
	if (GC.getUSE_USE_CANNOT_SPREAD_RELIGION_CALLBACK())
	{
		CyArgsList argsList;
		argsList.add(getOwnerINLINE());
		argsList.add(getID());
		argsList.add((int) eReligion);
		argsList.add(pPlot->getX());
		argsList.add(pPlot->getY());
		long lResult=0;
		gDLL->getPythonIFace()->callFunction(PYGameModule, "cannotSpreadReligion", argsList.makeFunctionArgs(), &lResult);
		if (lResult > 0)
		{
			return false;
		}
	}
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                        END                                                  */
/*************************************************************************************************/

	return true;
}


bool CvUnit::spread(ReligionTypes eReligion)
{
	CvCity* pCity;
	CvWString szBuffer;
	int iSpreadProb;

	if (!canSpread(plot(), eReligion))
	{
		return false;
	}

	pCity = plot()->getPlotCity();

	if (pCity != NULL)
	{
		iSpreadProb = m_pUnitInfo->getReligionSpreads(eReligion);

		if (pCity->getTeam() != getTeam())
		{
			iSpreadProb /= 2;
		}

		bool bSuccess;

		iSpreadProb += (((GC.getNumReligionInfos() - pCity->getReligionCount()) * (100 - iSpreadProb)) / GC.getNumReligionInfos());

		if (GC.getGameINLINE().getSorenRandNum(100, "Unit Spread Religion") < iSpreadProb)
		{
			pCity->setHasReligion(eReligion, true, true, false);
			// Sanguo Mod Official Belief, start, by poyuzhe 12.01.08
			if (pCity->getOwnerINLINE() != getOwnerINLINE())
			{
			    pCity->changeCulture(getOwnerINLINE(), (GC.getDefineINT("MISSIONARY_CULTURE_SPREAD_AMOUNT")) * (GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitGreatWorkPercent()) / 100, true, true);
			    if ((!GC.getGameINLINE().isOption(GAMEOPTION_NO_ESPIONAGE)) && (GET_PLAYER(getOwnerINLINE()).getTeam() != GET_PLAYER(pCity->getOwnerINLINE()).getTeam()))
			    {
                    GET_TEAM(GET_PLAYER(getOwnerINLINE()).getTeam()).changeEspionagePointsAgainstTeam(GET_PLAYER(pCity->getOwnerINLINE()).getTeam(), (GC.getDefineINT("MISSIONARY_ESPIONAGE_POINT_AMOUNT")) * (GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitGreatWorkPercent()) / 100);
			    }
			}
			// Sanguo Mod Official Belief, end
			bSuccess = true;
		}
		else
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_RELIGION_FAILED_TO_SPREAD", getNameKey(), GC.getReligionInfo(eReligion).getChar(), pCity->getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_NOSPREAD", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE());
			bSuccess = false;
		}

		// Python Event
		CvEventReporter::getInstance().unitSpreadReligionAttempt(this, eReligion, bSuccess);
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_SPREAD);
	}

	kill(true);

	return true;
}


bool CvUnit::canSpreadCorporation(const CvPlot* pPlot, CorporationTypes eCorporation, bool bTestVisible) const
{
	if (NO_CORPORATION == eCorporation)
	{
		return false;
	}

	if (!GET_PLAYER(getOwnerINLINE()).isActiveCorporation(eCorporation))
	{
		return false;
	}

	if (m_pUnitInfo->getCorporationSpreads(eCorporation) <= 0)
	{
		return false;
	}

	CvCity* pCity = pPlot->getPlotCity();

	if (NULL == pCity)
	{
		return false;
	}

	if (pCity->isHasCorporation(eCorporation))
	{
		return false;
	}

	if (!canEnterArea(pPlot->getTeam(), pPlot->area()))
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (!GET_PLAYER(pCity->getOwnerINLINE()).isActiveCorporation(eCorporation))
		{
			return false;
		}

		for (int iCorporation = 0; iCorporation < GC.getNumCorporationInfos(); ++iCorporation)
		{
			if (pCity->isHeadquarters((CorporationTypes)iCorporation))
			{
				if (GC.getGameINLINE().isCompetingCorporation((CorporationTypes)iCorporation, eCorporation))
				{
					return false;
				}
			}
		}

		bool bValid = false;
		for (int i = 0; i < GC.getNUM_CORPORATION_PREREQ_BONUSES(); ++i)
		{
			BonusTypes eBonus = (BonusTypes)GC.getCorporationInfo(eCorporation).getPrereqBonus(i);
			if (NO_BONUS != eBonus)
			{
				if (pCity->hasBonus(eBonus))
				{
					bValid = true;
					break;
				}
			}
		}

		if (!bValid)
		{
			return false;
		}

		if (GET_PLAYER(getOwnerINLINE()).getGold() < spreadCorporationCost(eCorporation, pCity))
		{
			return false;
		}
	}

	return true;
}

int CvUnit::spreadCorporationCost(CorporationTypes eCorporation, CvCity* pCity) const
{
	int iCost = std::max(0, GC.getCorporationInfo(eCorporation).getSpreadCost() * (100 + GET_PLAYER(getOwnerINLINE()).calculateInflationRate()));
	iCost /= 100;

	if (NULL != pCity)
	{
		if (getTeam() != pCity->getTeam() && !GET_TEAM(pCity->getTeam()).isVassal(getTeam()))
		{
			iCost *= GC.getDefineINT("CORPORATION_FOREIGN_SPREAD_COST_PERCENT");
			iCost /= 100;
		}

		for (int iCorp = 0; iCorp < GC.getNumCorporationInfos(); ++iCorp)
		{
			if (iCorp != eCorporation)
			{
				if (pCity->isActiveCorporation((CorporationTypes)iCorp))
				{
					if (GC.getGameINLINE().isCompetingCorporation(eCorporation, (CorporationTypes)iCorp))
					{
						iCost *= 100 + GC.getCorporationInfo((CorporationTypes)iCorp).getSpreadFactor();
						iCost /= 100;
					}
				}
			}
		}
	}

	return iCost;
}

bool CvUnit::spreadCorporation(CorporationTypes eCorporation)
{
	int iSpreadProb;

	if (!canSpreadCorporation(plot(), eCorporation))
	{
		return false;
	}

	CvCity* pCity = plot()->getPlotCity();

	if (NULL != pCity)
	{
		GET_PLAYER(getOwnerINLINE()).changeGold(-spreadCorporationCost(eCorporation, pCity));

		iSpreadProb = m_pUnitInfo->getCorporationSpreads(eCorporation);

		if (pCity->getTeam() != getTeam())
		{
			iSpreadProb /= 2;
		}

		iSpreadProb += (((GC.getNumCorporationInfos() - pCity->getCorporationCount()) * (100 - iSpreadProb)) / GC.getNumCorporationInfos());

		if (GC.getGameINLINE().getSorenRandNum(100, "Unit Spread Corporation") < iSpreadProb)
		{
			pCity->setHasCorporation(eCorporation, true, true, false);
		}
		else
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_CORPORATION_FAILED_TO_SPREAD", getNameKey(), GC.getCorporationInfo(eCorporation).getChar(), pCity->getNameKey());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_NOSPREAD", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE());
		}
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_SPREAD_CORPORATION);
	}

	kill(true);

	return true;
}


bool CvUnit::canJoin(const CvPlot* pPlot, SpecialistTypes eSpecialist) const
{
	CvCity* pCity;

	if (eSpecialist == NO_SPECIALIST)
	{
		return false;
	}

	if (!(m_pUnitInfo->getGreatPeoples(eSpecialist)))
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (!(pCity->canJoin()))
	{
		return false;
	}

	if (pCity->getTeam() != getTeam())
	{
		return false;
	}

	if (isDelayedDeath())
	{
		return false;
	}

	return true;
}


bool CvUnit::join(SpecialistTypes eSpecialist)
{
	CvCity* pCity;

	if (!canJoin(plot(), eSpecialist))
	{
		return false;
	}

	pCity = plot()->getPlotCity();

	if (pCity != NULL)
	{
		pCity->changeFreeSpecialistCount(eSpecialist, 1);
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_JOIN);
	}

	kill(true);

	return true;
}


bool CvUnit::canConstruct(const CvPlot* pPlot, BuildingTypes eBuilding, bool bTestVisible) const
{
	CvCity* pCity;

	if (eBuilding == NO_BUILDING)
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (getTeam() != pCity->getTeam())
	{
		return false;
	}

	if (pCity->getNumRealBuilding(eBuilding) > 0)
	{
		return false;
	}

	if (!(m_pUnitInfo->getForceBuildings(eBuilding)))
	{
		if (!(m_pUnitInfo->getBuildings(eBuilding)))
		{
			return false;
		}

		if (!(pCity->canConstruct(eBuilding, false, bTestVisible, true)))
		{
			return false;
		}
	}

	if (isDelayedDeath())
	{
		return false;
	}

	return true;
}


bool CvUnit::construct(BuildingTypes eBuilding)
{
	CvCity* pCity;

	if (!canConstruct(plot(), eBuilding))
	{
		return false;
	}

	pCity = plot()->getPlotCity();

	if (pCity != NULL)
	{
		pCity->setNumRealBuilding(eBuilding, pCity->getNumRealBuilding(eBuilding) + 1);

		CvEventReporter::getInstance().buildingBuilt(pCity, eBuilding);
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_CONSTRUCT);
	}

	kill(true);

	return true;
}


TechTypes CvUnit::getDiscoveryTech() const
{
	return ::getDiscoveryTech(getUnitType(), getOwnerINLINE());
}


int CvUnit::getDiscoverResearch(TechTypes eTech) const
{
	int iResearch;

	iResearch = (m_pUnitInfo->getBaseDiscover() + (m_pUnitInfo->getDiscoverMultiplier() * GET_TEAM(getTeam()).getTotalPopulation()));

	iResearch *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitDiscoverPercent();
	iResearch /= 100;

    if (eTech != NO_TECH)
    {
        iResearch = std::min(GET_TEAM(getTeam()).getResearchLeft(eTech), iResearch);
    }

	return std::max(0, iResearch);
}


bool CvUnit::canDiscover(const CvPlot* pPlot) const
{
	TechTypes eTech;

	eTech = getDiscoveryTech();

	if (eTech == NO_TECH)
	{
		return false;
	}

	if (getDiscoverResearch(eTech) == 0)
	{
		return false;
	}

	if (isDelayedDeath())
	{
		return false;
	}

	return true;
}


bool CvUnit::discover()
{
	TechTypes eDiscoveryTech;

	if (!canDiscover(plot()))
	{
		return false;
	}

	eDiscoveryTech = getDiscoveryTech();
	FAssertMsg(eDiscoveryTech != NO_TECH, "DiscoveryTech is not assigned a valid value");

	GET_TEAM(getTeam()).changeResearchProgress(eDiscoveryTech, getDiscoverResearch(eDiscoveryTech), getOwnerINLINE());

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_DISCOVER);
	}

	kill(true);

	return true;
}


int CvUnit::getMaxHurryProduction(CvCity* pCity) const
{
	int iProduction;

	iProduction = (m_pUnitInfo->getBaseHurry() + (m_pUnitInfo->getHurryMultiplier() * pCity->getPopulation()));

	iProduction *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitHurryPercent();
	iProduction /= 100;

	return std::max(0, iProduction);
}


int CvUnit::getHurryProduction(const CvPlot* pPlot) const
{
	CvCity* pCity;
	int iProduction;

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return 0;
	}

	iProduction = getMaxHurryProduction(pCity);

	iProduction = std::min(pCity->productionLeft(), iProduction);

	return std::max(0, iProduction);
}


bool CvUnit::canHurry(const CvPlot* pPlot, bool bTestVisible) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	CvCity* pCity;

	if (getHurryProduction(pPlot) == 0)
	{
		return false;
	}

	pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (pCity->getProductionTurnsLeft() == 1)
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (!(pCity->isProductionBuilding()))
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::hurry()
{
	CvCity* pCity;

	if (!canHurry(plot()))
	{
		return false;
	}

	pCity = plot()->getPlotCity();

	if (pCity != NULL)
	{
		pCity->changeProduction(getHurryProduction(plot()));
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_HURRY);
	}

	kill(true);

	return true;
}


int CvUnit::getTradeGold(const CvPlot* pPlot) const
{
	CvCity* pCapitalCity;
	CvCity* pCity;
	int iGold;

	pCity = pPlot->getPlotCity();
	pCapitalCity = GET_PLAYER(getOwnerINLINE()).getCapitalCity();

	if (pCity == NULL)
	{
		return 0;
	}

	iGold = (m_pUnitInfo->getBaseTrade() + (m_pUnitInfo->getTradeMultiplier() * ((pCapitalCity != NULL) ? pCity->calculateTradeProfit(pCapitalCity) : 0)));

	iGold *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitTradePercent();
	iGold /= 100;

	return std::max(0, iGold);
}


bool CvUnit::canTrade(const CvPlot* pPlot, bool bTestVisible) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	CvCity* pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (getTradeGold(pPlot) == 0)
	{
		return false;
	}

	if (!canEnterArea(pPlot->getTeam(), pPlot->area()))
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (pCity->getTeam() == getTeam())
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::trade()
{
	if (!canTrade(plot()))
	{
		return false;
	}

	GET_PLAYER(getOwnerINLINE()).changeGold(getTradeGold(plot()));

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_TRADE);
	}

	kill(true);

	return true;
}


int CvUnit::getGreatWorkCulture(const CvPlot* pPlot) const
{
	int iCulture;

	iCulture = m_pUnitInfo->getGreatWorkCulture();

	iCulture *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitGreatWorkPercent();
	iCulture /= 100;

	return std::max(0, iCulture);
}


bool CvUnit::canGreatWork(const CvPlot* pPlot) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	CvCity* pCity = pPlot->getPlotCity();

	if (pCity == NULL)
	{
		return false;
	}

	if (pCity->getOwnerINLINE() != getOwnerINLINE())
	{
		return false;
	}

	if (getGreatWorkCulture(pPlot) == 0)
	{
		return false;
	}

	return true;
}


bool CvUnit::greatWork()
{
	if (!canGreatWork(plot()))
	{
		return false;
	}

	CvCity* pCity = plot()->getPlotCity();

	if (pCity != NULL)
	{
		pCity->setCultureUpdateTimer(0);
		pCity->setOccupationTimer(0);

		int iCultureToAdd = 100 * getGreatWorkCulture(plot());
		int iNumTurnsApplied = (GC.getDefineINT("GREAT_WORKS_CULTURE_TURNS") * GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitGreatWorkPercent()) / 100;

		for (int i = 0; i < iNumTurnsApplied; ++i)
		{
			pCity->changeCultureTimes100(getOwnerINLINE(), iCultureToAdd / iNumTurnsApplied, true, true);
		}

		if (iNumTurnsApplied > 0)
		{
			pCity->changeCultureTimes100(getOwnerINLINE(), iCultureToAdd % iNumTurnsApplied, false, true);
		}
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_GREAT_WORK);
	}

	kill(true);

	return true;
}


int CvUnit::getEspionagePoints(const CvPlot* pPlot) const
{
	int iEspionagePoints;

	iEspionagePoints = m_pUnitInfo->getEspionagePoints();

	iEspionagePoints *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getUnitGreatWorkPercent();
	iEspionagePoints /= 100;

	return std::max(0, iEspionagePoints);
}

bool CvUnit::canInfiltrate(const CvPlot* pPlot, bool bTestVisible) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	if (GC.getGameINLINE().isOption(GAMEOPTION_NO_ESPIONAGE))
	{
		return false;
	}

	if (getEspionagePoints(NULL) == 0)
	{
		return false;
	}

	CvCity* pCity = pPlot->getPlotCity();
	if (pCity == NULL || pCity->isBarbarian())
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (NULL != pCity && pCity->getTeam() == getTeam())
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::infiltrate()
{
	if (!canInfiltrate(plot()))
	{
		return false;
	}

	int iPoints = getEspionagePoints(NULL);
	GET_TEAM(getTeam()).changeEspionagePointsAgainstTeam(GET_PLAYER(plot()->getOwnerINLINE()).getTeam(), iPoints);
	GET_TEAM(getTeam()).changeEspionagePointsEver(iPoints);

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_INFILTRATE);
	}

	kill(true);

	return true;
}


bool CvUnit::canEspionage(const CvPlot* pPlot, bool bTestVisible) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	if (!isSpy())
	{
		return false;
	}

	if (GC.getGameINLINE().isOption(GAMEOPTION_NO_ESPIONAGE))
	{
		return false;
	}

	PlayerTypes ePlotOwner = pPlot->getOwnerINLINE();
	if (NO_PLAYER == ePlotOwner)
	{
		return false;
	}

	CvPlayer& kTarget = GET_PLAYER(ePlotOwner);

	if (kTarget.isBarbarian())
	{
		return false;
	}

	if (kTarget.getTeam() == getTeam())
	{
		return false;
	}

	if (GET_TEAM(getTeam()).isVassal(kTarget.getTeam()))
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (isMadeAttack())
		{
			return false;
		}

		if (hasMoved())
		{
			return false;
		}

		if (kTarget.getTeam() != getTeam() && !isInvisible(kTarget.getTeam(), false))
		{
			return false;
		}
	}

	return true;
}

bool CvUnit::espionage(EspionageMissionTypes eMission, int iData)
{
	if (!canEspionage(plot()))
	{
		return false;
	}

	PlayerTypes eTargetPlayer = plot()->getOwnerINLINE();

	if (NO_ESPIONAGEMISSION == eMission)
	{
		FAssert(GET_PLAYER(getOwnerINLINE()).isHuman());
		CvPopupInfo* pInfo = new CvPopupInfo(BUTTONPOPUP_DOESPIONAGE);
		if (NULL != pInfo)
		{
			gDLL->getInterfaceIFace()->addPopup(pInfo, getOwnerINLINE(), true);
		}
	}
	else if (GC.getEspionageMissionInfo(eMission).isTwoPhases() && -1 == iData)
	{
		FAssert(GET_PLAYER(getOwnerINLINE()).isHuman());
		CvPopupInfo* pInfo = new CvPopupInfo(BUTTONPOPUP_DOESPIONAGE_TARGET);
		if (NULL != pInfo)
		{
			pInfo->setData1(eMission);
			gDLL->getInterfaceIFace()->addPopup(pInfo, getOwnerINLINE(), true);
		}
	}
	else
	{
		if (testSpyIntercepted(eTargetPlayer, GC.getEspionageMissionInfo(eMission).getDifficultyMod()))
		{
			return false;
		}

		if (GET_PLAYER(getOwnerINLINE()).doEspionageMission(eMission, eTargetPlayer, plot(), iData, this))
		{
			if (plot()->isActiveVisible(false))
			{
				NotifyEntity(MISSION_ESPIONAGE);
			}

			if (!testSpyIntercepted(eTargetPlayer, GC.getDefineINT("ESPIONAGE_SPY_MISSION_ESCAPE_MOD")))
			{
				setFortifyTurns(0);
				setMadeAttack(true);
				finishMoves();

				CvCity* pCapital = GET_PLAYER(getOwnerINLINE()).getCapitalCity();
				if (NULL != pCapital)
				{
					setXY(pCapital->getX_INLINE(), pCapital->getY_INLINE(), false, false, false);

					CvWString szBuffer = gDLL->getText("TXT_KEY_ESPIONAGE_SPY_SUCCESS", getNameKey(), pCapital->getNameKey());
					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_POSITIVE_DINK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_WHITE"), pCapital->getX_INLINE(), pCapital->getY_INLINE(), true, true);
				}
			}

			return true;
		}
	}

	return false;
}

bool CvUnit::testSpyIntercepted(PlayerTypes eTargetPlayer, int iModifier)
{
	CvPlayer& kTargetPlayer = GET_PLAYER(eTargetPlayer);

	if (kTargetPlayer.isBarbarian())
	{
		return false;
	}

	if (GC.getGameINLINE().getSorenRandNum(10000, "Spy Interception") >= getSpyInterceptPercent(kTargetPlayer.getTeam()) * (100 + iModifier))
	{
		return false;
	}

	CvString szFormatNoReveal;
	CvString szFormatReveal;

	if (GET_TEAM(kTargetPlayer.getTeam()).getCounterespionageModAgainstTeam(getTeam()) > 0)
	{
		szFormatNoReveal = "TXT_KEY_SPY_INTERCEPTED_MISSION";
		szFormatReveal = "TXT_KEY_SPY_INTERCEPTED_MISSION_REVEAL";
	}
	else if (plot()->isEspionageCounterSpy(kTargetPlayer.getTeam()))
	{
		szFormatNoReveal = "TXT_KEY_SPY_INTERCEPTED_SPY";
		szFormatReveal = "TXT_KEY_SPY_INTERCEPTED_SPY_REVEAL";
	}
	else
	{
		szFormatNoReveal = "TXT_KEY_SPY_INTERCEPTED";
		szFormatReveal = "TXT_KEY_SPY_INTERCEPTED_REVEAL";
	}

	CvWString szCityName = kTargetPlayer.getCivilizationShortDescription();
	CvCity* pClosestCity = GC.getMapINLINE().findCity(getX_INLINE(), getY_INLINE(), eTargetPlayer, kTargetPlayer.getTeam(), true, false);
	if (pClosestCity != NULL)
	{
		szCityName = pClosestCity->getName();
	}

	CvWString szBuffer = gDLL->getText(szFormatReveal.GetCString(), GET_PLAYER(getOwnerINLINE()).getCivilizationAdjectiveKey(), getNameKey(), kTargetPlayer.getCivilizationAdjectiveKey(), szCityName.GetCString());
	gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE(), true, true);

	if (GC.getGameINLINE().getSorenRandNum(100, "Spy Reveal identity") < GC.getDefineINT("ESPIONAGE_SPY_REVEAL_IDENTITY_PERCENT"))
	{
		if (!isEnemy(kTargetPlayer.getTeam()))
		{
			GET_PLAYER(eTargetPlayer).AI_changeMemoryCount(getOwnerINLINE(), MEMORY_SPY_CAUGHT, 1);
		}

		gDLL->getInterfaceIFace()->addMessage(eTargetPlayer, true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	}
	else
	{
		szBuffer = gDLL->getText(szFormatNoReveal.GetCString(), getNameKey(), kTargetPlayer.getCivilizationAdjectiveKey(), szCityName.GetCString());
		gDLL->getInterfaceIFace()->addMessage(eTargetPlayer, true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_EXPOSE", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	}

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_SURRENDER);
	}

	kill(true);

	return true;
}

int CvUnit::getSpyInterceptPercent(TeamTypes eTargetTeam) const
{
	FAssert(isSpy());
	FAssert(getTeam() != eTargetTeam);

	int iSuccess = 0;

	int iTargetPoints = GET_TEAM(eTargetTeam).getEspionagePointsEver();
	int iOurPoints = GET_TEAM(getTeam()).getEspionagePointsEver();
	iSuccess += (GC.getDefineINT("ESPIONAGE_INTERCEPT_SPENDING_MAX") * iTargetPoints) / std::max(1, iTargetPoints + iOurPoints);

	if (plot()->isEspionageCounterSpy(eTargetTeam))
	{
		iSuccess += GC.getDefineINT("ESPIONAGE_INTERCEPT_COUNTERSPY");
	}

	if (GET_TEAM(eTargetTeam).getCounterespionageModAgainstTeam(getTeam()) > 0)
	{
		iSuccess += GC.getDefineINT("ESPIONAGE_INTERCEPT_COUNTERESPIONAGE_MISSION");
	}

	if (0 == getFortifyTurns() || plot()->plotCount(PUF_isSpy, -1, -1, NO_PLAYER, getTeam()) > 1)
	{
		iSuccess += GC.getDefineINT("ESPIONAGE_INTERCEPT_RECENT_MISSION");
	}

	// Sanguo Mod Civilization Trait, start, by poyuzhe 12.21.08
    // Kong Rong, TRAIT_CELEBRITY
    bool bCelebrity = false;
    for (int iI = 0; iI < MAX_CIV_PLAYERS; iI++)
    {
        if (bCelebrity == true)
        {
            break;
        }
        if (eTargetTeam != NO_TEAM && GET_PLAYER((PlayerTypes)iI).isAlive() && GET_PLAYER((PlayerTypes)iI).getTeam() == eTargetTeam)
        {
            for (int iJ = 0; iJ < GC.getNumTraitInfos(); iJ++)
            {
                if (GET_PLAYER((PlayerTypes)iI).hasTrait((TraitTypes)iJ) && GC.getTraitInfo((TraitTypes)iJ).isCelebrity())
                {
                    iSuccess *= (100 + GC.getDefineINT("CELEBRITY_SPY_INTERCEPTED_CHANCE_MODIFIER"));
                    iSuccess /= 100;
                    bCelebrity = true;
                    break;
                }
            }
        }
    }
    // Sanguo Mod Civilization Trait, end

	return std::min(100, std::max(0, iSuccess));
}

bool CvUnit::isIntruding() const
{
	TeamTypes eLocalTeam = plot()->getTeam();

	if (NO_TEAM == eLocalTeam || eLocalTeam == getTeam())
	{
		return false;
	}

	// UNOFFICIAL_PATCH Start
	// * Vassal's spies no longer caught in master's territory
	//if (GET_TEAM(eLocalTeam).isVassal(getTeam()))
	if (GET_TEAM(eLocalTeam).isVassal(getTeam()) || GET_TEAM(getTeam()).isVassal(eLocalTeam))
	// UNOFFICIAL_PATCH End
	{
		return false;
	}

	return true;
}

bool CvUnit::canGoldenAge(const CvPlot* pPlot, bool bTestVisible) const
{
	if (!isGoldenAge())
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (GET_PLAYER(getOwnerINLINE()).unitsRequiredForGoldenAge() > GET_PLAYER(getOwnerINLINE()).unitsGoldenAgeReady())
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::goldenAge()
{
	if (!canGoldenAge(plot()))
	{
		return false;
	}

	GET_PLAYER(getOwnerINLINE()).killGoldenAgeUnits(this);

	GET_PLAYER(getOwnerINLINE()).changeGoldenAgeTurns(GET_PLAYER(getOwnerINLINE()).getGoldenAgeLength());
	GET_PLAYER(getOwnerINLINE()).changeNumUnitGoldenAges(1);

	if (plot()->isActiveVisible(false))
	{
		NotifyEntity(MISSION_GOLDEN_AGE);
	}

	kill(true);

	return true;
}


bool CvUnit::canBuild(const CvPlot* pPlot, BuildTypes eBuild, bool bTestVisible) const
{
    FAssertMsg(eBuild < GC.getNumBuildInfos(), "Index out of bounds");
	if (!(m_pUnitInfo->getBuilds(eBuild)))
	{
		return false;
	}

	if (!(GET_PLAYER(getOwnerINLINE()).canBuild(pPlot, eBuild, false, bTestVisible)))
	{
		return false;
	}

	if (!pPlot->isValidDomainForAction(*this))
	{
		return false;
	}

	return true;
}

// Returns true if build finished...
bool CvUnit::build(BuildTypes eBuild)
{
	bool bFinished;

	FAssertMsg(eBuild < GC.getNumBuildInfos(), "Invalid Build");

	if (!canBuild(plot(), eBuild))
	{
		return false;
	}

	// Note: notify entity must come before changeBuildProgress - because once the unit is done building,
	// that function will notify the entity to stop building.
	NotifyEntity((MissionTypes)GC.getBuildInfo(eBuild).getMissionType());

	GET_PLAYER(getOwnerINLINE()).changeGold(-(GET_PLAYER(getOwnerINLINE()).getBuildCost(plot(), eBuild)));

	bFinished = plot()->changeBuildProgress(eBuild, workRate(false), getTeam());

	finishMoves(); // needs to be at bottom because movesLeft() can affect workRate()...

	if (bFinished)
	{
		if (GC.getBuildInfo(eBuild).isKill())
		{
			kill(true);
		}
	}

	// Python Event
	CvEventReporter::getInstance().unitBuildImprovement(this, eBuild, bFinished);

	return bFinished;
}


bool CvUnit::canPromote(PromotionTypes ePromotion, int iLeaderUnitId) const
{
	if (iLeaderUnitId >= 0)
	{
		if (iLeaderUnitId == getID())
		{
			return false;
		}

		// The command is always possible if it's coming from a Warlord unit that gives just experience points
		CvUnit* pWarlord = GET_PLAYER(getOwnerINLINE()).getUnit(iLeaderUnitId);
		if (pWarlord &&
			NO_UNIT != pWarlord->getUnitType() &&
			pWarlord->getUnitInfo().getLeaderExperience() > 0 &&
			NO_PROMOTION == pWarlord->getUnitInfo().getLeaderPromotion() &&
			canAcquirePromotionAny())
		{
			return true;
		}
	}

	if (ePromotion == NO_PROMOTION)
	{
		return false;
	}

	if (!canAcquirePromotion(ePromotion))
	{
		return false;
	}

	if (GC.getPromotionInfo(ePromotion).isLeader())
	{
		if (iLeaderUnitId >= 0)
		{
			CvUnit* pWarlord = GET_PLAYER(getOwnerINLINE()).getUnit(iLeaderUnitId);
			// Sanguo Mod Hero, start, added by poyuzhe 01.18.09
			if (getLeaderUnitType() != NO_UNIT)
			{
			    return false;
			}
            // Sanguo Mod Hero, end
			if (pWarlord && NO_UNIT != pWarlord->getUnitType())
			{
				return (pWarlord->getUnitInfo().getLeaderPromotion() == ePromotion);
			}
		}
		return false;
	}
	else
	{
		if (!isPromotionReady())
		{
			return false;
		}
	}

	return true;
}

void CvUnit::promote(PromotionTypes ePromotion, int iLeaderUnitId)
{
	if (!canPromote(ePromotion, iLeaderUnitId))
	{
		return;
	}

	if (iLeaderUnitId >= 0)
	{
		CvUnit* pWarlord = GET_PLAYER(getOwnerINLINE()).getUnit(iLeaderUnitId);
		if (pWarlord)
		{
			pWarlord->giveExperience();
			if (!pWarlord->getNameNoDesc().empty())
			{
				setName(pWarlord->getNameKey());
			}

			//update graphics models
			m_eLeaderUnitType = pWarlord->getUnitType();
			reloadEntity();
		}
	}

	if (!GC.getPromotionInfo(ePromotion).isLeader())
	{
		changeLevel(1);
		changeDamage(-(getDamage() / 2));
	}

	setHasPromotion(ePromotion, true);

	testPromotionReady();

	if (IsSelected())
	{
		gDLL->getInterfaceIFace()->playGeneralSound(GC.getPromotionInfo(ePromotion).getSound());

		gDLL->getInterfaceIFace()->setDirty(UnitInfo_DIRTY_BIT, true);
	}
	else
	{
		setInfoBarDirty(true);
	}

	CvEventReporter::getInstance().unitPromoted(this, ePromotion);
}

bool CvUnit::lead(int iUnitId)
{
	if (!canLead(plot(), iUnitId))
	{
		return false;
	}

	PromotionTypes eLeaderPromotion = (PromotionTypes)m_pUnitInfo->getLeaderPromotion();

	if (-1 == iUnitId)
	{
		CvPopupInfo* pInfo = new CvPopupInfo(BUTTONPOPUP_LEADUNIT, eLeaderPromotion, getID());
		if (pInfo)
		{
			gDLL->getInterfaceIFace()->addPopup(pInfo, getOwnerINLINE(), true);
		}
		return false;
	}
	else
	{
		CvUnit* pUnit = GET_PLAYER(getOwnerINLINE()).getUnit(iUnitId);

		if (!pUnit || !pUnit->canPromote(eLeaderPromotion, getID()))
		{
			return false;
		}

		pUnit->joinGroup(NULL, true, true);

		pUnit->promote(eLeaderPromotion, getID());

		if (plot()->isActiveVisible(false))
		{
			NotifyEntity(MISSION_LEAD);
		}

		kill(true);

		return true;
	}
}


int CvUnit::canLead(const CvPlot* pPlot, int iUnitId) const
{
	PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return 0;
	}

	if (NO_UNIT == getUnitType())
	{
		return 0;
	}

	int iNumUnits = 0;
	CvUnitInfo& kUnitInfo = getUnitInfo();

	if (-1 == iUnitId)
	{
		CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();
		while(pUnitNode != NULL)
		{
			CvUnit* pUnit = ::getUnit(pUnitNode->m_data);
			pUnitNode = pPlot->nextUnitNode(pUnitNode);

			if (pUnit && pUnit != this && pUnit->getOwnerINLINE() == getOwnerINLINE() && pUnit->canPromote((PromotionTypes)kUnitInfo.getLeaderPromotion(), getID()))
			{
				++iNumUnits;
			}
		}
	}
	else
	{
		CvUnit* pUnit = GET_PLAYER(getOwnerINLINE()).getUnit(iUnitId);
		if (pUnit && pUnit != this && pUnit->canPromote((PromotionTypes)kUnitInfo.getLeaderPromotion(), getID()))
		{
			iNumUnits = 1;
		}
	}
	return iNumUnits;
}


int CvUnit::canGiveExperience(const CvPlot* pPlot) const
{
	int iNumUnits = 0;

	if (NO_UNIT != getUnitType() && m_pUnitInfo->getLeaderExperience() > 0)
	{
		CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();
		while(pUnitNode != NULL)
		{
			CvUnit* pUnit = ::getUnit(pUnitNode->m_data);
			pUnitNode = pPlot->nextUnitNode(pUnitNode);

			if (pUnit && pUnit != this && pUnit->getOwnerINLINE() == getOwnerINLINE() && pUnit->canAcquirePromotionAny())
			{
				++iNumUnits;
			}
		}
	}

	return iNumUnits;
}

bool CvUnit::giveExperience()
{
	CvPlot* pPlot = plot();

	if (pPlot)
	{
		int iNumUnits = canGiveExperience(pPlot);
		if (iNumUnits > 0)
		{
			int iTotalExperience = getStackExperienceToGive(iNumUnits);

			int iMinExperiencePerUnit = iTotalExperience / iNumUnits;
			int iRemainder = iTotalExperience % iNumUnits;

			CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();
			int i = 0;
			while(pUnitNode != NULL)
			{
				CvUnit* pUnit = ::getUnit(pUnitNode->m_data);
				pUnitNode = pPlot->nextUnitNode(pUnitNode);

				if (pUnit && pUnit != this && pUnit->getOwnerINLINE() == getOwnerINLINE() && pUnit->canAcquirePromotionAny())
				{
					// Sanguo Mod Hero, start, added by poyuzhe 02.22.09
					// pUnit->changeExperience(i < iRemainder ? iMinExperiencePerUnit+1 : iMinExperiencePerUnit);
					pUnit->changeExperience(i < iRemainder ? iMinExperiencePerUnit+1 : iMinExperiencePerUnit, -1, false, false, false, NO_TERRAIN, false);
					// Sanguo Mod Hero, end
					pUnit->testPromotionReady();
				}

				i++;
			}

			return true;
		}
	}

	return false;
}

int CvUnit::getStackExperienceToGive(int iNumUnits) const
{
	return (m_pUnitInfo->getLeaderExperience() * (100 + std::min(50, (iNumUnits - 1) * GC.getDefineINT("WARLORD_EXTRA_EXPERIENCE_PER_UNIT_PERCENT")))) / 100;
}

int CvUnit::upgradePrice(UnitTypes eUnit) const
{
	int iPrice;

	CyArgsList argsList;
	argsList.add(getOwnerINLINE());
	argsList.add(getID());
	argsList.add((int) eUnit);
	long lResult=0;
	gDLL->getPythonIFace()->callFunction(PYGameModule, "getUpgradePriceOverride", argsList.makeFunctionArgs(), &lResult);
	if (lResult >= 0)
	{
		return lResult;
	}

	if (isBarbarian())
	{
		return 0;
	}

	iPrice = GC.getDefineINT("BASE_UNIT_UPGRADE_COST");

	iPrice += (std::max(0, (GET_PLAYER(getOwnerINLINE()).getProductionNeeded(eUnit) - GET_PLAYER(getOwnerINLINE()).getProductionNeeded(getUnitType()))) * GC.getDefineINT("UNIT_UPGRADE_COST_PER_PRODUCTION"));

	if (!isHuman() && !isBarbarian())
	{
		iPrice *= GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIUnitUpgradePercent();
		iPrice /= 100;

		iPrice *= std::max(0, ((GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIPerEraModifier() * GET_PLAYER(getOwnerINLINE()).getCurrentEra()) + 100));
		iPrice /= 100;
	}

	iPrice -= (iPrice * getUpgradeDiscount()) / 100;

	return iPrice;
}


bool CvUnit::upgradeAvailable(UnitTypes eFromUnit, UnitClassTypes eToUnitClass, int iCount) const
{
	// Sanguo Mod Performance start, added by poyuzhe 07.27.09
	/*UnitTypes eLoopUnit;
	int iI;
	int numUnitClassInfos = GC.getNumUnitClassInfos();

	if (iCount > numUnitClassInfos)
	{
		return false;
	}

	CvUnitInfo &fromUnitInfo = GC.getUnitInfo(eFromUnit);

	if (fromUnitInfo.getUpgradeUnitClass(eToUnitClass))
	{
		return true;
	}

	for (iI = 0; iI < numUnitClassInfos; iI++)
	{
		if (fromUnitInfo.getUpgradeUnitClass(iI))
		{
			eLoopUnit = ((UnitTypes)(GC.getCivilizationInfo(getCivilizationType()).getCivilizationUnits(iI)));

			if (eLoopUnit != NO_UNIT)
			{
				if (upgradeAvailable(eLoopUnit, eToUnitClass, (iCount + 1)))
				{
					return true;
				}
			}
		}
	}*/

	int numUnitClassInfos = GC.getNumUnitClassInfos();

	if (iCount > numUnitClassInfos)
	{
		return false;
	}

	if (GC.getUnitInfo(eFromUnit).isUpgradeUnitClassTypes(eToUnitClass))
	{
		return true;
	}
	// Sanguo Mod Performance, end

	return false;
}


bool CvUnit::canUpgrade(UnitTypes eUnit, bool bTestVisible) const
{
	if (eUnit == NO_UNIT)
	{
		return false;
	}

	if(!isReadyForUpgrade())
	{
		return false;
	}

	if (!bTestVisible)
	{
		if (GET_PLAYER(getOwnerINLINE()).getGold() < upgradePrice(eUnit))
		{
			return false;
		}
	}

	if (hasUpgrade(eUnit))
	{
		return true;
	}

	return false;
}

bool CvUnit::isReadyForUpgrade() const
{
	if (!canMove())
	{
		return false;
	}

	if (plot()->getTeam() != getTeam())
	{
		return false;
	}

	return true;
}

// has upgrade is used to determine if an upgrade is possible,
// it specifically does not check whether the unit can move, whether the current plot is owned, enough gold
// those are checked in canUpgrade()
// does not search all cities, only checks the closest one
bool CvUnit::hasUpgrade(bool bSearch) const
{
	return (getUpgradeCity(bSearch) != NULL);
}

// has upgrade is used to determine if an upgrade is possible,
// it specifically does not check whether the unit can move, whether the current plot is owned, enough gold
// those are checked in canUpgrade()
// does not search all cities, only checks the closest one
bool CvUnit::hasUpgrade(UnitTypes eUnit, bool bSearch) const
{
	return (getUpgradeCity(eUnit, bSearch) != NULL);
}

// finds the 'best' city which has a valid upgrade for the unit,
// it specifically does not check whether the unit can move, or if the player has enough gold to upgrade
// those are checked in canUpgrade()
// if bSearch is true, it will check every city, if not, it will only check the closest valid city
// NULL result means the upgrade is not possible
CvCity* CvUnit::getUpgradeCity(bool bSearch) const
{
	CvPlayerAI& kPlayer = GET_PLAYER(getOwnerINLINE());
	UnitAITypes eUnitAI = AI_getUnitAIType();
	CvArea* pArea = area();

	int iCurrentValue = kPlayer.AI_unitValue(getUnitType(), eUnitAI, pArea);

	int iBestSearchValue = MAX_INT;
	CvCity* pBestUpgradeCity = NULL;

	for (int iI = 0; iI < GC.getNumUnitInfos(); iI++)
	{
		int iNewValue = kPlayer.AI_unitValue(((UnitTypes)iI), eUnitAI, pArea);
		if (iNewValue > iCurrentValue)
		{
			int iSearchValue;
			CvCity* pUpgradeCity = getUpgradeCity((UnitTypes)iI, bSearch, &iSearchValue);
			if (pUpgradeCity != NULL)
			{
				// if not searching or close enough, then this match will do
				if (!bSearch || iSearchValue < 16)
				{
					return pUpgradeCity;
				}

				if (iSearchValue < iBestSearchValue)
				{
					iBestSearchValue = iSearchValue;
					pBestUpgradeCity = pUpgradeCity;
				}
			}
		}
	}

	return pBestUpgradeCity;
}

// finds the 'best' city which has a valid upgrade for the unit, to eUnit type
// it specifically does not check whether the unit can move, or if the player has enough gold to upgrade
// those are checked in canUpgrade()
// if bSearch is true, it will check every city, if not, it will only check the closest valid city
// if iSearchValue non NULL, then on return it will be the city's proximity value, lower is better
// NULL result means the upgrade is not possible
CvCity* CvUnit::getUpgradeCity(UnitTypes eUnit, bool bSearch, int* iSearchValue) const
{
	PROFILE_FUNC();

	if (eUnit == NO_UNIT)
	{
		return false;
	}

	CvPlayerAI& kPlayer = GET_PLAYER(getOwnerINLINE());
	CvUnitInfo& kUnitInfo = GC.getUnitInfo(eUnit);

	if (GC.getCivilizationInfo(kPlayer.getCivilizationType()).getCivilizationUnits(kUnitInfo.getUnitClassType()) != eUnit)
	{
		return false;
	}

	if (!upgradeAvailable(getUnitType(), ((UnitClassTypes)(kUnitInfo.getUnitClassType()))))
	{
		return false;
	}

	if (kUnitInfo.getCargoSpace() < getCargo())
	{
		return false;
	}

	CLLNode<IDInfo>* pUnitNode = plot()->headUnitNode();
	while (pUnitNode != NULL)
	{
		CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = plot()->nextUnitNode(pUnitNode);

		if (pLoopUnit->getTransportUnit() == this)
		{
			if (kUnitInfo.getSpecialCargo() != NO_SPECIALUNIT)
			{
				if (kUnitInfo.getSpecialCargo() != pLoopUnit->getSpecialUnitType())
				{
					return false;
				}
			}

			if (kUnitInfo.getDomainCargo() != NO_DOMAIN)
			{
				if (kUnitInfo.getDomainCargo() != pLoopUnit->getDomainType())
				{
					return false;
				}
			}
		}
	}

	// sea units must be built on the coast
	bool bCoastalOnly = (getDomainType() == DOMAIN_SEA);

	// results
	int iBestValue = MAX_INT;
	CvCity* pBestCity = NULL;

	// if search is true, check every city for our team
	if (bSearch)
	{
		// air units can travel any distance
		bool bIgnoreDistance = (getDomainType() == DOMAIN_AIR);

		TeamTypes eTeam = getTeam();
		int iArea = getArea();
		int iX = getX_INLINE(), iY = getY_INLINE();

		// check every player on our team's cities
		for (int iI = 0; iI < MAX_PLAYERS; iI++)
		{
			// is this player on our team?
			CvPlayerAI& kLoopPlayer = GET_PLAYER((PlayerTypes)iI);
			if (kLoopPlayer.isAlive() && kLoopPlayer.getTeam() == eTeam)
			{
				int iLoop;
				for (CvCity* pLoopCity = kLoopPlayer.firstCity(&iLoop); pLoopCity != NULL; pLoopCity = kLoopPlayer.nextCity(&iLoop))
				{
					// if coastal only, then make sure we are coast
					CvArea* pWaterArea = NULL;
					if (!bCoastalOnly || ((pWaterArea = pLoopCity->waterArea()) != NULL && !pWaterArea->isLake()))
					{
						// can this city tran this unit?
						if (pLoopCity->canTrain(eUnit, false, false, true))
						{
							// if we do not care about distance, then the first match will do
							if (bIgnoreDistance)
							{
								// if we do not care about distance, then return 1 for value
								if (iSearchValue != NULL)
								{
									*iSearchValue = 1;
								}

								return pLoopCity;
							}

							int iValue = plotDistance(iX, iY, pLoopCity->getX_INLINE(), pLoopCity->getY_INLINE());

							// if not same area, not as good (lower numbers are better)
							if (iArea != pLoopCity->getArea() && (!bCoastalOnly || iArea != pWaterArea->getID()))
							{
								iValue *= 16;
							}

							// if we cannot path there, not as good (lower numbers are better)
							if (!generatePath(pLoopCity->plot(), 0, true))
							{
								iValue *= 16;
							}

							if (iValue < iBestValue)
							{
								iBestValue = iValue;
								pBestCity = pLoopCity;
							}
						}
					}
				}
			}
		}
	}
	else
	{
		// find the closest city
		CvCity* pClosestCity = GC.getMapINLINE().findCity(getX_INLINE(), getY_INLINE(), NO_PLAYER, getTeam(), true, bCoastalOnly);
		if (pClosestCity != NULL)
		{
			// if we can train, then return this city (otherwise it will return NULL)
			if (pClosestCity->canTrain(eUnit, false, false, true))
			{
				// did not search, always return 1 for search value
				iBestValue = 1;

				pBestCity = pClosestCity;
			}
		}
	}

	// return the best value, if non-NULL
	if (iSearchValue != NULL)
	{
		*iSearchValue = iBestValue;
	}

	return pBestCity;
}

void CvUnit::upgrade(UnitTypes eUnit)
{
	CvUnit* pUpgradeUnit;

	if (!canUpgrade(eUnit))
	{
		return;
	}

	GET_PLAYER(getOwnerINLINE()).changeGold(-(upgradePrice(eUnit)));

	pUpgradeUnit = GET_PLAYER(getOwnerINLINE()).initUnit(eUnit, getX_INLINE(), getY_INLINE(), AI_getUnitAIType());

	FAssertMsg(pUpgradeUnit != NULL, "UpgradeUnit is not assigned a valid value");

	pUpgradeUnit->joinGroup(getGroup());

	pUpgradeUnit->convert(this);

	pUpgradeUnit->finishMoves();

	// Sanguo Mod Hero, start, added by poyuzhe 07.27.09
	// if (pUpgradeUnit->getLeaderUnitType() == NO_UNIT)
	/*{
		if (pUpgradeUnit->getExperience() > GC.getDefineINT("MAX_EXPERIENCE_AFTER_UPGRADE"))
		{
			pUpgradeUnit->setExperience(GC.getDefineINT("MAX_EXPERIENCE_AFTER_UPGRADE"));
		}
	}*/
	if (pUpgradeUnit->getExperience() > GC.getDefineINT("MAX_EXPERIENCE_AFTER_UPGRADE"))
	{
		int iKeepExperience = GC.getDefineINT("MAX_EXPERIENCE_AFTER_UPGRADE");
		for (int iI = 0; iI < GC.getNumTechInfos(); iI++)
		{
			if (GC.getTechInfo((TechTypes)iI).getAdditionalUpgradeRetainExperience() != 0)
			{
				if (GET_TEAM(getTeam()).isHasTech((TechTypes)iI))
				{
					iKeepExperience = (iKeepExperience == GC.getDefineINT("MAX_EXPERIENCE_AFTER_UPGRADE")) ? GC.getTechInfo((TechTypes)iI).getAdditionalUpgradeRetainExperience() : (iKeepExperience + GC.getTechInfo((TechTypes)iI).getAdditionalUpgradeRetainExperience());
				}
			}
		}
		if (pUpgradeUnit->getExperience() > iKeepExperience)
		{
			pUpgradeUnit->setExperience(iKeepExperience);
		}
	}
	// Sanguo Mod Hero, end
}


HandicapTypes CvUnit::getHandicapType() const
{
	return GET_PLAYER(getOwnerINLINE()).getHandicapType();
}


CivilizationTypes CvUnit::getCivilizationType() const
{
	return GET_PLAYER(getOwnerINLINE()).getCivilizationType();
}

const wchar* CvUnit::getVisualCivAdjective(TeamTypes eForTeam) const
{
	if (getVisualOwner(eForTeam) == getOwnerINLINE())
	{
		return GC.getCivilizationInfo(getCivilizationType()).getAdjectiveKey();
	}

	return L"";
}

SpecialUnitTypes CvUnit::getSpecialUnitType() const
{
	return ((SpecialUnitTypes)(m_pUnitInfo->getSpecialUnitType()));
}


UnitTypes CvUnit::getCaptureUnitType(CivilizationTypes eCivilization) const
{
	FAssert(eCivilization != NO_CIVILIZATION);
	return ((m_pUnitInfo->getUnitCaptureClassType() == NO_UNITCLASS) ? NO_UNIT : (UnitTypes)GC.getCivilizationInfo(eCivilization).getCivilizationUnits(m_pUnitInfo->getUnitCaptureClassType()));
}


UnitCombatTypes CvUnit::getUnitCombatType() const
{
	return ((UnitCombatTypes)(m_pUnitInfo->getUnitCombatType()));
}


DomainTypes CvUnit::getDomainType() const
{
	return ((DomainTypes)(m_pUnitInfo->getDomainType()));
}


InvisibleTypes CvUnit::getInvisibleType() const
{
	// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
	// the following line is commented out
//	return ((InvisibleTypes)(m_pUnitInfo->getInvisibleType()));
    return m_iInvisibleType;
	// Sanguo Mod Hero, end
}

int CvUnit::getNumSeeInvisibleTypes() const
{
	// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
	// the following line is commented out
	// return m_pUnitInfo->getNumSeeInvisibleTypes();
	if (getHeroType() != NO_HERO)
	{
		return GC.getNumInvisibleInfos();
	}
	return 0;
	// Sanguo Mod Hero, end
}

InvisibleTypes CvUnit::getSeeInvisibleType(int i) const
{
	// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
	// the following line is commented out
	// return (InvisibleTypes)(m_pUnitInfo->getSeeInvisibleType(i));
	if (getHeroType() != NO_HERO)
	{
		return (GC.getGameINLINE().getHeroSeeInvisibleCount(getHeroType(), (InvisibleTypes)i) > 0) ? (InvisibleTypes)i : NO_INVISIBLE;
	}
	return NO_INVISIBLE;
	// Sanguo Mod Hero, end
}


int CvUnit::flavorValue(FlavorTypes eFlavor) const
{
	return m_pUnitInfo->getFlavorValue(eFlavor);
}


bool CvUnit::isBarbarian() const
{
	return GET_PLAYER(getOwnerINLINE()).isBarbarian();
}


bool CvUnit::isHuman() const
{
	return GET_PLAYER(getOwnerINLINE()).isHuman();
}


int CvUnit::visibilityRange() const
{
	return (GC.getDefineINT("UNIT_VISIBILITY_RANGE") + getExtraVisibilityRange());
}


int CvUnit::baseMoves() const
{
	return (m_pUnitInfo->getMoves() + getExtraMoves() + GET_TEAM(getTeam()).getExtraMoves(getDomainType()));
}


int CvUnit::maxMoves() const
{
	return (baseMoves() * GC.getMOVE_DENOMINATOR());
}


int CvUnit::movesLeft() const
{
	return std::max(0, (maxMoves() - getMoves()));
}


bool CvUnit::canMove() const
{
	if (isDead())
	{
		return false;
	}

	if (getMoves() >= maxMoves())
	{
		return false;
	}

	if (getImmobileTimer() > 0)
	{
		return false;
	}

	return true;
}


bool CvUnit::hasMoved()	const
{
	return (getMoves() > 0);
}


int CvUnit::airRange() const
{
	return (m_pUnitInfo->getAirRange() + getExtraAirRange());
}


int CvUnit::nukeRange() const
{
	return m_pUnitInfo->getNukeRange();
}


// XXX should this test for coal?
bool CvUnit::canBuildRoute() const
{
	int iI;

	for (iI = 0; iI < GC.getNumBuildInfos(); iI++)
	{
		if (GC.getBuildInfo((BuildTypes)iI).getRoute() != NO_ROUTE)
		{
			if (m_pUnitInfo->getBuilds(iI))
			{
				if (GET_TEAM(getTeam()).isHasTech((TechTypes)(GC.getBuildInfo((BuildTypes)iI).getTechPrereq())))
				{
					return true;
				}
			}
		}
	}

	return false;
}

BuildTypes CvUnit::getBuildType() const
{
	BuildTypes eBuild;

	if (getGroup()->headMissionQueueNode() != NULL)
	{
		switch (getGroup()->headMissionQueueNode()->m_data.eMissionType)
		{
		case MISSION_MOVE_TO:
			break;

		case MISSION_ROUTE_TO:
			if (getGroup()->getBestBuildRoute(plot(), &eBuild) != NO_ROUTE)
			{
				return eBuild;
			}
			break;

		case MISSION_MOVE_TO_UNIT:
		case MISSION_SKIP:
		case MISSION_SLEEP:
		case MISSION_FORTIFY:
		case MISSION_PLUNDER:
		case MISSION_AIRPATROL:
		case MISSION_SEAPATROL:
		case MISSION_HEAL:
		case MISSION_SENTRY:
		case MISSION_AIRLIFT:
		case MISSION_NUKE:
		case MISSION_RECON:
		case MISSION_PARADROP:
		case MISSION_AIRBOMB:
		case MISSION_BOMBARD:
		case MISSION_RANGE_ATTACK:
		case MISSION_PILLAGE:
		case MISSION_SABOTAGE:
		case MISSION_DESTROY:
		case MISSION_STEAL_PLANS:
		case MISSION_FOUND:
		case MISSION_SPREAD:
		case MISSION_SPREAD_CORPORATION:
		case MISSION_JOIN:
		case MISSION_CONSTRUCT:
		case MISSION_DISCOVER:
		case MISSION_HURRY:
		case MISSION_TRADE:
		case MISSION_GREAT_WORK:
		case MISSION_INFILTRATE:
		case MISSION_GOLDEN_AGE:
		case MISSION_LEAD:
		case MISSION_ESPIONAGE:
		case MISSION_DIE_ANIMATION:
		// Sanguo Mod Hero, start, added by poyuzhe 01.07.09
		case MISSION_FORM_LEGION:
		case MISSION_JOIN_LEGION:
		case MISSION_REGROUP_LEGION:
        case MISSION_DISMISS_LEGION:
        case MISSION_QUIT_LEGION:
		case MISSION_RBOMBARD:
		case MISSION_ABOMBARD:
		case MISSION_COMBINE_CAMP:
		case MISSION_NINE_TRANSFORMATION:
        case MISSION_FORM_ADVANCED_LEGION:
        case MISSION_JOIN_ADVANCED_LEGION:
        case MISSION_REGROUP_ADVANCED_LEGION:
        case MISSION_DISMISS_ADVANCED_LEGION:
        case MISSION_QUIT_ADVANCED_LEGION:
        // Sanguo Mod Hero, end
			break;

		case MISSION_BUILD:
			return (BuildTypes)getGroup()->headMissionQueueNode()->m_data.iData1;
			break;

		default:
			FAssert(false);
			break;
		}
	}

	return NO_BUILD;
}


int CvUnit::workRate(bool bMax) const
{
	int iRate;

	if (!bMax)
	{
		if (!canMove())
		{
			return 0;
		}
	}

	iRate = m_pUnitInfo->getWorkRate();

	iRate *= std::max(0, (GET_PLAYER(getOwnerINLINE()).getWorkerSpeedModifier() + 100));
	iRate /= 100;

	if (!isHuman() && !isBarbarian())
	{
		iRate *= std::max(0, (GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIWorkRateModifier() + 100));
		iRate /= 100;
	}

	return iRate;
}


bool CvUnit::isAnimal() const
{
	return m_pUnitInfo->isAnimal();
}


bool CvUnit::isNoBadGoodies() const
{
	return m_pUnitInfo->isNoBadGoodies();
}


bool CvUnit::isOnlyDefensive() const
{
	return m_pUnitInfo->isOnlyDefensive();
}


bool CvUnit::isNoCapture() const
{
	return m_pUnitInfo->isNoCapture();
}


bool CvUnit::isRivalTerritory() const
{
	return m_pUnitInfo->isRivalTerritory();
}


bool CvUnit::isMilitaryHappiness() const
{
	return m_pUnitInfo->isMilitaryHappiness();
}


bool CvUnit::isInvestigate() const
{
	return m_pUnitInfo->isInvestigate();
}


bool CvUnit::isCounterSpy() const
{
	return m_pUnitInfo->isCounterSpy();
}


bool CvUnit::isSpy() const
{
	return m_pUnitInfo->isSpy();
}


bool CvUnit::isFound() const
{
	return m_pUnitInfo->isFound();
}


bool CvUnit::isGoldenAge() const
{
	if (isDelayedDeath())
	{
		return false;
	}

	return m_pUnitInfo->isGoldenAge();
}

bool CvUnit::canCoexistWithEnemyUnit(TeamTypes eTeam) const
{
	if (NO_TEAM == eTeam)
	{
		if(alwaysInvisible())
		{
			return true;
		}

		return false;
	}

	if(isInvisible(eTeam, false))
	{
		return true;
	}

	return false;
}

bool CvUnit::isFighting() const
{
	return (getCombatUnit() != NULL);
}


bool CvUnit::isAttacking() const
{
	return (getAttackPlot() != NULL && !isDelayedDeath());
}


bool CvUnit::isDefending() const
{
	return (isFighting() && !isAttacking());
}


bool CvUnit::isCombat() const
{
	return (isFighting() || isAttacking());
}


int CvUnit::maxHitPoints() const
{
	return GC.getMAX_HIT_POINTS();
}


int CvUnit::currHitPoints()	const
{
	return (maxHitPoints() - getDamage());
}


bool CvUnit::isHurt() const
{
	return (getDamage() > 0);
}


bool CvUnit::isDead() const
{
	return (getDamage() >= maxHitPoints());
}


void CvUnit::setBaseCombatStr(int iCombat)
{
	m_iBaseCombat = iCombat;
}

int CvUnit::baseCombatStr() const
{
	return m_iBaseCombat;
}


// maxCombatStr can be called in four different configurations
//		pPlot == NULL, pAttacker == NULL for combat when this is the attacker
//		pPlot valid, pAttacker valid for combat when this is the defender
//		pPlot valid, pAttacker == NULL (new case), when this is the defender, attacker unknown
//		pPlot valid, pAttacker == this (new case), when the defender is unknown, but we want to calc approx str
//			note, in this last case, it is expected pCombatDetails == NULL, it does not have to be, but some
//			values may be unexpectedly reversed in this case (iModifierTotal will be the negative sum)
int CvUnit::maxCombatStr(const CvPlot* pPlot, const CvUnit* pAttacker, CombatDetails* pCombatDetails) const
{
	PROFILE_FUNC();

	int iCombat;

	FAssertMsg((pPlot == NULL) || (pPlot->getTerrainType() != NO_TERRAIN), "(pPlot == NULL) || (pPlot->getTerrainType() is not expected to be equal with NO_TERRAIN)");

	// handle our new special case
	const	CvPlot*	pAttackedPlot = NULL;
	bool	bAttackingUnknownDefender = false;
	if (pAttacker == this)
	{
		bAttackingUnknownDefender = true;
		pAttackedPlot = pPlot;

		// reset these values, we will fiddle with them below
		pPlot = NULL;
		pAttacker = NULL;
	}
	// otherwise, attack plot is the plot of us (the defender)
	else if (pAttacker != NULL)
	{
		pAttackedPlot = plot();
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iExtraCombatPercent = 0;
		pCombatDetails->iAnimalCombatModifierTA = 0;
		pCombatDetails->iAIAnimalCombatModifierTA = 0;
		pCombatDetails->iAnimalCombatModifierAA = 0;
		pCombatDetails->iAIAnimalCombatModifierAA = 0;
		pCombatDetails->iBarbarianCombatModifierTB = 0;
		pCombatDetails->iAIBarbarianCombatModifierTB = 0;
		pCombatDetails->iBarbarianCombatModifierAB = 0;
		pCombatDetails->iAIBarbarianCombatModifierAB = 0;
		pCombatDetails->iPlotDefenseModifier = 0;
		pCombatDetails->iFortifyModifier = 0;
		pCombatDetails->iCityDefenseModifier = 0;
		pCombatDetails->iHillsAttackModifier = 0;
		pCombatDetails->iHillsDefenseModifier = 0;
		pCombatDetails->iFeatureAttackModifier = 0;
		pCombatDetails->iFeatureDefenseModifier = 0;
		pCombatDetails->iTerrainAttackModifier = 0;
		pCombatDetails->iTerrainDefenseModifier = 0;
		pCombatDetails->iCityAttackModifier = 0;
		pCombatDetails->iDomainDefenseModifier = 0;
		pCombatDetails->iCityBarbarianDefenseModifier = 0;
		pCombatDetails->iClassDefenseModifier = 0;
		pCombatDetails->iClassAttackModifier = 0;
		pCombatDetails->iCombatModifierA = 0;
		pCombatDetails->iCombatModifierT = 0;
		pCombatDetails->iDomainModifierA = 0;
		pCombatDetails->iDomainModifierT = 0;
		pCombatDetails->iAnimalCombatModifierA = 0;
		pCombatDetails->iAnimalCombatModifierT = 0;
		pCombatDetails->iRiverAttackModifier = 0;
		pCombatDetails->iAmphibAttackModifier = 0;
		pCombatDetails->iKamikazeModifier = 0;
		pCombatDetails->iModifierTotal = 0;
		pCombatDetails->iBaseCombatStr = 0;
		pCombatDetails->iCombat = 0;
		pCombatDetails->iMaxCombatStr = 0;
		pCombatDetails->iCurrHitPoints = 0;
		pCombatDetails->iMaxHitPoints = 0;
		pCombatDetails->iCurrCombatStr = 0;
		pCombatDetails->eOwner = getOwnerINLINE();
		pCombatDetails->eVisualOwner = getVisualOwner();
		pCombatDetails->sUnitName = getName().GetCString();
	}

	if (baseCombatStr() == 0)
	{
		return 0;
	}

	int iModifier = 0;
	int iExtraModifier;

	iExtraModifier = getExtraCombatPercent();
	iModifier += iExtraModifier;
	if (pCombatDetails != NULL)
	{
		pCombatDetails->iExtraCombatPercent = iExtraModifier;
	}

    // Sanguo Mod Hero, Might strength bonus, start, by poyuzhe 09.15.08
    iExtraModifier = getMightStrengthModifier();
    iModifier += iExtraModifier;
    // Sanguo Mod Hero, end

	// Sanguo Mod Civilization Trait, start, by poyuzhe 11.23.08
    // Lu Bu, TRAIT_TOUGH
    if (GET_PLAYER(getOwnerINLINE()).isTough())
    {
        if (NULL != pPlot && bAttackingUnknownDefender == false)
        {
            iExtraModifier = GC.getDefineINT("TOUGH_DEFENSE_STRENGTH_MODIFIER");
            iModifier += iExtraModifier;
        }
        else if (NULL == pPlot && NULL == pAttacker)
        {
            iExtraModifier = GC.getDefineINT("TOUGH_ATTACK_STRENGTH_MODIFIER");
            iModifier += iExtraModifier;
        }
    }
    // Sanguo Mod Civilization Trait, end

    // Sanguo Mod Hero, start, added by poyuzhe 01.24.09
    if (m_iTempStrengthModifier > 0)
    {
        iModifier += m_iTempStrengthModifier;
    }

	int iCombatOdds = -1;
    // Sanguo Mod Hero, end

	// do modifiers for animals and barbarians (leaving these out for bAttackingUnknownDefender case)
	if (pAttacker != NULL)
	{
		if (isAnimal())
		{
			if (pAttacker->isHuman())
			{
				iExtraModifier = GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierTA = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIAnimalCombatModifierTA = iExtraModifier;
				}
			}
		}

		if (pAttacker->isAnimal())
		{
			if (isHuman())
			{
				iExtraModifier = -GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierAA = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = -GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIAnimalCombatModifierAA = iExtraModifier;
				}
			}
		}

		if (isBarbarian())
		{
			if (pAttacker->isHuman())
			{
				iExtraModifier = GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			// Sanguo Mod Hero, start, added by poyuzhe 02.03.09
			if (pAttacker->getLegionHero() != NO_HERO)
			{
			    PROFILE_BEGIN("MaxCombatStr::BarbarianModifier");
				int iBarbarianModifier = 0;
			    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
			    {
			        if (GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier() != 0)
			        {
			            if (pAttacker->isHasPromotion((PromotionTypes)iI))
			            {
			                iBarbarianModifier += GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier();
			            }
			        }
			    }

			    if (iBarbarianModifier != 0)
			    {
			        iExtraModifier = -iBarbarianModifier;
			        iModifier += iExtraModifier;
			        if (pAttacker->isHuman())
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iBarbarianCombatModifierTB += iExtraModifier;
			            }
			        }
			        else
			        {
			            if (pCombatDetails != NULL)
			            {
                            pCombatDetails->iAIBarbarianCombatModifierTB += iExtraModifier;
			            }
			        }
			    }

				PROFILE_END();
			}
            // Sanguo Mod Hero, end
		}

		if (pAttacker->isBarbarian())
		{
			if (isHuman())
			{
				iExtraModifier = -GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iBarbarianCombatModifierAB = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = -GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			// Sanguo Mod Hero, start, added by poyuzhe 02.03.09
			if (getLegionHero() != NO_HERO)
			{
			    int iBarbarianModifier = 0;
			    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
			    {
			        if (GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier() != 0)
			        {
			            if (isHasPromotion((PromotionTypes)iI))
			            {
			                iBarbarianModifier += GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier();
			            }
			        }
			    }

			    if (iBarbarianModifier != 0)
			    {
			        iExtraModifier = iBarbarianModifier;
			        iModifier += iExtraModifier;
			        if (isHuman())
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iBarbarianCombatModifierAB += iExtraModifier;
			            }
			        }
			        else
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iAIBarbarianCombatModifierAB += iExtraModifier;
			            }
			        }
			    }
			}
            // Sanguo Mod Hero, end
		}

		// Sanguo Mod Hero, start, added by poyuzhe 02.03.09
		if (getLegionHero() != NO_HERO || getLeaderUnitType() != NO_UNIT)
		{
		    if (pAttacker->getHeroType() != NO_HERO)
		    {
				PROFILE_BEGIN("maxCombatStr::legionStrMod");
		        int iLegionStrModifier = 0;
		        for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		        {
		            if (GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier() != 0)
		            {
		                if (pAttacker->isHasPromotion((PromotionTypes)iI))
		                {
		                    iLegionStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier();
		                }
		            }
		        }

		        if (iLegionStrModifier != 0)
		        {
		            iExtraModifier = -iLegionStrModifier;
		            iModifier += iExtraModifier;
		        }

				PROFILE_END();
		    }
		}

		if (pAttacker->getLegionHero() != NO_HERO || pAttacker->getLeaderUnitType() != NO_UNIT)
		{
		    if (getHeroType() != NO_HERO)
		    {
		        int iLegionStrModifier = 0;
		        for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		        {
		            if (GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier() != 0)
		            {
		                if (isHasPromotion((PromotionTypes)iI))
		                {
		                    iLegionStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier();
		                }
		            }
		        }

		        if (iLegionStrModifier != 0)
		        {
		            iExtraModifier = iLegionStrModifier;
		            iModifier += iExtraModifier;
		        }
		    }
		}

		if (pAttacker->getLegionHero() != NO_HERO)
		{
		    if (GET_PLAYER(getOwnerINLINE()).getStateReligion() == NO_RELIGION || !GC.getReligionInfo((ReligionTypes)GET_PLAYER(getOwnerINLINE()).getStateReligion()).isGoodReligion())
            {
                PROFILE_BEGIN("maxCombatStr::religionStrMod");
				int iNonGoodReligionStrengthModifier = 0;
                for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                {
                    if (pAttacker->isHasPromotion((PromotionTypes)iI))
                    {
                        iNonGoodReligionStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getNonGoodReligionStrengthModifier();
                    }
                }
                if (iNonGoodReligionStrengthModifier > 0)
                {
                    iExtraModifier = -iNonGoodReligionStrengthModifier;
		            iModifier += iExtraModifier;
                }
				PROFILE_END();
            }
		}

		if (getLegionHero() != NO_HERO)
		{
		    if (GET_PLAYER(pAttacker->getOwnerINLINE()).getStateReligion() == NO_RELIGION || !GC.getReligionInfo((ReligionTypes)GET_PLAYER(pAttacker->getOwnerINLINE()).getStateReligion()).isGoodReligion())
            {
                int iNonGoodReligionStrengthModifier = 0;
                for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                {
                    if (isHasPromotion((PromotionTypes)iI))
                    {
                        iNonGoodReligionStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getNonGoodReligionStrengthModifier();
                    }
                }
                if (iNonGoodReligionStrengthModifier > 0)
                {
                    iExtraModifier = iNonGoodReligionStrengthModifier;
		            iModifier += iExtraModifier;
                }
            }
		}

        if (isWarGod())
        {
            if (iCombatOdds == -1)
			{
				iCombatOdds = getNoStrChangeCombatOdds(pAttacker, this);
			}
			iExtraModifier = iCombatOdds / 10;
            iModifier += iExtraModifier;
        }
		if (pAttacker->isWarGod())
        {
            if (iCombatOdds == -1)
			{
				iCombatOdds = getNoStrChangeCombatOdds(pAttacker, this);
			}
            iExtraModifier = -(1000 - iCombatOdds) / 10;
            iModifier += iExtraModifier;
        }

		if (pAttacker->isCoreTroop())
		{
            if (isGuard())
            {
                iExtraModifier = GC.getDefineINT("HERO_GUARDSMAN_CORE_TROOP_MODIFIER");
                iModifier += iExtraModifier;
            }
		}

		// Sanguo Mod Hero, start, added by poyuzhe 07.18.09
		if (pAttacker->getLegionHero() != NO_HERO)
		{
		    if (pAttacker->withdrawalProbability() > 0)
			{
				PROFILE_BEGIN("maxCombatStr::withdrawalChanceToStrength");
				bool bWithdrawalChanceToStrength = false;
				for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
				{
					if (pAttacker->isHasPromotion((PromotionTypes)iI))
					{
						if (GC.getPromotionInfo((PromotionTypes)iI).isNoEnemyWithdrawal())
						{
							bWithdrawalChanceToStrength = true;
						}
					}
				}
				if (bWithdrawalChanceToStrength)
				{
					if (iCombatOdds == -1)
					{
						iCombatOdds = getNoStrChangeCombatOdds(pAttacker, this);
					}
					iExtraModifier = -std::min(100, pAttacker->withdrawalProbability()) * (1000 - iCombatOdds) / 1000;
					iModifier += iExtraModifier;
				}
				PROFILE_END();
			}
		}
        // Sanguo Mod Hero, end
	}

	// add defensive bonuses (leaving these out for bAttackingUnknownDefender case)
	if (pPlot != NULL)
	{
		// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
//		if (!noDefensiveBonus())
		if (!noDefensiveBonus() && (pAttacker == NULL || !(!pPlot->isCity(true) && pAttacker->isIgnoreDefenderTerrain())))
		// Sanguo Mod Hero, end
		{
			iExtraModifier = pPlot->defenseModifier(getTeam(), (pAttacker != NULL) ? pAttacker->ignoreBuildingDefense() : true);
			// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
			if (getPlotDefenseModifier() != 0)
			{
			    iExtraModifier *= (100 + getPlotDefenseModifier());
			    iExtraModifier /= 100;
			}
            // Sanguo Mod Hero, end
			iModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iPlotDefenseModifier = iExtraModifier;
			}
		}

		iExtraModifier = fortifyModifier();
		iModifier += iExtraModifier;
		if (pCombatDetails != NULL)
		{
			pCombatDetails->iFortifyModifier = iExtraModifier;
		}

		if (pPlot->isCity(true, getTeam()))
		{
			iExtraModifier = cityDefenseModifier();
			iModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iCityDefenseModifier = iExtraModifier;
			}
		}

        // Sanguo Mod Hero, start, added by poyuzhe 02.01.09
        if (pAttacker == NULL || !(!pPlot->isCity(true) && pAttacker->isIgnoreDefenderTerrain()))
        {
            // The following lines indented once
            if (pPlot->isHills())
            {
                iExtraModifier = hillsDefenseModifier();
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iHillsDefenseModifier = iExtraModifier;
                }
            }

            if (pPlot->getFeatureType() != NO_FEATURE)
            {
                iExtraModifier = featureDefenseModifier(pPlot->getFeatureType());
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iFeatureDefenseModifier = iExtraModifier;
                }
            }
            else
            {
                iExtraModifier = terrainDefenseModifier(pPlot->getTerrainType());
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iTerrainDefenseModifier = iExtraModifier;
                }
            }
        }
		// Sanguo Mod Hero, end
	}

	// if we are attacking to an plot with an unknown defender, the calc the modifier in reverse
	if (bAttackingUnknownDefender)
	{
		pAttacker = this;
	}

	// calc attacker bonueses
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                       09/20/09                                jdog5000      */
/**                                                                                              */
/** Bugfix                                                                                       */
/*************************************************************************************************/
/* original code
	if (pAttacker != NULL)
*/
	if (pAttacker != NULL && pAttackedPlot != NULL)
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                        END                                                  */
/*************************************************************************************************/
	{
		int iTempModifier = 0;

		if (pAttackedPlot->isCity(true, getTeam()))
		{
			iExtraModifier = -pAttacker->cityAttackModifier();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iCityAttackModifier = iExtraModifier;
			}

			if (pAttacker->isBarbarian())
			{
				iExtraModifier = GC.getDefineINT("CITY_BARBARIAN_DEFENSE_MODIFIER");
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCityBarbarianDefenseModifier = iExtraModifier;
				}
			}
		}

		if (pAttackedPlot->isHills())
		{
			iExtraModifier = -pAttacker->hillsAttackModifier();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iHillsAttackModifier = iExtraModifier;
			}
		}

		if (pAttackedPlot->getFeatureType() != NO_FEATURE)
		{
			iExtraModifier = -pAttacker->featureAttackModifier(pAttackedPlot->getFeatureType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iFeatureAttackModifier = iExtraModifier;
			}
		}
		else
		{
			iExtraModifier = -pAttacker->terrainAttackModifier(pAttackedPlot->getTerrainType());
			// Sanguo Mod Hero, start, added by poyuzhe 02.01.09
			// BTS Bug Fix, commented out the following line
//			iModifier += iExtraModifier;
            iTempModifier += iExtraModifier;
			// Sanguo Mod Hero, end
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iTerrainAttackModifier = iExtraModifier;
			}
		}

		// Sanguo Mod Hero, start, added by poyuzhe 07.18.09
		if (pAttacker->getLegionHero() != NO_HERO)
		{
		    if (pAttacker->withdrawalProbability() + pAttacker->getExtraWithdrawal() > 0)
			{
				PROFILE_BEGIN("maxCombatStr::withdrawalChanceToStrength");
				bool bWithdrawalChanceToStrength = false;
				for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
				{
					if (pAttacker->isHasPromotion((PromotionTypes)iI))
					{
						if (GC.getPromotionInfo((PromotionTypes)iI).isNoEnemyWithdrawal())
						{
							bWithdrawalChanceToStrength = true;
						}
					}
				}
				if (bWithdrawalChanceToStrength)
				{
					iExtraModifier = -(pAttacker->withdrawalProbability() + pAttacker->getExtraWithdrawal());
					iTempModifier += iExtraModifier;
				}
				PROFILE_END();
			}
		}
        // Sanguo Mod Hero, end

		// only compute comparisions if we are the defender with a known attacker
		if (!bAttackingUnknownDefender)
		{
			FAssertMsg(pAttacker != this, "pAttacker is not expected to be equal with this");

			iExtraModifier = unitClassDefenseModifier(pAttacker->getUnitClassType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iClassDefenseModifier = iExtraModifier;
			}

			iExtraModifier = -pAttacker->unitClassAttackModifier(getUnitClassType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iClassAttackModifier = iExtraModifier;
			}

			if (pAttacker->getUnitCombatType() != NO_UNITCOMBAT)
			{
				iExtraModifier = unitCombatModifier(pAttacker->getUnitCombatType());
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCombatModifierA = iExtraModifier;
				}
			}
			if (getUnitCombatType() != NO_UNITCOMBAT)
			{
				iExtraModifier = -pAttacker->unitCombatModifier(getUnitCombatType());
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCombatModifierT = iExtraModifier;
				}
			}

			iExtraModifier = domainModifier(pAttacker->getDomainType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iDomainModifierA = iExtraModifier;
			}

			iExtraModifier = -pAttacker->domainModifier(getDomainType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iDomainModifierT = iExtraModifier;
			}

			if (pAttacker->isAnimal())
			{
				iExtraModifier = animalCombatModifier();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierA = iExtraModifier;
				}
			}

			if (isAnimal())
			{
				iExtraModifier = -pAttacker->animalCombatModifier();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierT = iExtraModifier;
				}
			}
		}

		if (!(pAttacker->isRiver()))
		{
			if (pAttacker->plot()->isRiverCrossing(directionXY(pAttacker->plot(), pAttackedPlot)))
			{
				iExtraModifier = -GC.getRIVER_ATTACK_MODIFIER();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iRiverAttackModifier = iExtraModifier;
				}
			}
		}

		if (!(pAttacker->isAmphib()))
		{
			if (!(pAttackedPlot->isWater()) && pAttacker->plot()->isWater())
			{
				iExtraModifier = -GC.getAMPHIB_ATTACK_MODIFIER();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAmphibAttackModifier = iExtraModifier;
				}
			}
		}

		if (pAttacker->getKamikazePercent() != 0)
		{
			iExtraModifier = pAttacker->getKamikazePercent();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iKamikazeModifier = iExtraModifier;
			}
		}

		// Sanguo Mod Hero, start, added by poyuzhe 02.03.09
		if (pAttackedPlot->isRiver() || pAttackedPlot->isCoastalLand(0))
		{
		    if (pAttacker != this)
		    {
	            if (getWaterSideStrengthModifier() != 0)
	            {
	                iExtraModifier = getWaterSideStrengthModifier();
	                iTempModifier += iExtraModifier;
	            }
		    }

			if (pAttacker->getWaterSideStrengthModifier() != 0)
            {
                iExtraModifier = -pAttacker->getWaterSideStrengthModifier();
                iTempModifier += iExtraModifier;
            }
        }

        if (pAttacker->movesLeft() > GC.getMOVE_DENOMINATOR())
        {
            int iPostAttackMove = pAttacker->movesLeft() - std::max(GC.getMOVE_DENOMINATOR(), pAttackedPlot->movementCost(pAttacker, pAttacker->plot()));
            iPostAttackMove /= GC.getMOVE_DENOMINATOR();
            if (iPostAttackMove > 0)
            {
                if (pAttacker->getLegionHero() != NO_HERO)
                {
                    PROFILE_BEGIN("maxCombatStr::moveAttackMod");
					int iMoveAttackStrModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getMoveAttackStrengthModifier() != 0)
                        {
                            if (pAttacker->isHasPromotion((PromotionTypes)iI))
                            {
                                iMoveAttackStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getMoveAttackStrengthModifier();
                            }
                        }
                    }

                    if (iMoveAttackStrModifier != 0)
                    {
                        iExtraModifier = -(iMoveAttackStrModifier * iPostAttackMove);
                        iTempModifier += iExtraModifier;
                    }
					PROFILE_END();
                }
            }
        }
        // Sanguo Mod Hero, end

		// if we are attacking an unknown defender, then use the reverse of the modifier
		if (bAttackingUnknownDefender)
		{
			iModifier -= iTempModifier;
		}
		else
		{
			iModifier += iTempModifier;
		}
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iModifierTotal = iModifier;
		pCombatDetails->iBaseCombatStr = baseCombatStr();
	}

	if (iModifier > 0)
	{
		iCombat = (baseCombatStr() * (iModifier + 100));
	}
	else
	{
		iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iCombat = iCombat;
		pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
		pCombatDetails->iCurrHitPoints = currHitPoints();
		pCombatDetails->iMaxHitPoints = maxHitPoints();
		pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
	}

	// Sanguo Mod Hero, start, added by poyuzhe 02.03.09
	if (pAttacker != NULL && pAttacker != this)
	{
	    // if this is the defender with a known attacker
	    if (iCombatOdds == -1 && getLegionHero() != NO_HERO || pAttacker->getLegionHero() != NO_HERO)
	    {
	        int iCurrCombatStr = std::max(1, iCombat) * currHitPoints() / maxHitPoints();
	        if (iCurrCombatStr < pAttacker->currNoStrChangeCombatStr(NULL, NULL))
	        {
	            if (getLegionHero() != NO_HERO)
	            {
					PROFILE_BEGIN("maxCombatStr::betterSTrengthMod");                   
					int iBetterStrengthEnemyModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier() != 0)
                        {
                            if (isHasPromotion((PromotionTypes)iI))
                            {
                                iBetterStrengthEnemyModifier += GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier();
                            }
                        }
                    }

                    if (iBetterStrengthEnemyModifier != 0)
                    {
                        iModifier += iBetterStrengthEnemyModifier;

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iModifierTotal = iModifier;
                            pCombatDetails->iBaseCombatStr = baseCombatStr();
                        }

                        if (iModifier > 0)
                        {
                            iCombat = (baseCombatStr() * (iModifier + 100));
                        }
                        else
                        {
                            iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
                        }

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iCombat = iCombat;
                            pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
                            pCombatDetails->iCurrHitPoints = currHitPoints();
                            pCombatDetails->iMaxHitPoints = maxHitPoints();
                            pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
                        }
	                }
					PROFILE_END();
	            }
	        }
	        else if (iCurrCombatStr > pAttacker->currNoStrChangeCombatStr(NULL, NULL))
	        {
	            if (pAttacker->getLegionHero() != NO_HERO)
	            {
                    int iBetterStrengthEnemyModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier() != 0)
                        {
                            if (pAttacker->isHasPromotion((PromotionTypes)iI))
                            {
                                iBetterStrengthEnemyModifier += GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier();
                            }
                        }
                    }

                    if (iBetterStrengthEnemyModifier != 0)
                    {
                        iModifier += -iBetterStrengthEnemyModifier;

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iModifierTotal = iModifier;
                            pCombatDetails->iBaseCombatStr = baseCombatStr();
                        }

                        if (iModifier > 0)
                        {
                            iCombat = (baseCombatStr() * (iModifier + 100));
                        }
                        else
                        {
                            iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
                        }

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iCombat = iCombat;
                            pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
                            pCombatDetails->iCurrHitPoints = currHitPoints();
                            pCombatDetails->iMaxHitPoints = maxHitPoints();
                            pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
                        }
	                }
	            }
	        }
	    }
	}
    // Sanguo Mod Hero, end

	return std::max(1, iCombat);
}


int CvUnit::currCombatStr(const CvPlot* pPlot, const CvUnit* pAttacker, CombatDetails* pCombatDetails) const
{
	return ((maxCombatStr(pPlot, pAttacker, pCombatDetails) * currHitPoints()) / maxHitPoints());
}


int CvUnit::currFirepower(const CvPlot* pPlot, const CvUnit* pAttacker) const
{
	return ((maxCombatStr(pPlot, pAttacker) + currCombatStr(pPlot, pAttacker) + 1) / 2);
}

// this nomalizes str by firepower, useful for quick odds calcs
// the effect is that a damaged unit will have an effective str lowered by firepower/maxFirepower
// doing the algebra, this means we mulitply by 1/2(1 + currHP)/maxHP = (maxHP + currHP) / (2 * maxHP)
int CvUnit::currEffectiveStr(const CvPlot* pPlot, const CvUnit* pAttacker, CombatDetails* pCombatDetails) const
{
	int currStr = currCombatStr(pPlot, pAttacker, pCombatDetails);

	currStr *= (maxHitPoints() + currHitPoints());
	currStr /= (2 * maxHitPoints());

	return currStr;
}

float CvUnit::maxCombatStrFloat(const CvPlot* pPlot, const CvUnit* pAttacker) const
{
	return (((float)(maxCombatStr(pPlot, pAttacker))) / 100.0f);
}


float CvUnit::currCombatStrFloat(const CvPlot* pPlot, const CvUnit* pAttacker) const
{
	return (((float)(currCombatStr(pPlot, pAttacker))) / 100.0f);
}

bool CvUnit::canFight() const
{
	return (baseCombatStr() > 0);
}


bool CvUnit::canAttack() const
{
	if (!canFight())
	{
		return false;
	}

	if (isOnlyDefensive())
	{
		return false;
	}

	return true;
}
bool CvUnit::canAttack(const CvUnit& defender) const
{
	if (!canAttack())
	{
		return false;
	}

	if (defender.getDamage() >= combatLimit())
	{
		return false;
	}

	// Artillery can't amphibious attack
	if (plot()->isWater() && !defender.plot()->isWater())
	{
		if (combatLimit() < 100)
		{
			return false;
		}
	}

	return true;
}

bool CvUnit::canDefend(const CvPlot* pPlot) const
{
	if (pPlot == NULL)
	{
		pPlot = plot();
	}

	if (!canFight())
	{
		return false;
	}

	if (!pPlot->isValidDomainForAction(*this))
	{
		if (GC.getDefineINT("LAND_UNITS_CAN_ATTACK_WATER_CITIES") == 0)
		{
			return false;
		}
	}

	return true;
}


bool CvUnit::canSiege(TeamTypes eTeam) const
{
	if (!canDefend())
	{
		return false;
	}

	if (!isEnemy(eTeam))
	{
		return false;
	}

	if (!isNeverInvisible())
	{
		return false;
	}

	return true;
}


int CvUnit::airBaseCombatStr() const
{
	return m_pUnitInfo->getAirCombat();
}


int CvUnit::airMaxCombatStr(const CvUnit* pOther) const
{
	int iModifier;
	int iCombat;

	if (airBaseCombatStr() == 0)
	{
		return 0;
	}

	iModifier = getExtraCombatPercent();

	if (getKamikazePercent() != 0)
	{
		iModifier += getKamikazePercent();
	}

	/********************************************************************************/
	/**		BETTER_BTS_AI_MOD						8/16/08		DanF5771 & jdog5000	*/
	/**																				*/
	/**		Bugfix																	*/
	/********************************************************************************/
	/* original BTS code
	if (getExtraCombatPercent() != 0)
	{
		iModifier += getExtraCombatPercent();
	}
	*/
	// ExtraCombatPercent already counted above
	/********************************************************************************/
	/**		BETTER_BTS_AI_MOD						END								*/
	/********************************************************************************/

	if (NULL != pOther)
	{
		if (pOther->getUnitCombatType() != NO_UNITCOMBAT)
		{
			iModifier += unitCombatModifier(pOther->getUnitCombatType());
		}

		iModifier += domainModifier(pOther->getDomainType());

		if (pOther->isAnimal())
		{
			iModifier += animalCombatModifier();
		}
	}

	if (iModifier > 0)
	{
		iCombat = (airBaseCombatStr() * (iModifier + 100));
	}
	else
	{
		iCombat = ((airBaseCombatStr() * 10000) / (100 - iModifier));
	}

	return std::max(1, iCombat);
}


int CvUnit::airCurrCombatStr(const CvUnit* pOther) const
{
	return ((airMaxCombatStr(pOther) * currHitPoints()) / maxHitPoints());
}


float CvUnit::airMaxCombatStrFloat(const CvUnit* pOther) const
{
	return (((float)(airMaxCombatStr(pOther))) / 100.0f);
}


float CvUnit::airCurrCombatStrFloat(const CvUnit* pOther) const
{
	return (((float)(airCurrCombatStr(pOther))) / 100.0f);
}


int CvUnit::combatLimit() const
{
	return m_pUnitInfo->getCombatLimit();
}


int CvUnit::airCombatLimit() const
{
	return m_pUnitInfo->getAirCombatLimit();
}


bool CvUnit::canAirAttack() const
{
	return (airBaseCombatStr() > 0);
}


bool CvUnit::canAirDefend(const CvPlot* pPlot) const
{
	if (pPlot == NULL)
	{
		pPlot = plot();
	}

	if (maxInterceptionProbability() == 0)
	{
		return false;
	}

	if (getDomainType() != DOMAIN_AIR)
	{
/************************************************************************************************/
/* UNOFFICIAL_PATCH                       10/30/09                     Mongoose & jdog5000      */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
/* original bts code
		if (!pPlot->isValidDomainForLocation(*this))
*/
		// From Mongoose SDK
		// Land units which are cargo cannot intercept
		if (!pPlot->isValidDomainForLocation(*this) || isCargo())
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/
		{
			return false;
		}
	}

	return true;
}


int CvUnit::airCombatDamage(const CvUnit* pDefender) const
{
	CvCity* pCity;
	CvPlot* pPlot;
	int iOurStrength;
	int iTheirStrength;
	int iStrengthFactor;
	int iDamage;

	pPlot = pDefender->plot();

	iOurStrength = airCurrCombatStr(pDefender);
	FAssertMsg(iOurStrength > 0, "Air combat strength is expected to be greater than zero");
	iTheirStrength = pDefender->maxCombatStr(pPlot, this);

	iStrengthFactor = ((iOurStrength + iTheirStrength + 1) / 2);

	iDamage = std::max(1, ((GC.getDefineINT("AIR_COMBAT_DAMAGE") * (iOurStrength + iStrengthFactor)) / (iTheirStrength + iStrengthFactor)));

	pCity = pPlot->getPlotCity();

	if (pCity != NULL)
	{
		iDamage *= std::max(0, (pCity->getAirModifier() + 100));
		iDamage /= 100;
	}

	return iDamage;
}


int CvUnit::rangeCombatDamage(const CvUnit* pDefender) const
{
	CvPlot* pPlot;
	int iOurStrength;
	int iTheirStrength;
	int iStrengthFactor;
	int iDamage;

	pPlot = pDefender->plot();

	iOurStrength = airCurrCombatStr(pDefender);
	FAssertMsg(iOurStrength > 0, "Combat strength is expected to be greater than zero");
	iTheirStrength = pDefender->maxCombatStr(pPlot, this);

	iStrengthFactor = ((iOurStrength + iTheirStrength + 1) / 2);

	iDamage = std::max(1, ((GC.getDefineINT("RANGE_COMBAT_DAMAGE") * (iOurStrength + iStrengthFactor)) / (iTheirStrength + iStrengthFactor)));

	return iDamage;
}


CvUnit* CvUnit::bestInterceptor(const CvPlot* pPlot) const
{
	CvUnit* pLoopUnit;
	CvUnit* pBestUnit;
	int iValue;
	int iBestValue;
	int iLoop;
	int iI;

	iBestValue = 0;
	pBestUnit = NULL;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (isEnemy(GET_PLAYER((PlayerTypes)iI).getTeam()) && !isInvisible(GET_PLAYER((PlayerTypes)iI).getTeam(), false, false))
			{
				for(pLoopUnit = GET_PLAYER((PlayerTypes)iI).firstUnit(&iLoop); pLoopUnit != NULL; pLoopUnit = GET_PLAYER((PlayerTypes)iI).nextUnit(&iLoop))
				{
					if (pLoopUnit->canAirDefend())
					{
						if (!pLoopUnit->isMadeInterception())
						{
							if ((pLoopUnit->getDomainType() != DOMAIN_AIR) || !(pLoopUnit->hasMoved()))
							{
								if ((pLoopUnit->getDomainType() != DOMAIN_AIR) || (pLoopUnit->getGroup()->getActivityType() == ACTIVITY_INTERCEPT))
								{
									if (plotDistance(pLoopUnit->getX_INLINE(), pLoopUnit->getY_INLINE(), pPlot->getX_INLINE(), pPlot->getY_INLINE()) <= pLoopUnit->airRange())
									{
										iValue = pLoopUnit->currInterceptionProbability();

										if (iValue > iBestValue)
										{
											iBestValue = iValue;
											pBestUnit = pLoopUnit;
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	return pBestUnit;
}


CvUnit* CvUnit::bestSeaPillageInterceptor(CvUnit* pPillager, int iMinOdds) const
{
	CvUnit* pBestUnit = NULL;

	for (int iDX = -1; iDX <= 1; ++iDX)
	{
		for (int iDY = -1; iDY <= 1; ++iDY)
		{
			CvPlot* pLoopPlot = plotXY(pPillager->getX_INLINE(), pPillager->getY_INLINE(), iDX, iDY);

			if (NULL != pLoopPlot)
			{
				CLLNode<IDInfo>* pUnitNode = pLoopPlot->headUnitNode();

				while (NULL != pUnitNode)
				{
					CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
					pUnitNode = pLoopPlot->nextUnitNode(pUnitNode);

					if (NULL != pLoopUnit)
					{
						if (pLoopUnit->area() == pPillager->plot()->area())
						{
							if (!pLoopUnit->isInvisible(getTeam(), false))
							{
								if (isEnemy(pLoopUnit->getTeam()))
								{
									if (DOMAIN_SEA == pLoopUnit->getDomainType())
									{
										if (ACTIVITY_PATROL == pLoopUnit->getGroup()->getActivityType())
										{
											if (NULL == pBestUnit || pLoopUnit->isBetterDefenderThan(pBestUnit, this))
											{
												if (getCombatOdds(pPillager, pLoopUnit) < iMinOdds)
												{
													pBestUnit = pLoopUnit;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	return pBestUnit;
}


bool CvUnit::isAutomated() const
{
	return getGroup()->isAutomated();
}


bool CvUnit::isWaiting() const
{
	return getGroup()->isWaiting();
}


bool CvUnit::isFortifyable() const
{
	if (!canFight() || noDefensiveBonus() || ((getDomainType() != DOMAIN_LAND) && (getDomainType() != DOMAIN_IMMOBILE)))
	{
		return false;
	}

	return true;
}


int CvUnit::fortifyModifier() const
{
	if (!isFortifyable())
	{
		return 0;
	}

	return (getFortifyTurns() * GC.getFORTIFY_MODIFIER_PER_TURN());
}


int CvUnit::experienceNeeded() const
{
	// Use python to determine pillage amounts...
	int iExperienceNeeded;
	long lExperienceNeeded;

	lExperienceNeeded = 0;
	iExperienceNeeded = 0;

	CyArgsList argsList;
	argsList.add(getLevel());	// pass in the units level
	argsList.add(getOwnerINLINE());	// pass in the units
	// Sanguo Mod Hero, start, added by poyuzhe 01.17.09
	argsList.add(false);
    // Sanguo Mod Hero, end

	gDLL->getPythonIFace()->callFunction(PYGameModule, "getExperienceNeeded", argsList.makeFunctionArgs(),&lExperienceNeeded);

	iExperienceNeeded = (int)lExperienceNeeded;

	return iExperienceNeeded;
}


int CvUnit::attackXPValue() const
{
	return m_pUnitInfo->getXPValueAttack();
}


int CvUnit::defenseXPValue() const
{
	return m_pUnitInfo->getXPValueDefense();
}


int CvUnit::maxXPValue() const
{
	int iMaxValue;

	iMaxValue = MAX_INT;

	if (isAnimal())
	{
		iMaxValue = std::min(iMaxValue, GC.getDefineINT("ANIMAL_MAX_XP_VALUE"));
	}

	if (isBarbarian())
	{
		iMaxValue = std::min(iMaxValue, GC.getDefineINT("BARBARIAN_MAX_XP_VALUE"));
	}

	return iMaxValue;
}


int CvUnit::firstStrikes() const
{
	return std::max(0, (m_pUnitInfo->getFirstStrikes() + getExtraFirstStrikes()));
}


int CvUnit::chanceFirstStrikes() const
{
	return std::max(0, (m_pUnitInfo->getChanceFirstStrikes() + getExtraChanceFirstStrikes()));
}


int CvUnit::maxFirstStrikes() const
{
	return (firstStrikes() + chanceFirstStrikes());
}


bool CvUnit::isRanged() const
{
	int i;
	CvUnitInfo * pkUnitInfo = &getUnitInfo();
	for ( i = 0; i < pkUnitInfo->getGroupDefinitions(); i++ )
	{
		if ( !getArtInfo(i, GET_PLAYER(getOwnerINLINE()).getCurrentEra())->getActAsRanged() )
		{
			return false;
		}
	}
	return true;
}


bool CvUnit::alwaysInvisible() const
{
	return m_pUnitInfo->isInvisible();
}


bool CvUnit::immuneToFirstStrikes() const
{
	return (m_pUnitInfo->isFirstStrikeImmune() || (getImmuneToFirstStrikesCount() > 0));
}


bool CvUnit::noDefensiveBonus() const
{
	//Sanguo Mod Civilization Trait, start, added by nk 11.28.08
	// Ma Teng, TRAIT_AFFINITY
	if (getUnitCombatType() != NO_UNITCOMBAT && GET_PLAYER(getOwnerINLINE()).isForceDefensiveBonusUnitCombat(getUnitCombatType()))
	{
		return false;
	}
	//Sanguo Mod Civilization Trait, end

	// Sanguo Mod Hero, start, added by poyuzhe 03.10.09
	if (getLegionHero() != NO_HERO)
	{
		for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		{
			if (GC.getPromotionInfo((PromotionTypes)iI).isForceDefensiveBonus())
			{
				if (isHasPromotion((PromotionTypes)iI))
				{
					return false;
				}
			}
		}
	}
	// Sanguo Mod Hero, end

	return m_pUnitInfo->isNoDefensiveBonus();
}


bool CvUnit::ignoreBuildingDefense() const
{
	// Sanguo Mod Civilization Trait, start, by poyuzhe 11.23.08
    // Gongsun Zan, TRAIT_HORSE_ARCHER
    int iI;
    UnitCombatTypes eUnitCombat = getUnitCombatType();
    for (iI = 0; iI < GC.getNumTraitInfos(); iI++)
    {
        if (GET_PLAYER(getOwnerINLINE()).hasTrait((TraitTypes)iI))
        {
            if (eUnitCombat == (UnitCombatTypes)(GC.getTraitInfo((TraitTypes)iI).getIgnoreBuildingDefenseCombatType()))
            {
                return true;
            }
        }
    }
    // Sanguo Mod Civilization Trait, end
	return m_pUnitInfo->isIgnoreBuildingDefense();
}


bool CvUnit::canMoveImpassable() const
{
	return m_pUnitInfo->isCanMoveImpassable();
}

bool CvUnit::canMoveAllTerrain() const
{
	// Sanguo Mod Hero, start, by poyuzhe 10.15.09
	if (m_pUnitInfo->isHero())
	{
		return true;
	}
	// Sanguo Mod Hero, end
	return m_pUnitInfo->isCanMoveAllTerrain();
}

bool CvUnit::flatMovementCost() const
{
	return m_pUnitInfo->isFlatMovementCost();
}


bool CvUnit::ignoreTerrainCost() const
{
	return m_pUnitInfo->isIgnoreTerrainCost();
}


bool CvUnit::isNeverInvisible() const
{
	return (!alwaysInvisible() && (getInvisibleType() == NO_INVISIBLE));
}


bool CvUnit::isInvisible(TeamTypes eTeam, bool bDebug, bool bCheckCargo) const
{
	if (bDebug && GC.getGameINLINE().isDebugMode())
	{
		return false;
	}

	if (getTeam() == eTeam)
	{
		return false;
	}

	if (alwaysInvisible())
	{
		return true;
	}

	if (bCheckCargo && isCargo())
	{
		return true;
	}

	if (getInvisibleType() == NO_INVISIBLE)
	{
		return false;
	}

	// Sanguo Mod Hero, start, added by poyuzhe 08.16.09
	// return !(plot()->isInvisibleVisible(eTeam, getInvisibleType()));
	if (plot()->isInvisibleVisible(eTeam, getInvisibleType()))
	{
		return false;
	}

	if (getInvisibleType() == INVISIBLE_TERRAIN)
	{
	    bool bInvisible = false;
	    if (GC.getDefineINT("HERO_INVISIBLE_TERRAIN_HILL_INVISIBLE") == 1 && plot()->isHills())
	    {
	        bInvisible = true;
	    }

	    if (plot()->getFeatureType() == (FeatureTypes)GC.getInfoTypeForString(GC.getDefineSTRING("HERO_INVISIBLE_TERRAIN_FIRST_FEATURE")) || plot()->getFeatureType() == (FeatureTypes)GC.getInfoTypeForString(GC.getDefineSTRING("HERO_INVISIBLE_TERRAIN_SECOND_FEATURE")))
	    {
	        bInvisible = true;
	    }

	    if (bInvisible && !plot()->isCity(true))
	    {
	        return true;
	    }
	}

	if (getInvisibleType() == INVISIBLE_NAVAL)
	{
		if (plot()->isWater())
		{
			return true;
		}
	}

	return false;
	// Sanguo Mod Hero, end
}


bool CvUnit::isNukeImmune() const
{
	return m_pUnitInfo->isNukeImmune();
}


int CvUnit::maxInterceptionProbability() const
{
	return std::max(0, m_pUnitInfo->getInterceptionProbability() + getExtraIntercept());
}


int CvUnit::currInterceptionProbability() const
{
	if (getDomainType() != DOMAIN_AIR)
	{
		return maxInterceptionProbability();
	}
	else
	{
		return ((maxInterceptionProbability() * currHitPoints()) / maxHitPoints());
	}
}


int CvUnit::evasionProbability() const
{
	return std::max(0, m_pUnitInfo->getEvasionProbability() + getExtraEvasion());
}


int CvUnit::withdrawalProbability() const
{
	if (getDomainType() == DOMAIN_LAND && plot()->isWater())
	{
		return 0;
	}

	return std::max(0, (m_pUnitInfo->getWithdrawalProbability() + getExtraWithdrawal()));
}


int CvUnit::collateralDamage() const
{
	return std::max(0, (m_pUnitInfo->getCollateralDamage()));
}


int CvUnit::collateralDamageLimit() const
{
	return std::max(0, m_pUnitInfo->getCollateralDamageLimit() * GC.getMAX_HIT_POINTS() / 100);
}


int CvUnit::collateralDamageMaxUnits() const
{
	return std::max(0, m_pUnitInfo->getCollateralDamageMaxUnits());
}


int CvUnit::cityAttackModifier() const
{
	return (m_pUnitInfo->getCityAttackModifier() + getExtraCityAttackPercent());
}


int CvUnit::cityDefenseModifier() const
{
	return (m_pUnitInfo->getCityDefenseModifier() + getExtraCityDefensePercent());
}


int CvUnit::animalCombatModifier() const
{
	return m_pUnitInfo->getAnimalCombatModifier();
}


int CvUnit::hillsAttackModifier() const
{
	return (m_pUnitInfo->getHillsAttackModifier() + getExtraHillsAttackPercent());
}


int CvUnit::hillsDefenseModifier() const
{
	return (m_pUnitInfo->getHillsDefenseModifier() + getExtraHillsDefensePercent());
}


int CvUnit::terrainAttackModifier(TerrainTypes eTerrain) const
{
	FAssertMsg(eTerrain >= 0, "eTerrain is expected to be non-negative (invalid Index)");
	FAssertMsg(eTerrain < GC.getNumTerrainInfos(), "eTerrain is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getTerrainAttackModifier(eTerrain) + getExtraTerrainAttackPercent(eTerrain));
}


int CvUnit::terrainDefenseModifier(TerrainTypes eTerrain) const
{
	FAssertMsg(eTerrain >= 0, "eTerrain is expected to be non-negative (invalid Index)");
	FAssertMsg(eTerrain < GC.getNumTerrainInfos(), "eTerrain is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getTerrainDefenseModifier(eTerrain) + getExtraTerrainDefensePercent(eTerrain));
}


int CvUnit::featureAttackModifier(FeatureTypes eFeature) const
{
	FAssertMsg(eFeature >= 0, "eFeature is expected to be non-negative (invalid Index)");
	FAssertMsg(eFeature < GC.getNumFeatureInfos(), "eFeature is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getFeatureAttackModifier(eFeature) + getExtraFeatureAttackPercent(eFeature));
}

int CvUnit::featureDefenseModifier(FeatureTypes eFeature) const
{
	FAssertMsg(eFeature >= 0, "eFeature is expected to be non-negative (invalid Index)");
	FAssertMsg(eFeature < GC.getNumFeatureInfos(), "eFeature is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getFeatureDefenseModifier(eFeature) + getExtraFeatureDefensePercent(eFeature));
}

int CvUnit::unitClassAttackModifier(UnitClassTypes eUnitClass) const
{
	FAssertMsg(eUnitClass >= 0, "eUnitClass is expected to be non-negative (invalid Index)");
	FAssertMsg(eUnitClass < GC.getNumUnitClassInfos(), "eUnitClass is expected to be within maximum bounds (invalid Index)");
	return m_pUnitInfo->getUnitClassAttackModifier(eUnitClass);
}


int CvUnit::unitClassDefenseModifier(UnitClassTypes eUnitClass) const
{
	FAssertMsg(eUnitClass >= 0, "eUnitClass is expected to be non-negative (invalid Index)");
	FAssertMsg(eUnitClass < GC.getNumUnitClassInfos(), "eUnitClass is expected to be within maximum bounds (invalid Index)");
	return m_pUnitInfo->getUnitClassDefenseModifier(eUnitClass);
}


int CvUnit::unitCombatModifier(UnitCombatTypes eUnitCombat) const
{
	FAssertMsg(eUnitCombat >= 0, "eUnitCombat is expected to be non-negative (invalid Index)");
	FAssertMsg(eUnitCombat < GC.getNumUnitCombatInfos(), "eUnitCombat is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getUnitCombatModifier(eUnitCombat) + getExtraUnitCombatModifier(eUnitCombat));
}


int CvUnit::domainModifier(DomainTypes eDomain) const
{
	FAssertMsg(eDomain >= 0, "eDomain is expected to be non-negative (invalid Index)");
	FAssertMsg(eDomain < NUM_DOMAIN_TYPES, "eDomain is expected to be within maximum bounds (invalid Index)");
	return (m_pUnitInfo->getDomainModifier(eDomain) + getExtraDomainModifier(eDomain));
}


int CvUnit::bombardRate() const
{
	return (m_pUnitInfo->getBombardRate() + getExtraBombardRate());
}


int CvUnit::airBombBaseRate() const
{
	return m_pUnitInfo->getBombRate();
}


int CvUnit::airBombCurrRate() const
{
	return ((airBombBaseRate() * currHitPoints()) / maxHitPoints());
}


SpecialUnitTypes CvUnit::specialCargo() const
{
	return ((SpecialUnitTypes)(m_pUnitInfo->getSpecialCargo()));
}


DomainTypes CvUnit::domainCargo() const
{
	return ((DomainTypes)(m_pUnitInfo->getDomainCargo()));
}


int CvUnit::cargoSpace() const
{
	return m_iCargoCapacity;
}

void CvUnit::changeCargoSpace(int iChange)
{
	if (iChange != 0)
	{
		m_iCargoCapacity += iChange;
		FAssert(m_iCargoCapacity >= 0);
		setInfoBarDirty(true);
	}
}

bool CvUnit::isFull() const
{
	return (getCargo() >= cargoSpace());
}


int CvUnit::cargoSpaceAvailable(SpecialUnitTypes eSpecialCargo, DomainTypes eDomainCargo) const
{
	if (specialCargo() != NO_SPECIALUNIT)
	{
		if (specialCargo() != eSpecialCargo)
		{
			return 0;
		}
	}

	if (domainCargo() != NO_DOMAIN)
	{
		if (domainCargo() != eDomainCargo)
		{
			return 0;
		}
	}

	return std::max(0, (cargoSpace() - getCargo()));
}


bool CvUnit::hasCargo() const
{
	return (getCargo() > 0);
}


bool CvUnit::canCargoAllMove() const
{
	CLLNode<IDInfo>* pUnitNode;
	CvUnit* pLoopUnit;
	CvPlot* pPlot;

	pPlot = plot();

	pUnitNode = pPlot->headUnitNode();

	while (pUnitNode != NULL)
	{
		pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit->getTransportUnit() == this)
		{
			if (pLoopUnit->getDomainType() == DOMAIN_LAND)
			{
				if (!(pLoopUnit->canMove()))
				{
					return false;
				}
			}
		}
	}

	return true;
}

bool CvUnit::canCargoEnterArea(TeamTypes eTeam, const CvArea* pArea, bool bIgnoreRightOfPassage) const
{
	CvPlot* pPlot = plot();

	CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();

	while (pUnitNode != NULL)
	{
		CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit->getTransportUnit() == this)
		{
			if (!pLoopUnit->canEnterArea(eTeam, pArea, bIgnoreRightOfPassage))
			{
				return false;
			}
		}
	}

	return true;
}

int CvUnit::getUnitAICargo(UnitAITypes eUnitAI) const
{
	int iCount = 0;

	std::vector<CvUnit*> aCargoUnits;
	getCargoUnits(aCargoUnits);
	for (uint i = 0; i < aCargoUnits.size(); ++i)
	{
		if (aCargoUnits[i]->AI_getUnitAIType() == eUnitAI)
		{
			++iCount;
		}
	}

	return iCount;
}


int CvUnit::getID() const
{
	return m_iID;
}


int CvUnit::getIndex() const
{
	return (getID() & FLTA_INDEX_MASK);
}


IDInfo CvUnit::getIDInfo() const
{
	IDInfo unit(getOwnerINLINE(), getID());
	return unit;
}


void CvUnit::setID(int iID)
{
	m_iID = iID;
}


int CvUnit::getGroupID() const
{
	return m_iGroupID;
}


bool CvUnit::isInGroup() const
{
	return(getGroupID() != FFreeList::INVALID_INDEX);
}


bool CvUnit::isGroupHead() const // XXX is this used???
{
	return (getGroup()->getHeadUnit() == this);
}


CvSelectionGroup* CvUnit::getGroup() const
{
	return GET_PLAYER(getOwnerINLINE()).getSelectionGroup(getGroupID());
}


bool CvUnit::canJoinGroup(const CvPlot* pPlot, CvSelectionGroup* pSelectionGroup) const
{
	CvUnit* pHeadUnit;

	// do not allow someone to join a group that is about to be split apart
	// this prevents a case of a never-ending turn
	if (pSelectionGroup->AI_isForceSeparate())
	{
		return false;
	}

	if (pSelectionGroup->getOwnerINLINE() == NO_PLAYER)
	{
		pHeadUnit = pSelectionGroup->getHeadUnit();

		if (pHeadUnit != NULL)
		{
			if (pHeadUnit->getOwnerINLINE() != getOwnerINLINE())
			{
				return false;
			}
		}
	}
	else
	{
		if (pSelectionGroup->getOwnerINLINE() != getOwnerINLINE())
		{
			return false;
		}
	}

	if (pSelectionGroup->getNumUnits() > 0)
	{
		if (!(pSelectionGroup->atPlot(pPlot)))
		{
			return false;
		}

		if (pSelectionGroup->getDomainType() != getDomainType())
		{
			return false;
		}
	}

	return true;
}


void CvUnit::joinGroup(CvSelectionGroup* pSelectionGroup, bool bRemoveSelected, bool bRejoin)
{
	CvSelectionGroup* pOldSelectionGroup;
	CvSelectionGroup* pNewSelectionGroup;
	CvPlot* pPlot;

	pOldSelectionGroup = GET_PLAYER(getOwnerINLINE()).getSelectionGroup(getGroupID());

	if ((pSelectionGroup != pOldSelectionGroup) || (pOldSelectionGroup == NULL))
	{
		pPlot = plot();

		if (pSelectionGroup != NULL)
		{
			pNewSelectionGroup = pSelectionGroup;
		}
		else
		{
			if (bRejoin)
			{
				pNewSelectionGroup = GET_PLAYER(getOwnerINLINE()).addSelectionGroup();
				pNewSelectionGroup->init(pNewSelectionGroup->getID(), getOwnerINLINE());
			}
			else
			{
				pNewSelectionGroup = NULL;
			}
		}

		if ((pNewSelectionGroup == NULL) || canJoinGroup(pPlot, pNewSelectionGroup))
		{
			if (pOldSelectionGroup != NULL)
			{
				bool bWasHead = false;
				if (!isHuman())
				{
					if (pOldSelectionGroup->getNumUnits() > 1)
					{
						if (pOldSelectionGroup->getHeadUnit() == this)
						{
							bWasHead = true;
						}
					}
				}

				pOldSelectionGroup->removeUnit(this);

				// if we were the head, if the head unitAI changed, then force the group to separate (non-humans)
				if (bWasHead)
				{
					FAssert(pOldSelectionGroup->getHeadUnit() != NULL);
					if (pOldSelectionGroup->getHeadUnit()->AI_getUnitAIType() != AI_getUnitAIType())
					{
						pOldSelectionGroup->AI_makeForceSeparate();
					}
				}
			}

			if ((pNewSelectionGroup != NULL) && pNewSelectionGroup->addUnit(this, false))
			{
				m_iGroupID = pNewSelectionGroup->getID();
			}
			else
			{
				m_iGroupID = FFreeList::INVALID_INDEX;
			}

			if (getGroup() != NULL)
			{
				if (getGroup()->getNumUnits() > 1)
				{
					getGroup()->setActivityType(ACTIVITY_AWAKE);
				}
				else
				{
					GET_PLAYER(getOwnerINLINE()).updateGroupCycle(this);
				}
			}

			if (getTeam() == GC.getGameINLINE().getActiveTeam())
			{
				if (pPlot != NULL)
				{
					pPlot->setFlagDirty(true);
				}
			}

			if (pPlot == gDLL->getInterfaceIFace()->getSelectionPlot())
			{
				gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
			}
		}

		if (bRemoveSelected)
		{
			if (IsSelected())
			{
				gDLL->getInterfaceIFace()->removeFromSelectionList(this);
			}
		}
	}
}


int CvUnit::getHotKeyNumber()
{
	return m_iHotKeyNumber;
}


void CvUnit::setHotKeyNumber(int iNewValue)
{
	CvUnit* pLoopUnit;
	int iLoop;

	FAssert(getOwnerINLINE() != NO_PLAYER);

	if (getHotKeyNumber() != iNewValue)
	{
		if (iNewValue != -1)
		{
			for(pLoopUnit = GET_PLAYER(getOwnerINLINE()).firstUnit(&iLoop); pLoopUnit != NULL; pLoopUnit = GET_PLAYER(getOwnerINLINE()).nextUnit(&iLoop))
			{
				if (pLoopUnit->getHotKeyNumber() == iNewValue)
				{
					pLoopUnit->setHotKeyNumber(-1);
				}
			}
		}

		m_iHotKeyNumber = iNewValue;

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}
	}
}


int CvUnit::getX() const
{
	return m_iX;
}


int CvUnit::getY() const
{
	return m_iY;
}


void CvUnit::setXY(int iX, int iY, bool bGroup, bool bUpdate, bool bShow, bool bCheckPlotVisible)
{
	PROFILE_FUNC();
	
	CLLNode<IDInfo>* pUnitNode;
	CvCity* pOldCity;
	CvCity* pNewCity;
	CvCity* pWorkingCity;
	CvUnit* pTransportUnit;
	CvUnit* pLoopUnit;
	CvPlot* pOldPlot;
	CvPlot* pNewPlot;
	CvPlot* pLoopPlot;
	CLinkList<IDInfo> oldUnits;
	ActivityTypes eOldActivityType;
	int iI;

	// OOS!! Temporary for Out-of-Sync madness debugging...
	if (GC.getLogging())
	{
		if (gDLL->getChtLvl() > 0)
		{
			char szOut[1024];
			sprintf(szOut, "Player %d Unit %d (%S's %S) moving from %d:%d to %d:%d\n", getOwnerINLINE(), getID(), GET_PLAYER(getOwnerINLINE()).getNameKey(), getName().GetCString(), getX_INLINE(), getY_INLINE(), iX, iY);
			gDLL->messageControlLog(szOut);
		}
	}

	FAssert(!at(iX, iY));
	FAssert(!isFighting());
	FAssert((iX == INVALID_PLOT_COORD) || (GC.getMapINLINE().plotINLINE(iX, iY)->getX_INLINE() == iX));
	FAssert((iY == INVALID_PLOT_COORD) || (GC.getMapINLINE().plotINLINE(iX, iY)->getY_INLINE() == iY));

	if (getGroup() != NULL)
	{
		eOldActivityType = getGroup()->getActivityType();
	}
	else
	{
		eOldActivityType = NO_ACTIVITY;
	}

	setBlockading(false);

	if (!bGroup || isCargo())
	{
		joinGroup(NULL, true);
		bShow = false;
	}

	pNewPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (pNewPlot != NULL)
	{
		pTransportUnit = getTransportUnit();

		if (pTransportUnit != NULL)
		{
			if (!(pTransportUnit->atPlot(pNewPlot)))
			{
				setTransportUnit(NULL);
			}
		}

		if (canFight())
		{
			oldUnits.clear();

			pUnitNode = pNewPlot->headUnitNode();

			while (pUnitNode != NULL)
			{
				oldUnits.insertAtEnd(pUnitNode->m_data);
				pUnitNode = pNewPlot->nextUnitNode(pUnitNode);
			}

			pUnitNode = oldUnits.head();

			while (pUnitNode != NULL)
			{
				pLoopUnit = ::getUnit(pUnitNode->m_data);
				pUnitNode = oldUnits.next(pUnitNode);

				if (pLoopUnit != NULL)
				{
					if (isEnemy(pLoopUnit->getTeam(), pNewPlot) || pLoopUnit->isEnemy(getTeam()))
					{
						if (!pLoopUnit->canCoexistWithEnemyUnit(getTeam()))
						{
							if (NO_UNITCLASS == pLoopUnit->getUnitInfo().getUnitCaptureClassType() && pLoopUnit->canDefend(pNewPlot))
							{
								pLoopUnit->jumpToNearestValidPlot(); // can kill unit
							}
							else
							{
								// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
								// commented out 1 line
//								if (!m_pUnitInfo->isHiddenNationality() && !pLoopUnit->getUnitInfo().isHiddenNationality())
								if (!isHiddenNationality() && !pLoopUnit->isHiddenNationality())
								// Sanguo Mod Hero, end
								{
									GET_TEAM(pLoopUnit->getTeam()).changeWarWeariness(getTeam(), *pNewPlot, GC.getDefineINT("WW_UNIT_CAPTURED"));
									GET_TEAM(getTeam()).changeWarWeariness(pLoopUnit->getTeam(), *pNewPlot, GC.getDefineINT("WW_CAPTURED_UNIT"));
									GET_TEAM(getTeam()).AI_changeWarSuccess(pLoopUnit->getTeam(), GC.getDefineINT("WAR_SUCCESS_UNIT_CAPTURING"));
								}

								if (!isNoCapture())
								{
									pLoopUnit->setCapturingPlayer(getOwnerINLINE());
								}

								// Sanguo Mod Hero, start, added by poyuzhe 02.22.09
								if (pLoopUnit->getLegionHero() != NO_HERO)
								{
									bool bTempHiddenNationality = (pLoopUnit->isHiddenNationality() && !pLoopUnit->getUnitInfo().isHiddenNationality());
									pLoopUnit->doLegionMemberDefeated(this, false, false);
									if (bTempHiddenNationality)
									{
										pLoopUnit->setTempCheck(true);
									}
								}
								// Sanguo Mod Hero, end

								pLoopUnit->kill(false, getOwnerINLINE());
							}
						}
					}
				}
			}
		}

		if (pNewPlot->isGoody(getTeam()))
		{
			GET_PLAYER(getOwnerINLINE()).doGoody(pNewPlot, this);
		}
	}

	pOldPlot = plot();

	if (pOldPlot != NULL)
	{
		pOldPlot->removeUnit(this, bUpdate && !hasCargo());

		pOldPlot->changeAdjacentSight(getTeam(), visibilityRange(), false, this, true);

		pOldPlot->area()->changeUnitsPerPlayer(getOwnerINLINE(), -1);
		pOldPlot->area()->changePower(getOwnerINLINE(), -(m_pUnitInfo->getPowerValue()));

		if (AI_getUnitAIType() != NO_UNITAI)
		{
			pOldPlot->area()->changeNumAIUnits(getOwnerINLINE(), AI_getUnitAIType(), -1);
		}

		if (isAnimal())
		{
			pOldPlot->area()->changeAnimalsPerPlayer(getOwnerINLINE(), -1);
		}

		if (pOldPlot->getTeam() != getTeam() && (pOldPlot->getTeam() == NO_TEAM || !GET_TEAM(pOldPlot->getTeam()).isVassal(getTeam())))
		{
			GET_PLAYER(getOwnerINLINE()).changeNumOutsideUnits(-1);
		}

		setLastMoveTurn(GC.getGameINLINE().getTurnSlice());

		pOldCity = pOldPlot->getPlotCity();

		if (pOldCity != NULL)
		{
			if (isMilitaryHappiness())
			{
				pOldCity->changeMilitaryHappinessUnits(-1);
			}
		}

		pWorkingCity = pOldPlot->getWorkingCity();

		if (pWorkingCity != NULL)
		{
			if (canSiege(pWorkingCity->getTeam()))
			{
				pWorkingCity->AI_setAssignWorkDirty(true);
			}
		}

		if (pOldPlot->isWater())
		{
			for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
			{
				pLoopPlot = plotDirection(pOldPlot->getX_INLINE(), pOldPlot->getY_INLINE(), ((DirectionTypes)iI));

				if (pLoopPlot != NULL)
				{
					if (pLoopPlot->isWater())
					{
						pWorkingCity = pLoopPlot->getWorkingCity();

						if (pWorkingCity != NULL)
						{
							if (canSiege(pWorkingCity->getTeam()))
							{
								pWorkingCity->AI_setAssignWorkDirty(true);
							}
						}
					}
				}
			}
		}

		if (pOldPlot->isActiveVisible(true))
		{
			pOldPlot->updateMinimapColor();
		}

		if (pOldPlot == gDLL->getInterfaceIFace()->getSelectionPlot())
		{
			gDLL->getInterfaceIFace()->verifyPlotListColumn();

			gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
		}
	}

	if (pNewPlot != NULL)
	{
		m_iX = pNewPlot->getX_INLINE();
		m_iY = pNewPlot->getY_INLINE();
	}
	else
	{
		m_iX = INVALID_PLOT_COORD;
		m_iY = INVALID_PLOT_COORD;
	}

	FAssertMsg(plot() == pNewPlot, "plot is expected to equal pNewPlot");

	if (pNewPlot != NULL)
	{
		pNewCity = pNewPlot->getPlotCity();

		if (pNewCity != NULL)
		{
			if (isEnemy(pNewCity->getTeam()) && !canCoexistWithEnemyUnit(pNewCity->getTeam()) && canFight())
			{
				GET_TEAM(getTeam()).changeWarWeariness(pNewCity->getTeam(), *pNewPlot, GC.getDefineINT("WW_CAPTURED_CITY"));
/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                      06/14/09                                jdog5000      */
/**                                                                                              */
/** General AI                                                                                   */
/*************************************************************************************************/
/* original bts code
				GET_TEAM(getTeam()).AI_changeWarSuccess(pNewCity->getTeam(), GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING"));
*/
				// Double war success if capturing capital city, always a significant blow to enemy
				// pNewCity still points to old city here, hasn't been acquired yet
				GET_TEAM(getTeam()).AI_changeWarSuccess(pNewCity->getTeam(), (pNewCity->isCapital() ? 2 : 1)*GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING"));
/*************************************************************************************************/
/** BETTER_BTS_AI_MOD                       END                                                  */
/*************************************************************************************************/

				PlayerTypes eNewOwner = GET_PLAYER(getOwnerINLINE()).pickConqueredCityOwner(*pNewCity);

				if (NO_PLAYER != eNewOwner)
				{
					GET_PLAYER(eNewOwner).acquireCity(pNewCity, true, false, true); // will delete the pointer
					pNewCity = NULL;
				}
			}
		}

		//update facing direction
		if(pOldPlot != NULL)
		{
			DirectionTypes newDirection = estimateDirection(pOldPlot, pNewPlot);
			if(newDirection != NO_DIRECTION)
				m_eFacingDirection = newDirection;
		}

		//update cargo mission animations
		if (isCargo())
		{
			if (eOldActivityType != ACTIVITY_MISSION)
			{
				getGroup()->setActivityType(eOldActivityType);
			}
		}

		setFortifyTurns(0);

		pNewPlot->changeAdjacentSight(getTeam(), visibilityRange(), true, this, true); // needs to be here so that the square is considered visible when we move into it...

		pNewPlot->addUnit(this, bUpdate && !hasCargo());

		pNewPlot->area()->changeUnitsPerPlayer(getOwnerINLINE(), 1);
		pNewPlot->area()->changePower(getOwnerINLINE(), m_pUnitInfo->getPowerValue());

		if (AI_getUnitAIType() != NO_UNITAI)
		{
			pNewPlot->area()->changeNumAIUnits(getOwnerINLINE(), AI_getUnitAIType(), 1);
		}

		if (isAnimal())
		{
			pNewPlot->area()->changeAnimalsPerPlayer(getOwnerINLINE(), 1);
		}

		if (pNewPlot->getTeam() != getTeam() && (pNewPlot->getTeam() == NO_TEAM || !GET_TEAM(pNewPlot->getTeam()).isVassal(getTeam())))
		{
			GET_PLAYER(getOwnerINLINE()).changeNumOutsideUnits(1);
		}

		if (shouldLoadOnMove(pNewPlot))
		{
			load();
		}

		for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			if (GET_TEAM((TeamTypes)iI).isAlive())
			{
				if (!isInvisible(((TeamTypes)iI), false))
				{
					if (pNewPlot->isVisible((TeamTypes)iI, false))
					{
						GET_TEAM((TeamTypes)iI).meet(getTeam(), true);
					}
				}
			}
		}

		pNewCity = pNewPlot->getPlotCity();

		if (pNewCity != NULL)
		{
			if (isMilitaryHappiness())
			{
				pNewCity->changeMilitaryHappinessUnits(1);
			}
		}

		pWorkingCity = pNewPlot->getWorkingCity();

		if (pWorkingCity != NULL)
		{
			if (canSiege(pWorkingCity->getTeam()))
			{
				pWorkingCity->verifyWorkingPlot(pWorkingCity->getCityPlotIndex(pNewPlot));
			}
		}

		if (pNewPlot->isWater())
		{
			for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
			{
				pLoopPlot = plotDirection(pNewPlot->getX_INLINE(), pNewPlot->getY_INLINE(), ((DirectionTypes)iI));

				if (pLoopPlot != NULL)
				{
					if (pLoopPlot->isWater())
					{
						pWorkingCity = pLoopPlot->getWorkingCity();

						if (pWorkingCity != NULL)
						{
							if (canSiege(pWorkingCity->getTeam()))
							{
								pWorkingCity->verifyWorkingPlot(pWorkingCity->getCityPlotIndex(pLoopPlot));
							}
						}
					}
				}
			}
		}

		if (pNewPlot->isActiveVisible(true))
		{
			pNewPlot->updateMinimapColor();
		}

		if (GC.IsGraphicsInitialized())
		{
			//override bShow if check plot visible
			if(bCheckPlotVisible && pNewPlot->isVisibleToWatchingHuman())
				bShow = true;

			if (bShow)
			{
				QueueMove(pNewPlot);
			}
			else
			{
				SetPosition(pNewPlot);
			}
		}

		if (pNewPlot == gDLL->getInterfaceIFace()->getSelectionPlot())
		{
			gDLL->getInterfaceIFace()->verifyPlotListColumn();

			gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
		}
	}

	if (pOldPlot != NULL)
	{
		if (hasCargo())
		{
			pUnitNode = pOldPlot->headUnitNode();

			while (pUnitNode != NULL)
			{
				pLoopUnit = ::getUnit(pUnitNode->m_data);
				pUnitNode = pOldPlot->nextUnitNode(pUnitNode);

				if (pLoopUnit->getTransportUnit() == this)
				{
					pLoopUnit->setXY(iX, iY, bGroup, false);
				}
			}
		}
	}

	if (bUpdate && hasCargo())
	{
		if (pOldPlot != NULL)
		{
			pOldPlot->updateCenterUnit();
			pOldPlot->setFlagDirty(true);
		}

		if (pNewPlot != NULL)
		{
			pNewPlot->updateCenterUnit();
			pNewPlot->setFlagDirty(true);
		}
	}

	FAssert(pOldPlot != pNewPlot);
	GET_PLAYER(getOwnerINLINE()).updateGroupCycle(this);

	setInfoBarDirty(true);

	if (IsSelected())
	{
		if (isFound())
		{
			gDLL->getInterfaceIFace()->setDirty(GlobeLayer_DIRTY_BIT, true);
			gDLL->getEngineIFace()->updateFoundingBorder();
		}

		gDLL->getInterfaceIFace()->setDirty(ColoredPlots_DIRTY_BIT, true);
	}

	//update glow
	if (pNewPlot != NULL)
	{
		gDLL->getEntityIFace()->updateEnemyGlow(getUnitEntity());
	}

	// Sanguo Mod Performance, start, added by poyuzhe 08.12.09
	{PROFILE_BEGIN("setXY::plotdangercache");
	for (int iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (GET_TEAM(getTeam()).isAtWar((TeamTypes)iI))
		{
			for (int iDX = -DANGER_RANGE; iDX <= DANGER_RANGE; iDX++)
			{
				for (int iDY = -DANGER_RANGE; iDY <= DANGER_RANGE; iDY++)
				{
					if (pOldPlot != NULL)
					{
                        int iIndex = GC.getMapINLINE().plotNumINLINE(pOldPlot->getX_INLINE() + iDX, pOldPlot->getY_INLINE() + iDY);
                        if (iIndex > -1 && iIndex < GC.getMapINLINE().numPlotsINLINE())
                        {
                            for (int iJ = 0; iJ < GET_TEAM((TeamTypes)iI).getPlayerMemberListSize(); iJ++)
                            {
                                GET_PLAYER(GET_TEAM((TeamTypes)iI).getPlayerMemberAt(iJ)).AI_invalidatePlotDangerCache(iIndex);
                            }
                        }
					}
					if (pNewPlot != NULL)
					{
                        int iIndex = GC.getMapINLINE().plotNumINLINE(pNewPlot->getX_INLINE() + iDX, pNewPlot->getY_INLINE() + iDY);
                        if (iIndex > -1 && iIndex < GC.getMapINLINE().numPlotsINLINE())
                        {
                            for (int iJ = 0; iJ < GET_TEAM((TeamTypes)iI).getPlayerMemberListSize(); iJ++)
                            {
                                GET_PLAYER(GET_TEAM((TeamTypes)iI).getPlayerMemberAt(iJ)).AI_invalidatePlotDangerCache(iIndex);
                            }
                        }
					}
				}
			}
		}
	}
	PROFILE_END();}
	// Sanguo Mod Performance, end

	// Sanguo Mod Hero, start, added by poyuzhe 08.17.09
	CvWString szBuffer;
    if (isLeaderTroop())
    {
        for (int iI = 1; iI < GC.getGameINLINE().getHeroLegionSize(getLeaderHero()); iI++)
		{
			CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroLegionMemberAt(getLeaderHero(), iI));
			FAssert (pMember != NULL && pMember->getLeaderHero() == getLeaderHero());
			if (stepDistance(getX_INLINE(), getY_INLINE(), pMember->getX_INLINE(), pMember->getY_INLINE()) > GC.getDefineINT("ADVANCED_LEGION_TOO_FAR_MEMBER_QUIT_DISTANCE"))
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_HERO_AUTO_QUIT_ADVANCED_LEGION", GC.getHeroInfo(pMember->getHeroType()).getDescription(), GC.getHeroInfo(getLeaderHero()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, NULL, MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pMember->getX_INLINE(), pMember->getY_INLINE());
			    pMember->quitAdvancedLegion();
            }
		}
    }
	else if (isCoreTroop())
	{
		PROFILE_BEGIN("setXY::coreTroopDistance");
		for (int iI = 1; iI < GC.getGameINLINE().getHeroLegionSize(getHeroType()); iI++)
		{
			CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroLegionMemberAt(getLegionHero(), iI));
			FAssert (pMember != NULL && pMember->getLegionHero() == getLegionHero());
			if (stepDistance(getX_INLINE(), getY_INLINE(), pMember->getX_INLINE(), pMember->getY_INLINE()) > GC.getDefineINT("LEGION_TOO_FAR_MEMBER_QUIT_DISTANCE"))
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_AUTO_QUIT_LEGION", pMember->getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, NULL, MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pMember->getX_INLINE(), pMember->getY_INLINE());
			    pMember->quitLegion();
            }
		}
        if (getLeaderHero() != NO_HERO)
        {
            CvUnit* pLeaderTroop = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroUnitID(getLeaderHero()));
            FAssert (pLeaderTroop->isLeaderTroop());
            if (stepDistance(getX_INLINE(), getY_INLINE(), pLeaderTroop->getX_INLINE(), pLeaderTroop->getY_INLINE()) > GC.getDefineINT("ADVANCED_LEGION_TOO_FAR_MEMBER_QUIT_DISTANCE"))
		    {
                szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_HERO_AUTO_QUIT_ADVANCED_LEGION", GC.getHeroInfo(getHeroType()).getDescription(), GC.getHeroInfo(getLeaderHero()).getDescription());
			    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, NULL, MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE());
		        quitAdvancedLegion();
            }
        }
		PROFILE_END();
	}
	else if (getLegionHero() != NO_HERO)
	{
		PROFILE_BEGIN("setXY::legionDistance");
		CvUnit* pHero = GET_PLAYER(getOwnerINLINE()).getUnit(GC.getGameINLINE().getHeroUnitID(getLegionHero()));
		FAssert (pHero != NULL && pHero->getHeroType() == getLegionHero());
		if (stepDistance(getX_INLINE(), getY_INLINE(), pHero->getX_INLINE(), pHero->getY_INLINE()) > GC.getDefineINT("LEGION_TOO_FAR_MEMBER_QUIT_DISTANCE"))
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_LEGION_UNIT_AUTO_QUIT_LEGION", getNameKey(), GC.getHeroInfo(pHero->getHeroType()).getDescription());
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, NULL, MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE());
		    quitLegion();
        }
		PROFILE_END();
	}
	// Sanguo Mod Hero, end

	// report event to Python, along with some other key state
	CvEventReporter::getInstance().unitSetXY(pNewPlot, this);
}


bool CvUnit::at(int iX, int iY) const
{
	return((getX_INLINE() == iX) && (getY_INLINE() == iY));
}


bool CvUnit::atPlot(const CvPlot* pPlot) const
{
	return (plot() == pPlot);
}


CvPlot* CvUnit::plot() const
{
	return GC.getMapINLINE().plotSorenINLINE(getX_INLINE(), getY_INLINE());
}


int CvUnit::getArea() const
{
	return plot()->getArea();
}


CvArea* CvUnit::area() const
{
	return plot()->area();
}


bool CvUnit::onMap() const
{
	return (plot() != NULL);
}


int CvUnit::getLastMoveTurn() const
{
	return m_iLastMoveTurn;
}


void CvUnit::setLastMoveTurn(int iNewValue)
{
	m_iLastMoveTurn = iNewValue;
	FAssert(getLastMoveTurn() >= 0);
}


CvPlot* CvUnit::getReconPlot() const
{
	return GC.getMapINLINE().plotSorenINLINE(m_iReconX, m_iReconY);
}


void CvUnit::setReconPlot(CvPlot* pNewValue)
{
	CvPlot* pOldPlot;

	pOldPlot = getReconPlot();

	if (pOldPlot != pNewValue)
	{
		if (pOldPlot != NULL)
		{
			pOldPlot->changeAdjacentSight(getTeam(), GC.getDefineINT("RECON_VISIBILITY_RANGE"), false, this, true);
			pOldPlot->changeReconCount(-1); // changeAdjacentSight() tests for getReconCount()
		}

		if (pNewValue == NULL)
		{
			m_iReconX = INVALID_PLOT_COORD;
			m_iReconY = INVALID_PLOT_COORD;
		}
		else
		{
			m_iReconX = pNewValue->getX_INLINE();
			m_iReconY = pNewValue->getY_INLINE();

			pNewValue->changeReconCount(1); // changeAdjacentSight() tests for getReconCount()
			pNewValue->changeAdjacentSight(getTeam(), GC.getDefineINT("RECON_VISIBILITY_RANGE"), true, this, true);
		}
	}
}


int CvUnit::getGameTurnCreated() const
{
	return m_iGameTurnCreated;
}


void CvUnit::setGameTurnCreated(int iNewValue)
{
	m_iGameTurnCreated = iNewValue;
	FAssert(getGameTurnCreated() >= 0);
}


int CvUnit::getDamage() const
{
	return m_iDamage;
}


void CvUnit::setDamage(int iNewValue, PlayerTypes ePlayer, bool bNotifyEntity)
{
	int iOldValue;

	iOldValue = getDamage();

	m_iDamage = range(iNewValue, 0, maxHitPoints());

	FAssertMsg(currHitPoints() >= 0, "currHitPoints() is expected to be non-negative (invalid Index)");

	if (iOldValue != getDamage())
	{
		if (GC.getGameINLINE().isFinalInitialized() && bNotifyEntity)
		{
			NotifyEntity(MISSION_DAMAGE);
		}

		setInfoBarDirty(true);

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}

		if (plot() == gDLL->getInterfaceIFace()->getSelectionPlot())
		{
			gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
		}
	}

	if (isDead())
	{
		kill(true, ePlayer);
	}
}


void CvUnit::changeDamage(int iChange, PlayerTypes ePlayer)
{
	setDamage((getDamage() + iChange), ePlayer);
}


int CvUnit::getMoves() const
{
	return m_iMoves;
}


void CvUnit::setMoves(int iNewValue)
{
	CvPlot* pPlot;

	if (getMoves() != iNewValue)
	{
		pPlot = plot();

		m_iMoves = iNewValue;

		FAssert(getMoves() >= 0);

		// Sanguo Mod Hero, start, removed by poyuzhe 07.3.09
		if (m_iMoves > 0 && getLegionHero() != NO_HERO)
		{
		    GC.getGameINLINE().setLegionUnitMoved(getLegionHero(), true);
            if (getLeaderHero() != NO_HERO)
            {
                GC.getGameINLINE().setLegionUnitMoved(getLeaderHero(), true);
            }
		}
        // Sanguo Mod Hero, end

		if (getTeam() == GC.getGameINLINE().getActiveTeam())
		{
			if (pPlot != NULL)
			{
				pPlot->setFlagDirty(true);
			}
		}

		if (IsSelected())
		{
			gDLL->getFAStarIFace()->ForceReset(&GC.getInterfacePathFinder());

			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}

		if (pPlot == gDLL->getInterfaceIFace()->getSelectionPlot())
		{
			gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
		}

		// Sanguo Mod Performance, start, added by poyuzhe 08.13.09
        if (GC.getGameINLINE().isFinalInitialized())
        {
            for (int iI = 0; iI < MAX_TEAMS; iI++)
            {
                if (GET_TEAM(getTeam()).isAtWar((TeamTypes)iI))
                {
                    for (int iDX = -DANGER_RANGE; iDX <= DANGER_RANGE; iDX++)
                    {
                        for (int iDY = -DANGER_RANGE; iDY <= DANGER_RANGE; iDY++)
                        {
                            int iIndex = GC.getMapINLINE().plotNumINLINE(plot()->getX_INLINE() + iDX, plot()->getY_INLINE() + iDY);
                            if (iIndex > -1 && iIndex < GC.getMapINLINE().numPlotsINLINE())
                            {
                                for (int iJ = 0; iJ < GET_TEAM((TeamTypes)iI).getPlayerMemberListSize(); iJ++)
                                {
                                    GET_PLAYER(GET_TEAM((TeamTypes)iI).getPlayerMemberAt(iJ)).AI_invalidatePlotDangerCache(iIndex);
                                }
                            }
                        }
                    }
                }
            }
        }
        // Sanguo Mod Performance, end
	}
}


void CvUnit::changeMoves(int iChange)
{
	setMoves(getMoves() + iChange);
}


void CvUnit::finishMoves()
{
	setMoves(maxMoves());
}


int CvUnit::getExperience() const
{
	return m_iExperience;
}

void CvUnit::setExperience(int iNewValue, int iMax)
{
	if ((getExperience() != iNewValue) && (getExperience() < ((iMax == -1) ? MAX_INT : iMax)))
	{
		m_iExperience = std::min(((iMax == -1) ? MAX_INT : iMax), iNewValue);
		FAssert(getExperience() >= 0);

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}
	}
}
//Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
//Sun Ce, TRAIT_SEAFARING
//void CvUnit::changeExperience(int iChange, int iMax, bool bFromCombat, bool bInBorders, bool bUpdateGlobal)
void CvUnit::changeExperience(int iChange, int iMax, bool bFromCombat, bool bInBorders, bool bUpdateGlobal, TerrainTypes eTerrain, bool bGiveHeroXP)
//Sanguo Mod Civilization Trait, end
{
	// Sanguo Mod Hero, start, added by poyuzhe 01.17.09
	if (isNakedHero())  // if it's a naked hero, don't give him XP
	{
	    return;
	}
    // Sanguo Mod Hero, end

	int iUnitExperience = iChange;
	if (bFromCombat)
	{
		CvPlayer& kPlayer = GET_PLAYER(getOwnerINLINE());

		int iCombatExperienceMod = 100 + kPlayer.getGreatGeneralRateModifier();

		if (bInBorders)
		{
			iCombatExperienceMod += kPlayer.getDomesticGreatGeneralRateModifier() + kPlayer.getExpInBorderModifier();
			iUnitExperience += (iChange * kPlayer.getExpInBorderModifier()) / 100;
		}

		if (bUpdateGlobal)
		{
			kPlayer.changeCombatExperience((iChange * iCombatExperienceMod) / 100);
		}
        //Sanguo Mod Civilization Trait, start, by poyuzhe 11.21.08
        //Sun Ce, TRAIT_SEAFARING
		//if (getExperiencePercent() != 0)
		if (getExperiencePercent() != 0 || eTerrain != NO_TERRAIN)
		{
            int iExperiencePercent = getExperiencePercent();
            if (eTerrain != NO_TERRAIN)
            {
                iExperiencePercent += GET_PLAYER(getOwnerINLINE()).getTerrainExperienceModifier(eTerrain);
            }
            //iUnitExperience *= std::max(0, 100 + getExperiencePercent());
			iUnitExperience *= std::max(0, 100 + iExperiencePercent);
			//Sanguo Mod Civilization Trait, end
			iUnitExperience /= 100;
		}
	}

	setExperience((getExperience() + iUnitExperience), iMax);

	// Sanguo Mod Hero, start, added by poyuzhe 01.17.09
	if (getLegionHero() != NO_HERO && bGiveHeroXP)
	{
	    if (!isHuman())
	    {
			iUnitExperience *= (GC.getGameINLINE().getSorenRandNum(5, "AI hero experience Modifier") + 1);
		}
	    float fUnitExperience = (float)std::max(0, std::min(iUnitExperience, iMax - getExperience()));
        
        if (getLeaderHero() != NO_HERO)
        {
            GC.getGameINLINE().changeHeroExperience(getLeaderHero(), fUnitExperience / 4);
        }
		
        {PROFILE_BEGIN("changeExperince::loop");
		for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		{
			if (GC.getPromotionInfo((PromotionTypes)iI).getMatureLevel() > 0)
			{
				if (GC.getGameINLINE().getHeroPromotionStatus(getLegionHero(), (PromotionTypes)iI) < 3 && GC.getGameINLINE().getHeroPromotionStatus(getLegionHero(), (PromotionTypes)iI) > 0)
				{
					if (GC.getGameINLINE().getHeroLevel(getLegionHero()) <= GC.getPromotionInfo((PromotionTypes)iI).getMatureLevel())
					{
						fUnitExperience /= 2;
					}
					else
					{
						fUnitExperience *= 2;
					}
					break;
				}
			}
		}
		PROFILE_END();}
	    GC.getGameINLINE().changeHeroExperience(getLegionHero(), fUnitExperience);
	}
    // Sanguo Mod Hero, end
}

int CvUnit::getLevel() const
{
	return m_iLevel;
}

void CvUnit::setLevel(int iNewValue)
{
	if (getLevel() != iNewValue)
	{
		m_iLevel = iNewValue;
		FAssert(getLevel() >= 0);

		if (getLevel() > GET_PLAYER(getOwnerINLINE()).getHighestUnitLevel())
		{
			GET_PLAYER(getOwnerINLINE()).setHighestUnitLevel(getLevel());
		}

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}
	}
}

void CvUnit::changeLevel(int iChange)
{
	setLevel(getLevel() + iChange);
}

int CvUnit::getCargo() const
{
	return m_iCargo;
}

void CvUnit::changeCargo(int iChange)
{
	m_iCargo += iChange;
	FAssert(getCargo() >= 0);
}

void CvUnit::getCargoUnits(std::vector<CvUnit*>& aUnits) const
{
	aUnits.clear();

	if (hasCargo())
	{
		CvPlot* pPlot = plot();
		CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();
		while (pUnitNode != NULL)
		{
			CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
			pUnitNode = pPlot->nextUnitNode(pUnitNode);
			if (pLoopUnit->getTransportUnit() == this)
			{
				aUnits.push_back(pLoopUnit);
			}
		}
	}

	FAssert(getCargo() == aUnits.size());
}

CvPlot* CvUnit::getAttackPlot() const
{
	return GC.getMapINLINE().plotSorenINLINE(m_iAttackPlotX, m_iAttackPlotY);
}


void CvUnit::setAttackPlot(const CvPlot* pNewValue, bool bAirCombat)
{
	if (getAttackPlot() != pNewValue)
	{
		if (pNewValue != NULL)
		{
			m_iAttackPlotX = pNewValue->getX_INLINE();
			m_iAttackPlotY = pNewValue->getY_INLINE();
		}
		else
		{
			m_iAttackPlotX = INVALID_PLOT_COORD;
			m_iAttackPlotY = INVALID_PLOT_COORD;
		}
	}

	m_bAirCombat = bAirCombat;
}

bool CvUnit::isAirCombat() const
{
	return m_bAirCombat;
}

int CvUnit::getCombatTimer() const
{
	return m_iCombatTimer;
}

void CvUnit::setCombatTimer(int iNewValue)
{
	m_iCombatTimer = iNewValue;
	FAssert(getCombatTimer() >= 0);
}

void CvUnit::changeCombatTimer(int iChange)
{
	setCombatTimer(getCombatTimer() + iChange);
}

int CvUnit::getCombatFirstStrikes() const
{
	return m_iCombatFirstStrikes;
}

void CvUnit::setCombatFirstStrikes(int iNewValue)
{
	m_iCombatFirstStrikes = iNewValue;
	FAssert(getCombatFirstStrikes() >= 0);
}

void CvUnit::changeCombatFirstStrikes(int iChange)
{
	setCombatFirstStrikes(getCombatFirstStrikes() + iChange);
}

int CvUnit::getFortifyTurns() const
{
	return m_iFortifyTurns;
}

void CvUnit::setFortifyTurns(int iNewValue)
{
	iNewValue = range(iNewValue, 0, GC.getDefineINT("MAX_FORTIFY_TURNS"));

	if (iNewValue != getFortifyTurns())
	{
		m_iFortifyTurns = iNewValue;
		setInfoBarDirty(true);
	}
}

void CvUnit::changeFortifyTurns(int iChange)
{
	setFortifyTurns(getFortifyTurns() + iChange);
}

int CvUnit::getBlitzCount() const
{
	return m_iBlitzCount;
}

bool CvUnit::isBlitz() const
{
	return (getBlitzCount() > 0);
}

void CvUnit::changeBlitzCount(int iChange)
{
	m_iBlitzCount += iChange;
	FAssert(getBlitzCount() >= 0);
}

int CvUnit::getAmphibCount() const
{
	return m_iAmphibCount;
}

bool CvUnit::isAmphib() const
{
	return (getAmphibCount() > 0);
}

void CvUnit::changeAmphibCount(int iChange)
{
	m_iAmphibCount += iChange;
	FAssert(getAmphibCount() >= 0);
}

int CvUnit::getRiverCount() const
{
	return m_iRiverCount;
}

bool CvUnit::isRiver() const
{
	return (getRiverCount() > 0);
}

void CvUnit::changeRiverCount(int iChange)
{
	m_iRiverCount += iChange;
	FAssert(getRiverCount() >= 0);
}

int CvUnit::getEnemyRouteCount() const
{
	return m_iEnemyRouteCount;
}

bool CvUnit::isEnemyRoute() const
{
	return (getEnemyRouteCount() > 0);
}

void CvUnit::changeEnemyRouteCount(int iChange)
{
	m_iEnemyRouteCount += iChange;
	FAssert(getEnemyRouteCount() >= 0);
}

int CvUnit::getAlwaysHealCount() const
{
	return m_iAlwaysHealCount;
}

bool CvUnit::isAlwaysHeal() const
{
	return (getAlwaysHealCount() > 0);
}

void CvUnit::changeAlwaysHealCount(int iChange)
{
	m_iAlwaysHealCount += iChange;
	FAssert(getAlwaysHealCount() >= 0);
}

int CvUnit::getHillsDoubleMoveCount() const
{
	return m_iHillsDoubleMoveCount;
}

bool CvUnit::isHillsDoubleMove() const
{
	return (getHillsDoubleMoveCount() > 0);
}

void CvUnit::changeHillsDoubleMoveCount(int iChange)
{
	m_iHillsDoubleMoveCount += iChange;
	FAssert(getHillsDoubleMoveCount() >= 0);
}

int CvUnit::getImmuneToFirstStrikesCount() const
{
	return m_iImmuneToFirstStrikesCount;
}

void CvUnit::changeImmuneToFirstStrikesCount(int iChange)
{
	m_iImmuneToFirstStrikesCount += iChange;
	FAssert(getImmuneToFirstStrikesCount() >= 0);
}


int CvUnit::getExtraVisibilityRange() const
{
	return m_iExtraVisibilityRange;
}

void CvUnit::changeExtraVisibilityRange(int iChange)
{
	if (iChange != 0)
	{
		plot()->changeAdjacentSight(getTeam(), visibilityRange(), false, this, true);

		m_iExtraVisibilityRange += iChange;
		FAssert(getExtraVisibilityRange() >= 0);

		plot()->changeAdjacentSight(getTeam(), visibilityRange(), true, this, true);
	}
}

int CvUnit::getExtraMoves() const
{
	return m_iExtraMoves;
}


void CvUnit::changeExtraMoves(int iChange)
{
	m_iExtraMoves += iChange;
	// Sanguo Mod Hero, start, added by poyuzhe 03.11.09
	FAssert(baseMoves() > 0);
	// Sanguo Mod Hero, end
}


int CvUnit::getExtraMoveDiscount() const
{
	return m_iExtraMoveDiscount;
}


void CvUnit::changeExtraMoveDiscount(int iChange)
{
	m_iExtraMoveDiscount += iChange;
	FAssert(getExtraMoveDiscount() >= 0);
}

int CvUnit::getExtraAirRange() const
{
	return m_iExtraAirRange;
}

void CvUnit::changeExtraAirRange(int iChange)
{
	m_iExtraAirRange += iChange;
}

int CvUnit::getExtraIntercept() const
{
	return m_iExtraIntercept;
}

void CvUnit::changeExtraIntercept(int iChange)
{
	m_iExtraIntercept += iChange;
}

int CvUnit::getExtraEvasion() const
{
	return m_iExtraEvasion;
}

void CvUnit::changeExtraEvasion(int iChange)
{
	m_iExtraEvasion += iChange;
}

int CvUnit::getExtraFirstStrikes() const
{
	return m_iExtraFirstStrikes;
}

void CvUnit::changeExtraFirstStrikes(int iChange)
{
	m_iExtraFirstStrikes += iChange;
	FAssert(getExtraFirstStrikes() >= 0);
}

int CvUnit::getExtraChanceFirstStrikes() const
{
	return m_iExtraChanceFirstStrikes;
}

void CvUnit::changeExtraChanceFirstStrikes(int iChange)
{
	m_iExtraChanceFirstStrikes += iChange;
	FAssert(getExtraChanceFirstStrikes() >= 0);
}


int CvUnit::getExtraWithdrawal() const
{
	return m_iExtraWithdrawal;
}


void CvUnit::changeExtraWithdrawal(int iChange)
{
	m_iExtraWithdrawal += iChange;
	FAssert(getExtraWithdrawal() >= 0);
}

int CvUnit::getExtraCollateralDamage() const
{
	return m_iExtraCollateralDamage;
}

void CvUnit::changeExtraCollateralDamage(int iChange)
{
	m_iExtraCollateralDamage += iChange;
	FAssert(getExtraCollateralDamage() >= 0);
}

int CvUnit::getExtraBombardRate() const
{
	return m_iExtraBombardRate;
}

void CvUnit::changeExtraBombardRate(int iChange)
{
	m_iExtraBombardRate += iChange;
	FAssert(getExtraBombardRate() >= 0);
}

int CvUnit::getExtraEnemyHeal() const
{
	return m_iExtraEnemyHeal;
}

void CvUnit::changeExtraEnemyHeal(int iChange)
{
	m_iExtraEnemyHeal += iChange;
	FAssert(getExtraEnemyHeal() >= 0);
}

int CvUnit::getExtraNeutralHeal() const
{
	return m_iExtraNeutralHeal;
}

void CvUnit::changeExtraNeutralHeal(int iChange)
{
	m_iExtraNeutralHeal += iChange;
	FAssert(getExtraNeutralHeal() >= 0);
}

int CvUnit::getExtraFriendlyHeal() const
{
	return m_iExtraFriendlyHeal;
}


void CvUnit::changeExtraFriendlyHeal(int iChange)
{
	m_iExtraFriendlyHeal += iChange;
	FAssert(getExtraFriendlyHeal() >= 0);
}

int CvUnit::getSameTileHeal() const
{
	return m_iSameTileHeal;
}

void CvUnit::changeSameTileHeal(int iChange)
{
	m_iSameTileHeal += iChange;
	FAssert(getSameTileHeal() >= 0);
}

int CvUnit::getAdjacentTileHeal() const
{
	return m_iAdjacentTileHeal;
}

void CvUnit::changeAdjacentTileHeal(int iChange)
{
	m_iAdjacentTileHeal += iChange;
	FAssert(getAdjacentTileHeal() >= 0);
}

int CvUnit::getExtraCombatPercent() const
{
	return m_iExtraCombatPercent;
}

void CvUnit::changeExtraCombatPercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExtraCombatPercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraCityAttackPercent() const
{
	return m_iExtraCityAttackPercent;
}

void CvUnit::changeExtraCityAttackPercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExtraCityAttackPercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraCityDefensePercent() const
{
	return m_iExtraCityDefensePercent;
}

void CvUnit::changeExtraCityDefensePercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExtraCityDefensePercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraHillsAttackPercent() const
{
	return m_iExtraHillsAttackPercent;
}

void CvUnit::changeExtraHillsAttackPercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExtraHillsAttackPercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraHillsDefensePercent() const
{
	return m_iExtraHillsDefensePercent;
}

void CvUnit::changeExtraHillsDefensePercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExtraHillsDefensePercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getRevoltProtection() const
{
	return m_iRevoltProtection;
}

void CvUnit::changeRevoltProtection(int iChange)
{
	if (iChange != 0)
	{
		m_iRevoltProtection += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getCollateralDamageProtection() const
{
	return m_iCollateralDamageProtection;
}

void CvUnit::changeCollateralDamageProtection(int iChange)
{
	if (iChange != 0)
	{
		m_iCollateralDamageProtection += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getPillageChange() const
{
	return m_iPillageChange;
}

void CvUnit::changePillageChange(int iChange)
{
	if (iChange != 0)
	{
		m_iPillageChange += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getUpgradeDiscount() const
{
	return m_iUpgradeDiscount;
}

void CvUnit::changeUpgradeDiscount(int iChange)
{
	if (iChange != 0)
	{
		m_iUpgradeDiscount += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExperiencePercent() const
{
	// Sanguo Mod Hero, start, added by poyuzhe 08.16.09
	// return m_iExperiencePercent;
	PROFILE_FUNC();

	if (m_iLegionTurn > 0)
	{
		int iLegionExperienceMod = 0;
		for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		{
			if (GC.getPromotionInfo((PromotionTypes)iI).getExperienceModifierPerLegionTurn() > 0)
			{
				iLegionExperienceMod += GC.getPromotionInfo((PromotionTypes)iI).getExperienceModifierPerLegionTurn();
			}
		}

		return (m_iExperiencePercent + std::min(GC.getDefineINT("HERO_UNIT_EXPERIENCE_MOD_CAP"), m_iLegionTurn * iLegionExperienceMod));
	}
	else
	{
		return m_iExperiencePercent;
	}
	// Sanguo Mod Hero, end
}

void CvUnit::changeExperiencePercent(int iChange)
{
	if (iChange != 0)
	{
		m_iExperiencePercent += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getKamikazePercent() const
{
	return m_iKamikazePercent;
}

void CvUnit::changeKamikazePercent(int iChange)
{
	if (iChange != 0)
	{
		m_iKamikazePercent += iChange;

		setInfoBarDirty(true);
	}
}

DirectionTypes CvUnit::getFacingDirection(bool checkLineOfSightProperty) const
{
	if (checkLineOfSightProperty)
	{
		if (m_pUnitInfo->isLineOfSight())
		{
			return m_eFacingDirection; //only look in facing direction
		}
		else
		{
			return NO_DIRECTION; //look in all directions
		}
	}
	else
	{
		return m_eFacingDirection;
	}
}

void CvUnit::setFacingDirection(DirectionTypes eFacingDirection)
{
	if (eFacingDirection != m_eFacingDirection)
	{
		if (m_pUnitInfo->isLineOfSight())
		{
			//remove old fog
			plot()->changeAdjacentSight(getTeam(), visibilityRange(), false, this, true);

			//change direction
			m_eFacingDirection = eFacingDirection;

			//clear new fog
			plot()->changeAdjacentSight(getTeam(), visibilityRange(), true, this, true);

			gDLL->getInterfaceIFace()->setDirty(ColoredPlots_DIRTY_BIT, true);
		}
		else
		{
			m_eFacingDirection = eFacingDirection;
		}

		//update formation
		NotifyEntity(NO_MISSION);
	}
}

void CvUnit::rotateFacingDirectionClockwise()
{
	//change direction
	DirectionTypes eNewDirection = (DirectionTypes) ((m_eFacingDirection + 1) % NUM_DIRECTION_TYPES);
	setFacingDirection(eNewDirection);
}

void CvUnit::rotateFacingDirectionCounterClockwise()
{
	//change direction
	DirectionTypes eNewDirection = (DirectionTypes) ((m_eFacingDirection + NUM_DIRECTION_TYPES - 1) % NUM_DIRECTION_TYPES);
	setFacingDirection(eNewDirection);
}

int CvUnit::getImmobileTimer() const
{
	return m_iImmobileTimer;
}

void CvUnit::setImmobileTimer(int iNewValue)
{
	if (iNewValue != getImmobileTimer())
	{
		m_iImmobileTimer = iNewValue;

		setInfoBarDirty(true);
	}
}

void CvUnit::changeImmobileTimer(int iChange)
{
	if (iChange != 0)
	{
		setImmobileTimer(std::max(0, getImmobileTimer() + iChange));
	}
}

bool CvUnit::isMadeAttack() const
{
	return m_bMadeAttack;
}


void CvUnit::setMadeAttack(bool bNewValue)
{
	m_bMadeAttack = bNewValue;
}


bool CvUnit::isMadeInterception() const
{
	return m_bMadeInterception;
}


void CvUnit::setMadeInterception(bool bNewValue)
{
	m_bMadeInterception = bNewValue;
}


bool CvUnit::isPromotionReady() const
{
	return m_bPromotionReady;
}


void CvUnit::setPromotionReady(bool bNewValue)
{
	if (isPromotionReady() != bNewValue)
	{
		m_bPromotionReady = bNewValue;

		if (m_bPromotionReady)
		{
			getGroup()->setAutomateType(NO_AUTOMATE);
			getGroup()->clearMissionQueue();
			getGroup()->setActivityType(ACTIVITY_AWAKE);
		}

		gDLL->getEntityIFace()->showPromotionGlow(getUnitEntity(), bNewValue);

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(SelectionButtons_DIRTY_BIT, true);
		}
	}
}


void CvUnit::testPromotionReady()
{
	setPromotionReady((getExperience() >= experienceNeeded()) && canAcquirePromotionAny());
}


bool CvUnit::isDelayedDeath() const
{
	return m_bDeathDelay;
}


void CvUnit::startDelayedDeath()
{
	m_bDeathDelay = true;
}


// Returns true if killed...
bool CvUnit::doDelayedDeath()
{
	if (m_bDeathDelay && !isFighting())
	{
		kill(false);
		return true;
	}

	return false;
}


bool CvUnit::isCombatFocus() const
{
	return m_bCombatFocus;
}


bool CvUnit::isInfoBarDirty() const
{
	return m_bInfoBarDirty;
}


void CvUnit::setInfoBarDirty(bool bNewValue)
{
	m_bInfoBarDirty = bNewValue;
}

bool CvUnit::isBlockading() const
{
	return m_bBlockading;
}

void CvUnit::setBlockading(bool bNewValue)
{
	if (bNewValue != isBlockading())
	{
		m_bBlockading = bNewValue;

		updatePlunder(isBlockading() ? 1 : -1, true);
	}
}

void CvUnit::collectBlockadeGold()
{
	if (plot()->getTeam() == getTeam())
	{
		return;
	}

	int iBlockadeRange = GC.getDefineINT("SHIP_BLOCKADE_RANGE");

	for (int i = -iBlockadeRange; i <= iBlockadeRange; ++i)
	{
		for (int j = -iBlockadeRange; j <= iBlockadeRange; ++j)
		{
			CvPlot* pLoopPlot = ::plotXY(getX_INLINE(), getY_INLINE(), i, j);

			if (NULL != pLoopPlot && pLoopPlot->isRevealed(getTeam(), false))
			{
				CvCity* pCity = pLoopPlot->getPlotCity();

				if (NULL != pCity && !pCity->isPlundered() && isEnemy(pCity->getTeam()) && !atWar(pCity->getTeam(), getTeam()))
				{
					if (pCity->area() == area() || pCity->plot()->isAdjacentToArea(area()))
					{
						int iGold = pCity->calculateTradeProfit(pCity) * pCity->getTradeRoutes();
						if (iGold > 0)
						{
							pCity->setPlundered(true);
							GET_PLAYER(getOwnerINLINE()).changeGold(iGold);
							GET_PLAYER(pCity->getOwnerINLINE()).changeGold(-iGold);

							CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_TRADE_ROUTE_PLUNDERED", getNameKey(), pCity->getNameKey(), iGold);
							gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BUILD_BANK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());

							szBuffer = gDLL->getText("TXT_KEY_MISC_TRADE_ROUTE_PLUNDER", getNameKey(), pCity->getNameKey(), iGold);
							gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BUILD_BANK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE());
						}
					}
				}
			}
		}
	}
}


PlayerTypes CvUnit::getOwner() const
{
	return getOwnerINLINE();
}

PlayerTypes CvUnit::getVisualOwner(TeamTypes eForTeam) const
{
	if (NO_TEAM == eForTeam)
	{
		eForTeam = GC.getGameINLINE().getActiveTeam();
	}

	if (getTeam() != eForTeam && eForTeam != BARBARIAN_TEAM)
	{
		// Sanguo Mod Hero, start, added by poyuzhe 02.05.09
		// commented out 1 line
		if (isHiddenNationality())
		// Sanguo Mod Hero, end
		{
			if (!plot()->isCity(true, getTeam()))
			{
				return BARBARIAN_PLAYER;
			}
		}
	}

	return getOwnerINLINE();
}


PlayerTypes CvUnit::getCombatOwner(TeamTypes eForTeam, const CvPlot* pPlot) const
{
	if (eForTeam != NO_TEAM && getTeam() != eForTeam && eForTeam != BARBARIAN_TEAM)
	{
		if (isAlwaysHostile(pPlot))
		{
			return BARBARIAN_PLAYER;
		}
	}

	return getOwnerINLINE();
}

TeamTypes CvUnit::getTeam() const
{
	return GET_PLAYER(getOwnerINLINE()).getTeam();
}


PlayerTypes CvUnit::getCapturingPlayer() const
{
	return m_eCapturingPlayer;
}


void CvUnit::setCapturingPlayer(PlayerTypes eNewValue)
{
	m_eCapturingPlayer = eNewValue;
}


const UnitTypes CvUnit::getUnitType() const
{
	return m_eUnitType;
}

CvUnitInfo &CvUnit::getUnitInfo() const
{
	return *m_pUnitInfo;
}


UnitClassTypes CvUnit::getUnitClassType() const
{
	return (UnitClassTypes)m_pUnitInfo->getUnitClassType();
}

const UnitTypes CvUnit::getLeaderUnitType() const
{
	return m_eLeaderUnitType;
}

void CvUnit::setLeaderUnitType(UnitTypes leaderUnitType)
{
	if(m_eLeaderUnitType != leaderUnitType)
	{
		m_eLeaderUnitType = leaderUnitType;
		reloadEntity();
	}
}

CvUnit* CvUnit::getCombatUnit() const
{
	return getUnit(m_combatUnit);
}


void CvUnit::setCombatUnit(CvUnit* pCombatUnit, bool bAttacking)
{
	if (isCombatFocus())
	{
		gDLL->getInterfaceIFace()->setCombatFocus(false);
	}

	if (pCombatUnit != NULL)
	{
		if (bAttacking)
		{
			if (GC.getLogging())
			{
				if (gDLL->getChtLvl() > 0)
				{
					// Log info about this combat...
					char szOut[1024];
					sprintf( szOut, "*** KOMBAT!\n     ATTACKER: Player %d Unit %d (%S's %S), CombatStrength=%d\n     DEFENDER: Player %d Unit %d (%S's %S), CombatStrength=%d\n",
						getOwnerINLINE(), getID(), GET_PLAYER(getOwnerINLINE()).getName(), getName().GetCString(), currCombatStr(NULL, NULL),
						pCombatUnit->getOwnerINLINE(), pCombatUnit->getID(), GET_PLAYER(pCombatUnit->getOwnerINLINE()).getName(), pCombatUnit->getName().GetCString(), pCombatUnit->currCombatStr(pCombatUnit->plot(), this));
					gDLL->messageControlLog(szOut);
				}
			}

			if (getDomainType() == DOMAIN_LAND
				&& !m_pUnitInfo->isIgnoreBuildingDefense()
				&& pCombatUnit->plot()->getPlotCity()
				&& pCombatUnit->plot()->getPlotCity()->getBuildingDefense() > 0
				&& cityAttackModifier() >= GC.getDefineINT("MIN_CITY_ATTACK_MODIFIER_FOR_SIEGE_TOWER"))
			{
				CvDLLEntity::SetSiegeTower(true);
			}
		}

		FAssertMsg(getCombatUnit() == NULL, "Combat Unit is not expected to be assigned");
		FAssertMsg(!(plot()->isFighting()), "(plot()->isFighting()) did not return false as expected");
		m_bCombatFocus = (bAttacking && !(gDLL->getInterfaceIFace()->isFocusedWidget()) && ((getOwnerINLINE() == GC.getGameINLINE().getActivePlayer()) || ((pCombatUnit->getOwnerINLINE() == GC.getGameINLINE().getActivePlayer()) && !(GC.getGameINLINE().isMPOption(MPOPTION_SIMULTANEOUS_TURNS)))));
		m_combatUnit = pCombatUnit->getIDInfo();
		setCombatFirstStrikes((pCombatUnit->immuneToFirstStrikes()) ? 0 : (firstStrikes() + GC.getGameINLINE().getSorenRandNum(chanceFirstStrikes() + 1, "First Strike")));
	}
	else
	{
		if(getCombatUnit() != NULL)
		{
			FAssertMsg(getCombatUnit() != NULL, "getCombatUnit() is not expected to be equal with NULL");
			FAssertMsg(plot()->isFighting(), "plot()->isFighting is expected to be true");
			m_bCombatFocus = false;
			m_combatUnit.reset();
			setCombatFirstStrikes(0);

			if (IsSelected())
			{
				gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
			}

			if (plot() == gDLL->getInterfaceIFace()->getSelectionPlot())
			{
				gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);
			}

			CvDLLEntity::SetSiegeTower(false);
		}
	}

	setCombatTimer(0);
	setInfoBarDirty(true);

	if (isCombatFocus())
	{
		gDLL->getInterfaceIFace()->setCombatFocus(true);
	}
}


CvUnit* CvUnit::getTransportUnit() const
{
	return getUnit(m_transportUnit);
}


bool CvUnit::isCargo() const
{
	return (getTransportUnit() != NULL);
}


void CvUnit::setTransportUnit(CvUnit* pTransportUnit)
{
	CvUnit* pOldTransportUnit;

	pOldTransportUnit = getTransportUnit();

	if (pOldTransportUnit != pTransportUnit)
	{
		if (pOldTransportUnit != NULL)
		{
			pOldTransportUnit->changeCargo(-1);
		}

		if (pTransportUnit != NULL)
		{
			FAssertMsg(pTransportUnit->cargoSpaceAvailable(getSpecialUnitType(), getDomainType()) > 0, "Cargo space is expected to be available");

			joinGroup(NULL, true); // Because what if a group of 3 tries to get in a transport which can hold 2...

			m_transportUnit = pTransportUnit->getIDInfo();

			if (getDomainType() != DOMAIN_AIR)
			{
				getGroup()->setActivityType(ACTIVITY_SLEEP);
			}

			if (GC.getGameINLINE().isFinalInitialized())
			{
				finishMoves();
			}

			pTransportUnit->changeCargo(1);
			pTransportUnit->getGroup()->setActivityType(ACTIVITY_AWAKE);
		}
		else
		{
			m_transportUnit.reset();

			getGroup()->setActivityType(ACTIVITY_AWAKE);
		}

#ifdef _DEBUG
		std::vector<CvUnit*> aCargoUnits;
		if (pOldTransportUnit != NULL)
		{
			pOldTransportUnit->getCargoUnits(aCargoUnits);
		}
		if (pTransportUnit != NULL)
		{
			pTransportUnit->getCargoUnits(aCargoUnits);
		}
#endif

	}
}


int CvUnit::getExtraDomainModifier(DomainTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < NUM_DOMAIN_TYPES, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiExtraDomainModifier[eIndex];
}


void CvUnit::changeExtraDomainModifier(DomainTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < NUM_DOMAIN_TYPES, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiExtraDomainModifier[eIndex] = (m_aiExtraDomainModifier[eIndex] + iChange);
}


const CvWString CvUnit::getName(uint uiForm) const
{
	CvWString szBuffer;

	if (m_szName.empty())
	{
		// Sanguo Mod Hero, start, added by poyuzhe 07.19.09
		if (getHeroType() != NO_HERO && getLegionHero() == NO_HERO)
		{
		    return GC.getHeroInfo(getHeroType()).getDescription(uiForm);
		}
		// Sanguo Mod Hero, end
		return m_pUnitInfo->getDescription(uiForm);
	}

	szBuffer.Format(L"%s (%s)", m_szName.GetCString(), m_pUnitInfo->getDescription(uiForm));

	return szBuffer;
}


const wchar* CvUnit::getNameKey() const
{
	if (m_szName.empty())
	{
		// Sanguo Mod Hero, start, added by poyuzhe 07.19.09
		if (getHeroType() != NO_HERO && getLegionHero() == NO_HERO)
		{
		    return GC.getHeroInfo(getHeroType()).getTextKeyWide();
		}
		// Sanguo Mod Hero, end
		return m_pUnitInfo->getTextKeyWide();
	}
	else
	{
		return m_szName.GetCString();
	}
}


const CvWString& CvUnit::getNameNoDesc() const
{
	return m_szName;
}


void CvUnit::setName(CvWString szNewValue)
{
	gDLL->stripSpecialCharacters(szNewValue);

	// Sanguo Mod Hero, start, added by poyuzhe 08.24.09
	// hero rename
	// m_szName = szNewValue;
	if (isNakedHero() && GC.getHeroInfo(getHeroType()).isRandom())
	{
		GC.getHeroInfo(getHeroType()).reset();
		GC.getHeroInfo(getHeroType()).setText(szNewValue);
		GC.getHeroInfo(getHeroType()).setStrategy(gDLL->getText("TXT_KEY_HERO_GENERIC_STRATEGY", GC.getHeroInfo(getHeroType()).getDescription()));
	}
	else
	{
		m_szName = szNewValue;
	}
	// Sanguo Mod Hero, end

	if (IsSelected())
	{
		gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
	}
}


std::string CvUnit::getScriptData() const
{
	return m_szScriptData;
}


void CvUnit::setScriptData(std::string szNewValue)
{
	m_szScriptData = szNewValue;
}


int CvUnit::getTerrainDoubleMoveCount(TerrainTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiTerrainDoubleMoveCount[eIndex];
}


bool CvUnit::isTerrainDoubleMove(TerrainTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return (getTerrainDoubleMoveCount(eIndex) > 0);
}


void CvUnit::changeTerrainDoubleMoveCount(TerrainTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	m_paiTerrainDoubleMoveCount[eIndex] = (m_paiTerrainDoubleMoveCount[eIndex] + iChange);
	FAssert(getTerrainDoubleMoveCount(eIndex) >= 0);
}


int CvUnit::getFeatureDoubleMoveCount(FeatureTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiFeatureDoubleMoveCount[eIndex];
}


bool CvUnit::isFeatureDoubleMove(FeatureTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return (getFeatureDoubleMoveCount(eIndex) > 0);
}


void CvUnit::changeFeatureDoubleMoveCount(FeatureTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	m_paiFeatureDoubleMoveCount[eIndex] = (m_paiFeatureDoubleMoveCount[eIndex] + iChange);
	FAssert(getFeatureDoubleMoveCount(eIndex) >= 0);
}


int CvUnit::getExtraTerrainAttackPercent(TerrainTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiExtraTerrainAttackPercent[eIndex];
}


void CvUnit::changeExtraTerrainAttackPercent(TerrainTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");

	if (iChange != 0)
	{
		m_paiExtraTerrainAttackPercent[eIndex] += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraTerrainDefensePercent(TerrainTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiExtraTerrainDefensePercent[eIndex];
}


void CvUnit::changeExtraTerrainDefensePercent(TerrainTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumTerrainInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");

	if (iChange != 0)
	{
		m_paiExtraTerrainDefensePercent[eIndex] += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraFeatureAttackPercent(FeatureTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiExtraFeatureAttackPercent[eIndex];
}


void CvUnit::changeExtraFeatureAttackPercent(FeatureTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");

	if (iChange != 0)
	{
		m_paiExtraFeatureAttackPercent[eIndex] += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraFeatureDefensePercent(FeatureTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiExtraFeatureDefensePercent[eIndex];
}


void CvUnit::changeExtraFeatureDefensePercent(FeatureTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumFeatureInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");

	if (iChange != 0)
	{
		m_paiExtraFeatureDefensePercent[eIndex] += iChange;

		setInfoBarDirty(true);
	}
}

int CvUnit::getExtraUnitCombatModifier(UnitCombatTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumUnitCombatInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_paiExtraUnitCombatModifier[eIndex];
}


void CvUnit::changeExtraUnitCombatModifier(UnitCombatTypes eIndex, int iChange)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumUnitCombatInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	m_paiExtraUnitCombatModifier[eIndex] = (m_paiExtraUnitCombatModifier[eIndex] + iChange);
}


bool CvUnit::canAcquirePromotion(PromotionTypes ePromotion) const
{
	FAssertMsg(ePromotion >= 0, "ePromotion is expected to be non-negative (invalid Index)");
	FAssertMsg(ePromotion < GC.getNumPromotionInfos(), "ePromotion is expected to be within maximum bounds (invalid Index)");

	if (isHasPromotion(ePromotion))
	{
		return false;
	}

	if (GC.getPromotionInfo(ePromotion).getPrereqPromotion() != NO_PROMOTION)
	{
		if (!isHasPromotion((PromotionTypes)(GC.getPromotionInfo(ePromotion).getPrereqPromotion())))
		{
			return false;
		}
	}

	if (GC.getPromotionInfo(ePromotion).getPrereqOrPromotion1() != NO_PROMOTION)
	{
		if (!isHasPromotion((PromotionTypes)(GC.getPromotionInfo(ePromotion).getPrereqOrPromotion1())))
		{
			if ((GC.getPromotionInfo(ePromotion).getPrereqOrPromotion2() == NO_PROMOTION) || !isHasPromotion((PromotionTypes)(GC.getPromotionInfo(ePromotion).getPrereqOrPromotion2())))
			{
				return false;
			}
		}
	}

	if (GC.getPromotionInfo(ePromotion).getTechPrereq() != NO_TECH)
	{
		if (!(GET_TEAM(getTeam()).isHasTech((TechTypes)(GC.getPromotionInfo(ePromotion).getTechPrereq()))))
		{
			return false;
		}
	}

	if (GC.getPromotionInfo(ePromotion).getStateReligionPrereq() != NO_RELIGION)
	{
		if (GET_PLAYER(getOwnerINLINE()).getStateReligion() != GC.getPromotionInfo(ePromotion).getStateReligionPrereq())
		{
			return false;
		}
	}

	if (!isPromotionValid(ePromotion))
	{
		return false;
	}

	return true;
}

bool CvUnit::isPromotionValid(PromotionTypes ePromotion) const
{
	if (!::isPromotionValid(ePromotion, getUnitType(), true))
	{
		return false;
	}

	CvPromotionInfo& promotionInfo = GC.getPromotionInfo(ePromotion);

	if (promotionInfo.getWithdrawalChange() + m_pUnitInfo->getWithdrawalProbability() + getExtraWithdrawal() > GC.getDefineINT("MAX_WITHDRAWAL_PROBABILITY"))
	{
		return false;
	}

	if (promotionInfo.getInterceptChange() + maxInterceptionProbability() > GC.getDefineINT("MAX_INTERCEPTION_PROBABILITY"))
	{
		return false;
	}

	if (promotionInfo.getEvasionChange() + evasionProbability() > GC.getDefineINT("MAX_EVASION_PROBABILITY"))
	{
		return false;
	}

	return true;
}


bool CvUnit::canAcquirePromotionAny() const
{
	int iI;

	for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	{
		if (canAcquirePromotion((PromotionTypes)iI))
		{
			return true;
		}
	}

	return false;
}


bool CvUnit::isHasPromotion(PromotionTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < GC.getNumPromotionInfos(), "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_pabHasPromotion[eIndex];
}


void CvUnit::setHasPromotion(PromotionTypes eIndex, bool bNewValue)
{
	int iChange;
	int iI;

	// Sanguo Mod Hero, start, added by poyuzhe 03.10.09
	if (GC.getPromotionInfo(eIndex).getLimitToUnitCombatType() != NO_UNITCOMBAT)
	{
		if (getUnitCombatType() != GC.getPromotionInfo(eIndex).getLimitToUnitCombatType())
		{
			return;
		}
	}
	if (GC.getPromotionInfo(eIndex).getMovesChange() < 0 && bNewValue)
	{
		if (baseMoves() + GC.getPromotionInfo(eIndex).getMovesChange() <= 0)
		{
			return;
		}
	}
	if (GC.getPromotionInfo(eIndex).getActivePromotionPrereq() != NO_PROMOTION)
	{
		if (!isHasPromotion((PromotionTypes)GC.getPromotionInfo(eIndex).getActivePromotionPrereq()))
		{
			return;
		}
	}
    // Sanguo Mod Hero, end

	if (isHasPromotion(eIndex) != bNewValue)
	{
		m_pabHasPromotion[eIndex] = bNewValue;

		iChange = ((isHasPromotion(eIndex)) ? 1 : -1);

		changeBlitzCount((GC.getPromotionInfo(eIndex).isBlitz()) ? iChange : 0);
		changeAmphibCount((GC.getPromotionInfo(eIndex).isAmphib()) ? iChange : 0);
		changeRiverCount((GC.getPromotionInfo(eIndex).isRiver()) ? iChange : 0);
		changeEnemyRouteCount((GC.getPromotionInfo(eIndex).isEnemyRoute()) ? iChange : 0);
		changeAlwaysHealCount((GC.getPromotionInfo(eIndex).isAlwaysHeal()) ? iChange : 0);
		changeHillsDoubleMoveCount((GC.getPromotionInfo(eIndex).isHillsDoubleMove()) ? iChange : 0);
		changeImmuneToFirstStrikesCount((GC.getPromotionInfo(eIndex).isImmuneToFirstStrikes()) ? iChange : 0);

		changeExtraVisibilityRange(GC.getPromotionInfo(eIndex).getVisibilityChange() * iChange);
		changeExtraMoves(GC.getPromotionInfo(eIndex).getMovesChange() * iChange);
		changeExtraMoveDiscount(GC.getPromotionInfo(eIndex).getMoveDiscountChange() * iChange);
		changeExtraAirRange(GC.getPromotionInfo(eIndex).getAirRangeChange() * iChange);
		changeExtraIntercept(GC.getPromotionInfo(eIndex).getInterceptChange() * iChange);
		changeExtraEvasion(GC.getPromotionInfo(eIndex).getEvasionChange() * iChange);
		changeExtraFirstStrikes(GC.getPromotionInfo(eIndex).getFirstStrikesChange() * iChange);
		changeExtraChanceFirstStrikes(GC.getPromotionInfo(eIndex).getChanceFirstStrikesChange() * iChange);
		changeExtraWithdrawal(GC.getPromotionInfo(eIndex).getWithdrawalChange() * iChange);
		changeExtraCollateralDamage(GC.getPromotionInfo(eIndex).getCollateralDamageChange() * iChange);
		changeExtraBombardRate(GC.getPromotionInfo(eIndex).getBombardRateChange() * iChange);
		changeExtraEnemyHeal(GC.getPromotionInfo(eIndex).getEnemyHealChange() * iChange);
		changeExtraNeutralHeal(GC.getPromotionInfo(eIndex).getNeutralHealChange() * iChange);
		changeExtraFriendlyHeal(GC.getPromotionInfo(eIndex).getFriendlyHealChange() * iChange);
		changeSameTileHeal(GC.getPromotionInfo(eIndex).getSameTileHealChange() * iChange);
		changeAdjacentTileHeal(GC.getPromotionInfo(eIndex).getAdjacentTileHealChange() * iChange);
		changeExtraCombatPercent(GC.getPromotionInfo(eIndex).getCombatPercent() * iChange);
		changeExtraCityAttackPercent(GC.getPromotionInfo(eIndex).getCityAttackPercent() * iChange);
		changeExtraCityDefensePercent(GC.getPromotionInfo(eIndex).getCityDefensePercent() * iChange);
		changeExtraHillsAttackPercent(GC.getPromotionInfo(eIndex).getHillsAttackPercent() * iChange);
		changeExtraHillsDefensePercent(GC.getPromotionInfo(eIndex).getHillsDefensePercent() * iChange);
		changeRevoltProtection(GC.getPromotionInfo(eIndex).getRevoltProtection() * iChange);
		changeCollateralDamageProtection(GC.getPromotionInfo(eIndex).getCollateralDamageProtection() * iChange);
		changePillageChange(GC.getPromotionInfo(eIndex).getPillageChange() * iChange);
		changeUpgradeDiscount(GC.getPromotionInfo(eIndex).getUpgradeDiscount() * iChange);
		changeExperiencePercent(GC.getPromotionInfo(eIndex).getExperiencePercent() * iChange);
		changeKamikazePercent((GC.getPromotionInfo(eIndex).getKamikazePercent()) * iChange);
		changeCargoSpace(GC.getPromotionInfo(eIndex).getCargoChange() * iChange);

		// Sanguo Mod Hero, start, added by poyuzhe 02.04.09
        if (isRanged())
        {
            if (GC.getPromotionInfo(eIndex).canArcherBombard())
            {
                if (iChange == 1)
                {
                    m_iDCMBombRange += GC.getDefineINT("HERO_BASE_RANGE_BOMBARD_RANGE");
                }
                else
                {
                    m_iDCMBombRange = 0;
                }
            }

            if (iChange == 1)
            {
                m_iDCMBombRange += GC.getPromotionInfo(eIndex).getArcherBombardRangeChange();
            }
            else if (m_iDCMBombRange != 0)
            {
                m_iDCMBombRange -= GC.getPromotionInfo(eIndex).getArcherBombardRangeChange();
            }
        }

        if (GC.getPromotionInfo(eIndex).getInvisibleType() != NO_INVISIBLE)
        {
            m_iInvisibleType = ((iChange == 1) ? ((InvisibleTypes)GC.getPromotionInfo(eIndex).getInvisibleType()) : NO_INVISIBLE);
        }

        if (GC.getPromotionInfo(eIndex).isSeaLord() && getDomainType() == DOMAIN_SEA)
        {
            if (iChange == 1)
            {
                changeHiddenNationalityCount(1);
                changeAlwaysHostileCount(1);
                AI_setUnitAIType(UNITAI_PIRATE_SEA);
            }
            else
            {
				changeHiddenNationalityCount(-1);
                changeAlwaysHostileCount(-1);
				AI_setUnitAIType((UnitAITypes)GC.getUnitInfo(getUnitType()).getDefaultUnitAIType());
			}
        }

        changeSurroundingEnemyHealthChange(GC.getPromotionInfo(eIndex).getSurroundingEnemyHealthChange() * iChange);
        changeBaseCombatStrength(GC.getPromotionInfo(eIndex).getStrengthIncrease() * iChange);
		changeDefendFromChargeCount(GC.getPromotionInfo(eIndex).canDefendAgainstCharge() ? iChange : 0);
		changeAdditionalAttackChance(GC.getPromotionInfo(eIndex).getAdditionalAttackChance() * iChange);
        changeExtraCollateralDamage(GC.getPromotionInfo(eIndex).getFlankingDamageIncreasePercent() * iChange);
		changeWarGodCount(GC.getPromotionInfo(eIndex).isWarGod() ? iChange : 0);
		changeIgnoreDefenderTerrainCount(GC.getPromotionInfo(eIndex).isIgnoreDefenderTerrain() ? iChange : 0);
		changePlotDefenseModifier(GC.getPromotionInfo(eIndex).getDefenseBuildingEffectModifier() * iChange);
		changeWaterSideStrengthModifier(GC.getPromotionInfo(eIndex).getBesideRiverStrengthModifier() * iChange);
		// Sanguo Mod Hero, end

		for (iI = 0; iI < GC.getNumTerrainInfos(); iI++)
		{
			changeExtraTerrainAttackPercent(((TerrainTypes)iI), (GC.getPromotionInfo(eIndex).getTerrainAttackPercent(iI) * iChange));
			changeExtraTerrainDefensePercent(((TerrainTypes)iI), (GC.getPromotionInfo(eIndex).getTerrainDefensePercent(iI) * iChange));
			changeTerrainDoubleMoveCount(((TerrainTypes)iI), ((GC.getPromotionInfo(eIndex).getTerrainDoubleMove(iI)) ? iChange : 0));
		}

		for (iI = 0; iI < GC.getNumFeatureInfos(); iI++)
		{
			changeExtraFeatureAttackPercent(((FeatureTypes)iI), (GC.getPromotionInfo(eIndex).getFeatureAttackPercent(iI) * iChange));
			changeExtraFeatureDefensePercent(((FeatureTypes)iI), (GC.getPromotionInfo(eIndex).getFeatureDefensePercent(iI) * iChange));
			changeFeatureDoubleMoveCount(((FeatureTypes)iI), ((GC.getPromotionInfo(eIndex).getFeatureDoubleMove(iI)) ? iChange : 0));
		}

		for (iI = 0; iI < GC.getNumUnitCombatInfos(); iI++)
		{
			changeExtraUnitCombatModifier(((UnitCombatTypes)iI), (GC.getPromotionInfo(eIndex).getUnitCombatModifierPercent(iI) * iChange));
		}

		for (iI = 0; iI < NUM_DOMAIN_TYPES; iI++)
		{
			changeExtraDomainModifier(((DomainTypes)iI), (GC.getPromotionInfo(eIndex).getDomainModifierPercent(iI) * iChange));
		}

		if (IsSelected())
		{
			gDLL->getInterfaceIFace()->setDirty(SelectionButtons_DIRTY_BIT, true);
			gDLL->getInterfaceIFace()->setDirty(InfoPane_DIRTY_BIT, true);
		}

		//update graphics
		gDLL->getEntityIFace()->updatePromotionLayers(getUnitEntity());
	}
}


int CvUnit::getSubUnitCount() const
{
	return m_pUnitInfo->getGroupSize();
}


int CvUnit::getSubUnitsAlive() const
{
	return getSubUnitsAlive( getDamage());
}


int CvUnit::getSubUnitsAlive(int iDamage) const
{
	if (iDamage >= maxHitPoints())
	{
		return 0;
	}
	else
	{
		return std::max(1, (((m_pUnitInfo->getGroupSize() * (maxHitPoints() - iDamage)) + (maxHitPoints() / ((m_pUnitInfo->getGroupSize() * 2) + 1))) / maxHitPoints()));
	}
}
// returns true if unit can initiate a war action with plot (possibly by declaring war)
bool CvUnit::potentialWarAction(const CvPlot* pPlot) const
{
	TeamTypes ePlotTeam = pPlot->getTeam();
	TeamTypes eUnitTeam = getTeam();

	if (ePlotTeam == NO_TEAM)
	{
		return false;
	}

	if (isEnemy(ePlotTeam, pPlot))
	{
		return true;
	}

	if (getGroup()->AI_isDeclareWar(pPlot) && GET_TEAM(eUnitTeam).AI_getWarPlan(ePlotTeam) != NO_WARPLAN)
	{
		return true;
	}

	return false;
}

void CvUnit::read(FDataStreamBase* pStream)
{
	// Init data before load
	reset();

	uint uiFlag=0;
	pStream->Read(&uiFlag);	// flags for expansion

	pStream->Read(&m_iID);
	pStream->Read(&m_iGroupID);
	pStream->Read(&m_iHotKeyNumber);
	pStream->Read(&m_iX);
	pStream->Read(&m_iY);
	pStream->Read(&m_iLastMoveTurn);
	pStream->Read(&m_iReconX);
	pStream->Read(&m_iReconY);
	pStream->Read(&m_iGameTurnCreated);
	pStream->Read(&m_iDamage);
	pStream->Read(&m_iMoves);
	pStream->Read(&m_iExperience);
	pStream->Read(&m_iLevel);
	pStream->Read(&m_iCargo);
	pStream->Read(&m_iCargoCapacity);
	pStream->Read(&m_iAttackPlotX);
	pStream->Read(&m_iAttackPlotY);
	pStream->Read(&m_iCombatTimer);
	pStream->Read(&m_iCombatFirstStrikes);
	if (uiFlag < 2)
	{
		int iCombatDamage;
		pStream->Read(&iCombatDamage);
	}
	pStream->Read(&m_iFortifyTurns);
	pStream->Read(&m_iBlitzCount);
	pStream->Read(&m_iAmphibCount);
	pStream->Read(&m_iRiverCount);
	pStream->Read(&m_iEnemyRouteCount);
	pStream->Read(&m_iAlwaysHealCount);
	pStream->Read(&m_iHillsDoubleMoveCount);
	pStream->Read(&m_iImmuneToFirstStrikesCount);
	pStream->Read(&m_iExtraVisibilityRange);
	pStream->Read(&m_iExtraMoves);
	pStream->Read(&m_iExtraMoveDiscount);
	pStream->Read(&m_iExtraAirRange);
	pStream->Read(&m_iExtraIntercept);
	pStream->Read(&m_iExtraEvasion);
	pStream->Read(&m_iExtraFirstStrikes);
	pStream->Read(&m_iExtraChanceFirstStrikes);
	pStream->Read(&m_iExtraWithdrawal);
	pStream->Read(&m_iExtraCollateralDamage);
	pStream->Read(&m_iExtraBombardRate);
	pStream->Read(&m_iExtraEnemyHeal);
	pStream->Read(&m_iExtraNeutralHeal);
	pStream->Read(&m_iExtraFriendlyHeal);
	pStream->Read(&m_iSameTileHeal);
	pStream->Read(&m_iAdjacentTileHeal);
	pStream->Read(&m_iExtraCombatPercent);
	pStream->Read(&m_iExtraCityAttackPercent);
	pStream->Read(&m_iExtraCityDefensePercent);
	pStream->Read(&m_iExtraHillsAttackPercent);
	pStream->Read(&m_iExtraHillsDefensePercent);
	pStream->Read(&m_iRevoltProtection);
	pStream->Read(&m_iCollateralDamageProtection);
	pStream->Read(&m_iPillageChange);
	pStream->Read(&m_iUpgradeDiscount);
	pStream->Read(&m_iExperiencePercent);
	pStream->Read(&m_iKamikazePercent);
	pStream->Read(&m_iBaseCombat);
	pStream->Read((int*)&m_eFacingDirection);
	pStream->Read(&m_iImmobileTimer);

	pStream->Read(&m_bMadeAttack);
	pStream->Read(&m_bMadeInterception);
	pStream->Read(&m_bPromotionReady);
	pStream->Read(&m_bDeathDelay);
	pStream->Read(&m_bCombatFocus);
	// m_bInfoBarDirty not saved...
	pStream->Read(&m_bBlockading);
	if (uiFlag > 0)
	{
		pStream->Read(&m_bAirCombat);
	}

	pStream->Read((int*)&m_eOwner);
	pStream->Read((int*)&m_eCapturingPlayer);
	pStream->Read((int*)&m_eUnitType);
	FAssert(NO_UNIT != m_eUnitType);
	m_pUnitInfo = (NO_UNIT != m_eUnitType) ? &GC.getUnitInfo(m_eUnitType) : NULL;
	pStream->Read((int*)&m_eLeaderUnitType);

	pStream->Read((int*)&m_combatUnit.eOwner);
	pStream->Read(&m_combatUnit.iID);
	pStream->Read((int*)&m_transportUnit.eOwner);
	pStream->Read(&m_transportUnit.iID);

	pStream->Read(NUM_DOMAIN_TYPES, m_aiExtraDomainModifier);

	pStream->ReadString(m_szName);
	pStream->ReadString(m_szScriptData);

	pStream->Read(GC.getNumPromotionInfos(), m_pabHasPromotion);

	pStream->Read(GC.getNumTerrainInfos(), m_paiTerrainDoubleMoveCount);
	pStream->Read(GC.getNumFeatureInfos(), m_paiFeatureDoubleMoveCount);
	pStream->Read(GC.getNumTerrainInfos(), m_paiExtraTerrainAttackPercent);
	pStream->Read(GC.getNumTerrainInfos(), m_paiExtraTerrainDefensePercent);
	pStream->Read(GC.getNumFeatureInfos(), m_paiExtraFeatureAttackPercent);
	pStream->Read(GC.getNumFeatureInfos(), m_paiExtraFeatureDefensePercent);
	pStream->Read(GC.getNumUnitCombatInfos(), m_paiExtraUnitCombatModifier);

	// Sanguo Mod Hero, start, added by poyuzhe 01.08.09
	pStream->Read((int*)&m_eHeroType);
	pStream->Read((int*)&m_eLegionHero);
    pStream->Read((int*)&m_eLeaderHero);
	pStream->Read(&m_iTempStrengthModifier);
	pStream->Read(&m_iSurroundingEnemyHealthChange);
    pStream->Read(&m_iDCMBombRange);
    pStream->Read(&m_iDefendFromChargeCount);
    pStream->Read((int*)&m_iInvisibleType);
    pStream->Read(&m_iHiddenNationalityCount);
	pStream->Read(&m_iAlwaysHostileCount);
    pStream->Read(&m_iAdditionalAttackChance);
	pStream->Read(&m_iWarGodCount);
	pStream->Read(&m_iIgnoreDefenderTerrainCount);
	pStream->Read(&m_iPlotDefenseModifier);
	pStream->Read(&m_iWaterSideStrModifier);
	pStream->Read(&m_iGuardCount);
	pStream->Read(&m_iLegionTurn);
    // Sanguo Mod Hero, end

}


void CvUnit::write(FDataStreamBase* pStream)
{
	uint uiFlag=2;
	pStream->Write(uiFlag);		// flag for expansion

	pStream->Write(m_iID);
	pStream->Write(m_iGroupID);
	pStream->Write(m_iHotKeyNumber);
	pStream->Write(m_iX);
	pStream->Write(m_iY);
	pStream->Write(m_iLastMoveTurn);
	pStream->Write(m_iReconX);
	pStream->Write(m_iReconY);
	pStream->Write(m_iGameTurnCreated);
	pStream->Write(m_iDamage);
	pStream->Write(m_iMoves);
	pStream->Write(m_iExperience);
	pStream->Write(m_iLevel);
	pStream->Write(m_iCargo);
	pStream->Write(m_iCargoCapacity);
	pStream->Write(m_iAttackPlotX);
	pStream->Write(m_iAttackPlotY);
	pStream->Write(m_iCombatTimer);
	pStream->Write(m_iCombatFirstStrikes);
	pStream->Write(m_iFortifyTurns);
	pStream->Write(m_iBlitzCount);
	pStream->Write(m_iAmphibCount);
	pStream->Write(m_iRiverCount);
	pStream->Write(m_iEnemyRouteCount);
	pStream->Write(m_iAlwaysHealCount);
	pStream->Write(m_iHillsDoubleMoveCount);
	pStream->Write(m_iImmuneToFirstStrikesCount);
	pStream->Write(m_iExtraVisibilityRange);
	pStream->Write(m_iExtraMoves);
	pStream->Write(m_iExtraMoveDiscount);
	pStream->Write(m_iExtraAirRange);
	pStream->Write(m_iExtraIntercept);
	pStream->Write(m_iExtraEvasion);
	pStream->Write(m_iExtraFirstStrikes);
	pStream->Write(m_iExtraChanceFirstStrikes);
	pStream->Write(m_iExtraWithdrawal);
	pStream->Write(m_iExtraCollateralDamage);
	pStream->Write(m_iExtraBombardRate);
	pStream->Write(m_iExtraEnemyHeal);
	pStream->Write(m_iExtraNeutralHeal);
	pStream->Write(m_iExtraFriendlyHeal);
	pStream->Write(m_iSameTileHeal);
	pStream->Write(m_iAdjacentTileHeal);
	pStream->Write(m_iExtraCombatPercent);
	pStream->Write(m_iExtraCityAttackPercent);
	pStream->Write(m_iExtraCityDefensePercent);
	pStream->Write(m_iExtraHillsAttackPercent);
	pStream->Write(m_iExtraHillsDefensePercent);
	pStream->Write(m_iRevoltProtection);
	pStream->Write(m_iCollateralDamageProtection);
	pStream->Write(m_iPillageChange);
	pStream->Write(m_iUpgradeDiscount);
	pStream->Write(m_iExperiencePercent);
	pStream->Write(m_iKamikazePercent);
	pStream->Write(m_iBaseCombat);
	pStream->Write(m_eFacingDirection);
	pStream->Write(m_iImmobileTimer);

	pStream->Write(m_bMadeAttack);
	pStream->Write(m_bMadeInterception);
	pStream->Write(m_bPromotionReady);
	pStream->Write(m_bDeathDelay);
	pStream->Write(m_bCombatFocus);
	// m_bInfoBarDirty not saved...
	pStream->Write(m_bBlockading);
	pStream->Write(m_bAirCombat);

	pStream->Write(m_eOwner);
	pStream->Write(m_eCapturingPlayer);
	pStream->Write(m_eUnitType);
	pStream->Write(m_eLeaderUnitType);

	pStream->Write(m_combatUnit.eOwner);
	pStream->Write(m_combatUnit.iID);
	pStream->Write(m_transportUnit.eOwner);
	pStream->Write(m_transportUnit.iID);

	pStream->Write(NUM_DOMAIN_TYPES, m_aiExtraDomainModifier);

	pStream->WriteString(m_szName);
	pStream->WriteString(m_szScriptData);

	pStream->Write(GC.getNumPromotionInfos(), m_pabHasPromotion);

	pStream->Write(GC.getNumTerrainInfos(), m_paiTerrainDoubleMoveCount);
	pStream->Write(GC.getNumFeatureInfos(), m_paiFeatureDoubleMoveCount);
	pStream->Write(GC.getNumTerrainInfos(), m_paiExtraTerrainAttackPercent);
	pStream->Write(GC.getNumTerrainInfos(), m_paiExtraTerrainDefensePercent);
	pStream->Write(GC.getNumFeatureInfos(), m_paiExtraFeatureAttackPercent);
	pStream->Write(GC.getNumFeatureInfos(), m_paiExtraFeatureDefensePercent);
	pStream->Write(GC.getNumUnitCombatInfos(), m_paiExtraUnitCombatModifier);

	// Sanguo Mod Hero, start, added by poyuzhe 01.08.09
	pStream->Write(m_eHeroType);
	pStream->Write(m_eLegionHero);
    pStream->Write(m_eLeaderHero);
	pStream->Write(m_iTempStrengthModifier);
	pStream->Write(m_iSurroundingEnemyHealthChange);
	pStream->Write(m_iDCMBombRange);
	pStream->Write(m_iDefendFromChargeCount);
	pStream->Write(m_iInvisibleType);
	pStream->Write(m_iHiddenNationalityCount);
	pStream->Write(m_iAlwaysHostileCount);
	pStream->Write(m_iAdditionalAttackChance);
	pStream->Write(m_iWarGodCount);
	pStream->Write(m_iIgnoreDefenderTerrainCount);
	pStream->Write(m_iPlotDefenseModifier);
	pStream->Write(m_iWaterSideStrModifier);
	pStream->Write(m_iGuardCount);
	pStream->Write(m_iLegionTurn);
    // Sanguo Mod Hero, end
}

// Protected Functions...

bool CvUnit::canAdvance(const CvPlot* pPlot, int iThreshold) const
{
	FAssert(canFight());
	FAssert(!(isAnimal() && pPlot->isCity()));
	FAssert(getDomainType() != DOMAIN_AIR);
	FAssert(getDomainType() != DOMAIN_IMMOBILE);

	if (pPlot->getNumVisibleEnemyDefenders(this) > iThreshold)
	{
		return false;
	}

	if (isNoCapture())
	{
		if (pPlot->isEnemyCity(*this))
		{
			return false;
		}
	}

	return true;
}


void CvUnit::collateralCombat(const CvPlot* pPlot, CvUnit* pSkipUnit)
{
	CLLNode<IDInfo>* pUnitNode;
	CvUnit* pLoopUnit;
	CvUnit* pBestUnit;
	CvWString szBuffer;
	int iTheirStrength;
	int iStrengthFactor;
	int iCollateralDamage;
	int iUnitDamage;
	int iDamageCount;
	int iPossibleTargets;
	int iCount;
	int iValue;
	int iBestValue;
	std::map<CvUnit*, int> mapUnitDamage;
	std::map<CvUnit*, int>::iterator it;

	int iCollateralStrength = (getDomainType() == DOMAIN_AIR ? airBaseCombatStr() : baseCombatStr()) * collateralDamage() / 100;
	// UNOFFICIAL_PATCH Start
	// * Barrage promotions made working again on Tanks and other units with no base collateral ability
	if (iCollateralStrength == 0 && getExtraCollateralDamage() == 0)
	// UNOFFICIAL_PATCH End
	{
		return;
	}

	iPossibleTargets = std::min((pPlot->getNumVisibleEnemyDefenders(this) - 1), collateralDamageMaxUnits());

	pUnitNode = pPlot->headUnitNode();

	while (pUnitNode != NULL)
	{
		pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit != pSkipUnit)
		{
			if (isEnemy(pLoopUnit->getTeam(), pPlot))
			{
				if (!(pLoopUnit->isInvisible(getTeam(), false)))
				{
					if (pLoopUnit->canDefend())
					{
						iValue = (1 + GC.getGameINLINE().getSorenRandNum(10000, "Collateral Damage"));

						iValue *= pLoopUnit->currHitPoints();

						mapUnitDamage[pLoopUnit] = iValue;
					}
				}
			}
		}
	}

	CvCity* pCity = NULL;
	if (getDomainType() == DOMAIN_AIR)
	{
		pCity = pPlot->getPlotCity();
	}

	iDamageCount = 0;
	iCount = 0;

	while (iCount < iPossibleTargets)
	{
		iBestValue = 0;
		pBestUnit = NULL;

		for (it = mapUnitDamage.begin(); it != mapUnitDamage.end(); it++)
		{
			if (it->second > iBestValue)
			{
				iBestValue = it->second;
				pBestUnit = it->first;
			}
		}

		if (pBestUnit != NULL)
		{
			mapUnitDamage.erase(pBestUnit);

			if (NO_UNITCOMBAT == getUnitCombatType() || !pBestUnit->getUnitInfo().getUnitCombatCollateralImmune(getUnitCombatType()))
			{
				iTheirStrength = pBestUnit->baseCombatStr();

				iStrengthFactor = ((iCollateralStrength + iTheirStrength + 1) / 2);

				iCollateralDamage = (GC.getDefineINT("COLLATERAL_COMBAT_DAMAGE") * (iCollateralStrength + iStrengthFactor)) / (iTheirStrength + iStrengthFactor);

				iCollateralDamage *= 100 + getExtraCollateralDamage();

				iCollateralDamage *= std::max(0, 100 - pBestUnit->getCollateralDamageProtection());
				iCollateralDamage /= 100;

				if (pCity != NULL)
				{
					iCollateralDamage *= 100 + pCity->getAirModifier();
					iCollateralDamage /= 100;
				}

				iCollateralDamage /= 100;

				iCollateralDamage = std::max(0, iCollateralDamage);

				int iMaxDamage = std::min(collateralDamageLimit(), (collateralDamageLimit() * (iCollateralStrength + iStrengthFactor)) / (iTheirStrength + iStrengthFactor));
				iUnitDamage = std::max(pBestUnit->getDamage(), std::min(pBestUnit->getDamage() + iCollateralDamage, iMaxDamage));

				if (pBestUnit->getDamage() != iUnitDamage)
				{
					pBestUnit->setDamage(iUnitDamage, getOwnerINLINE());
					iDamageCount++;
				}
			}

			iCount++;
		}
		else
		{
			break;
		}
	}

	if (iDamageCount > 0)
	{
		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_SUFFER_COL_DMG", iDamageCount);
		gDLL->getInterfaceIFace()->addMessage(pSkipUnit->getOwnerINLINE(), (pSkipUnit->getDomainType() != DOMAIN_AIR), GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_COLLATERAL", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pSkipUnit->getX_INLINE(), pSkipUnit->getY_INLINE(), true, true);

		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_INFLICT_COL_DMG", getNameKey(), iDamageCount);
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_COLLATERAL", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pSkipUnit->getX_INLINE(), pSkipUnit->getY_INLINE());
	}
}


void CvUnit::flankingStrikeCombat(const CvPlot* pPlot, int iAttackerStrength, int iAttackerFirepower, int iDefenderOdds, int iDefenderDamage, CvUnit* pSkipUnit)
{
	if (pPlot->isCity(true, pSkipUnit->getTeam()))
	{
		return;
	}

	CLLNode<IDInfo>* pUnitNode = pPlot->headUnitNode();

	std::vector< std::pair<CvUnit*, int> > listFlankedUnits;
	while (NULL != pUnitNode)
	{
		CvUnit* pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit != pSkipUnit)
		{
			if (!pLoopUnit->isDead() && isEnemy(pLoopUnit->getTeam(), pPlot))
			{
				if (!(pLoopUnit->isInvisible(getTeam(), false)))
				{
					if (pLoopUnit->canDefend())
					{
						int iFlankingStrength = m_pUnitInfo->getFlankingStrikeUnitClass(pLoopUnit->getUnitClassType());

						if (iFlankingStrength > 0)
						{
							int iFlankedDefenderStrength;
							int iFlankedDefenderOdds;
							int iAttackerDamage;
							int iFlankedDefenderDamage;

							getDefenderCombatValues(*pLoopUnit, pPlot, iAttackerStrength, iAttackerFirepower, iFlankedDefenderOdds, iFlankedDefenderStrength, iAttackerDamage, iFlankedDefenderDamage);

							if (GC.getGameINLINE().getSorenRandNum(GC.getDefineINT("COMBAT_DIE_SIDES"), "Flanking Combat") >= iDefenderOdds)
							{
								int iCollateralDamage = (iFlankingStrength * iDefenderDamage) / 100;
								int iUnitDamage = std::max(pLoopUnit->getDamage(), std::min(pLoopUnit->getDamage() + iCollateralDamage, collateralDamageLimit()));

								if (pLoopUnit->getDamage() != iUnitDamage)
								{
									listFlankedUnits.push_back(std::make_pair(pLoopUnit, iUnitDamage));
								}
							}
						}
					}
				}
			}
		}
	}

	int iNumUnitsHit = std::min((int)listFlankedUnits.size(), collateralDamageMaxUnits());

	for (int i = 0; i < iNumUnitsHit; ++i)
	{
		int iIndexHit = GC.getGameINLINE().getSorenRandNum(listFlankedUnits.size(), "Pick Flanked Unit");
		CvUnit* pUnit = listFlankedUnits[iIndexHit].first;
		int iDamage = listFlankedUnits[iIndexHit].second;
		pUnit->setDamage(iDamage, getOwnerINLINE());
		if (pUnit->isDead())
		{
			CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_KILLED_UNIT_BY_FLANKING", getNameKey(), pUnit->getNameKey(), pUnit->getVisualCivAdjective(getTeam()));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_UNIT_DIED_BY_FLANKING", pUnit->getNameKey(), getNameKey(), getVisualCivAdjective(pUnit->getTeam()));
			gDLL->getInterfaceIFace()->addMessage(pUnit->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

			pUnit->kill(false);
		}

		listFlankedUnits.erase(std::remove(listFlankedUnits.begin(), listFlankedUnits.end(), listFlankedUnits[iIndexHit]));
	}

	if (iNumUnitsHit > 0)
	{
		CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_DAMAGED_UNITS_BY_FLANKING", getNameKey(), iNumUnitsHit);
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

		if (NULL != pSkipUnit)
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOUR_UNITS_DAMAGED_BY_FLANKING", getNameKey(), iNumUnitsHit);
			gDLL->getInterfaceIFace()->addMessage(pSkipUnit->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GC.getGameINLINE().getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		}
	}
}


// Returns true if we were intercepted...
bool CvUnit::interceptTest(const CvPlot* pPlot)
{
	if (GC.getGameINLINE().getSorenRandNum(100, "Evasion Rand") >= evasionProbability())
	{
		CvUnit* pInterceptor = bestInterceptor(pPlot);

		if (pInterceptor != NULL)
		{
			if (GC.getGameINLINE().getSorenRandNum(100, "Intercept Rand (Air)") < pInterceptor->currInterceptionProbability())
			{
				fightInterceptor(pPlot, false);

				return true;
			}
		}
	}

	return false;
}


CvUnit* CvUnit::airStrikeTarget(const CvPlot* pPlot) const
{
	CvUnit* pDefender;

	pDefender = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);

	if (pDefender != NULL)
	{
		if (!pDefender->isDead())
		{
			if (pDefender->canDefend())
			{
				return pDefender;
			}
		}
	}

	return NULL;
}


bool CvUnit::canAirStrike(const CvPlot* pPlot) const
{
	if (getDomainType() != DOMAIN_AIR)
	{
		return false;
	}

	if (!canAirAttack())
	{
		return false;
	}

	if (pPlot == plot())
	{
		return false;
	}

	if (!pPlot->isVisible(getTeam(), false))
	{
		return false;
	}

	if (plotDistance(getX_INLINE(), getY_INLINE(), pPlot->getX_INLINE(), pPlot->getY_INLINE()) > airRange())
	{
		return false;
	}

	if (airStrikeTarget(pPlot) == NULL)
	{
		return false;
	}

	return true;
}


bool CvUnit::airStrike(CvPlot* pPlot)
{
	if (!canAirStrike(pPlot))
	{
		return false;
	}

	if (interceptTest(pPlot))
	{
		return false;
	}

	CvUnit* pDefender = airStrikeTarget(pPlot);

	FAssert(pDefender != NULL);
	FAssert(pDefender->canDefend());

	setReconPlot(pPlot);

	setMadeAttack(true);
	changeMoves(GC.getMOVE_DENOMINATOR());

	int iDamage = airCombatDamage(pDefender);

	int iUnitDamage = std::max(pDefender->getDamage(), std::min((pDefender->getDamage() + iDamage), airCombatLimit()));

	CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ARE_ATTACKED_BY_AIR", pDefender->getNameKey(), getNameKey(), -(((iUnitDamage - pDefender->getDamage()) * 100) / pDefender->maxHitPoints()));
	gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);

	szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ATTACK_BY_AIR", getNameKey(), pDefender->getNameKey(), -(((iUnitDamage - pDefender->getDamage()) * 100) / pDefender->maxHitPoints()));
	gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACKED", MESSAGE_TYPE_INFO, pDefender->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

	collateralCombat(pPlot, pDefender);

	pDefender->setDamage(iUnitDamage, getOwnerINLINE());

	return true;
}

bool CvUnit::canRangeStrike() const
{
	if (getDomainType() == DOMAIN_AIR)
	{
		return false;
	}

	if (airRange() <= 0)
	{
		return false;
	}

	if (airBaseCombatStr() <= 0)
	{
		return false;
	}

	if (!canFight())
	{
		return false;
	}

		if (isMadeAttack() && !isBlitz())
		{
			return false;
		}

	if (!canMove() && getMoves() > 0)
	{
		return false;
	}

	return true;
}

bool CvUnit::canRangeStrikeAt(const CvPlot* pPlot, int iX, int iY) const
{
	if (!canRangeStrike())
	{
		return false;
	}

	CvPlot* pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (NULL == pTargetPlot)
	{
		return false;
	}

	if (!pPlot->isVisible(getTeam(), false))
	{
		return false;
	}

/************************************************************************************************/
/* UNOFFICIAL_PATCH                       05/10/10                             jdog5000         */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
	// Need to check target plot too
	if (!pTargetPlot->isVisible(getTeam(), false))
	{
		return false;
	}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/

	if (plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), pTargetPlot->getX_INLINE(), pTargetPlot->getY_INLINE()) > airRange())
	{
		return false;
	}

	CvUnit* pDefender = airStrikeTarget(pTargetPlot);
	if (NULL == pDefender)
	{
		return false;
	}

	if (!pPlot->canSeePlot(pTargetPlot, getTeam(), airRange(), getFacingDirection(true)))
	{
		return false;
	}

	return true;
}


bool CvUnit::rangeStrike(int iX, int iY)
{
	CvUnit* pDefender;
	CvWString szBuffer;
	int iUnitDamage;
	int iDamage;

	CvPlot* pPlot = GC.getMapINLINE().plot(iX, iY);
	if (NULL == pPlot)
	{
		return false;
	}

/************************************************************************************************/
/* UNOFFICIAL_PATCH                       05/10/10                             jdog5000         */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
/* original bts code
	if (!canRangeStrikeAt(pPlot, iX, iY))
	{
		return false;
	}
*/
	if (!canRangeStrikeAt(plot(), iX, iY))
	{
		return false;
	}
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/

	pDefender = airStrikeTarget(pPlot);

	FAssert(pDefender != NULL);
	FAssert(pDefender->canDefend());

	if (GC.getDefineINT("RANGED_ATTACKS_USE_MOVES") == 0)
	{
		setMadeAttack(true);
	}
	changeMoves(GC.getMOVE_DENOMINATOR());

	iDamage = rangeCombatDamage(pDefender);

	iUnitDamage = std::max(pDefender->getDamage(), std::min((pDefender->getDamage() + iDamage), airCombatLimit()));

	szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ARE_ATTACKED_BY_AIR", pDefender->getNameKey(), getNameKey(), -(((iUnitDamage - pDefender->getDamage()) * 100) / pDefender->maxHitPoints()));
	//red icon over attacking unit
	gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_COMBAT", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), this->getX_INLINE(), this->getY_INLINE(), true, true);
	//white icon over defending unit
	gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), false, 0, L"", "AS2D_COMBAT", MESSAGE_TYPE_DISPLAY_ONLY, pDefender->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_WHITE"), pDefender->getX_INLINE(), pDefender->getY_INLINE(), true, true);

	szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ATTACK_BY_AIR", getNameKey(), pDefender->getNameKey(), -(((iUnitDamage - pDefender->getDamage()) * 100) / pDefender->maxHitPoints()));
	gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_COMBAT", MESSAGE_TYPE_INFO, pDefender->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());

	collateralCombat(pPlot, pDefender);

	//set damage but don't update entity damage visibility
	pDefender->setDamage(iUnitDamage, getOwnerINLINE(), false);

	if (pPlot->isActiveVisible(false))
	{
		// Range strike entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_RANGE_ATTACK).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_RANGE_ATTACK);
		kDefiniton.setPlot(pDefender->plot());
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, pDefender);
		gDLL->getEntityIFace()->AddMission(&kDefiniton);

		//delay death
/************************************************************************************************/
/* UNOFFICIAL_PATCH                       05/10/10                             jdog5000         */
/*                                                                                              */
/* Bugfix                                                                                       */
/************************************************************************************************/
/* original bts code
		pDefender->getGroup()->setMissionTimer(GC.getMissionInfo(MISSION_RANGE_ATTACK).getTime());
*/
		// mission timer is not used like this in any other part of code, so it might cause OOS
		// issues ... at worst I think unit dies before animation is complete, so no real
		// harm in commenting it out.
/************************************************************************************************/
/* UNOFFICIAL_PATCH                        END                                                  */
/************************************************************************************************/
	}

	return true;
}

//------------------------------------------------------------------------------------------------
// FUNCTION:    CvUnit::planBattle
//! \brief      Determines in general how a battle will progress.
//!
//!				Note that the outcome of the battle is not determined here. This function plans
//!				how many sub-units die and in which 'rounds' of battle.
//! \param      kBattleDefinition The battle definition, which receives the battle plan.
//! \retval     The number of game turns that the battle should be given.
//------------------------------------------------------------------------------------------------
int CvUnit::planBattle( CvBattleDefinition & kBattleDefinition ) const
{
#define BATTLE_TURNS_SETUP 4
#define BATTLE_TURNS_ENDING 4
#define BATTLE_TURNS_MELEE 6
#define BATTLE_TURNS_RANGED 6
#define BATTLE_TURN_RECHECK 4

	int								aiUnitsBegin[BATTLE_UNIT_COUNT];
	int								aiUnitsEnd[BATTLE_UNIT_COUNT];
	int								aiToKillMelee[BATTLE_UNIT_COUNT];
	int								aiToKillRanged[BATTLE_UNIT_COUNT];
	CvBattleRoundVector::iterator	iIterator;
	int								i, j;
	bool							bIsLoser;
	int								iRoundIndex;
	int								iTotalRounds = 0;
	int								iRoundCheck = BATTLE_TURN_RECHECK;

	// Initial conditions
	kBattleDefinition.setNumRangedRounds(0);
	kBattleDefinition.setNumMeleeRounds(0);

	int iFirstStrikesDelta = kBattleDefinition.getFirstStrikes(BATTLE_UNIT_ATTACKER) - kBattleDefinition.getFirstStrikes(BATTLE_UNIT_DEFENDER);
	if (iFirstStrikesDelta > 0) // Attacker first strikes
	{
		int iKills = computeUnitsToDie( kBattleDefinition, true, BATTLE_UNIT_DEFENDER );
		kBattleDefinition.setNumRangedRounds(std::max(iFirstStrikesDelta, iKills / iFirstStrikesDelta));
	}
	else if (iFirstStrikesDelta < 0) // Defender first strikes
	{
		int iKills = computeUnitsToDie( kBattleDefinition, true, BATTLE_UNIT_ATTACKER );
		iFirstStrikesDelta = -iFirstStrikesDelta;
		kBattleDefinition.setNumRangedRounds(std::max(iFirstStrikesDelta, iKills / iFirstStrikesDelta));
	}
	increaseBattleRounds( kBattleDefinition);


	// Keep randomizing until we get something valid
	do
	{
		iRoundCheck++;
		if ( iRoundCheck >= BATTLE_TURN_RECHECK )
		{
			increaseBattleRounds( kBattleDefinition);
			iTotalRounds = kBattleDefinition.getNumRangedRounds() + kBattleDefinition.getNumMeleeRounds();
			iRoundCheck = 0;
		}

		// Make sure to clear the battle plan, we may have to do this again if we can't find a plan that works.
		kBattleDefinition.clearBattleRounds();

		// Create the round list
		CvBattleRound kRound;
		kBattleDefinition.setBattleRound(iTotalRounds, kRound);

		// For the attacker and defender
		for ( i = 0; i < BATTLE_UNIT_COUNT; i++ )
		{
			// Gather some initial information
			BattleUnitTypes unitType = (BattleUnitTypes) i;
			aiUnitsBegin[unitType] = kBattleDefinition.getUnit(unitType)->getSubUnitsAlive(kBattleDefinition.getDamage(unitType, BATTLE_TIME_BEGIN));
			aiToKillRanged[unitType] = computeUnitsToDie( kBattleDefinition, true, unitType);
			aiToKillMelee[unitType] = computeUnitsToDie( kBattleDefinition, false, unitType);
			aiUnitsEnd[unitType] = aiUnitsBegin[unitType] - aiToKillMelee[unitType] - aiToKillRanged[unitType];

			// Make sure that if they aren't dead at the end, they have at least one unit left
			if ( aiUnitsEnd[unitType] == 0 && !kBattleDefinition.getUnit(unitType)->isDead() )
			{
				aiUnitsEnd[unitType]++;
				if ( aiToKillMelee[unitType] > 0 )
				{
					aiToKillMelee[unitType]--;
				}
				else
				{
					aiToKillRanged[unitType]--;
				}
			}

			// If one unit is the loser, make sure that at least one of their units dies in the last round
			if ( aiUnitsEnd[unitType] == 0 )
			{
				kBattleDefinition.getBattleRound(iTotalRounds - 1).addNumKilled(unitType, 1);
				if ( aiToKillMelee[unitType] > 0)
				{
					aiToKillMelee[unitType]--;
				}
				else
				{
					aiToKillRanged[unitType]--;
				}
			}

			// Randomize in which round each death occurs
			bIsLoser = aiUnitsEnd[unitType] == 0;

			// Randomize the ranged deaths
			for ( j = 0; j < aiToKillRanged[unitType]; j++ )
			{
				iRoundIndex = GC.getGameINLINE().getSorenRandNum( range( kBattleDefinition.getNumRangedRounds(), 0, kBattleDefinition.getNumRangedRounds()), "Ranged combat death");
				kBattleDefinition.getBattleRound(iRoundIndex).addNumKilled(unitType, 1);
			}

			// Randomize the melee deaths
			for ( j = 0; j < aiToKillMelee[unitType]; j++ )
			{
				iRoundIndex = GC.getGameINLINE().getSorenRandNum( range( kBattleDefinition.getNumMeleeRounds() - (bIsLoser ? 1 : 2 ), 0, kBattleDefinition.getNumMeleeRounds()), "Melee combat death");
				kBattleDefinition.getBattleRound(kBattleDefinition.getNumRangedRounds() + iRoundIndex).addNumKilled(unitType, 1);
			}

			// Compute alive sums
			int iNumberKilled = 0;
			for(int j=0;j<kBattleDefinition.getNumBattleRounds();j++)
			{
				CvBattleRound &round = kBattleDefinition.getBattleRound(j);
				round.setRangedRound(j < kBattleDefinition.getNumRangedRounds());
				iNumberKilled += round.getNumKilled(unitType);
				round.setNumAlive(unitType, aiUnitsBegin[unitType] - iNumberKilled);
			}
		}

		// Now compute wave sizes
		for(int i=0;i<kBattleDefinition.getNumBattleRounds();i++)
		{
			CvBattleRound &round = kBattleDefinition.getBattleRound(i);
			round.setWaveSize(computeWaveSize(round.isRangedRound(), round.getNumAlive(BATTLE_UNIT_ATTACKER) + round.getNumKilled(BATTLE_UNIT_ATTACKER), round.getNumAlive(BATTLE_UNIT_DEFENDER) + round.getNumKilled(BATTLE_UNIT_DEFENDER)));
		}

		if ( iTotalRounds > 400 )
		{
			kBattleDefinition.setNumMeleeRounds(1);
			kBattleDefinition.setNumRangedRounds(0);
			break;
		}
	}
	while ( !verifyRoundsValid( kBattleDefinition ));

	//add a little extra time for leader to surrender
	bool attackerLeader = false;
	bool defenderLeader = false;
	bool attackerDie = false;
	bool defenderDie = false;
	int lastRound = kBattleDefinition.getNumBattleRounds() - 1;
	if(kBattleDefinition.getUnit(BATTLE_UNIT_ATTACKER)->getLeaderUnitType() != NO_UNIT)
		attackerLeader = true;
	if(kBattleDefinition.getUnit(BATTLE_UNIT_DEFENDER)->getLeaderUnitType() != NO_UNIT)
		defenderLeader = true;
	if(kBattleDefinition.getBattleRound(lastRound).getNumAlive(BATTLE_UNIT_ATTACKER) == 0)
		attackerDie = true;
	if(kBattleDefinition.getBattleRound(lastRound).getNumAlive(BATTLE_UNIT_DEFENDER) == 0)
		defenderDie = true;

	int extraTime = 0;
	if((attackerLeader && attackerDie) || (defenderLeader && defenderDie))
		extraTime = BATTLE_TURNS_MELEE;
	if(gDLL->getEntityIFace()->GetSiegeTower(kBattleDefinition.getUnit(BATTLE_UNIT_ATTACKER)->getUnitEntity()) || gDLL->getEntityIFace()->GetSiegeTower(kBattleDefinition.getUnit(BATTLE_UNIT_DEFENDER)->getUnitEntity()))
		extraTime = BATTLE_TURNS_MELEE;

	return BATTLE_TURNS_SETUP + BATTLE_TURNS_ENDING + kBattleDefinition.getNumMeleeRounds() * BATTLE_TURNS_MELEE + kBattleDefinition.getNumRangedRounds() * BATTLE_TURNS_MELEE + extraTime;
}

//------------------------------------------------------------------------------------------------
// FUNCTION:	CvBattleManager::computeDeadUnits
//! \brief		Computes the number of units dead, for either the ranged or melee portion of combat.
//! \param		kDefinition The battle definition.
//! \param		bRanged true if computing the number of units that die during the ranged portion of combat,
//!					false if computing the number of units that die during the melee portion of combat.
//! \param		iUnit The index of the unit to compute (BATTLE_UNIT_ATTACKER or BATTLE_UNIT_DEFENDER).
//! \retval		The number of units that should die for the given unit in the given portion of combat
//------------------------------------------------------------------------------------------------
int CvUnit::computeUnitsToDie( const CvBattleDefinition & kDefinition, bool bRanged, BattleUnitTypes iUnit ) const
{
	FAssertMsg( iUnit == BATTLE_UNIT_ATTACKER || iUnit == BATTLE_UNIT_DEFENDER, "Invalid unit index");

	BattleTimeTypes iBeginIndex = bRanged ? BATTLE_TIME_BEGIN : BATTLE_TIME_RANGED;
	BattleTimeTypes iEndIndex = bRanged ? BATTLE_TIME_RANGED : BATTLE_TIME_END;
	return kDefinition.getUnit(iUnit)->getSubUnitsAlive(kDefinition.getDamage(iUnit, iBeginIndex)) -
		kDefinition.getUnit(iUnit)->getSubUnitsAlive( kDefinition.getDamage(iUnit, iEndIndex));
}

//------------------------------------------------------------------------------------------------
// FUNCTION:    CvUnit::verifyRoundsValid
//! \brief      Verifies that all rounds in the battle plan are valid
//! \param      vctBattlePlan The battle plan
//! \retval     true if the battle plan (seems) valid, false otherwise
//------------------------------------------------------------------------------------------------
bool CvUnit::verifyRoundsValid( const CvBattleDefinition & battleDefinition ) const
{
	for(int i=0;i<battleDefinition.getNumBattleRounds();i++)
	{
		if(!battleDefinition.getBattleRound(i).isValid())
			return false;
	}
	return true;
}

//------------------------------------------------------------------------------------------------
// FUNCTION:    CvUnit::increaseBattleRounds
//! \brief      Increases the number of rounds in the battle.
//! \param      kBattleDefinition The definition of the battle
//------------------------------------------------------------------------------------------------
void CvUnit::increaseBattleRounds( CvBattleDefinition & kBattleDefinition ) const
{
	if ( kBattleDefinition.getUnit(BATTLE_UNIT_ATTACKER)->isRanged() && kBattleDefinition.getUnit(BATTLE_UNIT_DEFENDER)->isRanged())
	{
		kBattleDefinition.addNumRangedRounds(1);
	}
	else
	{
		kBattleDefinition.addNumMeleeRounds(1);
	}
}

//------------------------------------------------------------------------------------------------
// FUNCTION:    CvUnit::computeWaveSize
//! \brief      Computes the wave size for the round.
//! \param      bRangedRound true if the round is a ranged round
//! \param		iAttackerMax The maximum number of attackers that can participate in a wave (alive)
//! \param		iDefenderMax The maximum number of Defenders that can participate in a wave (alive)
//! \retval     The desired wave size for the given parameters
//------------------------------------------------------------------------------------------------
int CvUnit::computeWaveSize( bool bRangedRound, int iAttackerMax, int iDefenderMax ) const
{
	FAssertMsg( getCombatUnit() != NULL, "You must be fighting somebody!" );
	int aiDesiredSize[BATTLE_UNIT_COUNT];
	if ( bRangedRound )
	{
		aiDesiredSize[BATTLE_UNIT_ATTACKER] = getUnitInfo().getRangedWaveSize();
		aiDesiredSize[BATTLE_UNIT_DEFENDER] = getCombatUnit()->getUnitInfo().getRangedWaveSize();
	}
	else
	{
		aiDesiredSize[BATTLE_UNIT_ATTACKER] = getUnitInfo().getMeleeWaveSize();
		aiDesiredSize[BATTLE_UNIT_DEFENDER] = getCombatUnit()->getUnitInfo().getMeleeWaveSize();
	}

	aiDesiredSize[BATTLE_UNIT_DEFENDER] = aiDesiredSize[BATTLE_UNIT_DEFENDER] <= 0 ? iDefenderMax : aiDesiredSize[BATTLE_UNIT_DEFENDER];
	aiDesiredSize[BATTLE_UNIT_ATTACKER] = aiDesiredSize[BATTLE_UNIT_ATTACKER] <= 0 ? iDefenderMax : aiDesiredSize[BATTLE_UNIT_ATTACKER];
	return std::min( std::min( aiDesiredSize[BATTLE_UNIT_ATTACKER], iAttackerMax ), std::min( aiDesiredSize[BATTLE_UNIT_DEFENDER],
		iDefenderMax) );
}

bool CvUnit::isTargetOf(const CvUnit& attacker) const
{
	CvUnitInfo& attackerInfo = attacker.getUnitInfo();
	CvUnitInfo& ourInfo = getUnitInfo();

	if (!plot()->isCity(true, getTeam()))
	{
		if (NO_UNITCLASS != getUnitClassType() && attackerInfo.getTargetUnitClass(getUnitClassType()))
		{
			return true;
		}

		if (NO_UNITCOMBAT != getUnitCombatType() && attackerInfo.getTargetUnitCombat(getUnitCombatType()))
		{
			return true;
		}
	}

	if (NO_UNITCLASS != attackerInfo.getUnitClassType() && ourInfo.getDefenderUnitClass(attackerInfo.getUnitClassType()))
	{
		return true;
	}

	if (NO_UNITCOMBAT != attackerInfo.getUnitCombatType() && ourInfo.getDefenderUnitCombat(attackerInfo.getUnitCombatType()))
	{
		return true;
	}

	return false;
}

bool CvUnit::isEnemy(TeamTypes eTeam, const CvPlot* pPlot) const
{
	if (NULL == pPlot)
	{
		pPlot = plot();
	}

	return (atWar(GET_PLAYER(getCombatOwner(eTeam, pPlot)).getTeam(), eTeam));
}

bool CvUnit::isPotentialEnemy(TeamTypes eTeam, const CvPlot* pPlot) const
{
	if (NULL == pPlot)
	{
		pPlot = plot();
	}

	return (::isPotentialEnemy(GET_PLAYER(getCombatOwner(eTeam, pPlot)).getTeam(), eTeam));
}

bool CvUnit::isSuicide() const
{
	return (m_pUnitInfo->isSuicide() || getKamikazePercent() != 0);
}

int CvUnit::getDropRange() const
{
	return (m_pUnitInfo->getDropRange());
}

void CvUnit::getDefenderCombatValues(CvUnit& kDefender, const CvPlot* pPlot, int iOurStrength, int iOurFirepower, int& iTheirOdds, int& iTheirStrength, int& iOurDamage, int& iTheirDamage, CombatDetails* pTheirDetails) const
{
	iTheirStrength = kDefender.currCombatStr(pPlot, this, pTheirDetails);
	int iTheirFirepower = kDefender.currFirepower(pPlot, this);

	FAssert((iOurStrength + iTheirStrength) > 0);
	FAssert((iOurFirepower + iTheirFirepower) > 0);

	iTheirOdds = ((GC.getDefineINT("COMBAT_DIE_SIDES") * iTheirStrength) / (iOurStrength + iTheirStrength));

	if (kDefender.isBarbarian()
	// Sanguo Mod Barbarian, start, added by poyuzhe 07.14.09
	&& kDefender.getLegionHero() != NO_HERO)
	// Sanguo Mod Barbarian, end
	{
		if (GET_PLAYER(getOwnerINLINE()).getWinsVsBarbs() < GC.getHandicapInfo(GET_PLAYER(getOwnerINLINE()).getHandicapType()).getFreeWinsVsBarbs())
		{
			iTheirOdds = std::min((10 * GC.getDefineINT("COMBAT_DIE_SIDES")) / 100, iTheirOdds);
		}
	}
	if (isBarbarian()
	// Sanguo Mod Barbarian, start, added by poyuzhe 07.14.09
	&& getLegionHero() != NO_HERO)
	// Sanguo Mod Barbarian, end
	{
		if (GET_PLAYER(kDefender.getOwnerINLINE()).getWinsVsBarbs() < GC.getHandicapInfo(GET_PLAYER(kDefender.getOwnerINLINE()).getHandicapType()).getFreeWinsVsBarbs())
		{
			iTheirOdds =  std::max((90 * GC.getDefineINT("COMBAT_DIE_SIDES")) / 100, iTheirOdds);
		}
	}

	int iStrengthFactor = ((iOurFirepower + iTheirFirepower + 1) / 2);

	iOurDamage = std::max(1, ((GC.getDefineINT("COMBAT_DAMAGE") * (iTheirFirepower + iStrengthFactor)) / (iOurFirepower + iStrengthFactor)));
	iTheirDamage = std::max(1, ((GC.getDefineINT("COMBAT_DAMAGE") * (iOurFirepower + iStrengthFactor)) / (iTheirFirepower + iStrengthFactor)));
}

int CvUnit::getTriggerValue(EventTriggerTypes eTrigger, const CvPlot* pPlot, bool bCheckPlot) const
{
	CvEventTriggerInfo& kTrigger = GC.getEventTriggerInfo(eTrigger);
	if (kTrigger.getNumUnits() <= 0)
	{
		return MIN_INT;
	}

	if (isDead())
	{
		return MIN_INT;
	}

	if (!CvString(kTrigger.getPythonCanDoUnit()).empty())
	{
		long lResult;

		CyArgsList argsList;
		argsList.add(eTrigger);
		argsList.add(getOwnerINLINE());
		argsList.add(getID());

		gDLL->getPythonIFace()->callFunction(PYRandomEventModule, kTrigger.getPythonCanDoUnit(), argsList.makeFunctionArgs(), &lResult);

		if (0 == lResult)
		{
			return MIN_INT;
		}
	}

	if (kTrigger.getNumUnitsRequired() > 0)
	{
		bool bFoundValid = false;
		for (int i = 0; i < kTrigger.getNumUnitsRequired(); ++i)
		{
			if (getUnitClassType() == kTrigger.getUnitRequired(i))
			{
				bFoundValid = true;
				break;
			}
		}

		if (!bFoundValid)
		{
			return MIN_INT;
		}
	}

	if (bCheckPlot)
	{
		if (kTrigger.isUnitsOnPlot())
		{
			if (!plot()->canTrigger(eTrigger, getOwnerINLINE()))
			{
				return MIN_INT;
			}
		}
	}

	int iValue = 0;

	if (0 == getDamage() && kTrigger.getUnitDamagedWeight() > 0)
	{
		return MIN_INT;
	}

	iValue += getDamage() * kTrigger.getUnitDamagedWeight();

	iValue += getExperience() * kTrigger.getUnitExperienceWeight();

	if (NULL != pPlot)
	{
		iValue += plotDistance(getX_INLINE(), getY_INLINE(), pPlot->getX_INLINE(), pPlot->getY_INLINE()) * kTrigger.getUnitDistanceWeight();
	}

	return iValue;
}

bool CvUnit::canApplyEvent(EventTypes eEvent) const
{
	CvEventInfo& kEvent = GC.getEventInfo(eEvent);

	if (0 != kEvent.getUnitExperience())
	{
		if (!canAcquirePromotionAny())
		{
			return false;
		}
	}

	if (NO_PROMOTION != kEvent.getUnitPromotion())
	{
		if (!canAcquirePromotion((PromotionTypes)kEvent.getUnitPromotion()))
		{
			return false;
		}
	}

	if (kEvent.getUnitImmobileTurns() > 0)
	{
		if (!canAttack())
		{
			return false;
		}
	}

	return true;
}

void CvUnit::applyEvent(EventTypes eEvent)
{
	if (!canApplyEvent(eEvent))
	{
		return;
	}

	CvEventInfo& kEvent = GC.getEventInfo(eEvent);

	if (0 != kEvent.getUnitExperience())
	{
		setDamage(0);
		changeExperience(kEvent.getUnitExperience());
	}

	if (NO_PROMOTION != kEvent.getUnitPromotion())
	{
		setHasPromotion((PromotionTypes)kEvent.getUnitPromotion(), true);
	}

	if (kEvent.getUnitImmobileTurns() > 0)
	{
		changeImmobileTimer(kEvent.getUnitImmobileTurns());
		CvWString szText = gDLL->getText("TXT_KEY_EVENT_UNIT_IMMOBILE", getNameKey(), kEvent.getUnitImmobileTurns());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szText, "AS2D_UNITGIFTED", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_UNIT_TEXT"), getX_INLINE(), getY_INLINE(), true, true);
	}

	CvWString szNameKey(kEvent.getUnitNameKey());

	if (!szNameKey.empty())
	{
		setName(gDLL->getText(kEvent.getUnitNameKey()));
	}

	if (kEvent.isDisbandUnit())
	{
		kill(false);
	}
}

const CvArtInfoUnit* CvUnit::getArtInfo(int i, EraTypes eEra) const
{
	return m_pUnitInfo->getArtInfo(i, eEra, (UnitArtStyleTypes) GC.getCivilizationInfo(getCivilizationType()).getUnitArtStyleType());
}

const TCHAR* CvUnit::getButton() const
{
	const CvArtInfoUnit* pArtInfo = getArtInfo(0, GET_PLAYER(getOwnerINLINE()).getCurrentEra());

	// Sanguo Mod Hero, start, added by poyuzhe 07.19.09
	if (getUnitInfo().isHero() && getHeroType() != NO_HERO)
	{
	    return GC.getHeroInfo(getHeroType()).getButton();
	}
    // Sanguo Mod Hero, end
	if (NULL != pArtInfo)
	{
		return pArtInfo->getButton();
	}

	return m_pUnitInfo->getButton();
}

int CvUnit::getGroupSize() const
{
	return m_pUnitInfo->getGroupSize();
}

int CvUnit::getGroupDefinitions() const
{
	return m_pUnitInfo->getGroupDefinitions();
}

int CvUnit::getUnitGroupRequired(int i) const
{
	return m_pUnitInfo->getUnitGroupRequired(i);
}

bool CvUnit::isRenderAlways() const
{
	return m_pUnitInfo->isRenderAlways();
}

float CvUnit::getAnimationMaxSpeed() const
{
	return m_pUnitInfo->getUnitMaxSpeed();
}

float CvUnit::getAnimationPadTime() const
{
	return m_pUnitInfo->getUnitPadTime();
}

const char* CvUnit::getFormationType() const
{
	return m_pUnitInfo->getFormationType();
}

bool CvUnit::isMechUnit() const
{
	return m_pUnitInfo->isMechUnit();
}

bool CvUnit::isRenderBelowWater() const
{
	return m_pUnitInfo->isRenderBelowWater();
}

int CvUnit::getRenderPriority(UnitSubEntityTypes eUnitSubEntity, int iMeshGroupType, int UNIT_MAX_SUB_TYPES) const
{
	if (eUnitSubEntity == UNIT_SUB_ENTITY_SIEGE_TOWER)
	{
		return (getOwner() * (GC.getNumUnitInfos() + 2) * UNIT_MAX_SUB_TYPES) + iMeshGroupType;
	}
	else
	{
		return (getOwner() * (GC.getNumUnitInfos() + 2) * UNIT_MAX_SUB_TYPES) + m_eUnitType * UNIT_MAX_SUB_TYPES + iMeshGroupType;
	}
}

bool CvUnit::isAlwaysHostile(const CvPlot* pPlot) const
{
	// Sanguo Mod Hero, start, added by poyuzhe 02.23.09
	// if (!m_pUnitInfo->isAlwaysHostile())
	if (m_iAlwaysHostileCount == 0)
	// Sanguo Mod Hero, end
	{
		return false;
	}

	if (NULL != pPlot && pPlot->isCity(true, getTeam()))
	{
		return false;
	}

	return true;
}

bool CvUnit::verifyStackValid()
{
	if (!alwaysInvisible())
	{
		if (plot()->isVisibleEnemyUnit(this))
		{
			return jumpToNearestValidPlot();
		}
	}

	return true;
}

// Private Functions...

//check if quick combat
bool CvUnit::isCombatVisible(const CvUnit* pDefender) const
{
	bool bVisible = false;

	if (!m_pUnitInfo->isQuickCombat())
	{
		if (NULL == pDefender || !pDefender->getUnitInfo().isQuickCombat())
		{
			if (isHuman())
			{
				if (!GET_PLAYER(getOwnerINLINE()).isOption(PLAYEROPTION_QUICK_ATTACK))
				{
					bVisible = true;
				}
			}
			else if (NULL != pDefender && pDefender->isHuman())
			{
				if (!GET_PLAYER(pDefender->getOwnerINLINE()).isOption(PLAYEROPTION_QUICK_DEFENSE))
				{
					bVisible = true;
				}
			}
		}
	}

	return bVisible;
}

// used by the executable for the red glow and plot indicators
bool CvUnit::shouldShowEnemyGlow(TeamTypes eForTeam) const
{
	if (isDelayedDeath())
	{
		return false;
	}

	if (getDomainType() == DOMAIN_AIR)
	{
		return false;
	}

	if (!canFight())
	{
		return false;
	}

	CvPlot* pPlot = plot();
	if (pPlot == NULL)
	{
		return false;
	}

	TeamTypes ePlotTeam = pPlot->getTeam();
	if (ePlotTeam != eForTeam)
	{
		return false;
	}

	if (!isEnemy(ePlotTeam))
	{
		return false;
	}

	return true;
}

bool CvUnit::shouldShowFoundBorders() const
{
	return isFound();
}


void CvUnit::cheat(bool bCtrl, bool bAlt, bool bShift)
{
	if (gDLL->getChtLvl() > 0)
	{
		if (bCtrl)
		{
			setPromotionReady(true);
		}
	}
}

float CvUnit::getHealthBarModifier() const
{
	return (GC.getDefineFLOAT("HEALTH_BAR_WIDTH") / (GC.getGameINLINE().getBestLandUnitCombat() * 2));
}

void CvUnit::getLayerAnimationPaths(std::vector<AnimationPathTypes>& aAnimationPaths) const
{
	for (int i=0; i < GC.getNumPromotionInfos(); ++i)
	{
		PromotionTypes ePromotion = (PromotionTypes) i;
		if (isHasPromotion(ePromotion))
		{
			AnimationPathTypes eAnimationPath = (AnimationPathTypes) GC.getPromotionInfo(ePromotion).getLayerAnimationPath();
			if(eAnimationPath != ANIMATIONPATH_NONE)
			{
				aAnimationPaths.push_back(eAnimationPath);
			}
		}
	}
}

int CvUnit::getSelectionSoundScript() const
{
	int iScriptId = getArtInfo(0, GET_PLAYER(getOwnerINLINE()).getCurrentEra())->getSelectionSoundScriptId();
	if (iScriptId == -1)
	{
		iScriptId = GC.getCivilizationInfo(getCivilizationType()).getSelectionSoundScriptId();
	}
	return iScriptId;
}

// Sanguo Mod Civilization Trait, start, by poyuzhe 11.26.08
// Zhang Yang, TRAIT_FOREIGNER
SanguoCampTypes CvUnit::getCampType() const
{
	for (int iI = 0; iI < GC.getNumSanguoCampInfos(); iI++)
	{
		if (GC.getSanguoCampInfo((SanguoCampTypes)iI).getUnitType() == getUnitType())
		{
			return (SanguoCampTypes)iI;
		}
	}
	return NO_SANGUOCAMP;
}
// Sanguo Mod Civilization Trait, end

// Sanguo Mod Hero, start, added by poyuzhe 01.08.09
HeroTypes CvUnit::getHeroType() const
{
    return m_eHeroType;
}

void CvUnit::setHeroType(HeroTypes eHero)
{
    m_eHeroType = eHero;
}

bool CvUnit::isPotentialLegionMember(const CvUnit* pUnit) const
{
    FAssert (NULL != pUnit);
    if (getOwnerINLINE() != pUnit->getOwnerINLINE())
    {
        return false;
    }

    if (getID() == pUnit->getID())
    {
        return false;
    }

    if (pUnit->getHeroType() == NO_HERO)
    {
        return false;
    }

    if (getHeroType() != NO_HERO)
    {
        return false;
    }

    if (getLegionHero() != NO_HERO)
    {
		return false;
	}

    if (getUnitCombatType() == NO_UNITCOMBAT || getUnitCombatType() == UNITCOMBAT_RECON || getUnitCombatType() == UNITCOMBAT_HERO)
    {

        return false;
    }

    if (GAME.isHeroHasLegionMember(pUnit->getHeroType(), getID()))
    {
        return false;
    }

    return true;
}

bool CvUnit::canJoinLegion(const CvUnit* pUnit) const
{
    PROFILE_FUNC();

	FAssert (pUnit != NULL);
	FAssert (pUnit->getHeroType() != NO_HERO);

    if (!isPotentialLegionMember(pUnit))
    {
        return false;
    }

	if (GAME.getHeroLegionSize(pUnit->getHeroType()) > 0 && getUnitCombatType() != GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(pUnit->getHeroType()))->getUnitCombatType())
    {
        return false;
    }

	if (GAME.getHeroLegionSize(pUnit->getHeroType()) >= GAME.getHeroCapacity(pUnit->getHeroType()))
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(pUnit->getHeroType()))
    {
        return false;
    }

    return true;
}

bool CvUnit::canBeCoreTroop(const CvUnit* pUnit) const
{
    FAssert (NULL != pUnit);

    if (!canJoinLegion(pUnit))
    {
        return false;
    }

    if (m_eLeaderUnitType != NO_UNIT)
    {
        return false;
    }

    return true;
}

HeroTypes CvUnit::getLegionHero() const
{
    return m_eLegionHero;
}

void CvUnit::setLegionHero(HeroTypes eNewHero)
{
    FAssert (eNewHero >= -1 && eNewHero < GC.getNumHeroInfos());
	if (m_eLegionHero != eNewHero)
	{
		m_iLegionTurn = 0;
	}
    m_eLegionHero = eNewHero;
}

bool CvUnit::canFormLegion(const CvPlot* pPlot) const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (getHeroType() == NO_HERO)
    {
        return false;
    }

	if (hasMoved())
	{
		return false;
	}

    if (getLegionHero() != NO_HERO)
    {
        return false;
    }

    if (NULL == pPlot)
    {
        pPlot = plot();
    }

    FAssert (pPlot != NULL);

    if (pPlot->plotCount(PUF_canBeCoreTroop, getOwnerINLINE(), getID()) == 0)
    {
        return false;
    }

    return true;
}

bool CvUnit::formLegion(bool bRegroup, bool bRegroupDone)
{
	PROFILE_FUNC();

    FAssert (getHeroType() >= 0 && getHeroType() < GC.getNumHeroInfos());
    if (!bRegroup)
    {
        FAssert (getHeroType() != NO_HERO);
        if (GAME.getHeroLegionSize(getHeroType()) == 0)
        {
            CyArgsList argsList;
            argsList.add(getHeroType());
            argsList.add(bRegroup);
            gDLL->getPythonIFace()->callFunction(PYScreensModule, "showLegionScreen", argsList.makeFunctionArgs());
            return false;
        }
        else
        {
            CvUnit* pCoreTroop = processLegion(false, false, false, false);
            FAssert (pCoreTroop != NULL);
			GAME.setLegionUnitMoved(pCoreTroop->getHeroType(), true);

            if (plot()->isActiveVisible(false))
            {
                NotifyEntity(MISSION_LEAD);
            }

            if (isHuman() && getOwnerINLINE() == GC.getGame().getActivePlayer())
            {
				gDLL->getInterfaceIFace()->selectGroup(pCoreTroop, gDLL->shiftKey(), gDLL->ctrlKey(), gDLL->altKey());
			}

			// Sanguo Mod tutorial, start, by poyuzhe 09.27.09
			if (!GET_PLAYER(getOwnerINLINE()).isFeatAccomplished(FEAT_FORM_LEGION))
			{
				GET_PLAYER(getOwnerINLINE()).setFeatAccomplished(FEAT_FORM_LEGION, true);
				if (GET_PLAYER(getOwnerINLINE()).isFeatPopup())
				{
					CyArgsList argsList;

					argsList.add(getOwnerINLINE());
					argsList.add(pCoreTroop->getHeroType());

					gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doFormLegionFeat", argsList.makeFunctionArgs());
				}
			}
			// Sanguo Mod Tutorial, end

            kill(false);

            return true;
        }
    }
    else
    {
        FAssert (isCoreTroop());
        if (!bRegroupDone)
        {
            GAME.backUpHeroLegionMember(getHeroType());
            CyArgsList argsList;
            argsList.add(getHeroType());
            argsList.add(bRegroup);
            gDLL->getPythonIFace()->callFunction(PYScreensModule, "showLegionScreen", argsList.makeFunctionArgs());
            return false;
        }
        else
        {
            HeroTypes eLeaderHero = getLeaderHero();
			bool bLeaderTroop = isLeaderTroop();
            int iAdvancedLegionSize = eLeaderHero != NO_HERO ? GAME.getHeroLegionSize(eLeaderHero) : 0;
            int iLeaderTroopID = -1;
            CvUnit* pLeaderHero = NULL;
            GAME.swapHeroLegionList(getHeroType());
            if (eLeaderHero != NO_HERO)
            {
                if (iAdvancedLegionSize > 1)
                {
                    quitAdvancedLegion(true);
                }
                else
                {
                    pLeaderHero = dismissAdvancedLegion(true);
                }
            }
            GAME.swapHeroLegionList(getHeroType());

            CvUnit* pCoreTroop = processLegion(true, false, false, false);

            FAssert (pCoreTroop != NULL);

            if (!pCoreTroop->isNakedHero() && eLeaderHero != NO_HERO)
            {
                iLeaderTroopID = GAME.getHeroCoreTroop(eLeaderHero);
                if (iAdvancedLegionSize > 1)
                {
					if (bLeaderTroop)
					{
						pCoreTroop->joinAdvancedLegion(iLeaderTroopID, true);
					}
					else
					{
						pCoreTroop->joinAdvancedLegion(iLeaderTroopID);
					}
				}
                else
                {
                    GAME.addHeroLegionMember(eLeaderHero, pCoreTroop->getID());
                    pLeaderHero->formAdvancedLegion(false, true);
                }
            }

		    GAME.setLegionUnitMoved(pCoreTroop->getHeroType(), true);

            if (isHuman() && getOwnerINLINE() == GC.getGame().getActivePlayer())
            {
			    gDLL->getInterfaceIFace()->selectGroup(pCoreTroop, gDLL->shiftKey(), gDLL->ctrlKey(), gDLL->altKey());
		    }

            return true;
        }
    }
}

bool CvUnit::isSameLegion(const CvUnit* pUnit) const
{
    FAssert (NULL != pUnit);

    if (getID() == pUnit->getID())
    {
        return false;
    }

    if (getLegionHero() == pUnit->getLegionHero())
    {
        return true;
    }

    return false;
}

bool CvUnit::canRegroupLegion(const CvPlot* pPlot) const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (getHeroType() == NO_HERO)
    {
        return false;
    }

    if (getLegionHero() != getHeroType())
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(getHeroType()))
    {
		return false;
	}

    FAssert (GAME.getHeroLegionSize(getHeroType()) > 0);
    FAssert (GAME.getHeroCoreTroop(getHeroType()) == getID());

    return true;
}

CvUnit* CvUnit::processLegion(bool bRegroup, bool bPrePromotion, bool bPostPromotion, bool bAttacked)
{
    PROFILE_FUNC();

	FAssert (!(bPrePromotion && bPostPromotion));
    FAssert (!(bRegroup && bPrePromotion));
    FAssert (!(bRegroup && bPostPromotion));

    UnitCombatTypes eUnitCombat;
    HeroTypes eHero = getHeroType();
    FAssert (eHero >= 0 && eHero < GC.getNumHeroInfos());

	// pCoreTroop is:
    // 1. We are naked hero, then is the unit we are going to attach to
    // 2. We are a Core Troop, then is ourselves, or the new unit our hero will abandon us for, or is NULL if legion is cleared
    CvUnit* pCoreTroop = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(eHero));

    // Find the correct unitCombatType.
    if (!bRegroup)
    {
        // if it's Form Legion mission,
        // then there must be a Core Troop selected before processLegion() is called
        FAssert(pCoreTroop != NULL);
        eUnitCombat = pCoreTroop->getUnitCombatType();
    }
    else
    {
        // if it's Regroup Legion mission, then use our (the old Core Troop's) unitcombatType
        eUnitCombat = getUnitCombatType();
    }

    // Find the proper promotions to give and take.
    PromotionTypes eUnitCombatPromotion = NO_PROMOTION;
    std::vector<PromotionTypes> MemberPromotionVec;
    std::vector<PromotionTypes> UniquePromotionVec;

    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
    {
        if (GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatType() == eUnitCombat)
        {
            if (GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatLevel() == GAME.getHeroUnitCombatLevel(eHero, eUnitCombat))
            {
                eUnitCombatPromotion = (PromotionTypes)iI;
            }
        }
        else if (GAME.getHeroPromotionStatus(eHero, (PromotionTypes)iI) == 1)
        {
            UniquePromotionVec.push_back((PromotionTypes)iI);
        }
        else if (GAME.getHeroPromotionStatus(eHero, (PromotionTypes)iI) == 2)
        {
            MemberPromotionVec.push_back((PromotionTypes)iI);
        }
    }

#ifdef _DEBUG
    if (eUnitCombatPromotion == NO_PROMOTION)
    {
        FAssert (GAME.getHeroUnitCombatLevel(eHero, eUnitCombat) <= 0);
    }
#endif

    // If is Regrouping Mission or is before hero's promotion, strip all old members' promotions
    // If is Regrouping Mission, strip all old members' hero stats as well
    if (bRegroup || bPrePromotion)
    {
        for (int iI = 0; iI < GAME.getHeroLegionBackUpSize(eHero); iI++)
		{
			CvUnit* pLoopUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionBackUpMemberAt(eHero, iI));
            if (iI == 0)
            {
                if (bRegroup)
                {
					pLoopUnit->setHeroType(NO_HERO);
				}
                for (std::vector<PromotionTypes>::iterator iter1 = UniquePromotionVec.begin(); iter1 != UniquePromotionVec.end(); ++iter1)
                {
                    if (getLeaderHero() == NO_HERO || GAME.getHeroPromotionStatus(getLeaderHero(), *iter1) != 1)
                    {
                        pLoopUnit->setHasPromotion((*iter1), false);
                    }
                }
            }

            if (eUnitCombatPromotion != NO_PROMOTION)
            {
                pLoopUnit->setHasPromotion(eUnitCombatPromotion, false);
            }

            for (std::vector<PromotionTypes>::iterator iter2 = MemberPromotionVec.begin(); iter2 != MemberPromotionVec.end(); ++iter2)
            {
                pLoopUnit->setHasPromotion((*iter2), false);
            }

            if (bRegroup && !GAME.isHeroHasLegionMember(eHero, pLoopUnit->getID()))
            {
                pLoopUnit->setLegionHero(NO_HERO);
            }
        }
    }

    // if it's before promoting a hero, then it's time to stop
    if (bPrePromotion)
    {
        return pCoreTroop;
    }

    if (bRegroup)
    {
        // if Legion has been cleared, check to see if we should resurrect the hero
        if (pCoreTroop == NULL)
        {
            // if we are not under attack, then it's a peaceful regroup legion mission; resurrect the hero
            if (!bAttacked)
            {
                setLeaderUnitType(NO_UNIT);

				// set pCoreTroop = pHero so we could return a value that is not NULL and means we have resurrect a hero
				pCoreTroop = GET_PLAYER(getOwnerINLINE()).initHero(eHero, getX_INLINE(), getY_INLINE(), GAME.getHeroDebutStatus(eHero) != getOwnerINLINE());
            }

            // since the legion is cleared, it's not necessary to do the rest, stop here
            return pCoreTroop;
        }
        // else if Legion's unitcombattype has changed, find the new unitcombat promotion
        else if (eUnitCombat != pCoreTroop->getUnitCombatType())
        {
            FAssert (!bAttacked);
            eUnitCombat = pCoreTroop->getUnitCombatType();
            eUnitCombatPromotion = NO_PROMOTION;
            for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
            {
                if (GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatType() == eUnitCombat)
                {
                    if (GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatLevel() == GAME.getHeroUnitCombatLevel(eHero, eUnitCombat))
                    {
                        eUnitCombatPromotion = (PromotionTypes)iI;
                    }
                }
            }
        }
    }

    FAssert (pCoreTroop != NULL);

    // Loop through all legion members and give proper promotions, then join them as a group
    CvSelectionGroup* pNewSelectionGroup =  GET_PLAYER(getOwnerINLINE()).addSelectionGroup();
	pNewSelectionGroup->init(pNewSelectionGroup->getID(), getOwnerINLINE());
	bool bOffenseWar = (area()->getAreaAIType(getTeam()) == AREAAI_OFFENSIVE);

    for (int iI = 0; iI < GAME.getHeroLegionSize(eHero); iI++)
	{
        CvUnit* pLoopUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(eHero, iI));
        if (bOffenseWar && pLoopUnit->AI_getUnitAIType() == UNITAI_COUNTER)
        {
            pLoopUnit->AI_setUnitAIType(UNITAI_ATTACK);
        }
        if (iI == 0)
        {
            FAssert (pLoopUnit = pCoreTroop);
            if (!bPostPromotion)
            {
                pLoopUnit->setHeroType(eHero);
            }
            for (std::vector<PromotionTypes>::iterator iter1 = UniquePromotionVec.begin(); iter1 != UniquePromotionVec.end(); ++iter1)
            {
                pLoopUnit->setHasPromotion((*iter1), true);
            }
        }

        if (eUnitCombatPromotion != NO_PROMOTION)
        {
            pLoopUnit->setHasPromotion(eUnitCombatPromotion, true);
        }

        for (std::vector<PromotionTypes>::iterator iter2 = MemberPromotionVec.begin(); iter2 != MemberPromotionVec.end(); ++iter2)
        {
            pLoopUnit->setHasPromotion((*iter2), true);
        }

        if (!bPostPromotion)
        {
            pLoopUnit->setLegionHero(eHero);
            pLoopUnit->joinGroup(pNewSelectionGroup);
        }

        // There are two situations:
        // 1. It's a Form Legion Mission, then we should pass Legion Members from the Naked Hero to the Core Troop
        // 2. It's a Regroup Legion Mission, and the Core Troop has Changed, then we should pass Legion Members from Old Core Troop to New Core Troop
        /*if ((!bRegroup || (bRegroup && getID() != pCoreTroop->getID())) && !bPostPromotion)
        {
            pCoreTroop->addLegionMember(*iter);
        }*/
    }

    // if it's when a hero has been promoted, since all due promotions are given to legion members, it's time to stop
    if (bPostPromotion)
    {
        return pCoreTroop;
    }

    // If is regrouping mission, and the Core troop has changed, clear the old core troop's hero stats, set the New Hero ID
    if (bRegroup)
    {
        if (getID() != pCoreTroop->getID())
        {
            //clearLegionMember();
            setLeaderUnitType(NO_UNIT);
            GAME.setHeroUnitID(eHero, pCoreTroop->getID());
            pCoreTroop->setLeaderUnitType((UnitTypes)GC.getHeroInfo(eHero).getUnitType());
        }
    }
    // If is From Legion Mission, set the New Hero Unit ID
    else
    {
		GAME.setHeroUnitID(eHero, pCoreTroop->getID());
		pCoreTroop->setLeaderUnitType((UnitTypes)GC.getHeroInfo(eHero).getUnitType());
	}

	FAssert (pNewSelectionGroup->headUnitNode() != NULL);

    return pCoreTroop;
}

bool CvUnit::canDismissLegion() const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (getHeroType() == NO_HERO)
    {
        return false;
    }

    if (getLegionHero() != getHeroType())
    {
        return false;
    }

    if (plot()->isWater())
    {
		return false;
	}

	if (hasMoved())
	{
		return false;
	}

	if (GAME.isLegionUnitMoved(getHeroType()))
	{
	    return false;
	}

	for (int iI = 0; iI < GAME.getHeroLegionSize(getHeroType()); iI++)
	{
		CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(getHeroType(), iI));
		FAssert (pMember != NULL);
		if (pMember->isHurt())
		{
		    return false;
		}
	}

    FAssert ( GAME.getHeroLegionSize(getHeroType()) > 0);
    FAssert ( GAME.getHeroCoreTroop(getHeroType()) == getID());

    return true;
}

CvUnit* CvUnit::dismissLegion(bool bAttacked)
{
    FAssert (getHeroType() >= 0 && getHeroType() < GC.getNumHeroInfos());
    bool bInArmy = (getLeaderHero() != NO_HERO);
	PromotionTypes eNineTransformation = bInArmy ? GAME.getHeroCurrentNineTransformationPromotion(getHeroType()) : NO_PROMOTION;
	if (bInArmy)
    {
        quitAdvancedLegion(true);
		// if legion current sets a nine transformation promotion, then check to see if after quiting army that nine tranformation promotion is still valid
		if (eNineTransformation != NO_PROMOTION && GAME.isHeroNineTransformationPromotion(getHeroType(), eNineTransformation))
		{
			eNineTransformation = NO_PROMOTION;
		}
    }

	GAME.backUpHeroLegionMember(getHeroType());
    GAME.clearHeroLegionMember(getHeroType());

    CvUnit* pHero = processLegion(true, false, false, bAttacked);

	// only take away nine transformation promotion that is not valid any more
	if (eNineTransformation != NO_PROMOTION)
	{
		GAME.setHeroCurrentNineTransformationPromotion(pHero->getHeroType(), NO_PROMOTION);
	}

#ifndef _DEBUG
	if (pHero != NULL)
	{
		GAME.setLegionUnitMoved(pHero->getHeroType(), true);
	}
#endif

    return pHero;
}

bool CvUnit::canJoinLegion(const CvPlot* pPlot) const
{
    PROFILE_FUNC();

	if (getLegionHero() != NO_HERO)
    {
        return false;
    }

    if (pPlot == NULL)
    {
        pPlot = plot();
    }

    if (hasMoved())
    {
		return false;
	}

    if (pPlot->plotCount(PUF_canAcceptLegionMember, getOwnerINLINE(), getID(), NO_PLAYER, NO_TEAM, PUF_hasNotMoved) == 0)
    {
        return false;
    }

    return true;
}

bool CvUnit::joinLegion(int iID)
{
    if (!isHuman())
    {
        if (!canJoinLegion(plot()))
        {
            return false;
        }

        // AI do the join legion stuff
        return true;
    }
    else
    {
        if (iID == -1)
        {
            int iLegionCount = plot()->plotCount(PUF_canAcceptLegionMember, getOwnerINLINE(), getID());
            FAssert (iLegionCount > 0);

            if (iLegionCount == 1)
            {
                CvUnit* pHero = plot()->plotCheck(PUF_canAcceptLegionMember, getOwnerINLINE(), getID());
                FAssert (pHero != NULL);
                FAssert (pHero->getHeroType() >= 0 && pHero->getHeroType() < GC.getNumHeroInfos());

				GAME.backUpHeroLegionMember(pHero->getHeroType());
				GAME.addHeroLegionMember(pHero->getHeroType(), getID());
                pHero->processLegion(true, false, false, false);

                return true;
            }
            else
            {
                CvPopupInfo* pInfo = new CvPopupInfo(BUTTONPOPUP_PYTHON);
                if (NULL != pInfo)
                {
                    pInfo->setText(gDLL->getText("TXT_KEY_JOIN_LEGION"));
                    pInfo->setData1(getID());
                    pInfo->setData2(getOwnerINLINE());
                    pInfo->setPythonModule(PYSanguoModule);
                    pInfo->setOnClickedPythonCallback("doJoinLegion");

                    // find all possible legions to join
                    CvUnit* pHero = plot()->plotCheck(PUF_canAcceptLegionMember, getOwnerINLINE(), getID());
                    while (pHero != NULL)
                    {
                        pHero->setTempCheck(true);
                        GET_PLAYER(getOwnerINLINE()).pushToJoinLegionTempHeroList(pHero->getID());
						pInfo->addPythonButton(gDLL->getText("TXT_KEY_CHOOSE_LEGION_HELP", GC.getHeroInfo(pHero->getHeroType()).getDescription(), pHero->getNameKey(), GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(pHero->getHeroType()))->getLevel()), "");
                        pHero = plot()->plotCheck(PUF_canAcceptLegionMember, getOwnerINLINE(), getID(), NO_PLAYER, NO_TEAM, PUF_isNotTempCheck);
                    }

                    CvUnit* pCheckedUnit = plot()->plotCheck(PUF_isTempCheck);
                    while (pCheckedUnit != NULL)
                    {
                        pCheckedUnit->setTempCheck(false);
                        pCheckedUnit = plot()->plotCheck(PUF_isTempCheck);
                    }

                    pInfo->addPythonButton(gDLL->getText("TXT_KEY_SCREEN_CANCEL"), "");
					pInfo->setData3(GET_PLAYER(getOwnerINLINE()).getJoinLegionTempHeroListSize());
                    pInfo->setFlags(0);
                    gDLL->getInterfaceIFace()->addPopup(pInfo, getOwnerINLINE(), true);
                }

                return false;
            }
        }
        else
        {
            CvUnit* pHero = GET_PLAYER(getOwnerINLINE()).getUnit(iID);
            FAssert (pHero != NULL);
            FAssert (pHero->getHeroType() >= 0 && pHero->getHeroType() < GC.getNumHeroInfos());

            HeroTypes eLeaderHero = pHero->getLeaderHero();
            int iAdvancedLegionSize = eLeaderHero != NO_HERO ? GAME.getHeroLegionSize(eLeaderHero) : 0;
            CvUnit* pLeaderHero = NULL;
			
            if (eLeaderHero != NO_HERO)
            {
                if (iAdvancedLegionSize > 1)
                {
                    pHero->quitAdvancedLegion(true);
                }
                else
                {
                    pLeaderHero = pHero->dismissAdvancedLegion();
                }
            }		

			GAME.backUpHeroLegionMember(pHero->getHeroType());
            GAME.addHeroLegionMember(pHero->getHeroType(), getID());
            pHero->processLegion(true, false, false, false);

            if (eLeaderHero != NO_HERO)
            {
                if (iAdvancedLegionSize > 1)
                {
                    int iUnitID = GAME.getHeroCoreTroop(eLeaderHero);
                    pHero->joinAdvancedLegion(iUnitID);
                }
                else
                {
                    GAME.addHeroCoreTroop(eLeaderHero, pHero->getID());
                    pLeaderHero->formAdvancedLegion(false, true);
                }
            }

            return true;
        }
    }
}

bool CvUnit::canAcceptLegionMember(const CvUnit* pUnit) const
{
    if (getHeroType() == NO_HERO)
    {
        return false;
    }

    if (m_pUnitInfo->isHero())
    {
        return false;
    }

    if ( GAME.getHeroLegionSize(getHeroType()) == 0)
    {
        return false;
    }

    if (getOwnerINLINE() != pUnit->getOwnerINLINE())
    {
        return false;
    }

    FAssert (getID() ==  GAME.getHeroCoreTroop(getHeroType()));

    if (getUnitCombatType() != pUnit->getUnitCombatType())
    {
        return false;
    }

    if ( GAME.getHeroLegionSize(getHeroType()) >= GAME.getHeroCapacity(getHeroType()))
    {
        return false;
    }

    FAssert ( GAME.getHeroLegionSize(getHeroType()) < GAME.getHeroCapacity(getHeroType()));

    if (GAME.isLegionUnitMoved(getHeroType()))
    {
        return false;
    }

    return true;
}

void CvUnit::setTempCheck(bool bNewValue)
{
    m_bTempCheck = bNewValue;
}

bool CvUnit::isNotTempCheck() const
{
    return (!m_bTempCheck);
}

bool CvUnit::isTempCheck() const
{
    return m_bTempCheck;
}

bool CvUnit::canQuitLegion()
{
	PROFILE_FUNC();

	if (getLegionHero() == NO_HERO)
    {
        return false;
    }

    if (getHeroType() != NO_HERO)
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(getLegionHero()))
    {
		return false;
	}

	if (isHurt())
	{
	    return false;
	}

	CvUnit* pHero = GAME.getHeroUnit(getLegionHero());
	FAssert (pHero != NULL);
	if (pHero->hasMoved())
	{
		return false;
	}

    return true;
}

bool CvUnit::quitLegion(bool bAttacked)
{
    CvUnit* pCoreTroop = GAME.getHeroUnit(getLegionHero());
    FAssert (pCoreTroop != NULL);
    FAssert (pCoreTroop->getHeroType() >= 0 && pCoreTroop->getHeroType() < GC.getNumHeroInfos());

    HeroTypes eLeaderHero = getLeaderHero();
    bool bLeaderTroop = pCoreTroop->isLeaderTroop();
    CvUnit* pLeaderHero = NULL;
    if (eLeaderHero != NO_HERO)
    {
		// if the Army has more than 1 legion, then temp quit army
        if (GAME.getHeroLegionSize(eLeaderHero) > 1)
        {
            pCoreTroop->quitAdvancedLegion(true);
        }
		// else then we are the only legion in army, temp dismiss army
        else
        {
            pLeaderHero = pCoreTroop->dismissAdvancedLegion(true);
        }
    }

    GAME.backUpHeroLegionMember(getLegionHero());
    GAME.deleteHeroLegionMember(getLegionHero(), getID());
    pCoreTroop = pCoreTroop->processLegion(true, false, false, bAttacked);

    if (eLeaderHero != NO_HERO)
    {
		if (pCoreTroop != NULL)
		{
			if (pLeaderHero == NULL)
			{
				// if army is not dismissed
				int iUnitID = GAME.getHeroCoreTroop(eLeaderHero);
				FAssert (pCoreTroop != NULL && pCoreTroop->getHeroType() != NO_HERO);
				if (!bLeaderTroop)
				{
					// if we are not guardian of Army, join as member legion
					pCoreTroop->joinAdvancedLegion(iUnitID);
				}
				else
				{
					// join as guardian legion
					pCoreTroop->joinAdvancedLegion(iUnitID, true);
				}
			}
			else
			{
				// eLeaderHero's back up list should have army heroes previous to temp dismiss army
				// if pCoreTroop == NULL, which means the legion has 0 member now, then don't bother to form advanced legion
				GC.getGame().swapHeroLegionList(eLeaderHero);
				pLeaderHero->formAdvancedLegion(false, true);
			}
		}
    }

    return true;
}

void CvUnit::processCombatWin(CvUnit* pLoser, bool bAttacking)
{
    PROFILE_FUNC();

	if (getLegionHero() == NO_HERO && pLoser->getLegionHero() == NO_HERO)
    {
        return;
    }

    bool bCoreTroop = false;
    bool bLoserCoreTroop = false;
    bool bStealHeroPromotion = false;
    bool bDefeatGeneralRestoreMoves = false;
    bool bCaptureBarbarian = false;
    bool bEnemyCaptured = false;
    bool bWinCombatSprawlBarbarian = false;
    bool bWinCombatSpreadCulture = false;
    bool bScatterEnemyStack = false;
    bool bWinnerTargetCoreTroopFirst = false;
    int iStartGoldenAgeChancePerLevel = 0;
    int iLowerCityDefensePercent = 0;
    int iSameTurnWinStrengthModifier = 0;
    int iConvertWarlordChance = 0;
    int iCaptureEnemyChance = 0;
    int iCoreTroopDefeatedStrengthModifier = 0;
    int iDefeatCoreTroopStrengthModifier = 0;
    int iUponDeathShareExperienceAmount = 0;
    int iSeaBattleWinPlunderPercent = 0;
    int iRestoreHealthPercent = 0;
	int iSecondAttackChance = 0;
    int iI;
    CvWString szBuffer;
	CvPlot* pLoserPlot = pLoser->plot();

    if (isCoreTroop())
    {
        bCoreTroop = true;
    }

    if (pLoser->isCoreTroop())
    {
        bLoserCoreTroop = true;
    }

    for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
    {
        if (isHasPromotion((PromotionTypes)iI) && GC.getPromotionInfo((PromotionTypes)iI).isHero())
        {
            iStartGoldenAgeChancePerLevel += GC.getPromotionInfo((PromotionTypes)iI).getKillCoreTroopGoldenAgeChancePerLevel();
            iLowerCityDefensePercent += GC.getPromotionInfo((PromotionTypes)iI).getAttackedCityDefenseChange();
            iSameTurnWinStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getSameTurnWinStrengthModifier();
            iConvertWarlordChance += GC.getPromotionInfo((PromotionTypes)iI).getConvertWarlordChance();
            iCaptureEnemyChance += GC.getPromotionInfo((PromotionTypes)iI).getCaptureEnemyChance();
            iDefeatCoreTroopStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getDefeatCoreTroopStrengthModifier();
            iSeaBattleWinPlunderPercent += GC.getPromotionInfo((PromotionTypes)iI).getSeaBattleWinPlunderPercent();
            iRestoreHealthPercent += GC.getPromotionInfo((PromotionTypes)iI).getWinCombatRestoreHealthPercent();
			iSecondAttackChance += GC.getPromotionInfo((PromotionTypes)iI).getSecondAttackChance();

            if (GC.getPromotionInfo((PromotionTypes)iI).canStealHeroPromotion())
            {
                bStealHeroPromotion = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isDefeatGeneralRestoreMoves())
            {
                bDefeatGeneralRestoreMoves = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).canCaptureBarbarian())
            {
                bCaptureBarbarian = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isWinCombatSprawlBarbarian())
            {
                bWinCombatSprawlBarbarian = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isWinCombatSpreadCulture())
            {
                bWinCombatSpreadCulture = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).canScatterEnemyStack())
            {
                bScatterEnemyStack = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isTargetCoreTroopFirst())
            {
                bWinnerTargetCoreTroopFirst = true;
            }
        }
        if (pLoser->isHasPromotion((PromotionTypes)iI))
        {
            if (pLoser->getLegionHero() != NO_HERO && GC.getPromotionInfo((PromotionTypes)iI).isHero())
            {
                iUponDeathShareExperienceAmount += GC.getPromotionInfo((PromotionTypes)iI).getUponDeathShareExperienceAmount();
                if (bLoserCoreTroop)
                {
                    iCoreTroopDefeatedStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getCoreTroopDefeatedStrengthModifier();
                }
            }
        }
    }

    if (bCoreTroop && bLoserCoreTroop && iStartGoldenAgeChancePerLevel > 0)
    {
		if (GAME.getSorenRandNum(100, "Start a Golden Age") < iStartGoldenAgeChancePerLevel * (GAME.getHeroLevel(pLoser->getHeroType())))
        {
            GET_PLAYER(getOwnerINLINE()).changeGoldenAgeTurns(GET_PLAYER(getOwnerINLINE()).getGoldenAgeLength());
            if (isHuman())
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_TRIUMPH", GC.getHeroInfo(getLegionHero()).getDescription(), GC.getHeroInfo(pLoser->getLegionHero()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_WELOVEKING", MESSAGE_TYPE_MINOR_EVENT, ARTFILEMGR.getInterfaceArtInfo("INTERFACE_HAPPY_PERSON")->getPath(), (ColorTypes)GC.getInfoTypeForString("COLOR_YELLOW"), getX_INLINE(), getY_INLINE(), true, true);
            }
        }
    }

    if (iLowerCityDefensePercent > 0 && pLoser->plot()->isCity())
    {
        int iBombardModifier = 0;
        CvCity* pBombardCity = pLoser->plot()->getPlotCity();
        FAssert (pBombardCity != NULL);
        if (!ignoreBuildingDefense())
        {
            iBombardModifier -= pBombardCity->getBuildingBombardDefense();
        }

        pBombardCity->changeDefenseModifier(-(iLowerCityDefensePercent * std::max(0, 100 + iBombardModifier)) / 100);

        if (isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_MISC_DEFENSES_IN_CITY_REDUCED_TO", pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false), GET_PLAYER(getOwnerINLINE()).getNameKey());
            gDLL->getInterfaceIFace()->addMessage(pBombardCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE(), true, true);
        }

        if (pLoser->isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_REDUCE_CITY_DEFENSES", GC.getHeroInfo(getLegionHero()).getDescription(), pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false));
            gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE());
	    }
    }

	if (iSameTurnWinStrengthModifier > 0)
	{
	    changeTempStrengthModifier(iSameTurnWinStrengthModifier);
	    if (isHuman())
	    {
            szBuffer = gDLL->getText("TXT_KEY_HERO_SAME_TURN_WIN_EXTRA_STRENGTH", GC.getHeroInfo(getLegionHero()).getDescription());
            gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_CITYRAZE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	    }
	}

	if (bCoreTroop && bLoserCoreTroop && bStealHeroPromotion)
	{
	    for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	    {
	        if (GC.getPromotionInfo((PromotionTypes)iI).isHero())
	        {
	            if (!isHasPromotion((PromotionTypes)iI))
	            {
	                if (pLoser->isHasPromotion((PromotionTypes)iI))
	                {
	                    if (GAME.canHeroAcquirePromotion(getHeroType(), (PromotionTypes)iI, true))
	                    {
	                        if (GAME.getSorenRandNum(100, "Steal Hero Promotion") < GC.getDefineINT("HERO_STEAL_PROMOTION_PERCENT"))
	                        {
	                            GAME.setHeroPromotionStatus(getHeroType(), (PromotionTypes)iI, GC.getPromotionInfo((PromotionTypes)iI).getHeroPromotionStatus(), false);
	                            if (isHuman())
	                            {
                                    szBuffer = gDLL->getText("TXT_KEY_HERO_STEALT_PROMOTION", GC.getHeroInfo(getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(pLoser->getLegionHero()).getDescription());
                                    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
                                }
                                if (pLoser->isHuman())
                                {
                                    szBuffer = gDLL->getText("TXT_KEY_HERO_PROMOTION_STEALT", GC.getHeroInfo(getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(pLoser->getLegionHero()).getDescription());
                                    gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pLoser->getX_INLINE(), pLoser->getY_INLINE(), true, true);
                                }
	                            break;
	                        }
	                    }
	                }
	            }
	        }
	    }
	}

	//if (iConvertWarlordChance > 0)
	//{
	//    if (pLoser->getHeroType() == NO_HERO && pLoser->getLeaderUnitType() != NO_UNIT)
	//    {
	//        if (GAME.getSorenRandNum(100, "Convert Warlord") < iConvertWarlordChance)
 //           {
 //               CvUnit* pWarlord = GET_PLAYER(getOwnerINLINE()).initUnit(pLoser->getLeaderUnitType(), getX_INLINE(), getY_INLINE());
	//			if (pWarlord != NULL)
	//			{
	//				GAME.removeGreatPersonBornName(pWarlord->getNameNoDesc());
	//				CvWString szOldName = pLoser->getNameNoDesc();
	//				if (GAME.isGreatPersonBorn(szOldName))
	//				{
	//					pWarlord->setName(szOldName);
	//				}
	//				if (isHuman())
	//				{
	//					szBuffer = gDLL->getText("TXT_KEY_WARLORD_CONVERTED", GC.getHeroInfo(getLegionHero()).getDescription(), pLoser->getNameNoDesc().GetCString());
	//					gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	//				}
	//				if (pLoser->isHuman())
	//				{
	//					szBuffer = gDLL->getText("TXT_KEY_WARLORD_BETRAYED", pLoser->getNameNoDesc().GetCString(), GC.getHeroInfo(getLegionHero()).getDescription());
	//					gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_CITYRAZED", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE(), true, true);
	//				}
	//			}
 //           }
 //           else
 //           {
 //               CvCity* pLoserCapital = GET_PLAYER(pLoser->getOwnerINLINE()).getCapitalCity();
 //               if (pLoserCapital != NULL && !pLoser->atPlot(pLoserCapital->plot()))
 //               {
 //                   CvUnit* pWarlord = GET_PLAYER(pLoser->getOwnerINLINE()).initUnit(pLoser->getLeaderUnitType(), pLoserCapital->getX_INLINE(), pLoserCapital->getY_INLINE());
	//				if (pWarlord != NULL)
	//				{
	//					GAME.removeGreatPersonBornName(pWarlord->getNameNoDesc());
	//					CvWString szOldName = pLoser->getNameNoDesc();
	//					if (GAME.isGreatPersonBorn(szOldName))
	//					{
	//						pWarlord->setName(szOldName);
	//					}

	//					if (isHuman())
	//					{
	//						szBuffer = gDLL->getText("TXT_KEY_WARLORD_RELEASED", GC.getHeroInfo(getLegionHero()).getDescription(), pLoser->getNameNoDesc().GetCString());
	//						gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	//					}
	//					if (pLoser->isHuman())
	//					{
	//						szBuffer = gDLL->getText("TXT_KEY_WARLORD_BEEN_RELEASED", pLoser->getNameNoDesc().GetCString(), GC.getHeroInfo(getLegionHero()).getDescription());
	//						gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_CITYRAZED", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE(), true, true);
	//					}
	//				}
 //               }
 //           }
	//    }
	//}

	if (bDefeatGeneralRestoreMoves)
	{
	    if (pLoser->getLeaderUnitType() != NO_UNIT)
	    {
            setMoves(0);
            setMadeAttack(false);
            if (isHuman())
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_RESTORE_MOVES", GC.getHeroInfo(getLegionHero()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
            }
	    }
	}

	if (bCaptureBarbarian && !bEnemyCaptured)
	{
	    if (pLoser->isBarbarian() && !pLoser->isNoCapture() && !pLoser->isNakedHero())
	    {
	        CvUnit* pNewUnit = GET_PLAYER(getOwnerINLINE()).initUnit(pLoser->getUnitType(), getX_INLINE(), getY_INLINE());
	        pNewUnit->finishMoves();
	        pNewUnit->setDamage(99);
	        pNewUnit->setExperience(pLoser->getExperience(), -1);
	        pNewUnit->setLevel(pLoser->getLevel());

	        for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	        {
	            if (pLoser->isHasPromotion((PromotionTypes)iI))
	            {
	                if (!GC.getPromotionInfo((PromotionTypes)iI).isHero() && GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatLevel() == 0)
	                {
	                    pNewUnit->setHasPromotion((PromotionTypes)iI, true);
	                }
	            }
	        }

	        bEnemyCaptured = true;
	    }
	}

    if (!bEnemyCaptured && iCaptureEnemyChance > 0)
    {
        // only capture the enemy if it's not a warlord, not a naked hero, not a no capture unit, and not a capturable unit whose capture UnitType is the same as his own UnitType
        if ((pLoser->getLeaderUnitType() == NO_UNIT || (pLoser->getLeaderUnitType() != NO_UNIT && pLoser->getHeroType() != NO_HERO)) && !pLoser->isNakedHero() && !pLoser->isNoCapture() && pLoser->getCaptureUnitType(getCivilizationType()) != pLoser->getUnitType())
        {
            if (GAME.getSorenRandNum(100, "Capture Enemy Unit") < iCaptureEnemyChance)
            {
                CvUnit* pNewUnit = GET_PLAYER(getOwnerINLINE()).initUnit(pLoser->getUnitType(), getX_INLINE(), getY_INLINE());
                pNewUnit->finishMoves();
                pNewUnit->setDamage(99);
                pNewUnit->setExperience(pLoser->getExperience(), -1);
                pNewUnit->setLevel(pLoser->getLevel());

                for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                {
                    if (pLoser->isHasPromotion((PromotionTypes)iI))
                    {
                        if (!GC.getPromotionInfo((PromotionTypes)iI).isHero() && GC.getPromotionInfo((PromotionTypes)iI).getPromotionUnitCombatLevel() == 0)
                        {
                            pNewUnit->setHasPromotion((PromotionTypes)iI, true);
                        }
                    }
                }

                bEnemyCaptured = true;
            }
        }
    }

    if (bWinCombatSprawlBarbarian)
    {
        bool bPeasant = false;
        for (int iTrait = 0; iTrait < GC.getNumTraitInfos(); iTrait++)
        {
            if (GET_PLAYER((PlayerTypes)getOwnerINLINE()).hasTrait((TraitTypes)iTrait) && GC.getTraitInfo((TraitTypes)iTrait).isPeasant())
            {
                bPeasant = true;
                break;
            }
        }
        if (bPeasant && plot()->getOwner() != getOwnerINLINE() && pLoser->plot()->getOwner() != getOwnerINLINE())
        {
            UnitTypes eBestUnit = NO_UNIT;
            int iValue;
            int iBestValue = 0;
            UnitTypes eLoopUnit;

            for (iI = 0; iI < GC.getNumUnitClassInfos(); iI++)
            {
                bool bValid = false;
                eLoopUnit = ((UnitTypes)(GC.getCivilizationInfo(GET_PLAYER(BARBARIAN_PLAYER).getCivilizationType()).getCivilizationUnits(iI)));

                if (eLoopUnit != NO_UNIT)
                {
                    CvUnitInfo& kUnit = GC.getUnitInfo(eLoopUnit);

                    bValid = (kUnit.getCombat() > 0 && !kUnit.isOnlyDefensive());

                    if (bValid)
                    {
                        if (plot()->isWater() && kUnit.getDomainType() != DOMAIN_SEA)
                        {
                            bValid = false;
                        }
                        else if (!plot()->isWater() && kUnit.getDomainType() != DOMAIN_LAND)
                        {
                            bValid = false;
                        }
                    }

                    if (bValid)
                    {
                        if (!GET_PLAYER(BARBARIAN_PLAYER).canTrain(eLoopUnit))
                        {
                            bValid = false;
                        }
                    }

                    if (bValid)
                    {
                        if (NO_BONUS != kUnit.getPrereqAndBonus())
                        {
                            if (!GET_TEAM(BARBARIAN_TEAM).isHasTech((TechTypes)GC.getBonusInfo((BonusTypes)kUnit.getPrereqAndBonus()).getTechCityTrade()))
                            {
                                bValid = false;
                            }
                        }
                    }

                    if (bValid)
                    {
                        bool bFound = false;
                        bool bRequires = false;
                        for (int i = 0; i < GC.getNUM_UNIT_PREREQ_OR_BONUSES(); ++i)
                        {
                            if (NO_BONUS != kUnit.getPrereqOrBonuses(i))
                            {
                                TechTypes eTech = (TechTypes)GC.getBonusInfo((BonusTypes)kUnit.getPrereqOrBonuses(i)).getTechCityTrade();
                                if (NO_TECH != eTech)
                                {
                                    bRequires = true;

                                    if (GET_TEAM(BARBARIAN_TEAM).isHasTech(eTech))
                                    {
                                        bFound = true;
                                        break;
                                    }
                                }
                            }
                        }

                        if (bRequires && !bFound)
                        {
                            bValid = false;
                        }
                    }

                    if (bValid)
                    {
                        iValue = (1 + GAME.getSorenRandNum(1000, "Barb Unit Selection"));

                        if (iValue > iBestValue)
                        {
                            eBestUnit = eLoopUnit;
                            iBestValue = iValue;
                        }
                    }
                }
            }

            if (eBestUnit != NO_UNIT)
            {
                GET_PLAYER(BARBARIAN_PLAYER).initUnit(eBestUnit, getX_INLINE(), getY_INLINE());
            }
        }
    }

    if (bWinCombatSpreadCulture)
    {
        doVictoryInfluence(pLoser, bAttacking);
        if (isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_HERO_STEAL_CULTURE", GC.getHeroInfo(getLegionHero()).getDescription());
            gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE());
        }

        if (pLoser->isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_HERO_CULTURE_STEALT", GC.getHeroInfo(getLegionHero()).getDescription());
            gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
        }
    }

    if (iCoreTroopDefeatedStrengthModifier > 0)
    {
        if (bLoserCoreTroop)
        {
            CvUnit* pUnit = GET_PLAYER(pLoser->getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(pLoser->getLegionHero(), pLoser->getID()));
            while(pUnit != NULL && pUnit->getID() != pLoser->getID())
            {
                pUnit->changeTempStrengthModifier(iCoreTroopDefeatedStrengthModifier);
				pUnit = GET_PLAYER(pLoser->getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(pLoser->getLegionHero(), pUnit->getID()));
            }

            if (pLoser->isHuman())
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_CORE_TROOP_DEAD_STRENGTH_BOOST", GC.getHeroInfo(pLoser->getHeroType()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
            }
        }
    }
    else
    {
        if (bLoserCoreTroop)
        {
			CvUnit* pUnit = GET_PLAYER(pLoser->getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(pLoser->getLegionHero(), pLoser->getID()));
            while(pUnit != NULL && pUnit->getID() != pLoser->getID())
            {
                pUnit->changeTempStrengthModifier(GC.getDefineINT("HERO_CORE_TROOP_DEFEATED_STRENGTH_DOWN"));
				pUnit = GET_PLAYER(pLoser->getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(pLoser->getLegionHero(), pUnit->getID()));
            }

			if (pLoser->isHuman() && GAME.getHeroLegionSize(pLoser->getLegionHero()) > 1)
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_CORE_TROOP_DEAD_STRENGTH_DOWN", GC.getHeroInfo(pLoser->getHeroType()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(pLoser->getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
            }
        }
    }

    if (iDefeatCoreTroopStrengthModifier > 0)
    {
        if (bLoserCoreTroop)
        {
            changeTempStrengthModifier(iDefeatCoreTroopStrengthModifier);
            CvUnit* pUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(getLegionHero(), getID()));
            while(pUnit != NULL && pUnit->getID() != getID())
            {
                pUnit->changeTempStrengthModifier(iDefeatCoreTroopStrengthModifier);
                pUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(getLegionHero(), pUnit->getID()));
            }
            if (isHuman())
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_DEFEAT_CORE_TROOP_STRENGTH_BOOST", GC.getHeroInfo(pLoser->getHeroType()).getDescription(), GC.getHeroInfo(getLegionHero()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE());
            }
        }
    }

    if (iUponDeathShareExperienceAmount > 0)
    {
        if (pLoser->getLegionHero() != NO_HERO)
        {
			for (iI = 0; iI < GAME.getHeroLegionSize(pLoser->getLegionHero()); iI++)
            {
				int iUnitID = GAME.getHeroLegionMemberAt(pLoser->getLegionHero(), iI);
				if (iUnitID != pLoser->getID())
				{
					FAssert ( GAME.getHeroLegionSize(pLoser->getLegionHero()) >= 2);
					CvUnit* pUnit = GET_PLAYER(pLoser->getOwnerINLINE()).getUnit(iUnitID);
					pUnit->changeExperience(pLoser->getExperience() * iUponDeathShareExperienceAmount / (( GAME.getHeroLegionSize(pLoser->getLegionHero()) - 1) * 100), -1, false, false, false, NO_TERRAIN, false);
                }
            }
            if (pLoser->isHuman())
            {
                szBuffer = gDLL->getText("TXT_KEY_HERO_lEGION_MEMBER_SHARE_EXPERIENCE", pLoser->getNameKey(), GC.getHeroInfo(pLoser->getLegionHero()).getDescription());
                gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(pLoser->getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
            }
        }
    }

    if (bAttacking && bScatterEnemyStack)
    {
        if (!pLoser->plot()->isCity())
        {
            CvUnit* pLoopUnit;
            CLLNode<IDInfo>* pUnitNode;
            pUnitNode = pLoser->plot()->headUnitNode();
            while (pUnitNode != NULL)
            {
                pLoopUnit = ::getUnit(pUnitNode->m_data);
                pUnitNode = pLoser->plot()->nextUnitNode(pUnitNode);
                if (pLoopUnit == pLoser)
                {
                    continue;
                }
                if (pLoopUnit->isEnemy(getTeam()) && !pLoopUnit->isDefendFromCharge())
                {
                    if (GAME.getSorenRandNum(100, "Scatter enemy stack") < GC.getDefineINT("HERO_SCATTER_ENEMY_STACK_CHANCE"))
                    {
                        pLoopUnit->joinGroup(NULL);
                        CvPlot* pLoopPlot;
                        int iValue;
                        int iBestValue = 0;
                        CvPlot* pBestPlot;
                        for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
                        {
                            pLoopPlot = plotDirection(pLoser->plot()->getX_INLINE(), pLoser->plot()->getY_INLINE(), ((DirectionTypes)iI));

                            if (pLoopPlot != NULL && pLoopPlot->isValidDomainForLocation(*pLoopUnit))
                            {
                                if (pLoopUnit->canMoveInto(pLoopPlot))
                                {
                                    if (pLoopUnit->canEnterArea(pLoopPlot->getTeam(), pLoopPlot->area()))
                                    {
                                        FAssertMsg(!pLoopUnit->atPlot(pLoopPlot), "atPlot(pLoopPlot) did not return false as expected");

                                        if ((pLoopUnit->getDomainType() != DOMAIN_AIR) || pLoopPlot->isFriendlyCity(*pLoopUnit, true))
                                        {
                                            if (pLoopPlot->isRevealed(pLoopUnit->getTeam(), false))
                                            {
                                                iValue = GAME.getSorenRandNum(10, "Random Plot Value");

                                                if (iValue > iBestValue)
                                                {
                                                    iBestValue = iValue;
                                                    pBestPlot = pLoopPlot;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (pBestPlot != NULL)
                        {
                            pLoopUnit->setXY(pBestPlot->getX_INLINE(), pBestPlot->getY_INLINE());
                        }
                    }
                }
            }
        }
    }

    if ((plot()->isWater() && !bAttacking) || (pLoser->plot()->isWater() && bAttacking))
    {
        if (iSeaBattleWinPlunderPercent > 0)
        {
            int iCost = pLoser->getUnitInfo().getProductionCost();

            iCost *= GC.getGameSpeedInfo(GAME.getGameSpeedType()).getTrainPercent();
            iCost /= 100;

            iCost *= GC.getEraInfo(GAME.getStartEra()).getTrainPercent();
            iCost /= 100;

            if (iCost > 0)
            {
                int iGold = iCost * iSeaBattleWinPlunderPercent / 100;

                iGold = std::max(GAME.getSorenRandNum(iGold, "Random Plunder Gold"), iGold / 2);
                if (iGold > GET_PLAYER(pLoser->getOwnerINLINE()).getGold())
                {
                    iGold = GET_PLAYER(pLoser->getOwnerINLINE()).getGold();
                }

                GET_PLAYER(getOwnerINLINE()).changeGold(iGold);
                GET_PLAYER(pLoser->getOwnerINLINE()).changeGold(-iGold);

                if (isHuman())
                {
                    szBuffer = gDLL->getText("TXT_KEY_HERO_SEA_PLUNDER", getNameKey(), iGold, GET_PLAYER(pLoser->getOwnerINLINE()).getCivilizationDescriptionKey());
                    gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());
                }
                if (pLoser->isHuman())
                {
                    szBuffer = gDLL->getText("TXT_KEY_HERO_SEA_PLUNDERED", pLoser->getNameKey(), GET_PLAYER(pLoser->getOwnerINLINE()).getCivilizationDescriptionKey(), iGold);
                    gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
                }
            }
        }
    }

    if (iRestoreHealthPercent > 0 && bCoreTroop)
    {
		changeDamage(-iRestoreHealthPercent);
		if (isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_HERO_RESTORE_HEALTH", GC.getHeroInfo(getHeroType()).getDescription());
            gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());
        }
        if (pLoser->isHuman())
        {
            szBuffer = gDLL->getText("TXT_KEY_HERO_RESTORE_HEALTH", GC.getHeroInfo(getHeroType()).getDescription());
            gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
        }
    }

    if (bLoserCoreTroop && bWinnerTargetCoreTroopFirst)
    {
        CvUnit* pLoopUnit;
        CLLNode<IDInfo>* pUnitNode;
        pUnitNode = pLoser->plot()->headUnitNode();
		bool bAssaulted = false;
        while (pUnitNode != NULL)
        {
            pLoopUnit = ::getUnit(pUnitNode->m_data);
            pUnitNode = pLoser->plot()->nextUnitNode(pUnitNode);
			if (pLoopUnit->getLegionHero() == pLoser->getLegionHero())
			{
				pLoopUnit->changeTempStrengthModifier(-GAME.getSorenRandNum(GC.getDefineINT("HERO_ASSAULT_CORE_TROOP_HEALTH_CHANGE"), "Assault Random Hit Point Change"));
				bAssaulted = true;
			}
		}
		if (bAssaulted)
		{
			if (isHuman())
			{
				szBuffer = gDLL->getText("TXT_KEY_HERO_ASSAULT_CORE_TROOP_HEALTH_CHANGE", GC.getHeroInfo(getHeroType()).getDescription(), GC.getHeroInfo(pLoser->getHeroType()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());
			}
			if (pLoser->isHuman())
			{
				szBuffer = gDLL->getText("TXT_KEY_HERO_ASSAULT_CORE_TROOP_HEALTH_CHANGE", GC.getHeroInfo(getHeroType()).getDescription(), GC.getHeroInfo(pLoser->getHeroType()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO, GC.getHeroInfo(pLoser->getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoser->getX_INLINE(), pLoser->getY_INLINE());
			}
		}
    }

	if (bCoreTroop && bAttacking && iSecondAttackChance > 0)
	{
		if (GAME.getSorenRandNum(100, "Second Attack Chance") < iSecondAttackChance)
		{
			CvUnit* pBestDefender = pLoserPlot->getBestDefender(pLoser->getOwnerINLINE(), getOwnerINLINE(), this, true);
			if (pBestDefender != pLoser && pBestDefender != NULL && getCombatOdds(this, pBestDefender) > GC.getDefineINT("HERO_SECOND_ATTACK_ODDS") * 10)
			{
				szBuffer = gDLL->getText("TXT_KEY_HERO_MORALE_SECOND_ATTACK", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_POSITIVE_DING", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoserPlot->getX_INLINE(), pLoserPlot->getY_INLINE());
				szBuffer = gDLL->getText("TXT_KEY_HERO_MORALE_SECOND_ATTACKED", getNameKey(), GC.getHeroInfo(getLegionHero()).getDescription());
				gDLL->getInterfaceIFace()->addMessage(pLoser->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_NEGATIVE_DING", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pLoserPlot->getX_INLINE(), pLoserPlot->getY_INLINE());
				if (canMoveInto(pLoserPlot, true))
				{
					attack(pLoserPlot, false, pBestDefender);
				}
				return;
			}
		}
	}
}

void CvUnit::processCombatWithdrawal(CvUnit* pDefender, bool bAttackerWithdrawal)
{
    bool bAttackedCity = false;
    bool bStealHeroPromotion = false;
    bool bCoreTroop = false;
    bool bDefenderCoreTroop = false;
    bool bDefenderStealHeroPromotion = false;
    bool bAttackWithSameLegion = false;
	bool bDefenderForceWithdrawal = false;
    int iLowerCityDefensePercent = 0;
    int iI;
	CvPlot* pDefenderPlot = pDefender->plot();
	PlayerTypes eDefendPlayer = pDefender->getOwnerINLINE();

    if (isCoreTroop())
    {
        bCoreTroop = true;
    }

    if (pDefender->isCoreTroop())
    {
        bDefenderCoreTroop = true;
    }

    if (pDefender->plot()->isCity())
    {
        bAttackedCity = true;
    }

    for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
    {
        if (isHasPromotion((PromotionTypes)iI))
        {
            if (GC.getPromotionInfo((PromotionTypes)iI).getAttackedCityDefenseChange() > 0)
            {
                iLowerCityDefensePercent += GC.getPromotionInfo((PromotionTypes)iI).getAttackedCityDefenseChange();
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).canStealHeroPromotion())
            {
                bStealHeroPromotion = true;
            }
            if (GC.getPromotionInfo((PromotionTypes)iI).isAttackWithSameLegion())
            {
				bAttackWithSameLegion = true;
			}
        }
        if (pDefender->isHasPromotion((PromotionTypes)iI))
        {
            if (GC.getPromotionInfo((PromotionTypes)iI).canStealHeroPromotion())
            {
                bDefenderStealHeroPromotion = true;
            }
			if (GC.getPromotionInfo((PromotionTypes)iI).isForceDefensiveWithdrawal())
			{
				bDefenderForceWithdrawal = true;
			}
        }
    }

    if (bAttackedCity && iLowerCityDefensePercent > 0)
    {
        int iBombardModifier = 0;
        CvCity* pBombardCity = pDefender->plot()->getPlotCity();
        FAssert (pBombardCity != NULL);
        if (!ignoreBuildingDefense())
        {
            iBombardModifier -= pBombardCity->getBuildingBombardDefense();
        }

        pBombardCity->changeDefenseModifier(-(iLowerCityDefensePercent * std::max(0, 100 + iBombardModifier)) / 100);

        CvWString szBuffer = gDLL->getText("TXT_KEY_MISC_DEFENSES_IN_CITY_REDUCED_TO", pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false), GET_PLAYER(getOwnerINLINE()).getNameKey());
        gDLL->getInterfaceIFace()->addMessage(pBombardCity->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE(), true, true);

        szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_REDUCE_CITY_DEFENSES", GC.getHeroInfo(getLegionHero()).getDescription(), pBombardCity->getNameKey(), pBombardCity->getDefenseModifier(false));
        gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pBombardCity->getX_INLINE(), pBombardCity->getY_INLINE());
	}

	if (bCoreTroop && bDefenderCoreTroop && (bStealHeroPromotion || bDefenderStealHeroPromotion))
	{
	    bool bAttackerAcquired = false;
	    bool bDefenderAcquired = false;
	    for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	    {
	        if (bAttackerAcquired && bDefenderAcquired)
	        {
	            break;
	        }
	        if (GC.getPromotionInfo((PromotionTypes)iI).isHero())
	        {
	            if (bStealHeroPromotion && !bAttackerAcquired && !isHasPromotion((PromotionTypes)iI))
	            {
	                if (pDefender->isHasPromotion((PromotionTypes)iI))
	                {
	                    if (GAME.canHeroAcquirePromotion(getHeroType(), (PromotionTypes)iI, true))
	                    {
	                        if (GAME.getSorenRandNum(100, "Steal Hero Promotion") < GC.getDefineINT("HERO_STEAL_PROMOTION_PERCENT"))
	                        {
	                            GAME.setHeroPromotionStatus(getHeroType(), (PromotionTypes)iI, GC.getPromotionInfo((PromotionTypes)iI).getHeroPromotionStatus(), false);
	                            CvWString szBuffer = gDLL->getText("TXT_KEY_HERO_STEALT_PROMOTION", GC.getHeroInfo(getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
                                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
                                szBuffer = gDLL->getText("TXT_KEY_HERO_PROMOTION_STEALT", GC.getHeroInfo(getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(pDefender->getLegionHero()).getDescription());
                                gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pDefender->getX_INLINE(), pDefender->getY_INLINE(), true, true);
	                            bAttackerAcquired = true;
	                        }
	                    }
	                }
	            }
	            if (bDefenderStealHeroPromotion && !bDefenderAcquired && !pDefender->isHasPromotion((PromotionTypes)iI))
	            {
	                if (isHasPromotion((PromotionTypes)iI))
	                {
	                    if (GAME.canHeroAcquirePromotion(pDefender->getHeroType(), (PromotionTypes)iI, true))
	                    {
	                        if (GAME.getSorenRandNum(100, "Steal Hero Promotion") < GC.getDefineINT("HERO_STEAL_PROMOTION_PERCENT"))
	                        {
	                            GAME.setHeroPromotionStatus(pDefender->getHeroType(), (PromotionTypes)iI, GC.getPromotionInfo((PromotionTypes)iI).getHeroPromotionStatus(), false);
	                            CvWString szBuffer = gDLL->getText("TXT_KEY_HERO_STEALT_PROMOTION", GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(getLegionHero()).getDescription());
                                gDLL->getInterfaceIFace()->addMessage(pDefender->getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pDefender->getX_INLINE(), pDefender->getY_INLINE(), true, true);
                                szBuffer = gDLL->getText("TXT_KEY_HERO_PROMOTION_STEALT", GC.getHeroInfo(pDefender->getLegionHero()).getDescription(), GC.getPromotionInfo((PromotionTypes)iI).getDescription(), GC.getHeroInfo(getLegionHero()).getDescription());
                                gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_UNIT_GREATPEOPLE", MESSAGE_TYPE_INFO, GC.getHeroInfo(getLegionHero()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE(), true, true);
	                            bDefenderAcquired = true;
	                        }
	                    }
	                }
	            }
	        }
	    }
	}

	if (getLegionHero() != NO_HERO && bAttackWithSameLegion && bAttackerWithdrawal)
	{
		setTempCheck(true);
		CvUnit* pNextMember = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(getLegionHero(), getID()));
		while (pNextMember->getID() != getID())
		{
			if (!pNextMember->isTempCheck() && plotDistance(pNextMember->getX_INLINE(), pNextMember->getY_INLINE(), pDefender->getX_INLINE(), pDefender->getY_INLINE()) == 1)
			{
				if (getCombatOdds(pNextMember, pDefender) > GC.getDefineINT("HERO_SECOND_ATTACK_ODDS") * 10)
				{
					pNextMember->setTempCheck(true);
					pNextMember->attack(pDefender->plot(), false, pDefender);
					return;
				}
			}
			pNextMember = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroNextLegionMember(getLegionHero(), pNextMember->getID()));
		}
		for (iI = 0; iI < GAME.getHeroLegionSize(getLegionHero()); iI++)
		{
			pNextMember = GET_PLAYER(getOwnerINLINE()).getUnit( GAME.getHeroLegionMemberAt(getLegionHero(), iI));
			pNextMember->setTempCheck(false);
		}
	}

	if (bDefenderForceWithdrawal && !bAttackerWithdrawal)
	{
		if (pDefenderPlot->plotCount(PUF_isPlayer, pDefender->getOwnerINLINE(), -1, NO_PLAYER, NO_TEAM, PUF_canDefend) == 1)
		{
			CvPlot* pAdjacentPlot;
			std::vector<CvPlot*> pPotentialPlots;
			for (iI = 0; iI < NUM_DIRECTION_TYPES; iI++)
			{
				pAdjacentPlot = plotDirection(pDefenderPlot->getX_INLINE(), pDefenderPlot->getY_INLINE(), ((DirectionTypes)iI));
				if (pAdjacentPlot != NULL && pAdjacentPlot != pDefenderPlot)
				{
					if (pAdjacentPlot->plotCount(PUF_isEnemy, pDefender->getOwnerINLINE(), 0, NO_PLAYER, NO_TEAM, PUF_canDefend) == 0)
					{
						if (pAdjacentPlot->isValidDomainForLocation(*pDefender))
						{
							if (pDefender->canMoveInto(pAdjacentPlot))
							{
								pPotentialPlots.push_back(pAdjacentPlot);
							}
						}
					}
				}
			}

			if (pPotentialPlots.size() > 0)
			{
				int iRand = GAME.getSorenRandNum(pPotentialPlots.size(), "Random Plot Pick");
				setXY(pDefender->getX_INLINE(), pDefender->getY_INLINE());
				pDefender->setXY(pPotentialPlots[iRand]->getX_INLINE(), pPotentialPlots[iRand]->getY_INLINE());
				return;
			}
		}
	}

	CLLNode<IDInfo>* pUnitNode;
	CvUnit* pLoopUnit;

	pUnitNode = plot()->headUnitNode();

	std::vector< std::pair<int, std::pair<int, int> > > AdditionalAttackUnits;
	std::vector< std::pair<int, std::pair<int, int> > >::iterator iter;

	while (pUnitNode != NULL)
	{
		pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = plot()->nextUnitNode(pUnitNode);

		if (pLoopUnit->getOwnerINLINE() == getOwnerINLINE())
		{
		    if (pLoopUnit->getID() != getID())
		    {
		        if (pLoopUnit->getAdditionalAttackChance() > 0)
		        {
					int iWinOdds = getCombatOdds(pLoopUnit, pDefender);
					if (iWinOdds > GC.getDefineINT("HERO_SECOND_ATTACK_ODDS") * 10)
					{
						for (iter = AdditionalAttackUnits.begin(); iter != AdditionalAttackUnits.end(); ++iter)
						{
							if (iWinOdds > (*iter).second.second)
							{
								AdditionalAttackUnits.insert(iter, std::make_pair(pLoopUnit->getID(), std::make_pair(pLoopUnit->getAdditionalAttackChance(), iWinOdds)));
								break;
							}
						}

						if (iter == AdditionalAttackUnits.end())
						{
							AdditionalAttackUnits.push_back(std::make_pair(pLoopUnit->getID(), std::make_pair(pLoopUnit->getAdditionalAttackChance(), iWinOdds)));
						}
					}
		        }
		    }
		}
	}

	for (iter = AdditionalAttackUnits.begin(); iter != AdditionalAttackUnits.end(); ++iter)
	{
	    if (GAME.getSorenRandNum(100, "Additional Attack") < (*iter).second.first)
	    {
	        CvUnit* pAttackUnit = GET_PLAYER(getOwnerINLINE()).getUnit((*iter).first);
	        FAssert (pAttackUnit != NULL);

	        pAttackUnit->setTempCheck(true);
	        pAttackUnit->attack(pDefender->plot(), false, pDefender);

	        break;
	    }
	}


    // the other side chase withdrawal units
	pUnitNode = pDefenderPlot->headUnitNode();
    AdditionalAttackUnits.clear();

	while (pUnitNode != NULL)
	{
		pLoopUnit = ::getUnit(pUnitNode->m_data);
		pUnitNode = pDefenderPlot->nextUnitNode(pUnitNode);

		if (pLoopUnit->getOwnerINLINE() == eDefendPlayer)
		{
		    if (pDefender == NULL || pLoopUnit->getID() != pDefender->getID())
		    {
		        if (pLoopUnit->getAdditionalAttackChance() > 0)
		        {
                    int iWinOdds = getCombatOdds(pLoopUnit, this);
					
					if (iWinOdds > GC.getDefineINT("HERO_SECOND_ATTACK_ODDS") * 10)
					{
						for (iter = AdditionalAttackUnits.begin(); iter != AdditionalAttackUnits.end(); ++iter)
						{
							if (iWinOdds > (*iter).second.second)
							{
								AdditionalAttackUnits.insert(iter, std::make_pair(pLoopUnit->getID(), std::make_pair(pLoopUnit->getAdditionalAttackChance(), iWinOdds)));
								break;
							}
						}

						if (iter == AdditionalAttackUnits.end())
						{
							AdditionalAttackUnits.push_back(std::make_pair(pLoopUnit->getID(), std::make_pair(pLoopUnit->getAdditionalAttackChance(), iWinOdds)));
						}
					}
		        }
		    }
		}
	}

	for (iter = AdditionalAttackUnits.begin(); iter != AdditionalAttackUnits.end(); ++iter)
	{
	    if (GAME.getSorenRandNum(100, "Additional Attack") < (*iter).second.first)
	    {
	        CvUnit* pAttackUnit = GET_PLAYER(eDefendPlayer).getUnit((*iter).first);
	        FAssert (pAttackUnit != NULL);

	        pAttackUnit->setTempCheck(true);
	        pAttackUnit->attack(plot(), false, this);

	        break;
	    }
	}
}

int CvUnit::getTempStrengthModifier() const
{
    return m_iTempStrengthModifier;
}

void CvUnit::setTempStrengthModifier(int iNewValue)
{
    m_iTempStrengthModifier = iNewValue;
}

void CvUnit::changeTempStrengthModifier(int iChange)
{
    setTempStrengthModifier(iChange + getTempStrengthModifier());
}

// unit influences combat area after victory
// returns influence % in defended plot
float CvUnit::doVictoryInfluence(const CvUnit* pLoserUnit, bool bAttacking)
{
    if (isBarbarian() || pLoserUnit->isBarbarian())
    {
        return 0.0f;
    }

    if (DOMAIN_SEA == getDomainType())
    {
        return 0.0f;
    }

	CvPlot* pWinnerPlot = plot();
	CvPlot* pLoserPlot = pLoserUnit->plot();
	CvPlot* pDefenderPlot = NULL;
	if (!bAttacking)
	{
		pDefenderPlot = pWinnerPlot;
	}
	else
	{
		pDefenderPlot = pLoserPlot;
	}
	int iWinnerCultureBefore = pDefenderPlot->getCulture(getOwnerINLINE()); //used later for influence %

	float fWinnerPlotMultiplier = 1.0f; // by default: same influence in WinnerPlot and LoserPlot
//	if (GC.getDefineFLOAT("IDW_WINNER_PLOT_MULTIPLIER"))
//		fWinnerPlotMultiplier = GC.getDefineFLOAT("IDW_WINNER_PLOT_MULTIPLIER");

	float fLoserPlotMultiplier = 1.0f; // by default: same influence in WinnerPlot and LoserPlot
//	if (GC.getDefineFLOAT("IDW_LOSER_PLOT_MULTIPLIER"))
//		fLoserPlotMultiplier = GC.getDefineFLOAT("IDW_LOSER_PLOT_MULTIPLIER");

	if (pLoserPlot->isCity()) // city combat
	{
		if (pLoserPlot->getNumVisibleEnemyDefenders(this) > 1)
		{
			// if there are still some city defenders ->
			// we use same influence rules as for field combat
			influencePlots(pLoserPlot, pLoserUnit->getOwnerINLINE(), fLoserPlotMultiplier);
			influencePlots(pWinnerPlot, pLoserUnit->getOwnerINLINE(), fWinnerPlotMultiplier);
		}
		else // last defender is dead
		{
			float fNoCityDefenderMultiplier = 2.5; // default: 250%
//			if (GC.getDefineFLOAT("IDW_NO_CITY_DEFENDER_MULTIPLIER"))
//				fNoCityDefenderMultiplier = GC.getDefineFLOAT("IDW_NO_CITY_DEFENDER_MULTIPLIER");

			// last city defender is dead -> influence is increased
			influencePlots(pLoserPlot, pLoserUnit->getOwnerINLINE(), fLoserPlotMultiplier * fNoCityDefenderMultiplier);
			influencePlots(pWinnerPlot, pLoserUnit->getOwnerINLINE(), fWinnerPlotMultiplier * fNoCityDefenderMultiplier);
		}
	}
	else // field combat
	{
		if (!pLoserUnit->canDefend())
		{
			// no influence from worker capture
			return 0.0f;
		}

		if (pLoserPlot->getImprovementType() != NO_IMPROVEMENT
			&& GC.getImprovementInfo(pLoserPlot->getImprovementType()).getDefenseModifier() > 0
			&& pLoserPlot->getNumVisibleEnemyDefenders(this) > 1)
		{
			// fort captured
			float fFortCaptureMultiplier = 2.0f; // default: 200%
//			if (GC.getDefineFLOAT("IDW_FORT_CAPTURE_MULTIPLIER"))
//				fFortCaptureMultiplier = GC.getDefineFLOAT("IDW_FORT_CAPTURE_MULTIPLIER");

			// influence is increased
			influencePlots(pLoserPlot, pLoserUnit->getOwnerINLINE(), fLoserPlotMultiplier * fFortCaptureMultiplier);
			influencePlots(pWinnerPlot, pLoserUnit->getOwnerINLINE(), fWinnerPlotMultiplier * fFortCaptureMultiplier);

		}
		else
		{
			influencePlots(pLoserPlot, pLoserUnit->getOwnerINLINE(), fLoserPlotMultiplier);
			influencePlots(pWinnerPlot, pLoserUnit->getOwnerINLINE(), fWinnerPlotMultiplier);
		}
	}

	// calculate influence % in defended plot (to be displayed in game log)

	int iWinnerCultureAfter = pDefenderPlot->getCulture(getOwnerINLINE());
	int iTotalCulture = pDefenderPlot->countTotalCulture();
	float fInfluenceRatio = 0.0f;
	if (iTotalCulture > 0)
	{
		fInfluenceRatio = ((iWinnerCultureAfter-iWinnerCultureBefore)*100.0f)/iTotalCulture;
	}
    return fInfluenceRatio;
}

// unit influences given plot and surounding area i.e. transfers culture from target civ to unit's owner
void CvUnit::influencePlots(const CvPlot* pCentralPlot, PlayerTypes eTargetPlayer, float fLocationMultiplier)
{
	float fBaseCombatInfluence = 4.0f;
//	if (GC.getDefineFLOAT("IDW_BASE_COMBAT_INFLUENCE"))
//        fBaseCombatInfluence = GC.getDefineFLOAT("IDW_BASE_COMBAT_INFLUENCE");

	// calculate base multiplier used for all plots
	float fGameSpeedMultiplier = (float) GC.getGameSpeedInfo(GAME.getGameSpeedType()).getConstructPercent();
	fGameSpeedMultiplier /= 100;
	fGameSpeedMultiplier *= GC.getEraInfo(GAME.getStartEra()).getConstructPercent();
	fGameSpeedMultiplier /= 100;
	fGameSpeedMultiplier = sqrt(fGameSpeedMultiplier);

	float fExperienceFactor = 0.01f;  // default: each point of experience increases influence by 1%
//	if (GC.getDefineFLOAT("IDW_EXPERIENCE_FACTOR"))
//        fExperienceFactor = GC.getDefineFLOAT("IDW_EXPERIENCE_FACTOR");
    float fExperienceMultiplier = 1.0f + (getExperience() * 0.01f);
    if (getHeroType() != NO_HERO)
    {
        fExperienceMultiplier += fExperienceMultiplier + (GAME.getHeroExperience(getHeroType()) * 0.01f);
    }

	float fWarlordMultiplier = 1.0;
	if (NO_UNIT != getLeaderUnitType()) // warlord or hero is here
	{
		fWarlordMultiplier = 1.5; // default: +50%
//		if (GC.getDefineFLOAT("IDW_WARLORD_MULTIPLIER"))
//			fWarlordMultiplier = GC.getDefineFLOAT("IDW_WARLORD_MULTIPLIER");
	}

	float fBaseMultiplier = fBaseCombatInfluence * fGameSpeedMultiplier * fLocationMultiplier * fExperienceMultiplier * fWarlordMultiplier;
	if (fBaseMultiplier <= 0.0f)
		return;

	// get influence radius
	int iInfluenceRadius = 2; // default: like 2square city workable radius
//	if (GC.getDefineINT("IDW_INFLUENCE_RADIUS"))
//		iInfluenceRadius = GC.getDefineINT("IDW_INFLUENCE_RADIUS");
//	if (iInfluenceRadius < 0)
//		return;

	float fPlotDistanceFactor = 0.2f; // default: influence decreases by 20% with plot distance
//	if (GC.getDefineFLOAT("IDW_PLOT_DISTANCE_FACTOR"))
//        fPlotDistanceFactor = GC.getDefineFLOAT("IDW_PLOT_DISTANCE_FACTOR");

//	CvWString szBuffer;
//	szBuffer.Format(L"Factors: %.1f, %.1f, %.1f, %.1f, %.1f, %.1f, %.3f, %d", fBaseCombatInfluence, fLocationMultiplier, fGameSpeedMultiplier, fPlotDistanceFactor, fExperienceMultiplier, fWarlordMultiplier, fBaseMultiplier, iInfluenceRadius);
//	gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_UNIT_BUILD_UNIT", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCentralPlot->getX_INLINE(), pCentralPlot->getY_INLINE());

	for (int iDX = -iInfluenceRadius; iDX <= iInfluenceRadius; iDX++)
	{
		for (int iDY = -iInfluenceRadius; iDY <= iInfluenceRadius; iDY++)
		{
			int iDistance = plotDistance(0, 0, iDX, iDY);

			if (iDistance <= iInfluenceRadius)
			{
				CvPlot* pLoopPlot = plotXY(pCentralPlot->getX_INLINE(), pCentralPlot->getY_INLINE(), iDX, iDY);

				if (pLoopPlot != NULL)
				{
					// calculate distance multiplier for current plot
					float fDistanceMultiplier = 0.5f+0.5f*fPlotDistanceFactor-fPlotDistanceFactor*iDistance;
					if (fDistanceMultiplier <= 0.0f)
						continue;
					int iTargetCulture = pLoopPlot->getCulture(eTargetPlayer);
					if (iTargetCulture <= 0)
						continue;
					int iCultureTransfer = int (fBaseMultiplier * fDistanceMultiplier * sqrt((float) iTargetCulture));
					if (iTargetCulture < iCultureTransfer)
					{
						// cannot transfer more culture than remaining target culure
						iCultureTransfer = iTargetCulture;
					}
					if (iCultureTransfer == 0 && iTargetCulture > 0)
					{
						// always at least 1 point of culture must be transfered
						// othervise we may have the problems with capturing of very low culture cities.
						iCultureTransfer = 1;
					}

					if (iCultureTransfer > 0)
					{
						// target player's culture in plot is lowered
						pLoopPlot->changeCulture(eTargetPlayer, -iCultureTransfer, false);
						// owners's culture in plot is raised
						pLoopPlot->changeCulture(getOwnerINLINE(), iCultureTransfer, true);
					}
				}
			}
		}
	}
}

// Dale - RB: Field Bombard START
bool CvUnit::canRBombard(const CvPlot* pPlot) const
{
    if(getDCMBombRange() < 1)
    {
        return false;
    }

	if (bombardRate() <= 0)
	{
		return false;
	}

	if (isMadeAttack())
	{
		return false;
	}

	if (isCargo())
	{
		return false;
	}

	return true;
}

bool CvUnit::canBombardAtRanged(const CvPlot* pPlot, int iX, int iY) const
{
	CvCity* pCity;
	CvPlot* pTargetPlot;
	if (!canRBombard(pPlot))
	{
		return false;
	}

	if (iX < 0 || iY < 0)
	{
		return false;
	}

	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	if (plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), pTargetPlot->getX_INLINE(), pTargetPlot->getY_INLINE()) > getDCMBombRange())
	{
		return false;
	}

	if (pTargetPlot->isOwned())
	{
		if (pTargetPlot->getTeam() != getTeam())
		{
			if (!atWar(pTargetPlot->getTeam(), getTeam()))
			{
				return false;
			}
		}
	}

	pCity = pTargetPlot->getPlotCity();

	if (pCity != NULL)
	{
		if (!(pCity->isBombardable(this)))
		{
			if(pTargetPlot->getNumVisibleEnemyDefenders(this) == 0)
			{
                return false;
			}
		}
	}
	else
	{
		if(pTargetPlot->getNumVisibleEnemyDefenders(this) == 0)
		{
			if (pTargetPlot->getOwnerINLINE() == getOwnerINLINE())
			{
				return false;
			}
			if (pTargetPlot->getImprovementType() == NO_IMPROVEMENT)
			{
				return false;
			}
			if (GC.getImprovementInfo(pTargetPlot->getImprovementType()).isPermanent())
			{
				return false;
			}
			if (GC.getImprovementInfo(pTargetPlot->getImprovementType()).getAirBombDefense() == -1)
			{
				return false;
			}
		}
	}

	return true;
}

bool CvUnit::bombardRanged(int iX, int iY, bool sAttack)
{
	CvCity* pCity;
	CvPlot* pPlot;
	CvWString szBuffer;
	CvUnit* pLoopUnit = NULL;
	int iDefenderNum, iUnitDamage;
	bool bTarget = false;

	if (!canBombardAtRanged(plot(), iX, iY))
	{
		return false;
	}

	pPlot = GC.getMapINLINE().plotINLINE(iX, iY);
	pCity = pPlot->getPlotCity();
	iDefenderNum = pPlot->getNumVisibleEnemyDefenders(this);

	bool bSharpShooter = false;

	if(iDefenderNum > 0)
	{
		pLoopUnit = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
		for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
        {
            if (isHasPromotion((PromotionTypes)iI))
            {
                if (GC.getPromotionInfo((PromotionTypes)iI).getArcherBombardRangeChange() > 0)
                {
                    bSharpShooter = true;
                    break;
                }
            }
        }
		iUnitDamage = GAME.getSorenRandNum(bSharpShooter ? bombardRate() * 2 : bombardRate(), "Bombard damage");//(rand() % (bombardRate()));
	}

	if (GAME.getSorenRandNum(100, "Bombard Accuracy") <= GC.getDefineINT("HERO_RANGED_BOMBARD_ACCURACY"))
	{
		bTarget = true;
	}
	else
	{
		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_BOMB_MISSED", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE(), true, true);
		if (NULL != pLoopUnit)
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_BOMB_MISSED", getNameKey());
			gDLL->getInterfaceIFace()->addMessage(pLoopUnit->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());
		}
	}

	if (pCity != NULL)
	{
		if(pCity->isBombardable(this))
		{
			int iBombardModifier = 0;
			if (!ignoreBuildingDefense())
			{
				iBombardModifier -= pCity->getBuildingBombardDefense();
			}

			pCity->changeDefenseModifier(-(bombardRate() * std::max(0, 100 + iBombardModifier)) / 100);
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_DEFENSES_REDUCED_TO", pCity->getNameKey(), pCity->getDefenseModifier(false), getNameKey());
			gDLL->getInterfaceIFace()->addMessage(pCity->getOwnerINLINE(), false, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pCity->getX_INLINE(), pCity->getY_INLINE(), true, true);
			szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_DEFENSES_REDUCED_TO", getNameKey(), pCity->getNameKey(), pCity->getDefenseModifier(false));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pCity->getX_INLINE(), pCity->getY_INLINE());
		}
	}
	if (bTarget)
	{
		if(iDefenderNum > 0)
		{
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ARE_ATTACKED_BY_AIR", pLoopUnit->getNameKey(), getNameKey(), -(((iUnitDamage - pLoopUnit->getDamage()) * 100) / pLoopUnit->maxHitPoints()));
			gDLL->getInterfaceIFace()->addMessage(pLoopUnit->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
			szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ATTACK_BY_AIR", getNameKey(), pLoopUnit->getNameKey(), -(((iUnitDamage - pLoopUnit->getDamage()) * 100) / pLoopUnit->maxHitPoints()));
			gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACKED", MESSAGE_TYPE_INFO, pLoopUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			collateralCombat(pPlot, pLoopUnit);
			pLoopUnit->changeDamage(iUnitDamage, getOwner());
		}
		else if (pPlot->getImprovementType() != NO_IMPROVEMENT)
		{
			if (GAME.getSorenRandNum(bombardRate(), "Bomb - Offense") >= GAME.getSorenRandNum(GC.getImprovementInfo(pPlot->getImprovementType()).getAirBombDefense(), "Bomb - Defense"))
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_DESTROYED_IMP", getNameKey(), GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_PILLAGE", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
				if (pPlot->isOwned())
				{
					szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_IMP_WAS_DESTROYED", GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide(), getNameKey(), GET_PLAYER(getOwnerINLINE()).getCivilizationAdjectiveKey());
					gDLL->getInterfaceIFace()->addMessage(pPlot->getOwnerINLINE(), false, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_PILLAGED", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
				}
				pPlot->setImprovementType((ImprovementTypes)(GC.getImprovementInfo(pPlot->getImprovementType()).getImprovementPillage()));
			}
			else
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_UNIT_FAIL_DESTROY_IMP", getNameKey(), GC.getImprovementInfo(pPlot->getImprovementType()).getTextKeyWide());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMB_FAILS", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
			}
		}
	}

	if (!sAttack)
	{
		setMadeAttack(true);
		changeMoves(GC.getMOVE_DENOMINATOR());
	}

	if (pPlot->isActiveVisible(false))
	{
		// Bombard entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_RBOMBARD).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_BOMBARD);
		kDefiniton.setPlot(pPlot);
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, pLoopUnit);
		gDLL->getEntityIFace()->AddMission(&kDefiniton);
	}

	return true;
}

int CvUnit::getDCMBombRange() const
{
	return m_iDCMBombRange;
}
// Dale - RB: Field Bombard END

// Dale - ARB: Archer Bombard START
bool CvUnit::canArcherBombard(const CvPlot* pPlot) const
{
	if(getDCMBombRange() < 1)
    {
        return false;
    }

	if (bombardRate() > 0)
	{
		return false;
	}

	if (isMadeAttack())
	{
		return false;
	}

	if (isCargo())
	{
		return false;
	}

	return true;
}

bool CvUnit::canArcherBombardAt(const CvPlot* pPlot, int iX, int iY) const
{
	CvPlot* pTargetPlot;
	if (!canArcherBombard(pPlot))
	{
		return false;
	}

	if(iX < 0 || iY < 0)
	{
		return false;
	}

	pTargetPlot = GC.getMapINLINE().plotINLINE(iX, iY);

	FAssert (NULL != pTargetPlot);

	if (pTargetPlot->isWater())
	{
	    return false;
	}

	if (plotDistance(pPlot->getX_INLINE(), pPlot->getY_INLINE(), pTargetPlot->getX_INLINE(), pTargetPlot->getY_INLINE()) > getDCMBombRange())
	{
		return false;
	}

	if(pTargetPlot->getNumVisibleEnemyDefenders(this) == 0)
	{
		return false;
	}

	if (pTargetPlot->isOwned())
	{
		if(pTargetPlot->getTeam() != getTeam())
		{
			if (!atWar(pTargetPlot->getTeam(), getTeam()))
			{
				return false;
			}
		}
	}

	return true;
}

bool CvUnit::archerBombard(int iX, int iY, bool sAttack)
{
	CvPlot* pPlot;
	CvWString szBuffer;
	CvUnit* pLoopUnit = NULL;
	int iDefenderNum, iUnitDamage;
	bool bTarget = false;

	if (!canArcherBombardAt(plot(), iX, iY))
	{
		return false;
	}

	pPlot = GC.getMapINLINE().plotINLINE(iX, iY);
	iDefenderNum = pPlot->getNumVisibleEnemyDefenders(this);

	if(iDefenderNum == 0)
	{
		return false;
	}

	bool bSharpShooter = false;
	for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
	{
	    if (isHasPromotion((PromotionTypes)iI))
	    {
	        if (GC.getPromotionInfo((PromotionTypes)iI).getArcherBombardRangeChange() > 0)
	        {
	            bSharpShooter = true;
	            break;
	        }
	    }
	}

	if (GAME.getSorenRandNum(100, "Bombard Accuracy") <= (bSharpShooter ? 100 : GC.getDefineINT("HERO_RANGED_BOMBARD_ACCURACY")))
	{
		bTarget = true;
	}
	else
	{
		pLoopUnit = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
		if (NULL == pLoopUnit)
		{
			return false;
		}
		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_BOMB_MISSED", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARDED", MESSAGE_TYPE_INFO, GC.getUnitInfo(getUnitType()).getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), getX_INLINE(), getY_INLINE(), true, true);
		szBuffer = gDLL->getText("TXT_KEY_MISC_ENEMY_BOMB_MISSED", getNameKey());
		gDLL->getInterfaceIFace()->addMessage(pLoopUnit->getOwnerINLINE(), true, GC.getDefineINT("EVENT_MESSAGE_TIME"), szBuffer, "AS2D_BOMBARD", MESSAGE_TYPE_INFO, NULL, (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), getX_INLINE(), getY_INLINE());
	}

	if (bTarget)
	{
		pLoopUnit = pPlot->getBestDefender(NO_PLAYER, getOwnerINLINE(), this, true);
		if (pLoopUnit == NULL)
		{
			return false;
		}
		iUnitDamage = std::max(1, (((bSharpShooter ? 2 * GC.getDefineINT("COMBAT_DAMAGE") : GC.getDefineINT("COMBAT_DAMAGE")) * (currFirepower(NULL, NULL) + ((currFirepower(NULL, NULL) + pLoopUnit->currFirepower(NULL, NULL) + 1) / 2))) / (pLoopUnit->currFirepower(pPlot, this) + ((currFirepower(NULL, NULL) + pLoopUnit->currFirepower(NULL, NULL) + 1) / 2))));
		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ARE_ATTACKED_BY_AIR", pLoopUnit->getNameKey(), getNameKey(), -(((iUnitDamage - pLoopUnit->getDamage()) * 100) / pLoopUnit->maxHitPoints()));
		gDLL->getInterfaceIFace()->addMessage(pLoopUnit->getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACK", MESSAGE_TYPE_INFO, getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_RED"), pPlot->getX_INLINE(), pPlot->getY_INLINE(), true, true);
		szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_ATTACK_BY_AIR", getNameKey(), pLoopUnit->getNameKey(), -(((iUnitDamage - pLoopUnit->getDamage()) * 100) / pLoopUnit->maxHitPoints()));
		gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), true, GC.getEVENT_MESSAGE_TIME(), szBuffer, "AS2D_AIR_ATTACKED", MESSAGE_TYPE_INFO, pLoopUnit->getButton(), (ColorTypes)GC.getInfoTypeForString("COLOR_GREEN"), pPlot->getX_INLINE(), pPlot->getY_INLINE());
		collateralCombat(pPlot, pLoopUnit);
		pLoopUnit->changeDamage(iUnitDamage, getOwner());
	}

	if (!sAttack)
	{
		setMadeAttack(true);
		changeMoves(GC.getMOVE_DENOMINATOR());
	}

	if (pPlot->isActiveVisible(false))
	{
		// Bombard entity mission
		CvMissionDefinition kDefiniton;
		kDefiniton.setMissionTime(GC.getMissionInfo(MISSION_ABOMBARD).getTime() * gDLL->getSecsPerTurn());
		kDefiniton.setMissionType(MISSION_BOMBARD);
		kDefiniton.setPlot(pPlot);
		kDefiniton.setUnit(BATTLE_UNIT_ATTACKER, this);
		kDefiniton.setUnit(BATTLE_UNIT_DEFENDER, pLoopUnit);
		gDLL->getEntityIFace()->AddMission(&kDefiniton);
	}

	return true;
}
// Dale - ARB: Archer Bombard END

void CvUnit::stopDelayedDeath()
{
    m_bDeathDelay = false;
}

void CvUnit::doLegionMemberDefeated(const CvUnit* pEnemy, bool bAttacking, bool bNoSurrender, PlayerTypes ePlayer)
{
    PROFILE_FUNC();

	CvUnit* pHero = GAME.getHeroUnit(getLegionHero());
    FAssert (pHero != NULL);

    PlayerTypes eEnemyPlayer;
    if (pEnemy != NULL)
    {
		eEnemyPlayer = pEnemy->getOwnerINLINE();
	}
	else if (ePlayer != NO_PLAYER)
	{
		eEnemyPlayer = ePlayer;
	}
	else
	{
		eEnemyPlayer = NO_PLAYER;
	}

    if (getHeroType() == NO_HERO)
    {
        // if it's a plain member unit
        GAME.deleteHeroLegionMember(getLegionHero(), getID());
        setLegionHero(NO_HERO);
    }
    else
    {
		// If we killed our core troop ourselves, happens when core troop is on a transport, and the the transport is deleted.
		if (eEnemyPlayer == NO_PLAYER)
		{
			return;
		}

        // If it's a core troop
        CvEventReporter::getInstance().unitKilled(this, eEnemyPlayer);
        HeroTypes eHero = getHeroType();
        FAssert (pHero->getID() == getID());
        FAssert (eHero >= 0 && eHero < GC.getNumHeroInfos());
        CvWString szBuffer;

        int iDieRand = GAME.getSorenRandNum(100, "Hero Die Rand");
        bool bHeroProcessed = false;
        CvUnit* pNewCore = NULL;
        bool bLegionProcessed = false;
        bool bCurrentLeader = (GC.getHeroInfo(getHeroType()).getLeaderType() == GET_PLAYER(getOwnerINLINE()).getLeaderType());
        bool bCurrentCiv = (GAME.getCurrentDefaultCivilization(getHeroType()) == getCivilizationType());
		bool bHighDeathChance = GAME.isOption(GAMEOPTION_HIGH_HERO_DEATH_CHANCE);

		if (iDieRand < ((pEnemy != NULL && pEnemy->isCoreTroop()) ? std::max(GC.getDefineINT("HERO_DIE_CHANCE") + std::min((GC.getHeroInfo(pEnemy->getHeroType()).getMight() - GC.getHeroInfo(eHero).getMight()), 10), 0) * (bHighDeathChance ? 4 : 1) : GC.getDefineINT("HERO_DIE_CHANCE") * (bHighDeathChance ? 4 : 1)))
        {
            if (!GAME.isOption(GAMEOPTION_NO_HERO_DEATH_FROM_COMBAT))
            {
				for (int iI = 0; iI < MAX_PLAYERS; iI++)
				{
					if (GET_PLAYER((PlayerTypes)iI).isAlive())
					{
						szBuffer = gDLL->getText("TXT_KEY_MISC_HERO_DIED", GC.getHeroInfo(eHero).getDescription());
						gDLL->getInterfaceIFace()->addMessage(((PlayerTypes)iI), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
					}
				}

                GAME.doHeroDeath(eHero, false);

				bHeroProcessed = true;
				bLegionProcessed = true;
            }
        }

        if (!bHeroProcessed && !isBarbarian())
        {
            // if the enemy is a barbarian, is current leader or we can't surrender, then we must escape
            if ((GET_PLAYER(eEnemyPlayer).isBarbarian()) ? true
                : (iDieRand < ((pEnemy != NULL && pEnemy->isCoreTroop()) ? (std::max(1, GC.getDefineINT("HERO_ESCAPE_CHANCE") + std::min(20, GC.getHeroInfo(eHero).getMight() - GC.getHeroInfo(pEnemy->getHeroType()).getMight()))) : GC.getDefineINT("HERO_ESCAPE_CHANCE"))))
            {
                bool bOneUnitLegion = (GAME.getHeroLegionSize(eHero) == 1);
                quitLegion(true);
                if (bOneUnitLegion)
                {
                    pNewCore = NULL;
                }
                else
                {
                    pNewCore = GAME.getHeroUnit(eHero);
                }
                bLegionProcessed = true;

                // if there is at least one more unit in the legion, do the following
                if (pNewCore != NULL)
                {
                    pNewCore->finishMoves();
                    bHeroProcessed = true;
                }
                // if the legion is empty
                else
                {
                    // if there's fightable own units on the same plot, stay here for a second chance
                    if (!plot()->isWater() && plot()->plotCount(PUF_isPlayer, getOwnerINLINE(), -1, NO_PLAYER, NO_TEAM, PUF_canDefend) > 1)
                    {
                        CvUnit* pNewHero = GET_PLAYER(getOwnerINLINE()).initHero(eHero, plot()->getX_INLINE(), plot()->getY_INLINE(), GAME.getHeroDebutStatus(eHero) != getOwnerINLINE());
                        pNewHero->finishMoves();
                        bHeroProcessed = true;
                    }
                    else
                    {
                        CvCity* pCapital = GET_PLAYER(getOwnerINLINE()).getCapitalCity();
                        if (pCapital)
                        {
							CvUnit* pNewHero = NULL;
                            // flee to nearest city, or already at capital while not attacking, which means capital has fallen, then surrender if we have only 1 city left
							if (!atPlot(pCapital->plot()) || GET_PLAYER(getOwnerINLINE()).getNumCities() > 1)
                            {
								int iX, iY;
								CvCity* pNearestCity = GC.getMapINLINE().findCity(plot()->getX_INLINE(), plot()->getY_INLINE(), getOwnerINLINE(), NO_TEAM, !(plot()->isWater()), false, NO_TEAM, NO_DIRECTION, (plot()->isCity() ? plot()->getPlotCity() : NULL));
								if (pNearestCity == NULL)
								{
									pNearestCity = GC.getMapINLINE().findCity(plot()->getX_INLINE(), plot()->getY_INLINE(), getOwnerINLINE(), NO_TEAM, false, false, NO_TEAM, NO_DIRECTION, (plot()->isCity() ? plot()->getPlotCity() : NULL));
								}
								if (pNearestCity == NULL)
								{
									GET_PLAYER(getOwnerINLINE()).findXY(&iX, &iY, true);
								}
								else
								{
									iX = pNearestCity->getX_INLINE();
									iY = pNearestCity->getY_INLINE();
								}

								pNewHero = GET_PLAYER(getOwnerINLINE()).initHero(eHero, iX, iY, GAME.getHeroDebutStatus(eHero) != getOwnerINLINE());
							}
                            else if (bAttacking)
                            {
								pNewHero = GET_PLAYER(getOwnerINLINE()).initHero(eHero, plot()->getX_INLINE(), plot()->getY_INLINE(), GAME.getHeroDebutStatus(eHero) != getOwnerINLINE());
                            }
							if (pNewHero != NULL)
							{
								pNewHero->finishMoves();
							}
							bHeroProcessed = true;
                        }
                    }
                    if (bHeroProcessed)
                    {
                        szBuffer = gDLL->getText("TXT_KEY_MISC_HERO_ESCAPED", GC.getHeroInfo(eHero).getDescription());
                        gDLL->getInterfaceIFace()->addMessage(eEnemyPlayer, false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
                        gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_MAJOR_EVENT);

						int iWoundChance = GC.getDefineINT("HERO_WOUND_CHANCE");
						bool bUsReckless = false;
						bool bTheirReckless = false;
						for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
						{
							if (GC.getPromotionInfo((PromotionTypes)iI).isReckless())
							{
								if (GAME.getHeroPromotionStatus(eHero, (PromotionTypes)iI) > 0 && GAME.getHeroPromotionStatus(eHero, (PromotionTypes)iI) < 3)
								{
									bUsReckless = true;
								}
								if (pEnemy != NULL && pEnemy->isHasPromotion((PromotionTypes)iI))
								{
									bTheirReckless = true;
								}
							}
						}
						if (bUsReckless && !bTheirReckless)
						{
							iWoundChance /= 4;
						}
						else if (!bUsReckless && bTheirReckless)
						{
							iWoundChance *= 2;
						}

                        if (!GAME.isHeroWounded(eHero) && GAME.getSorenRandNum(100, "Hero Wounded Rand") < ((pEnemy != NULL && pEnemy->isCoreTroop()) ? (std::max(1, iWoundChance + std::min(10, GC.getHeroInfo(pEnemy->getHeroType()).getMight() - GC.getHeroInfo(eHero).getMight()))) : iWoundChance))
						{
							GAME.doHeroWounded(eHero);
							szBuffer = gDLL->getText("TXT_KEY_MISC_HERO_WOUNDED", GC.getHeroInfo(eHero).getDescription());
							gDLL->getInterfaceIFace()->addMessage(eEnemyPlayer, false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_MAJOR_EVENT);
							gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_MAJOR_EVENT);
						}
                    }
                }
            }
        }

        // Hero Capture
        if (!bHeroProcessed)
        {
			if (getOwnerINLINE() != BARBARIAN_PLAYER)
            {
                int iEnemyHeroic = 0;
                if (pEnemy != NULL && pEnemy->isCoreTroop())
                {
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (pEnemy->isHasPromotion((PromotionTypes)iI))
                        {
                            iEnemyHeroic += GC.getPromotionInfo((PromotionTypes)iI).getConvertWarlordChance();
                        }
                    }
                }
                if (iEnemyHeroic > 0)
                {
                    GC.getHeroInfo(eHero).setCivilizationLoyalty(eEnemyPlayer, std::min(100, GC.getHeroInfo(eHero).getCivilizationLoyalty(getOwnerINLINE()) + iEnemyHeroic));
                }
            }
            
            // if we are still that core troop
            if (pNewCore == NULL)
            {
                if (!bLegionProcessed)
                {
                    dismissLegion(true);
                }
            }
            else
            {
                pNewCore->dismissLegion(true);
            }

			GAME.setHeroCapturePlayer(eHero, eEnemyPlayer);
			GAME.setHeroUnitID(eHero, -1);
			GET_PLAYER(getOwnerINLINE()).changeHeroMaintenance(eHero, false, true, 0);

			if (GET_PLAYER(eEnemyPlayer).isHuman())
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_YOU_CAPTURED_HERO", GC.getHeroInfo(eHero).getDescription());
				gDLL->getInterfaceIFace()->addMessage(eEnemyPlayer, false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitVictoryScript(), MESSAGE_TYPE_INFO);
			}
			if (GET_PLAYER(getOwnerINLINE()).isHuman())
			{
				szBuffer = gDLL->getText("TXT_KEY_MISC_OUR_HERO_CAPTURED", GC.getHeroInfo(eHero).getDescription(), GET_PLAYER(eEnemyPlayer).getCivilizationDescription());
				gDLL->getInterfaceIFace()->addMessage(getOwnerINLINE(), false, GC.getEVENT_MESSAGE_TIME(), szBuffer, GC.getEraInfo(GAME.getCurrentEra()).getAudioUnitDefeatScript(), MESSAGE_TYPE_INFO);
			}

			// Sanguo Mod tutorial, start, by poyuzhe 09.27.09
			if (!GET_PLAYER(eEnemyPlayer).isFeatAccomplished(FEAT_CAPTURE_HERO))
			{
				GET_PLAYER(eEnemyPlayer).setFeatAccomplished(FEAT_CAPTURE_HERO, true);
				if (GET_PLAYER(eEnemyPlayer).isFeatPopup())
				{
					CyArgsList argsList;

					argsList.add(getOwnerINLINE());
					argsList.add(eHero);

					gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doCaptureHeroFeat", argsList.makeFunctionArgs());
				}
			}
			// Sanguo Mod Tutorial, end

			return;
        }
    }
}

bool CvUnit::isCoreTroop() const
{
    return (getHeroType() != NO_HERO && getLegionHero() != NO_HERO);
}

bool CvUnit::isNakedHero() const
{
    return (getHeroType() != NO_HERO && getLegionHero() == NO_HERO);
}

int CvUnit::maxNoStrChangeCombatStr(const CvPlot* pPlot, const CvUnit* pAttacker, CombatDetails* pCombatDetails) const
{
	PROFILE_FUNC();

	int iCombat;

	FAssertMsg((pPlot == NULL) || (pPlot->getTerrainType() != NO_TERRAIN), "(pPlot == NULL) || (pPlot->getTerrainType() is not expected to be equal with NO_TERRAIN)");

	// handle our new special case
	const	CvPlot*	pAttackedPlot = NULL;
	bool	bAttackingUnknownDefender = false;
	if (pAttacker == this)
	{
		bAttackingUnknownDefender = true;
		pAttackedPlot = pPlot;

		// reset these values, we will fiddle with them below
		pPlot = NULL;
		pAttacker = NULL;
	}
	// otherwise, attack plot is the plot of us (the defender)
	else if (pAttacker != NULL)
	{
		pAttackedPlot = plot();
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iExtraCombatPercent = 0;
		pCombatDetails->iAnimalCombatModifierTA = 0;
		pCombatDetails->iAIAnimalCombatModifierTA = 0;
		pCombatDetails->iAnimalCombatModifierAA = 0;
		pCombatDetails->iAIAnimalCombatModifierAA = 0;
		pCombatDetails->iBarbarianCombatModifierTB = 0;
		pCombatDetails->iAIBarbarianCombatModifierTB = 0;
		pCombatDetails->iBarbarianCombatModifierAB = 0;
		pCombatDetails->iAIBarbarianCombatModifierAB = 0;
		pCombatDetails->iPlotDefenseModifier = 0;
		pCombatDetails->iFortifyModifier = 0;
		pCombatDetails->iCityDefenseModifier = 0;
		pCombatDetails->iHillsAttackModifier = 0;
		pCombatDetails->iHillsDefenseModifier = 0;
		pCombatDetails->iFeatureAttackModifier = 0;
		pCombatDetails->iFeatureDefenseModifier = 0;
		pCombatDetails->iTerrainAttackModifier = 0;
		pCombatDetails->iTerrainDefenseModifier = 0;
		pCombatDetails->iCityAttackModifier = 0;
		pCombatDetails->iDomainDefenseModifier = 0;
		pCombatDetails->iCityBarbarianDefenseModifier = 0;
		pCombatDetails->iClassDefenseModifier = 0;
		pCombatDetails->iClassAttackModifier = 0;
		pCombatDetails->iCombatModifierA = 0;
		pCombatDetails->iCombatModifierT = 0;
		pCombatDetails->iDomainModifierA = 0;
		pCombatDetails->iDomainModifierT = 0;
		pCombatDetails->iAnimalCombatModifierA = 0;
		pCombatDetails->iAnimalCombatModifierT = 0;
		pCombatDetails->iRiverAttackModifier = 0;
		pCombatDetails->iAmphibAttackModifier = 0;
		pCombatDetails->iKamikazeModifier = 0;
		pCombatDetails->iModifierTotal = 0;
		pCombatDetails->iBaseCombatStr = 0;
		pCombatDetails->iCombat = 0;
		pCombatDetails->iMaxCombatStr = 0;
		pCombatDetails->iCurrHitPoints = 0;
		pCombatDetails->iMaxHitPoints = 0;
		pCombatDetails->iCurrCombatStr = 0;
		pCombatDetails->eOwner = getOwnerINLINE();
		pCombatDetails->eVisualOwner = getVisualOwner();
		pCombatDetails->sUnitName = getName().GetCString();
	}

	if (baseCombatStr() == 0)
	{
		return 0;
	}

	int iModifier = 0;
	int iExtraModifier;

	iExtraModifier = getExtraCombatPercent();
	iModifier += iExtraModifier;
	if (pCombatDetails != NULL)
	{
		pCombatDetails->iExtraCombatPercent = iExtraModifier;
	}

    iExtraModifier = getMightStrengthModifier();
    iModifier += iExtraModifier;

    if (GET_PLAYER(getOwnerINLINE()).isTough())
    {
        if (NULL != pPlot && bAttackingUnknownDefender == false)
        {
            iExtraModifier = GC.getDefineINT("TOUGH_DEFENSE_STRENGTH_MODIFIER");
            iModifier += iExtraModifier;
        }
        else if (NULL == pPlot && NULL == pAttacker)
        {
            iExtraModifier = GC.getDefineINT("TOUGH_ATTACK_STRENGTH_MODIFIER");
            iModifier += iExtraModifier;
        }
    }

    if (m_iTempStrengthModifier > 0)
    {
        iModifier += m_iTempStrengthModifier;
    }

	// do modifiers for animals and barbarians (leaving these out for bAttackingUnknownDefender case)
	if (pAttacker != NULL)
	{
		if (isAnimal())
		{
			if (pAttacker->isHuman())
			{
				iExtraModifier = GC.getHandicapInfo(GAME.getHandicapType()).getAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierTA = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = GC.getHandicapInfo(GAME.getHandicapType()).getAIAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIAnimalCombatModifierTA = iExtraModifier;
				}
			}
		}

		if (pAttacker->isAnimal())
		{
			if (isHuman())
			{
				iExtraModifier = -GC.getHandicapInfo(GAME.getHandicapType()).getAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierAA = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = -GC.getHandicapInfo(GAME.getHandicapType()).getAIAnimalCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIAnimalCombatModifierAA = iExtraModifier;
				}
			}
		}

		if (isBarbarian())
		{
			if (pAttacker->isHuman())
			{
				iExtraModifier = GC.getHandicapInfo(GAME.getHandicapType()).getBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = GC.getHandicapInfo(GAME.getHandicapType()).getAIBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			if (pAttacker->getLegionHero() != NO_HERO)
			{
				int iBarbarianModifier = 0;
			    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
			    {
			        if (GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier() != 0)
			        {
			            if (pAttacker->isHasPromotion((PromotionTypes)iI))
			            {
			                iBarbarianModifier += GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier();
			            }
			        }
			    }

			    if (iBarbarianModifier != 0)
			    {
			        iExtraModifier = -iBarbarianModifier;
			        iModifier += iExtraModifier;
			        if (pAttacker->isHuman())
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iBarbarianCombatModifierTB += iExtraModifier;
			            }
			        }
			        else
			        {
			            if (pCombatDetails != NULL)
			            {
                            pCombatDetails->iAIBarbarianCombatModifierTB += iExtraModifier;
			            }
			        }
			    }
			}
		}

		if (pAttacker->isBarbarian())
		{
			if (isHuman())
			{
				iExtraModifier = -GC.getHandicapInfo(GAME.getHandicapType()).getBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iBarbarianCombatModifierAB = iExtraModifier;
				}
			}
			else
			{
				iExtraModifier = -GC.getHandicapInfo(GAME.getHandicapType()).getAIBarbarianCombatModifier();
				iModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAIBarbarianCombatModifierTB = iExtraModifier;
				}
			}
			if (getLegionHero() != NO_HERO)
			{
			    int iBarbarianModifier = 0;
			    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
			    {
			        if (GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier() != 0)
			        {
			            if (isHasPromotion((PromotionTypes)iI))
			            {
			                iBarbarianModifier += GC.getPromotionInfo((PromotionTypes)iI).getBarbarianModifier();
			            }
			        }
			    }

			    if (iBarbarianModifier != 0)
			    {
			        iExtraModifier = iBarbarianModifier;
			        iModifier += iExtraModifier;
			        if (isHuman())
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iBarbarianCombatModifierAB += iExtraModifier;
			            }
			        }
			        else
			        {
			            if (pCombatDetails != NULL)
			            {
			                pCombatDetails->iAIBarbarianCombatModifierAB += iExtraModifier;
			            }
			        }
			    }
			}
		}

		if (getLegionHero() != NO_HERO || getLeaderUnitType() != NO_UNIT)
		{
		    if (pAttacker->getHeroType() != NO_HERO)
		    {
				PROFILE_BEGIN("maxCombatStr::legionStrMod");
		        int iLegionStrModifier = 0;
		        for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		        {
		            if (GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier() != 0)
		            {
		                if (pAttacker->isHasPromotion((PromotionTypes)iI))
		                {
		                    iLegionStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier();
		                }
		            }
		        }

		        if (iLegionStrModifier != 0)
		        {
		            iExtraModifier = -iLegionStrModifier;
		            iModifier += iExtraModifier;
		        }

				PROFILE_END();
		    }
		}

		if (pAttacker->getLegionHero() != NO_HERO || pAttacker->getLeaderUnitType() != NO_UNIT)
		{
		    if (getHeroType() != NO_HERO)
		    {
		        int iLegionStrModifier = 0;
		        for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		        {
		            if (GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier() != 0)
		            {
		                if (isHasPromotion((PromotionTypes)iI))
		                {
		                    iLegionStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getVersusEnemyCoreTroopStrengthModifier();
		                }
		            }
		        }

		        if (iLegionStrModifier != 0)
		        {
		            iExtraModifier = iLegionStrModifier;
		            iModifier += iExtraModifier;
		        }
		    }
		}

		if (pAttacker->getLegionHero() != NO_HERO)
		{
		    if (GET_PLAYER(getOwnerINLINE()).getStateReligion() == NO_RELIGION || !GC.getReligionInfo((ReligionTypes)GET_PLAYER(getOwnerINLINE()).getStateReligion()).isGoodReligion())
            {
                PROFILE_BEGIN("maxCombatStr::religionStrMod");
				int iNonGoodReligionStrengthModifier = 0;
                for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                {
                    if (pAttacker->isHasPromotion((PromotionTypes)iI))
                    {
                        iNonGoodReligionStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getNonGoodReligionStrengthModifier();
                    }
                }
                if (iNonGoodReligionStrengthModifier > 0)
                {
                    iExtraModifier = -iNonGoodReligionStrengthModifier;
		            iModifier += iExtraModifier;
                }
				PROFILE_END();
            }
		}

		if (getLegionHero() != NO_HERO)
		{
		    if (GET_PLAYER(pAttacker->getOwnerINLINE()).getStateReligion() == NO_RELIGION || !GC.getReligionInfo((ReligionTypes)GET_PLAYER(pAttacker->getOwnerINLINE()).getStateReligion()).isGoodReligion())
            {
                int iNonGoodReligionStrengthModifier = 0;
                for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                {
                    if (isHasPromotion((PromotionTypes)iI))
                    {
                        iNonGoodReligionStrengthModifier += GC.getPromotionInfo((PromotionTypes)iI).getNonGoodReligionStrengthModifier();
                    }
                }
                if (iNonGoodReligionStrengthModifier > 0)
                {
                    iExtraModifier = iNonGoodReligionStrengthModifier;
		            iModifier += iExtraModifier;
                }
            }
		}

		if (pAttacker->isCoreTroop())
		{
            if (isGuard())
            {
                iExtraModifier = GC.getDefineINT("HERO_GUARDSMAN_CORE_TROOP_MODIFIER");
                iModifier += iExtraModifier;
            }
		}
	}

	// add defensive bonuses (leaving these out for bAttackingUnknownDefender case)
	if (pPlot != NULL)
	{
		if (!noDefensiveBonus() && (pAttacker == NULL || !(!pPlot->isCity(true) && pAttacker->isIgnoreDefenderTerrain())))
		{
			iExtraModifier = pPlot->defenseModifier(getTeam(), (pAttacker != NULL) ? pAttacker->ignoreBuildingDefense() : true);
			if (getPlotDefenseModifier() != 0)
			{
			    iExtraModifier *= (100 + getPlotDefenseModifier());
			    iExtraModifier /= 100;
			}

			iModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iPlotDefenseModifier = iExtraModifier;
			}
		}

		iExtraModifier = fortifyModifier();
		iModifier += iExtraModifier;
		if (pCombatDetails != NULL)
		{
			pCombatDetails->iFortifyModifier = iExtraModifier;
		}

		if (pPlot->isCity(true, getTeam()))
		{
			iExtraModifier = cityDefenseModifier();
			iModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iCityDefenseModifier = iExtraModifier;
			}
		}

        if (pAttacker == NULL || !(!pPlot->isCity(true) && pAttacker->isIgnoreDefenderTerrain()))
        {
            // The following lines indented once
            if (pPlot->isHills())
            {
                iExtraModifier = hillsDefenseModifier();
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iHillsDefenseModifier = iExtraModifier;
                }
            }

            if (pPlot->getFeatureType() != NO_FEATURE)
            {
                iExtraModifier = featureDefenseModifier(pPlot->getFeatureType());
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iFeatureDefenseModifier = iExtraModifier;
                }
            }
            else
            {
                iExtraModifier = terrainDefenseModifier(pPlot->getTerrainType());
                iModifier += iExtraModifier;
                if (pCombatDetails != NULL)
                {
                    pCombatDetails->iTerrainDefenseModifier = iExtraModifier;
                }
            }
        }
	}

	// if we are attacking to an plot with an unknown defender, the calc the modifier in reverse
	if (bAttackingUnknownDefender)
	{
		pAttacker = this;
	}

	// calc attacker bonueses
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                       09/20/09                                jdog5000      */
/**                                                                                              */
/** Bugfix                                                                                       */
/*************************************************************************************************/
/* original code
	if (pAttacker != NULL)
*/
	if (pAttacker != NULL && pAttackedPlot != NULL)
/*************************************************************************************************/
/** UNOFFICIAL_PATCH                        END                                                  */
/*************************************************************************************************/
	{
		int iTempModifier = 0;

		if (pAttackedPlot->isCity(true, getTeam()))
		{
			iExtraModifier = -pAttacker->cityAttackModifier();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iCityAttackModifier = iExtraModifier;
			}

			if (pAttacker->isBarbarian())
			{
				iExtraModifier = GC.getDefineINT("CITY_BARBARIAN_DEFENSE_MODIFIER");
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCityBarbarianDefenseModifier = iExtraModifier;
				}
			}
		}

		if (pAttackedPlot->isHills())
		{
			iExtraModifier = -pAttacker->hillsAttackModifier();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iHillsAttackModifier = iExtraModifier;
			}
		}

		if (pAttackedPlot->getFeatureType() != NO_FEATURE)
		{
			iExtraModifier = -pAttacker->featureAttackModifier(pAttackedPlot->getFeatureType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iFeatureAttackModifier = iExtraModifier;
			}
		}
		else
		{
			iExtraModifier = -pAttacker->terrainAttackModifier(pAttackedPlot->getTerrainType());
            iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iTerrainAttackModifier = iExtraModifier;
			}
		}

		// only compute comparisions if we are the defender with a known attacker
		if (!bAttackingUnknownDefender)
		{
			FAssertMsg(pAttacker != this, "pAttacker is not expected to be equal with this");

			iExtraModifier = unitClassDefenseModifier(pAttacker->getUnitClassType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iClassDefenseModifier = iExtraModifier;
			}

			iExtraModifier = -pAttacker->unitClassAttackModifier(getUnitClassType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iClassAttackModifier = iExtraModifier;
			}

			if (pAttacker->getUnitCombatType() != NO_UNITCOMBAT)
			{
				iExtraModifier = unitCombatModifier(pAttacker->getUnitCombatType());
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCombatModifierA = iExtraModifier;
				}
			}
			if (getUnitCombatType() != NO_UNITCOMBAT)
			{
				iExtraModifier = -pAttacker->unitCombatModifier(getUnitCombatType());
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iCombatModifierT = iExtraModifier;
				}
			}

			iExtraModifier = domainModifier(pAttacker->getDomainType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iDomainModifierA = iExtraModifier;
			}

			iExtraModifier = -pAttacker->domainModifier(getDomainType());
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iDomainModifierT = iExtraModifier;
			}

			if (pAttacker->isAnimal())
			{
				iExtraModifier = animalCombatModifier();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierA = iExtraModifier;
				}
			}

			if (isAnimal())
			{
				iExtraModifier = -pAttacker->animalCombatModifier();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAnimalCombatModifierT = iExtraModifier;
				}
			}
		}

		if (!(pAttacker->isRiver()))
		{
			if (pAttacker->plot()->isRiverCrossing(directionXY(pAttacker->plot(), pAttackedPlot)))
			{
				iExtraModifier = -GC.getRIVER_ATTACK_MODIFIER();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iRiverAttackModifier = iExtraModifier;
				}
			}
		}

		if (!(pAttacker->isAmphib()))
		{
			if (!(pAttackedPlot->isWater()) && pAttacker->plot()->isWater())
			{
				iExtraModifier = -GC.getAMPHIB_ATTACK_MODIFIER();
				iTempModifier += iExtraModifier;
				if (pCombatDetails != NULL)
				{
					pCombatDetails->iAmphibAttackModifier = iExtraModifier;
				}
			}
		}

		if (pAttacker->getKamikazePercent() != 0)
		{
			iExtraModifier = pAttacker->getKamikazePercent();
			iTempModifier += iExtraModifier;
			if (pCombatDetails != NULL)
			{
				pCombatDetails->iKamikazeModifier = iExtraModifier;
			}
		}

		if (pAttackedPlot->isRiver() || pAttackedPlot->isCoastalLand(0))
		{
		    if (pAttacker != this)
		    {
	            if (getWaterSideStrengthModifier() != 0)
	            {
	                iExtraModifier = getWaterSideStrengthModifier();
	                iTempModifier += iExtraModifier;
	            }
		    }

			if (pAttacker->getWaterSideStrengthModifier() != 0)
            {
                iExtraModifier = -pAttacker->getWaterSideStrengthModifier();
                iTempModifier += iExtraModifier;
            }
        }

        if (pAttacker->movesLeft() > GC.getMOVE_DENOMINATOR())
        {
            int iPostAttackMove = pAttacker->movesLeft() - std::max(GC.getMOVE_DENOMINATOR(), pAttackedPlot->movementCost(pAttacker, pAttacker->plot()));
            iPostAttackMove /= GC.getMOVE_DENOMINATOR();
            if (iPostAttackMove > 0)
            {
                if (pAttacker->getLegionHero() != NO_HERO)
                {
					int iMoveAttackStrModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getMoveAttackStrengthModifier() != 0)
                        {
                            if (pAttacker->isHasPromotion((PromotionTypes)iI))
                            {
                                iMoveAttackStrModifier += GC.getPromotionInfo((PromotionTypes)iI).getMoveAttackStrengthModifier();
                            }
                        }
                    }

                    if (iMoveAttackStrModifier != 0)
                    {
                        iExtraModifier = -(iMoveAttackStrModifier * iPostAttackMove);
                        iTempModifier += iExtraModifier;
                    }
                }
            }
        }

		// if we are attacking an unknown defender, then use the reverse of the modifier
		if (bAttackingUnknownDefender)
		{
			iModifier -= iTempModifier;
		}
		else
		{
			iModifier += iTempModifier;
		}
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iModifierTotal = iModifier;
		pCombatDetails->iBaseCombatStr = baseCombatStr();
	}

	if (iModifier > 0)
	{
		iCombat = (baseCombatStr() * (iModifier + 100));
	}
	else
	{
		iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
	}

	if (pCombatDetails != NULL)
	{
		pCombatDetails->iCombat = iCombat;
		pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
		pCombatDetails->iCurrHitPoints = currHitPoints();
		pCombatDetails->iMaxHitPoints = maxHitPoints();
		pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
	}

	if (pAttacker != NULL && pAttacker != this)
	{
	    // if this is the defender with a known attacker
	    if (getLegionHero() != NO_HERO || pAttacker->getLegionHero() != NO_HERO)
	    {
	        int iCurrCombatStr = std::max(1, iCombat) * currHitPoints() / maxHitPoints();
	        if (iCurrCombatStr < pAttacker->currNoStrChangeCombatStr(NULL, NULL))
	        {
	            if (getLegionHero() != NO_HERO)
	            {                 
					int iBetterStrengthEnemyModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier() != 0)
                        {
                            if (isHasPromotion((PromotionTypes)iI))
                            {
                                iBetterStrengthEnemyModifier += GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier();
                            }
                        }
                    }

                    if (iBetterStrengthEnemyModifier != 0)
                    {
                        iModifier += iBetterStrengthEnemyModifier;

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iModifierTotal = iModifier;
                            pCombatDetails->iBaseCombatStr = baseCombatStr();
                        }

                        if (iModifier > 0)
                        {
                            iCombat = (baseCombatStr() * (iModifier + 100));
                        }
                        else
                        {
                            iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
                        }

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iCombat = iCombat;
                            pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
                            pCombatDetails->iCurrHitPoints = currHitPoints();
                            pCombatDetails->iMaxHitPoints = maxHitPoints();
                            pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
                        }
	                }
	            }
	        }
	        else if (iCurrCombatStr > pAttacker->currNoStrChangeCombatStr(NULL, NULL))
	        {
	            if (pAttacker->getLegionHero() != NO_HERO)
	            {
                    int iBetterStrengthEnemyModifier = 0;
                    for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
                    {
                        if (GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier() != 0)
                        {
                            if (pAttacker->isHasPromotion((PromotionTypes)iI))
                            {
                                iBetterStrengthEnemyModifier += GC.getPromotionInfo((PromotionTypes)iI).getBetterStrengthEnemyModifier();
                            }
                        }
                    }

                    if (iBetterStrengthEnemyModifier != 0)
                    {
                        iModifier += -iBetterStrengthEnemyModifier;

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iModifierTotal = iModifier;
                            pCombatDetails->iBaseCombatStr = baseCombatStr();
                        }

                        if (iModifier > 0)
                        {
                            iCombat = (baseCombatStr() * (iModifier + 100));
                        }
                        else
                        {
                            iCombat = ((baseCombatStr() * 10000) / (100 - iModifier));
                        }

                        if (pCombatDetails != NULL)
                        {
                            pCombatDetails->iCombat = iCombat;
                            pCombatDetails->iMaxCombatStr = std::max(1, iCombat);
                            pCombatDetails->iCurrHitPoints = currHitPoints();
                            pCombatDetails->iMaxHitPoints = maxHitPoints();
                            pCombatDetails->iCurrCombatStr = ((pCombatDetails->iMaxCombatStr * pCombatDetails->iCurrHitPoints) / pCombatDetails->iMaxHitPoints);
                        }
	                }
	            }
	        }
	    }
	}

	return std::max(1, iCombat);
}


int CvUnit::currNoStrChangeCombatStr(const CvPlot* pPlot, const CvUnit* pAttacker, CombatDetails* pCombatDetails) const
{
	return ((maxNoStrChangeCombatStr(pPlot, pAttacker, pCombatDetails) * currHitPoints()) / maxHitPoints());
}


int CvUnit::currNoStrChangeFirepower(const CvPlot* pPlot, const CvUnit* pAttacker) const
{
	return ((maxNoStrChangeCombatStr(pPlot, pAttacker, NULL) + currNoStrChangeCombatStr(pPlot, pAttacker, NULL) + 1) / 2);
}

bool CvUnit::canCombineCamp(CvPlot* pPlot) const
{
	PROFILE_FUNC();

    if (!getUnitInfo().isCamp())
    {
        return false;
    }

    if (pPlot == NULL)
    {
        return false;
    }

    if (plot() == NULL)
    {
        return false;
    }

	SanguoCampTypes eCamp = getCampType();

    if (GC.getSanguoCampInfo(eCamp).getUpgradeUnitType() == NO_UNIT)
    {
        return false;
    }

    int iCount = pPlot->plotCount(PUF_isUnitType, getUnitType(), -1, getOwnerINLINE());
    if (getX() == pPlot->getX() && getY() == pPlot->getY())
    {
        if (iCount < 2)
        {
            return false;
        }
    }
    else if (iCount < 1)
    {
        return false;
    }

    TraitTypes eTrait = (TraitTypes)GC.getSanguoCampInfo(eCamp).getBonusUnitSprawlingTrait();
    if (eTrait == NO_TRAIT)
    {
        return false;
    }
    else if (!GET_PLAYER(getOwnerINLINE()).hasTrait(eTrait))
    {
        return false;
    }

    return true;
}

bool CvUnit::combineCamp()
{
    if (!canCombineCamp(plot()))
    {
        return false;
    }

    CvUnit* pCombineWithCamp = plot()->plotCheck(PUF_isUnitType, getUnitType(), -1, getOwnerINLINE(), NO_TEAM, PUF_isNotSameUnit, getID(), -1);
    FAssert (pCombineWithCamp != NULL);

    CvUnit* pCombinedCamp = GET_PLAYER(getOwnerINLINE()).initUnit((UnitTypes)GC.getSanguoCampInfo(getCampType()).getUpgradeUnitType(), plot()->getX_INLINE(), plot()->getY_INLINE());
    FAssert (pCombinedCamp != NULL);
    pCombineWithCamp->kill(false);
    pCombinedCamp->NotifyEntity(MISSION_FOUND);
    kill(false);

    return true;
}

bool CvUnit::canNineTransformation() const
{
	PROFILE_FUNC();

	if (getHeroType() == NO_HERO)
	{
		return false;
	}

    if (getLegionHero() == NO_HERO)
    {
        return false;
    }
	
	if (hasMoved())
	{
		return false;
	}

	if (!GC.getHeroInfo(getHeroType()).isNineTransformation())
	{
        if (getLeaderHero() == NO_HERO)
        {
            return false;
        }
        else
        {
            bool bFound = false;
            for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
            {
                if (GAME.getHeroPromotionStatus(getLeaderHero(), (PromotionTypes)iI) == 2)
                {
                    if (GAME.getHeroPromotionStatus(getHeroType(), (PromotionTypes)iI) != 2)
                    {
                        bFound = true;
                        break;
                    }
                }
            }
            if (!bFound)
            {
                return false;
            }
        }
    }

	if (GAME.isLegionUnitMoved(getHeroType()))
	{
		return false;
	}

	return true;
}

bool CvUnit::nineTransformation()
{
	PROFILE_FUNC();

	if (isHuman())
	{
		CyArgsList argsList;
        argsList.add(getHeroType());
        gDLL->getPythonIFace()->callFunction(PYScreensModule, "showNineTransformationScreen", argsList.makeFunctionArgs());
	    GAME.setLegionUnitMoved(getHeroType(), true);
		return true;
	}
	else
	{
		int iBestValue = 0, iValue = 0;
		PromotionTypes eBestPromotion = NO_PROMOTION;
		for (int iI = 0; iI < GC.getNumPromotionInfos(); iI++)
		{
			if (GAME.isHeroNineTransformationPromotion(getHeroType(), (PromotionTypes)iI))
			{
				iValue = GAME.AI_heroPromotionValue(getHeroType(), (PromotionTypes)iI, true);
				if (iValue > iBestValue)
				{
					iBestValue = iValue;
					eBestPromotion = (PromotionTypes)iI;
				}
			}
		}
		if (eBestPromotion != NO_PROMOTION)
		{
			GAME.setHeroCurrentNineTransformationPromotion(getHeroType(), eBestPromotion);
			return true;
		}
		return false;
	}
}

int CvUnit::getSurroundingEnemyHealthChange() const
{
    return m_iSurroundingEnemyHealthChange;
}

void CvUnit::changeSurroundingEnemyHealthChange(int iChange)
{
	m_iSurroundingEnemyHealthChange += iChange;
	FAssert (m_iSurroundingEnemyHealthChange <= 0);
}

void CvUnit::changeBaseCombatStrength(int iChange)
{
	m_iBaseCombat += iChange;
	FAssert (m_iBaseCombat >= 0);
}

bool CvUnit::isDefendFromCharge() const
{
    return (m_iDefendFromChargeCount > 0);
}

void CvUnit::changeDefendFromChargeCount(int iChange)
{
	m_iDefendFromChargeCount += iChange;
	FAssert (m_iDefendFromChargeCount >= 0);
}

bool CvUnit::isHiddenNationality() const
{
    return (m_iHiddenNationalityCount > 0);
}

void CvUnit::changeHiddenNationalityCount(int iChange)
{
	m_iHiddenNationalityCount += iChange;
	FAssert (m_iHiddenNationalityCount >= 0);
}

void CvUnit::changeAlwaysHostileCount(int iChange)
{
	m_iAlwaysHostileCount += iChange;
	FAssert (m_iAlwaysHostileCount >= 0);
}

int CvUnit::getAdditionalAttackChance() const
{
    return m_iAdditionalAttackChance;
}

void CvUnit::changeAdditionalAttackChance(int iChange)
{
	m_iAdditionalAttackChance += iChange;
	FAssert (m_iAdditionalAttackChance >= 0);
}	

bool CvUnit::isWarGod() const
{
	return (m_iWarGodCount > 0);
}

void CvUnit::changeWarGodCount(int iChange)
{
	m_iWarGodCount += iChange;
	FAssert (m_iWarGodCount >= 0);
}

bool CvUnit::isIgnoreDefenderTerrain() const
{
	return (m_iIgnoreDefenderTerrainCount > 0);
}

void CvUnit::changeIgnoreDefenderTerrainCount(int iChange)
{
	m_iIgnoreDefenderTerrainCount += iChange;
	FAssert (m_iIgnoreDefenderTerrainCount >= 0);
}

int CvUnit::getPlotDefenseModifier() const
{
	return m_iPlotDefenseModifier;
}

void CvUnit::changePlotDefenseModifier(int iChange)
{
	m_iPlotDefenseModifier += iChange;
}

int CvUnit::getWaterSideStrengthModifier() const
{
	return m_iWaterSideStrModifier;
}

void CvUnit::changeWaterSideStrengthModifier(int iChange)
{
	m_iWaterSideStrModifier += iChange;
}

bool CvUnit::isGuard() const
{
	return (m_iGuardCount > 0);
}

void CvUnit::changeGuardCount(int iChange)
{
	m_iGuardCount += iChange;
	FAssert (m_iGuardCount >= 0);
}

int CvUnit::getLegionTurn() const
{
	return m_iLegionTurn;
}

int CvUnit::getMightStrengthModifier() const
{
    if (getHeroType() == NO_HERO || getLegionHero() == NO_HERO)
    {
        return 0;
    }

    int iMight = GC.getHeroInfo(getHeroType()).getMight();
    if (iMight <= 60)
    {
        return (iMight / 20);
    }
    if (iMight <= 70)
    {
        return (3 + (iMight - 60) / 10);
    }
    if (iMight <= 80)
    {
        return (4 + (iMight - 70) / 5);
    }
    if (iMight <= 90)
    {
        return (6 + (iMight - 80) / 2);
    }
    return (11 + (iMight - 90));
}

bool CvUnit::formAdvancedLegion(bool bRegroup, bool bRegroupDone)
{
    FAssert (getHeroType() >= 0 && getHeroType() < GC.getNumHeroInfos());

    if (!bRegroup)
    {
        FAssert (isNakedHero());
        if (GAME.getHeroLegionSize(getHeroType()) == 0)
        {
            CyArgsList argsList;
            argsList.add(getHeroType());
            argsList.add(bRegroup);
            gDLL->getPythonIFace()->callFunction(PYScreensModule, "showAdvancedLegionScreen", argsList.makeFunctionArgs());
            return false;
        }
        else
        {
            GAME.clearHeroBackUpList(getHeroType());
            CvUnit* pLeaderTroop = processAdvancedLegion(false, false);
            FAssert (pLeaderTroop != NULL);
			GAME.setLegionUnitMoved(pLeaderTroop->getLeaderHero(), true);
            GAME.setHeroUnitID(pLeaderTroop->getLeaderHero(), pLeaderTroop->getID());

            if (plot()->isActiveVisible(false))
            {
                NotifyEntity(MISSION_LEAD);
            }

            if (isHuman() && getOwnerINLINE() == GC.getGame().getActivePlayer())
            {
				gDLL->getInterfaceIFace()->selectGroup(pLeaderTroop, gDLL->shiftKey(), gDLL->ctrlKey(), gDLL->altKey());
			}

			// Sanguo Mod tutorial, start, by poyuzhe 09.27.09
			if (!GET_PLAYER(getOwnerINLINE()).isFeatAccomplished(FEAT_FORM_ADVANCED_LEGION))
			{
				GET_PLAYER(getOwnerINLINE()).setFeatAccomplished(FEAT_FORM_ADVANCED_LEGION, true);
				if (GET_PLAYER(getOwnerINLINE()).isFeatPopup())
				{
					CyArgsList argsList;

					argsList.add(getOwnerINLINE());
					argsList.add(pLeaderTroop->getLeaderHero());

					gDLL->getPythonIFace()->callFunction(PYSanguoModule, "doFormAdvancedLegionFeat", argsList.makeFunctionArgs());
				}
			}
			// Sanguo Mod Tutorial, end

            kill(false);

            return true;
        }
    }
    else
    {
        if (!bRegroupDone)
        {
            FAssert (isLeaderTroop());
            GAME.backUpHeroLegionMember(getLeaderHero());
            CyArgsList argsList;
            argsList.add(getLeaderHero());
            argsList.add(bRegroup);
            gDLL->getPythonIFace()->callFunction(PYScreensModule, "showAdvancedLegionScreen", argsList.makeFunctionArgs());
            return false;
        }
        else
        {
            FAssert (isCoreTroop());

			GAME.swapHeroLegionList(getLeaderHero());

			std::vector< std::pair<HeroTypes, PromotionTypes> > aNineTransformations;

			for (int iI = 0; iI < GAME.getHeroLegionSize(getLeaderHero()); iI++)
			{
				CvUnit* pLoopUnit = GET_PLAYER((PlayerTypes)GAME.getHeroDebutStatus(getLeaderHero())).getUnit(GAME.getHeroLegionMemberAt(getLeaderHero(), iI));
				if (pLoopUnit != NULL)
				{
					HeroTypes eLoopHero = pLoopUnit->getHeroType();
					PromotionTypes eLoopPromotion = GAME.getHeroCurrentNineTransformationPromotion(eLoopHero);
					if (eLoopPromotion != NO_PROMOTION)
					{
						aNineTransformations.push_back( std::make_pair(eLoopHero, eLoopPromotion) );
					}
				}
			}

			GAME.swapHeroLegionList(getLeaderHero());

			CvUnit* pLeaderTroop = processAdvancedLegion(true, false, true);

			for (std::vector< std::pair<HeroTypes, PromotionTypes> >::const_iterator iter = aNineTransformations.begin(); iter != aNineTransformations.end(); ++iter)
			{
				if (!GAME.isHeroNineTransformationPromotion((*iter).first, (*iter).second))
				{
					GAME.setHeroCurrentNineTransformationPromotion((*iter).first, NO_PROMOTION);
				}
			}

            FAssert (pLeaderTroop != NULL);

            GAME.setLegionUnitMoved(pLeaderTroop->isNakedHero() ? pLeaderTroop->getHeroType() : pLeaderTroop->getLeaderHero(), true);
            GAME.setHeroUnitID(pLeaderTroop->isNakedHero() ? pLeaderTroop->getHeroType() : pLeaderTroop->getLeaderHero(), pLeaderTroop->getID());

            if (isHuman() && getOwnerINLINE() == GC.getGame().getActivePlayer())
            {
				gDLL->getInterfaceIFace()->selectGroup(pLeaderTroop, gDLL->shiftKey(), gDLL->ctrlKey(), gDLL->altKey());
			}

            return true;
        }
    }
}

bool CvUnit::canFormAdvancedLegion(const CvPlot* pPlot) const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (getHeroType() == NO_HERO)
    {
        return false;
    }

	if (hasMoved())
	{
		return false;
	}

    if (getLegionHero() != NO_HERO)
    {
        return false;
    }

    if (NULL == pPlot)
    {
        pPlot = plot();
    }

    if (GAME.isLegionUnitMoved(getHeroType()))
    {
        return false;
    }

    if (GC.getHeroInfo(getHeroType()).getLeaderType() != GET_PLAYER(getOwnerINLINE()).getLeaderType())
    {
        if (GAME.getHeroLevel(getHeroType()) < 8)
        {
            return false;
        }
    }

    FAssert (pPlot != NULL);

    if (pPlot->plotCount(PUF_canBeLeaderTroop, getOwnerINLINE(), getID()) == 0)
    {
        return false;
    }

    return true;
}

bool CvUnit::canDismissAdvancedLegion() const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (!isLeaderTroop())
    {
        return false;
    }

    if (plot()->isWater())
    {
		return false;
	}

	if (hasMoved())
	{
		return false;
	}

	if (GAME.isLegionUnitMoved(getLeaderHero()))
	{
	    return false;
    }

	for (int iI = 0; iI < GAME.getHeroLegionSize(getLeaderHero()); iI++)
	{
		CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(getLeaderHero(), iI));
		FAssert (pMember != NULL);
        if (GAME.isLegionUnitMoved(pMember->getHeroType()))
		{
		    return false;
		}
		if (pMember->isHurt())
		{
			return false;
		}
    }

    return true;
}

CvUnit* CvUnit::dismissAdvancedLegion(bool bTemp)
{
    FAssert (getHeroType() >= 0 && getHeroType() < GC.getNumHeroInfos());
    FAssert (isLeaderTroop());
	std::vector< std::pair<HeroTypes, PromotionTypes> > aNineTransformations;
	
	if (!bTemp)
	{
		for (int iI = 0; iI < GAME.getHeroLegionSize(getLeaderHero()); iI++)
		{
			CvUnit* pLoopUnit = GET_PLAYER((PlayerTypes)GAME.getHeroDebutStatus(getLeaderHero())).getUnit(GAME.getHeroLegionMemberAt(getLeaderHero(), iI));
			if (pLoopUnit != NULL)
			{
				HeroTypes eLoopHero = pLoopUnit->getHeroType();
				PromotionTypes eLoopPromotion = GAME.getHeroCurrentNineTransformationPromotion(eLoopHero);
				if (eLoopPromotion != NO_PROMOTION)
				{
					aNineTransformations.push_back( std::make_pair(eLoopHero, eLoopPromotion) );
				}
			}
		}
	}

	GAME.backUpHeroLegionMember(getLeaderHero());
    GAME.clearHeroLegionMember(getLeaderHero());

    CvUnit* pHero = processAdvancedLegion(true, false, true);

	for (std::vector< std::pair<HeroTypes, PromotionTypes> >::const_iterator iter = aNineTransformations.begin(); iter != aNineTransformations.end(); ++iter)
	{
		if (!GAME.isHeroNineTransformationPromotion((*iter).first, (*iter).second))
		{
			GAME.setHeroCurrentNineTransformationPromotion((*iter).first, NO_PROMOTION);
		}
	}

	if (pHero != NULL)
	{
		GAME.setLegionUnitMoved(pHero->getHeroType(), true);
	}

	gDLL->getInterfaceIFace()->setDirty(PlotListButtons_DIRTY_BIT, true);

    return pHero;
}

bool CvUnit::canJoinAdvancedLegion(const CvUnit* pUnit) const
{
    FAssert (pUnit != NULL);
    FAssert (pUnit->isNakedHero() || pUnit->isLeaderTroop());

    if (getOwnerINLINE() != pUnit->getOwnerINLINE())
    {
        return false;
    }

    if (getID() == pUnit->getID())
    {
        return false;
    }

    if (pUnit->getHeroType() == NO_HERO)
    {
        return false;
    }

    if (!pUnit->isNakedHero() && !pUnit->isLeaderTroop())
    {
        return false;
    }

    if (!isCoreTroop())
    {
        return false;
    }

    if (getLeaderHero() != NO_HERO)
    {
        return false;
    }

    HeroTypes eHero = (pUnit->isNakedHero() ? pUnit->getHeroType() : pUnit->getLeaderHero());
    FAssert (eHero != NO_HERO);
    
    if (GAME.isHeroHasLegionMember(eHero, getID()))
    {
        return false;
    }

	if (GAME.getHeroLegionSize(eHero) >= GAME.getHeroCapacity(eHero))
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(eHero))
    {
        return false;
    }

    return true;
}

CvUnit* CvUnit::processAdvancedLegion(bool bRegroup, bool bAttacked, bool bKeepNineTransformation)
{
    PROFILE_FUNC();

    HeroTypes eLeaderHero, eLoopHero;
    int iI, iJ;
    CvUnit* pLoopHero;
    CvUnit* pLoopUnit;
    UnitCombatTypes eLoopUnitCombat;
    PromotionTypes eUnitCombatPromotionHigh = NO_PROMOTION, eUnitCombatPromotionLow = NO_PROMOTION, eUnitCombatPromotionMiddle;
    CvUnit* pReturnHero = NULL;

    if (!bRegroup)
    {
        eLeaderHero = getHeroType();
    }
    else
    {
        eLeaderHero = getLeaderHero();
        pReturnHero = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(eLeaderHero));
    }
    FAssert (eLeaderHero != NO_HERO);

    // If regroup mission, find out the leader troop situation, then give and take unique promotion of the leader hero
    if (bRegroup)
    {
        int iOldLeaderTroop = GAME.getHeroLegionBackUpMemberAt(eLeaderHero, 0);
        // 1. the old leader troop no longer in the legion
        // 2. the old leader troop still in the legion, but is no longer the leader troop
        // 3. the old leader troop is still the leader troop, then do nothing
        if (GAME.getHeroCoreTroop(eLeaderHero) != iOldLeaderTroop)
        {
            CvUnit* pOldLeaderTroop = GET_PLAYER(getOwnerINLINE()).getUnit(iOldLeaderTroop);
            CvUnit* pNewLeaderTroop = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(eLeaderHero));

            FAssert (pOldLeaderTroop != NULL);
            FAssert (pOldLeaderTroop->isCoreTroop());

            pReturnHero = pNewLeaderTroop;
            if (pNewLeaderTroop != NULL)
            {
                GAME.setHeroUnitID(eLeaderHero, pNewLeaderTroop->getID());
            }

            for (iI = 0; iI < GC.getNumPromotionInfos(); iI++)
            {
                PromotionTypes ePromotion = (PromotionTypes)iI;
                if (GAME.getHeroPromotionStatus(eLeaderHero, ePromotion) == 1)
                {
                    if (pOldLeaderTroop->isHasPromotion(ePromotion))
                    {
                        if (GAME.getHeroPromotionStatus(pOldLeaderTroop->getHeroType(), ePromotion) != 1)
                        {
                            pOldLeaderTroop->setHasPromotion(ePromotion, false);
                        }
                    }
                    if (pNewLeaderTroop != NULL)
                    {
                        pNewLeaderTroop->setHasPromotion(ePromotion, true);
                    }
                }
            }
        }
    }

    // Process those legions that are no longer in the advanced legion
    if (bRegroup)
    {
        for (iI = 0; iI < GAME.getHeroLegionBackUpSize(eLeaderHero); iI++)
        {
            if (!GAME.isHeroHasLegionMember(eLeaderHero, GAME.getHeroLegionBackUpMemberAt(eLeaderHero, iI)))
            {
                // Check unit combat level
                pLoopHero = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionBackUpMemberAt(eLeaderHero, iI));
                FAssert (pLoopHero != NULL);
                eLoopHero = pLoopHero->getHeroType();
                eLoopUnitCombat = pLoopHero->getUnitCombatType();

                if (!bKeepNineTransformation)
				{
					GAME.setHeroCurrentNineTransformationPromotion(eLoopHero, NO_PROMOTION);
				}

                for (iJ = 0; iJ < GC.getNumPromotionInfos(); iJ++)
                {
                    if (GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatType() == eLoopUnitCombat)
                    {
                        if (pLoopHero->isHasPromotion((PromotionTypes)iJ))
                        {
                            eUnitCombatPromotionHigh = (PromotionTypes)iJ;
                        }
                        if (GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatLevel() == GAME.getHeroUnitCombatLevel(eLoopHero, eLoopUnitCombat))
                        {
                            eUnitCombatPromotionLow = (PromotionTypes)iJ;
                        }
                    }
                }

                for (iJ = 0; iJ < GAME.getHeroLegionSize(eLoopHero); iJ++)
                {
                    pLoopUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(eLoopHero, iJ));
                    FAssert (pLoopUnit != NULL);
                    if (eUnitCombatPromotionHigh > eUnitCombatPromotionLow)
                    {
                        pLoopUnit->setHasPromotion(eUnitCombatPromotionHigh, false);
                        if (eUnitCombatPromotionLow != NO_PROMOTION)
                        {
                            pLoopUnit->setHasPromotion(eUnitCombatPromotionLow, true);
                        }
                    }
                    pLoopUnit->setLeaderHero(NO_HERO);
                }
            }
        }
    }

    // if there's no new leader troop, which means advanced legion dismissed
    if (bRegroup && pReturnHero == NULL)
    {
        pReturnHero = GET_PLAYER(getOwnerINLINE()).initHero(eLeaderHero, getX_INLINE(), getY_INLINE(), false);
        return pReturnHero;
    }

    // Process those legions that are new to the advanced legion (special case the new leader troop is not in old legion)
    for (iI = 0; iI < GAME.getHeroLegionSize(eLeaderHero); iI++)
    {
        if (!GAME.isHeroHasBackUpMember(eLeaderHero, GAME.getHeroLegionMemberAt(eLeaderHero, iI)))
        {
			pLoopHero = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(eLeaderHero, iI));
			FAssert (pLoopHero != NULL);
			eLoopHero = pLoopHero->getHeroType();
			
			// Check unit combat level      
            eLoopUnitCombat = pLoopHero->getUnitCombatType();
            eUnitCombatPromotionHigh = NO_PROMOTION;
            eUnitCombatPromotionLow = NO_PROMOTION;
            for (iJ = 0; iJ < GC.getNumPromotionInfos(); iJ++)
            {
                if (GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatType() == eLoopUnitCombat)
                {
                    if (GAME.getHeroUnitCombatLevel(eLeaderHero, eLoopUnitCombat) == GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatLevel())
                    {
                        eUnitCombatPromotionHigh = (PromotionTypes)iJ;
                    }
                    if (GAME.getHeroUnitCombatLevel(eLoopHero, eLoopUnitCombat) == GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatLevel())
                    {
                        eUnitCombatPromotionLow = (PromotionTypes)iJ;
                    }
                    if (GAME.getHeroUnitCombatLevel(eLoopHero, eLoopUnitCombat) + 1 == GC.getPromotionInfo((PromotionTypes)iJ).getPromotionUnitCombatLevel())
                    {
                        eUnitCombatPromotionMiddle = (PromotionTypes)iJ;
                    }
                }
				else if (!bRegroup && GAME.getHeroPromotionStatus(eLeaderHero, (PromotionTypes)iJ) == 1)
				{
					if (GAME.getHeroCurrentNineTransformationPromotion(eLeaderHero) != (PromotionTypes)iJ)
					{
						pLoopHero->setHasPromotion((PromotionTypes)iJ, true);
					}
				}
            }

            if (eUnitCombatPromotionHigh <= eUnitCombatPromotionLow)
            {
                eUnitCombatPromotionHigh = eUnitCombatPromotionLow;
            }
            else
            {
                eUnitCombatPromotionHigh = eUnitCombatPromotionMiddle;
            }

            for (iJ = 0; iJ < GAME.getHeroLegionSize(eLoopHero); iJ++)
            {
                pLoopUnit = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(eLoopHero, iJ));
                FAssert (pLoopUnit != NULL);
                if (eUnitCombatPromotionLow != NO_PROMOTION)
                {
                    pLoopUnit->setHasPromotion(eUnitCombatPromotionLow, false);
                }
                if (eUnitCombatPromotionHigh != NO_PROMOTION)
                {
                    pLoopUnit->setHasPromotion(eUnitCombatPromotionHigh, true);
                }
                pLoopUnit->setLeaderHero(eLeaderHero);
            }
        }
    }

    return GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(eLeaderHero));
}

bool CvUnit::isLeaderTroop() const
{
    if (!isCoreTroop())
    {
        return false;
    }

    if (getLeaderHero() == NO_HERO)
    {
        return false;
    }

    if (GAME.getHeroCoreTroop(getLeaderHero()) != getID())
    {
        return false;
    }

    return true;
}

HeroTypes CvUnit::getLeaderHero() const
{
    return m_eLeaderHero;
}

void CvUnit::setLeaderHero(HeroTypes eHero)
{
    m_eLeaderHero = eHero;
}

bool CvUnit::canBeLeaderTroop(const CvUnit* pUnit) const
{
    FAssert (NULL != pUnit);

    if (!canJoinAdvancedLegion(pUnit))
    {
        return false;
    }

    return true;
}

bool CvUnit::canJoinAdvancedLegion(const CvPlot* pPlot) const
{
    if (!isCoreTroop())
    {
        return false;
    }

    if (getLeaderHero() != NO_HERO)
    {
        return false;
    }

    if (pPlot == NULL)
    {
        pPlot = plot();
    }

    if (hasMoved())
    {
		return false;
	}

    if (pPlot->plotCount(PUF_canAcceptAdvancedLegionMember, getOwnerINLINE(), getID(), NO_PLAYER, NO_TEAM, PUF_hasNotMoved) == 0)
    {
        return false;
    }

    return true;
}

bool CvUnit::canAcceptAdvancedLegionMember(const CvUnit* pUnit) const
{
    if (!isLeaderTroop())
    {
        return false;
    }

    if ( GAME.getHeroLegionSize(getLeaderHero()) == 0)
    {
        return false;
    }

    if (getOwnerINLINE() != pUnit->getOwnerINLINE())
    {
        return false;
    }

    if ( GAME.getHeroLegionSize(getLeaderHero()) >= GAME.getHeroCapacity(getLeaderHero()))
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(getLeaderHero()))
    {
        return false;
    }

    return true;
}

bool CvUnit::joinAdvancedLegion(int iUnitID, bool bAsLeader)
{
    if (!isHuman())
    {
        if (!canJoinAdvancedLegion(plot()))
        {
            return false;
        }

        // AI do the join legion stuff
        return true;
    }
    else
    {
        if (iUnitID == -1)
        {
            int iLegionCount = plot()->plotCount(PUF_canAcceptAdvancedLegionMember, getOwnerINLINE(), getID());
            FAssert (iLegionCount > 0);

            if (iLegionCount == 1)
            {
                CvUnit* pHero = plot()->plotCheck(PUF_canAcceptAdvancedLegionMember, getOwnerINLINE(), getID());
                FAssert (pHero->isLeaderTroop());

				GAME.backUpHeroLegionMember(pHero->getLeaderHero());
				GAME.addHeroLegionMember(pHero->getLeaderHero(), getID());
                pHero->processAdvancedLegion(true, false);
                GAME.setLegionUnitMoved(getHeroType(), true);

                return true;
            }
            else
            {
                CvPopupInfo* pInfo = new CvPopupInfo(BUTTONPOPUP_PYTHON);
                if (NULL != pInfo)
                {
                    pInfo->setText(gDLL->getText("TXT_KEY_JOIN_ADVANCED_LEGION"));
                    pInfo->setData1(getID());
                    pInfo->setData2(getOwnerINLINE());
                    pInfo->setPythonModule(PYSanguoModule);
                    pInfo->setOnClickedPythonCallback("doJoinLegion");

                    // find all possible advanced legions to join
                    CvUnit* pHero = plot()->plotCheck(PUF_canAcceptAdvancedLegionMember, getOwnerINLINE(), getID());
                    while (pHero != NULL)
                    {
                        pHero->setTempCheck(true);
                        GET_PLAYER(getOwnerINLINE()).pushToJoinLegionTempHeroList(pHero->getID());
                        pInfo->addPythonButton(gDLL->getText("TXT_KEY_CHOOSE_ADVANCED_LEGION_HELP", GC.getHeroInfo(pHero->getLeaderHero()).getDescription(), GAME.getHeroUnitCombatLevel(pHero->getLeaderHero(), getUnitCombatType()), GC.getHeroInfo(pHero->getLeaderHero()).getLeadership()), "");
                        pHero = plot()->plotCheck(PUF_canAcceptAdvancedLegionMember, getOwnerINLINE(), getID(), NO_PLAYER, NO_TEAM, PUF_isNotTempCheck);
                    }

                    CvUnit* pCheckedUnit = plot()->plotCheck(PUF_isTempCheck);
                    while (pCheckedUnit != NULL)
                    {
                        pCheckedUnit->setTempCheck(false);
                        pCheckedUnit = plot()->plotCheck(PUF_isTempCheck);
                    }

                    pInfo->addPythonButton(gDLL->getText("TXT_KEY_SCREEN_CANCEL"), "");
					pInfo->setData3(GET_PLAYER(getOwnerINLINE()).getJoinLegionTempHeroListSize());
                    pInfo->setFlags(1);
                    gDLL->getInterfaceIFace()->addPopup(pInfo, getOwnerINLINE(), true);
                }

                return false;
            }
        }
        else
        {
            CvUnit* pHero = GET_PLAYER(getOwnerINLINE()).getUnit(iUnitID);
            FAssert (pHero->isLeaderTroop());

			GAME.backUpHeroLegionMember(pHero->getLeaderHero());
            if (!bAsLeader)
            {
                GAME.addHeroLegionMember(pHero->getLeaderHero(), getID());
            }
            else
            {
                GC.getGame().addHeroCoreTroop(pHero->getLeaderHero(), getID());
            }

            pHero->processAdvancedLegion(true, false);
            GAME.setLegionUnitMoved(getHeroType(), true);

            return true;
        }
    }
}

bool CvUnit::canRegroupAdvancedLegion(const CvPlot* pPlot) const
{
    PROFILE_FUNC();

	if (isDelayedDeath())
	{
		return false;
	}

	if (NO_UNIT == getUnitType())
	{
		return false;
	}

    if (!isLeaderTroop())
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(getLeaderHero()))
    {
        return false;
    }

    for (int iI = 0; iI < GAME.getHeroLegionSize(getLeaderHero()); iI++)
	{
		CvUnit* pMember = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroLegionMemberAt(getLeaderHero(), iI));
		FAssert (pMember != NULL);
        if (GAME.isLegionUnitMoved(pMember->getHeroType()))
		{
		    return false;
		}
    }

    return true;
}

bool CvUnit::canQuitAdvancedLegion() const
{
	PROFILE_FUNC();

	if (!isCoreTroop())
    {
        return false;
    }

    if (getLeaderHero() == NO_HERO)
    {
        return false;
    }

    if (GAME.isLegionUnitMoved(getLegionHero()))
    {
		return false;
	}

	CvUnit* pHero = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(getLeaderHero()));
	FAssert (pHero != NULL);
	if (pHero->hasMoved())
	{
		return false;
	}

    return true;
}

bool CvUnit::quitAdvancedLegion(bool bTemp)
{
    CvUnit* pLeaderTroop = GET_PLAYER(getOwnerINLINE()).getUnit(GAME.getHeroCoreTroop(getLeaderHero()));
    FAssert (pLeaderTroop != NULL);

	PromotionTypes eNineTransformation = bTemp ? NO_PROMOTION : GAME.getHeroCurrentNineTransformationPromotion(getHeroType());

    GAME.backUpHeroLegionMember(getLeaderHero());
    GAME.deleteHeroLegionMember(getLeaderHero(), getID());
    pLeaderTroop->processAdvancedLegion(true, false, true);

	if (eNineTransformation != NO_PROMOTION && !GAME.isHeroNineTransformationPromotion(getHeroType(), eNineTransformation))
	{
		GAME.setHeroCurrentNineTransformationPromotion(getHeroType(), NO_PROMOTION);
	}

    GAME.setLegionUnitMoved(getHeroType(), true);

    return true;
}
// Sanguo Mod Hero, end

